/*
 * Created on May 1, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 *
 */
package bt;

import java.io.File;
import java.io.FileReader;
import java.util.HashMap;

import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

/**
 * @author snshor
 * 
 * This class must be used only for extracting information from simple tags,
 * i.e. those with no embedded tags whatsoever, except when you know what you
 * are doing.
 */
public class SimpleXMLExtractor extends DefaultHandler {
	protected static final String DEFAULT_PARSER_NAME = "org.apache.xerces.parsers.SAXParser";

	XMLReader reader = null;

	HashMap handlers = new HashMap();

	Handler currentHandler = null;

	public SimpleXMLExtractor() {
	}

	public void parse(InputSource is) throws Exception {
		// Modification made on 2005-03-30:
		// using SAXParserFactory, see http://sam/bugzilla/show_bug.cgi?id=60
		//  reader = XMLReaderFactory.createXMLReader(DEFAULT_PARSER_NAME);
		//  reader = new SAXParser();
		//	reader = XMLReaderFactory.createXMLReader();
		SAXParserFactory spf = SAXParserFactory.newInstance();
		spf.setNamespaceAware(true);
		spf.setValidating(false);
		reader = spf.newSAXParser().getXMLReader();

		reader.setContentHandler(this);
		reader.setErrorHandler(this);
		reader.parse(is);
	}

	public void parse(File f) throws Exception {
		parse(new InputSource(new FileReader(f)));
	}

	/** Start element. */
	public void startElement(String uri, String local, String raw,
			Attributes attrs) throws SAXException {

		currentHandler = (Handler) handlers.get(local);

		//    System.out.println("Local = (" + local + ")" + (currentHandler !=
		// null) + "/" + (attrs != null));

		if (currentHandler != null && attrs != null)
			currentHandler.handleAttributes(attrs);
	}

	public void endElement(String uri, String local, String raw)
			throws SAXException {
		currentHandler = null;
	}

	/** Characters. */
	public void characters(char ch[], int start, int length)
			throws SAXException {
		if (currentHandler != null)
			currentHandler.handleChars(new String(ch, start, length));

	} // characters(char[],int,int);

	public void registerHandler(String tagName, Handler h) {
		handlers.put(tagName, h);
	}

	public static abstract class Handler {
		public void handleAttributes(Attributes attrs) {
		}

		public void handleChars(String chars) {
		}
	}

}