package com.openrules.dao;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * 
 */

import java.io.ByteArrayOutputStream;
import java.io.Closeable;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Properties;

public class DaoUtils {
	// utility
	public static boolean isEmpty( String s ) {
		return ( s == null || s.trim().length() == 0 );
	}
	
	public static void copy(InputStream is, OutputStream os) throws IOException {
		byte[] buf = new byte[1024*16];
		int len = 0;
		for(;;) {
			len = is.read(buf);
			if ( len <= 0 ) break;
			os.write(buf,0,len);
		}
	}

	public static void copy(Reader reader, Writer writer ) throws IOException {
		char[] buf = new char[1024*16];
		int len = 0;
		for(;;) {
			len = reader.read(buf);
			if ( len <= 0 ) break;
			writer.write(buf,0,len);
		}
	}
	
	public static void close(Closeable  target ){
		try { 
			target.close(); 
		} catch(Exception e ) { 
			// ignore
		}
	}

	public static byte[] readFileToByteArray(File file) throws IOException {
		InputStream is = new FileInputStream( file );
		ByteArrayOutputStream os = new ByteArrayOutputStream( (int)file.length() );
		try {
			copy( is, os );
			return os.toByteArray();
		} finally {
			close( is );
		}
	}
	
	public static String readAsString( Reader reader ) throws IOException {
		StringWriter sw = new StringWriter();
		copy( reader, sw );
		return sw.toString();
	}
	
	@SuppressWarnings("unchecked")
	public static <T> T newInstance( String className ) {
		try {
			return (T) Class.forName(className).newInstance();
		} catch (Exception e) {
			throw new RuntimeException( "Can not initialize DAO", e );
		}
	}
	
	public static Properties loadPropertiesFromClasspath( Class klass, String config ) {
		InputStream is = klass.getResourceAsStream(config);
		if (is == null)
			throw new DAOException(String.format(
					"Configuration file [%s] not found in classpath", config));
		Properties props = new Properties();
	
		try {
			props.load(is);
			return props;
		} catch (IOException e) {
			throw new DAOException(String.format("Error while reading '%s'",
					config), e);
	
		} finally {
			close(is);
		}
		
	}
	
	public static Properties loadPropertiesFromFile(String fileName) {
		FileInputStream propFile = null;
		try {
			propFile = new FileInputStream(fileName);
			Properties props = new Properties();
			props.load(propFile);
			return props;
		} catch (java.io.FileNotFoundException e) {
			throw new DAOException("Cannot find " + fileName);
		} catch (java.io.IOException e) {
			throw new DAOException("Error loading db properties from " + fileName);
		} finally {
			close(propFile);
		}
	}
	
	public static Properties getDefaultProperties() {
		Properties props = new Properties();
		// OpenRules Data Source Protocol Properties
		props.setProperty("openrules.protocol.name", "db");
		props.setProperty("openrules.protocol.class", "com.openrules.db.DbDataSourceHandler");
		// DB Access Properties
		props.setProperty(DaoOptions.USER,"embedded");
		props.setProperty(DaoOptions.PASSWORD,"none");
		props.setProperty(DaoOptions.CONNECT_URL, "none"); // has to be defined by a user, e.g.
				//"jdbc:derby:/_SVN/openrules.examples/RulesRepositoryDB/db/dbstorage;create=true");
		props.setProperty(DaoOptions.DRIVER,"org.apache.derby.jdbc.EmbeddedDriver");
		props.setProperty(DaoOptions.SELECT_SQL,
				          "select content from dbstorage where name = ?");
		// Optional DB Administration Properties
		props.setProperty(DaoOptions.STATEMENT_FACTORY_CLASS,
				          "com.openrules.jdbc.StatementFactoryDerbyEmbedded");
		props.setProperty(DaoOptions.CREATE_DDL,
				"CREATE TABLE dbstorage (NAME VARCHAR(255) NOT NULL,CONTENT BLOB,PRIMARY KEY(NAME))");
		props.setProperty(DaoOptions.INSERT_SQL,
				          "insert into dbstorage(name,content) values(?,?)");
		props.setProperty(DaoOptions.LIST_SQL,
				          "select name from dbstorage where name like ?"); 
		props.setProperty(DaoOptions.DELETE_SQL,
				          "delete from dbstorage where name like ?");
		return props;
	}
	
	public static String[] commandLineWithDoubleQuotes( String[] args ) {
		ArrayList<String> tmp = new ArrayList<String>();

		boolean join = false;
		StringBuilder sb = new StringBuilder();
		
		for ( String arg : args ) {
			if ( arg.length() == 0 )
				continue;
			
			if (  arg.charAt(0) == '"' ) {
				join = true;
				arg = arg.substring(1);
			}
			
			if ( join ) {
				int pos = arg.length() - 1;
				if ( pos >= 0 && arg.charAt( pos ) == '"' ) {
					arg = arg.substring(0,pos);
					join = false;
				}
				sb.append(arg);
			
			} else {
				if ( sb.length() > 0 ) {
					arg = sb.toString();
					sb.setLength(0);
				}
				tmp.add(arg);
			}
		}
		return tmp.toArray( new String[ tmp.size() ] );
	}

}
