package com.openrules.jdbc;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * 
 */

import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import com.openrules.dao.DAOException;

public class JdbcHelper {

	private StatementFactory factory;

	public JdbcHelper(StatementFactory fatory) {
		this.factory = fatory;
	}

	public void executeDDL(String script) throws DAOException {
		Statement st = factory.createStatement();
		try {
			st.executeUpdate(script);
		} catch (SQLException e) {
			throw new DAOException(e);
		} finally {
			try {
				st.close();
			} catch (SQLException e) {
				//ignore
			}
		}
	}

	public <T> List<T> query(String sql, Object[] params, RowDigester<T> rd)
			throws DAOException {
		ArrayList<T> list = new ArrayList<T>();
		try {
			PreparedStatement st = this.factory.prepareStatement(sql);
			if (params != null) {
				for (int i = 0; i < params.length; ++i)
					st.setObject(i + 1, params[i]);
			}
			ResultSet rs = st.executeQuery();
			while (rs.next())
				list.add(rd.digest(rs));

			rs.close();
			st.close();

		} catch (SQLException e) {
			throw new DAOException(e);
		}

		return list;
	}

	public int executeUpdate(String sql, Object[] params) throws DAOException {
		try {
			PreparedStatement st = this.factory.prepareStatement(sql);
			try {
				if (params != null) {
					UpdateHelper uh = new UpdateHelper(st);
					for (int i = 0; i < params.length; ++i) {
						uh.setParameter(i+1, params[i] );
					}
				}
				return st.executeUpdate();
			} finally {
				st.close();
			}

		} catch (SQLException e) {
			throw new DAOException(e);
		}
	}

	public StatementFactory getFactory() {
		return factory;
	}

}
