package com.openrules.jdbc;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * 
 */

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;

import com.openrules.dao.DAOException;
import com.openrules.dao.DaoOptions;

public class StatementFactoryImpl implements StatementFactory {
	
	private Connection connection = null;
	
	public void initialize( Properties options )  {
		String user = options.getProperty(DaoOptions.USER);
		String password = options.getProperty(DaoOptions.PASSWORD);
		String url = options.getProperty(DaoOptions.CONNECT_URL);
		String driver = options.getProperty(DaoOptions.DRIVER );
		
		try {
			Class.forName(driver).newInstance();
			
			connection = DriverManager.getConnection(url, user, password);
		} catch (Exception e) {
			throw new DAOException( "Initializing StatementFactory failed", e );
		}
	}

	/* (non-Javadoc)
	 * @see com.openrules.repo.test.jdbc.StatementFactory#createStatement()
	 */
	public Statement createStatement() throws DAOException {
		try {
			return connection.createStatement();
		} catch (SQLException e) {
			throw new DAOException(e);
		}
	}
	
	/* (non-Javadoc)
	 * @see com.openrules.repo.test.jdbc.StatementFactory#closeStatement(java.sql.Statement)
	 */
	public void closeStatement(Statement st) throws DAOException {
		if ( st == null ) 
			return;
		try {
			st.close();
		} catch (SQLException e) {
			throw new DAOException(e);
		}
	}

	/* (non-Javadoc)
	 * @see com.openrules.repo.test.jdbc.StatementFactory#dispose()
	 */
	public void dispose() {
		//close connection
		if ( connection != null )
			try {
				connection.close();
			} catch (SQLException e) {
				throw new DAOException( e );
			}
	}

	/* (non-Javadoc)
	 * @see com.openrules.repo.test.jdbc.StatementFactory#prepareStatement(java.lang.String)
	 */
	public PreparedStatement prepareStatement(String sql) throws DAOException {
		try {
			return connection.prepareStatement(sql);
		} catch (SQLException e) {
			throw new DAOException(e);
		}
	}




}
