package com.openrules.jdbc;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * 
 */

import java.io.ByteArrayInputStream;
import java.io.Reader;
import java.io.StringReader;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.sql.Types;

public class UpdateHelper {
	private PreparedStatement st;

	public UpdateHelper(PreparedStatement st) {
		this.st = st;
	}

	public void setParameter( int idx, Object o ) throws SQLException {
		if ( o instanceof String ) 
			setString(idx,(String)o);
		else if ( o instanceof Integer )
			setInteger(idx,(Integer)o);
		else if (o instanceof Timestamp )
			setTimestamp(idx, (Timestamp)o);
		else if ( o instanceof BooleanYN )
			setBooleanYN(idx, (BooleanYN)o );
		else if ( o instanceof byte[] )
			setBlob(idx, (byte[])o);
		else if (o instanceof char[] )
			setClob(idx, (char[])o);
		else
			setObject(idx,o);
	}
	
	public void setObject(int idx, Object value) throws SQLException {
		if (value == null)
			st.setNull(idx, Types.OTHER);
		else
			st.setObject(idx, value);
	}
	

	public void setString(int idx, String value) throws SQLException {
		if (value == null)
			st.setNull(idx, Types.VARCHAR);
		else
			st.setString(idx, value);
	}

	//support boolean for a string property with 'Y' for true and 'N' for false
	public void setBooleanYN(int idx, BooleanYN value) throws SQLException {
		if (value == null)
			setString(idx, null);
		else
			st.setString(idx, value.getString() );
	}

	public void setInteger(int idx, Integer value) throws SQLException {
		if (value == null)
			st.setNull(idx, Types.INTEGER);
		else
			st.setInt(idx, value);
	}

	public void setClob(int idx, char[] data) throws SQLException {
		if (data == null)
			st.setNull(idx, Types.CLOB);
		else {
			Reader r = new StringReader( new String(data) );
			//st.setCharacterStream(idx, r); JDK 1.6 only -- IK
			// for JDK 1.5 
			// If the length of the stream is unknown, 
			// the length parameter may be set to -1 to indicate that the driver should accept the stream regardless of its length.
			st.setCharacterStream(idx, r, -1); // JF
		}
	}

	public void setBlob(int idx, byte[] data) throws SQLException {
		if (data == null)
			st.setNull(idx, Types.BLOB);
		else {
			ByteArrayInputStream is = new ByteArrayInputStream(data);
			st.setBinaryStream(idx, is, data.length);
		}
	}

	public void setTimestamp(int idx, Timestamp value) throws SQLException {
		if (value == null)
			st.setNull(idx, Types.TIMESTAMP);
		else
			st.setTimestamp(idx, value);
	}
}
