package com.openrules.db.admin;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * 
 */

import java.sql.Blob;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.Properties;

import com.openrules.dao.DAOException;
import com.openrules.dao.DaoOptions;
import com.openrules.dao.DaoUtils;
import com.openrules.jdbc.ErrorMessage;
import com.openrules.jdbc.JdbcHelper;
import com.openrules.jdbc.RowDigestSingleValue;
import com.openrules.jdbc.RowDigester;
import com.openrules.jdbc.StatementFactory;

public class Dao {

	private JdbcHelper jdbcHelper;

	private String deleteSql;

	private String listSql;

	private String selectSql;

	private String insertSql;

	private String createDDL;

	public Dao() {
	}

	protected String getRequiredOption( Properties options, String key ) {
		String value = options.getProperty(key);
		if ( value == null )
			throw new RuntimeException("required property '%s' is not defined");
		return value;
	}
	
	public void initialize(Properties options) throws DAOException {
		String statementFactoryClassName = getRequiredOption(options,DaoOptions.STATEMENT_FACTORY_CLASS);

		StatementFactory factory = DaoUtils.newInstance(statementFactoryClassName);
		factory.initialize(options);
		jdbcHelper = new JdbcHelper(factory);

		this.selectSql = getRequiredOption(options,DaoOptions.SELECT_SQL);
		this.listSql = getRequiredOption(options,DaoOptions.LIST_SQL);
		this.deleteSql = getRequiredOption(options,DaoOptions.DELETE_SQL);
		this.insertSql = getRequiredOption(options,DaoOptions.INSERT_SQL);
		this.createDDL = getRequiredOption(options,DaoOptions.CREATE_DDL);

	}

	public void dispose() {
		if (jdbcHelper != null)
			jdbcHelper.getFactory().dispose();
	}

	public void initStorage() throws DAOException {
		jdbcHelper.executeDDL(this.createDDL);
	}

	public void upload(String key, byte[] data) throws DAOException {
		jdbcHelper.executeUpdate(this.insertSql, new Object[] { key, data });
	}

	public int remove(String key) throws DAOException {
		return jdbcHelper.executeUpdate(this.deleteSql, new Object[] { key });
	}

	public List<String> list(String mask) throws DAOException {
		String m = mask.replace('*', '%').replace('?', '_');
		return jdbcHelper.query(this.listSql, new String[] { m },
				new RowDigestSingleValue<String>());
	}

	private RowDigester<byte[]> blobDigester = new RowDigester<byte[]>() {
		public byte[] digest(ResultSet rs) throws SQLException {
			Blob b = rs.getBlob(1);
			return b.getBytes(1, (int) b.length());
		}
	};

	public byte[] load(String key) throws DAOException {
		List<byte[]> l = jdbcHelper.query(this.selectSql, new Object[] { key },
				blobDigester);
		if (l.size() == 0)
			throw new ErrorMessage("Record with key [%s] not found", key);
		return l.get(0);
	}
}
