package com.openrules.db;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * 
 */

/**
 * This class implements a DataSourceProtocolHandler that allows to
 * access xls-files with OpenRules tables saved in a database. The database is
 * configured using a property file that also defines a protocol name and its
 * implementation class such as this one.
 */

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Properties;

import com.openrules.datasource.AbstractDataSource;
import com.openrules.datasource.IDataSource;
import com.openrules.datasource.IDataSourceFactory;
import com.openrules.datasource.IDataSourceProtocolHandler;

public class DbDataSourceHandler extends DbSourceDao implements IDataSourceProtocolHandler {

	class DbDataSource extends AbstractDataSource {
		private URI uri;
		IDataSourceFactory factory;
		
		
		public DbDataSource(URI uri, IDataSourceFactory factory  ) {
			this.uri = uri;
			this.factory =  factory;
		}
		
		public String getDescription() {
			return uri.toASCIIString();
		}

		public URI getURI() {
			return uri;
		}

		public URL getUrl() {
			try {
				return uri.toURL();
			} catch (MalformedURLException e) {
				throw new RuntimeException( e );
			}
		}

		public InputStream getInputStream() throws IOException {
			byte[] content = load( this.getURI().getSchemeSpecificPart() );
			if ( content == null )
				return null;
			
			return new ByteArrayInputStream( content );
		}
		
	}
	
	public DbDataSourceHandler() {
	}
	
	public IDataSource fromURI(URI uri, IDataSourceFactory factory) {
		return new DbDataSource(uri,factory); 
	}
	
	public Properties getDefaultProperties() {
		Properties props = new Properties();
		// OpenRules Data Source Protocol Properties
		props.setProperty("openrules.protocol.name", "db");
		props.setProperty("openrules.protocol.class", "com.openrules.db.DbDataSourceHandler");
		// DB Access Properties
		props.setProperty("db.user","embedded");
		props.setProperty("db.password","none");
//		props.setProperty("db.url", 
//				"jdbc:derby:/_SVN/openrules.examples/RulesRepositoryDB/db/dbstorage;create=true");
		props.setProperty("db.driver","org.apache.derby.jdbc.EmbeddedDriver");
		props.setProperty("db.selectSql",
				          "select content from dbstorage where name = ?");
		// Optional DB Administration Properties
		props.setProperty("db.statementfactory.class",
				          "com.openrules.jdbc.StatementFactoryDerbyEmbedded");
		props.setProperty("db.createDDL",
				"CREATE TABLE dbstorage (NAME VARCHAR(255) NOT NULL,CONTENT BLOB,PRIMARY KEY(NAME))");
		props.setProperty("db.insertSql",
				          "insert into dbstorage(name,content) values(?,?)");
		props.setProperty("db.listSql",
				          "select name from dbstorage where name like ?"); 
		props.setProperty("db.deleteSql",
				          "delete from dbstorage where name like ?");
		return props;
	}

}
