package com.openrules.db;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * Modified by JF
 * 
 */

/**
 * This class provides an access to a database, which configuration is
 * described by the following properties:
 * <ul>
 * <li> db.user
 * <li> db.password
 * <li> db.url such as "jdbc:derby:dist/dbstorage;create=false"
 * <li> db.driver such as "org.apache.derby.jdbc.EmbeddedDriver"
 * <ul>
 * 
 * It is assumed that the database includes a tables (e.g. table "dbstorage")
 * that saves different BLOB objects (such as xls-files) with unique names.
 * The table contains two attributes:
 * <br> key - a unique string
 * <br> a blob object
 * 
 * The blob objects may be loaded from this table using an SQL statement 
 * described in the property:
 * 
 * db.selectSql such as "select content from dbstorage where name = ?"
 * 
 */



import java.sql.Blob;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Properties;

import com.openrules.dao.DaoOptions;

public class DbSourceDao {

	private Connection connection;
	private String selectSql;
	
	public DbSourceDao() {
	}

	public void initialize(Properties options) {
		String user = options.getProperty(DaoOptions.USER);
		String password = options.getProperty(DaoOptions.PASSWORD);
		String url = options.getProperty(DaoOptions.CONNECT_URL);
		String driver = options.getProperty(DaoOptions.DRIVER);
			
		try {
			Class.forName(driver).newInstance();
			connection = DriverManager.getConnection(url, user, password);
		} catch (Exception e) {
			throw new RuntimeException( "Initializing data source failed", e );
		}
		
		this.selectSql = options.getProperty(DaoOptions.SELECT_SQL);

	}

	public void dispose() {
		if ( connection != null )
			try {
				connection.close();
			} catch (SQLException e) {
				throw new RuntimeException( e );
			}
	}

	public byte[] load(String key )   {
		try {
			PreparedStatement st = this.connection.prepareStatement(this.selectSql);
			st.setString(1,key);
			ResultSet rs = st.executeQuery();
			if ( rs.next() ) {
				Blob b = rs.getBlob(1);
				return b.getBytes(1, (int) b.length() );
			}
			return null;
		} catch (SQLException e) {
			throw new RuntimeException( e );
		}
	}
}
