package com.openrules.dbv.admin.impl;

/*
 * Copyright (C) 2007 OpenRules, Inc.
 * 
 * Created on April-May, 2007 by IK
 * Modified by JF
 * 
 */

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

import com.openrules.dao.DAOException;
import com.openrules.dao.DaoOptions;
import com.openrules.dao.DaoUtils;
import com.openrules.dbv.admin.RepoDAO;
import com.openrules.dbv.admin.RepoEntry;
import com.openrules.jdbc.BooleanYN;
import com.openrules.jdbc.JdbcHelper;
import com.openrules.jdbc.RowDigester;
import com.openrules.jdbc.SelectHelper;
import com.openrules.jdbc.StatementFactory;

public class RepoDAOImpl implements RepoDAO {

	//private StatementFactory factory;
	private JdbcHelper jdbcHelper;

	private String createDDL;
//	private String deleteSql;
//	private String listSql;
//	private String selectSql;
//	private String insertSql;


	public RepoDAOImpl() {
	}

//	protected <T> List<T> runQuery(String sql, Object[] params,
//			RowDigester<T> digester) {
//		return new JdbcHelper(this.factory).query(sql, params, digester);
//	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#initialize()
	 */
	public void initialize(Properties options) throws DAOException {
		String statementFactoryClassName = options
				.getProperty(DaoOptions.STATEMENT_FACTORY_CLASS);
		if (statementFactoryClassName == null)
			throw new DAOException("Property '"+DaoOptions.STATEMENT_FACTORY_CLASS+"' must be defined");

		StatementFactory factory = DaoUtils.newInstance(statementFactoryClassName);

		factory.initialize(options);
		
		this.jdbcHelper = new JdbcHelper(factory);
		
		createDDL = options.getProperty(DaoOptions.CREATE_DDL);
//		deleteSql = options.getProperty(DaoOptions.DELETE_SQL);
//		selectSql = options.getProperty(DaoOptions.SELECT_SQL);
//		insertSql = options.getProperty(DaoOptions.INSERT_SQL);
//		listSql = options.getProperty(DaoOptions.LIST_SQL);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#createRepository()
	 */
	public void createRepository() throws DAOException {
		jdbcHelper.executeDDL(createDDL);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#dispose()
	 */
	public void dispose() {
		if ( jdbcHelper != null)
			jdbcHelper.getFactory().dispose();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#revision()
	 */
	public int revision() throws DAOException {

		List<Integer> l = jdbcHelper.query("SELECT MAX(revision) FROM repo", null,
				new RowDigester<Integer>() {
					public Integer digest(ResultSet rs) throws SQLException {
						return new Integer(rs.getInt(1));
					}
				});

		return (l.size() == 0 ? -1 : l.get(0));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#findEntry(java.lang.String, int)
	 */
	public RepoEntry findEntry(String name, int revision) throws DAOException {
		List<RepoEntry> tmp = find(name, revision);
		if (tmp.size() == 0)
			return null;
		if (tmp.size() > 1)
			throw new RuntimeException("Invalid resource name '" + name + "'");
		return tmp.get(0);
	}

	protected int insertRecord(RepoEntry entry) throws DAOException {

		entry.setTimestamp(new Timestamp(System.currentTimeMillis()));

		int rev = revision() + 1;
		entry.setRevision(rev);

		String sql = "INSERT INTO repo (author,comments,content,deleted,name,revision,timestamp) VALUES (?,?,?,?,?,?,?)";
		Object[] params = new Object[] {
			entry.getAuthor(),
			entry.getComments().toCharArray(), //CLOB
			entry.getContent(), //BLOB
			new BooleanYN( entry.isDeleted() ),
			entry.getName(),
			entry.getRevision(),
			entry.getTimestamp() 
		};
		
		jdbcHelper.executeUpdate(sql, params);

		return rev;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#addEntry(java.lang.String,
	 *      byte[], java.lang.String, java.lang.String)
	 */
	public int addEntry(String name, byte[] content, String author,
			String comments) throws DAOException {
		RepoEntry tmp = new RepoEntry();
		tmp.setName(name);
		tmp.setAuthor(author);
		tmp.setComments(comments == null ? "" : comments);
		tmp.setContent(content);
		return insertRecord(tmp);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#removeEntry(java.lang.String,
	 *      java.lang.String, java.lang.String)
	 */
	public int removeEntry(String name, String author, String comments)
			throws DAOException {
		RepoEntry rm = findEntry(name, -1);
		if (rm == null)
			return -1;

		RepoEntry tmp = new RepoEntry();
		tmp.setName(name);
		tmp.setAuthor(author);
		tmp.setComments(comments == null ? "" : comments);
		tmp.setDeleted(true);

		return insertRecord(tmp);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#findAll(java.lang.String, int)
	 */
	public List<RepoEntry> findAll(String nameMask, int revision)
			throws DAOException {
		return find(nameMask.replace('*', '%').replace('?', '_'), revision);
	}

	protected List<RepoEntry> find(final String mask, final int revision)
			throws DAOException {

		String sql = "select * from repo r where r.deleted = 'N' and r.revision = ( select max(r2.revision) from repo r2 where r2.name = r.name";
		ArrayList<Object> params = new ArrayList<Object>();
		if (revision > -1) {
			sql = sql + " and r2.revision <= ?";
			params.add(revision);
		}
		sql += ") and r.name like ?";
		params.add(mask);

		return jdbcHelper.query(sql, params.toArray(), new RepoEntryDigester());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.dbv.test.RepoDAO#history(java.lang.String)
	 */
	public List<RepoEntry> history(String name) throws DAOException {
		String sql = "select * from repo r where r.name = ? order by r.revision desc";
		return jdbcHelper.query(sql, new Object[] { name }, new RepoEntryDigester());
	}

}

class RepoEntryDigester implements RowDigester<RepoEntry> {

	public RepoEntry digest(ResultSet rs) throws SQLException {
		SelectHelper select = new SelectHelper(rs);
		RepoEntry re = new RepoEntry();
		re.setAuthor(select.getString("author"));
		re.setComments(select.getClob("comments"));
		re.setContent(select.getBlob("content"));
		re.setDeleted(select.getBooleanYN("deleted"));
		re.setName(select.getString("name"));
		re.setRevision(select.getInteger("revision"));
		re.setTimestamp(select.getTimestamp("timestamp"));
		return re;
	}
}
