package com.openrules.dbv;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Properties;

import com.openrules.datasource.AbstractDataSource;
import com.openrules.datasource.IDataSource;
import com.openrules.datasource.IDataSourceFactory;
import com.openrules.datasource.IDataSourceProtocolHandler;
import com.openrules.dbv.admin.RepoDAO;
import com.openrules.dbv.admin.RepoEntry;
import com.openrules.dbv.admin.RepositoryMap;

// rulesInMemory://<repository_name>:[<revision>]/<resource_name>

public class DbvDataSourceHandler extends RepositoryMap implements IDataSourceProtocolHandler {

	class RepoDataSource extends AbstractDataSource {
		private URI uri;
		IDataSourceFactory factory;
		
		public RepoDataSource(URI uri, IDataSourceFactory factory  ) {
			this.uri = uri;
			this.factory =  factory;
		}
		
		public String getDescription() {
			return uri.toASCIIString();
		}

		public URI getURI() {
			return uri;
		}

		public URL getUrl() {
			try {
				return uri.toURL();
			} catch (MalformedURLException e) {
				//ignored
				return null;
			}
		}

		public InputStream getInputStream() throws IOException {
			RepoEntry e = findEntry();
			if ( e == null )
				return null;
			byte[] content = e.getContent();
			if ( content == null )
				return null;
			
			return new ByteArrayInputStream( content );
		}
		
		private RepoEntry findEntry() {
			RepoDAO dao = getDAO( uri.getHost() );
			return dao.findEntry(uri.getPath(), uri.getPort() );
		}
		
	}
	
	public IDataSource fromURI(URI uri, IDataSourceFactory factory) {
		return new RepoDataSource(uri,factory); 
	}

	public void initialize(Properties options) {
		//nothing
		//override to customize
	}

}
