package com.openrules.eclipse.core.actions;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 */

import java.util.Iterator;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPart;

import com.openrules.eclipse.core.builder.ErrorHelper;
import com.openrules.eclipse.core.plugin.OpenrulesProject;

public class ToggleNatureAction implements IObjectActionDelegate {

	private IWorkbenchPart targetPart;

	private IProgressMonitor monitor;

	public ToggleNatureAction() {
		super();
	}

	public IProgressMonitor getMonitor() {
		return monitor;
	}

	public void setMonitor(IProgressMonitor monitor) {
		this.monitor = monitor;
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
		this.targetPart = targetPart;
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {
		try {
			if (this.targetPart instanceof IViewPart) {
				ISelection selection = targetPart.getSite()
						.getSelectionProvider().getSelection();

				if ((selection != null)
						&& (selection instanceof IStructuredSelection)) {
					Object[] targets = ((IStructuredSelection) selection)
							.toArray();
					// assert projects.length == 1
					IProject project = getProject(targets[0]);
					if (project != null)
						OpenrulesProject.toggleNature(project);
				}
			}

		} catch (Exception e) {
			ErrorHelper.handleException(e);
		}

	}

	protected IProject getProject(Object object) {
		IProject project = null;
		if (object instanceof IProject) {
			project = (IProject) object;
		} else if (object instanceof IJavaProject) {
			project = ((IJavaProject) object).getProject();
		}
		return project;
	}

	/**
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		if ((selection != null) && (selection instanceof IStructuredSelection)) {
			Iterator it = ((IStructuredSelection) selection).iterator();
			if ( it.hasNext() ) {
				IProject project = getProject( it.next() );
				if ( project != null ) {
					try {
						if ( OpenrulesProject.hasNature(project) ) {
							action.setText(ActionMessages.remove_nature);
							action.setToolTipText(ActionMessages.remove_nature_tooltip);
						} else {
							action.setText(ActionMessages.add_nature);
							action.setToolTipText(ActionMessages.add_nature_tooltip);
						}
					} catch (CoreException e) {
						action.setText(ActionMessages.toggle_action_disabled);
						action.setEnabled(false);
					}
				}
			}
		}
	}

}
