package com.openrules.eclipse.core.builder;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 */

import java.lang.reflect.InvocationTargetException;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.openrules.eclipse.core.plugin.OpenrulesPlugin;

/**
 * @author IK
 */

public class ErrorHelper {

	/**
	 * Returns new CoreException with a given parameters.
	 */
	public static CoreException coreException(String message, Throwable t,
			int code, int severity) {
		return new CoreException(new Status(severity,
				OpenrulesPlugin.OPENRULES_PLUGIN_ID, code, message, t));
	}

	/**
	 * Returns new CoreException with a given parameters.
	 */
	public static CoreException coreException(String message) {
		return coreException(message, null, -1, IStatus.ERROR);
	}

	/**
	 * Returns new CoreException with a given parameters.
	 */
	static private Throwable unwrapInvocationTargetException(
			InvocationTargetException t) {
		Throwable cause;

		while ((cause = t.getCause()) instanceof InvocationTargetException) {
			t = (InvocationTargetException) cause;
		}

		return cause != null ? cause : t;
	}

	/**
	 * Returns new CoreException with a given parameters.
	 */
	public static CoreException coreException(Throwable t) {
		if (t instanceof CoreException)
			return (CoreException) t;

		if (t instanceof InvocationTargetException) {
			t = unwrapInvocationTargetException((InvocationTargetException) t);
		}

		return coreException( t.getMessage(), t, -1, IStatus.ERROR);
	}

	/**
	 * Generic exception handler.
	 */
	public static CoreException handleException(Throwable t) {
		CoreException ce = coreException( t );
		log(ce.getStatus());
		return ce;
	}

	public static void log(IStatus status) {
		OpenrulesPlugin.getDefault().getLog().log(status);
	}

	public static void removeMarkers(IResource resource) {
		int depth = IResource.DEPTH_INFINITE;
		try {
			resource.deleteMarkers(OpenrulesPlugin.OPENRULES_PROBLEM_MARKER, true,
					depth);
		} catch (Exception e) {
			ErrorHelper.handleException(e);
		}
	}

	public static void addMarker(IResource resource, String message,
			int severity) throws CoreException {
		Map attributes = new HashMap();

		attributes.put(IMarker.MESSAGE, message);
		attributes.put(IMarker.SEVERITY, new Integer(severity));
		createMarker(resource, OpenrulesPlugin.OPENRULES_PROBLEM_MARKER,
				attributes);
	}

	public static void addMarker(IResource resource, IStatus status)
			throws CoreException {
		Map attributes = new HashMap();

		attributes.put(IMarker.MESSAGE, status.getMessage());
		attributes.put(IMarker.SEVERITY, new Integer(status.getSeverity()));
		createMarker(resource, OpenrulesPlugin.OPENRULES_PROBLEM_MARKER,
				attributes);
	}

	protected static IMarker createMarker(IResource resource, String type,
			Map attributes) throws CoreException {
		IMarker m = resource.createMarker(type);
		m.setAttributes(attributes);
		return m;
	}

	public static Throwable unwrap(Throwable t) {
		Throwable tmp = t;
		while (tmp != null) {
			if (tmp.getMessage() != null) {
				t = tmp;
				break;
			}
			tmp = tmp.getCause();
		}
		return t;
	}

}
