package com.openrules.eclipse.core.plugin;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 */

import java.io.IOException;
import java.io.PrintStream;

import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.IConsoleManager;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;

import com.openrules.eclipse.core.builder.ErrorHelper;
import com.openrules.eclipse.core.builder.Messages;
import com.openrules.eclipse.xls.launching.XlsConsoleListener;

public class OpenrulesConsoleOutput  {

	private MessageConsole console = null;
	private MessageConsoleStream out = null;

	private String getConsoleName() {
		return Messages.builder_console_title;
	}
	
	public OpenrulesConsoleOutput() {
		console = findConsole( getConsoleName() );
		//console.clearConsole();
		out = console.newMessageStream();
		out.setActivateOnWrite(true);
		console.addPatternMatchListener( new XlsConsoleListener() );
	}
	
	public void print( String str) {
		out.print(str);
	}

	public void println( String str) {
		out.println(str);
	}

	private MessageConsole findConsole(String name) {
		ConsolePlugin plugin = ConsolePlugin.getDefault();
		IConsoleManager conMan = plugin.getConsoleManager();
		IConsole[] existing = conMan.getConsoles();
		for (int i = 0; i < existing.length; i++)
			if (name.equals(existing[i].getName()))
				return (MessageConsole) existing[i];
		// no console found, so create a new one
		MessageConsole myConsole = new MessageConsole( getConsoleName(), null);
		conMan.addConsoles(new IConsole[] { myConsole });
		return myConsole;
	}


	public void close()  {
		if ( out != null && ! out.isClosed() )
			try {
				out.close();
			} catch (IOException e) {
				ErrorHelper.coreException(e);
			}
		out = null;
	}

	public PrintStream getOutputStream() {
		return new PrintStream(out);
	}


}
