package com.openrules.eclipse.core.plugin;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 */

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.ICommand;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IProjectNature;
import org.eclipse.core.runtime.CoreException;

import com.openrules.eclipse.core.builder.ErrorHelper;

public class OpenrulesProject implements IProjectNature {

	private IProject project;
	
	public IProject getProject() {
		return project;
	}

	public void setProject(IProject project) {
		this.project = project;
	}

	public void configure() throws CoreException {
		//assert project != null;
		addBuilder(project);
		
	}

	public void deconfigure() throws CoreException {
		//assert project != null;
		removeBuilder(project);
	}
	

	//helpers ==================================================================
	public static void toggleNature(IProject project) throws CoreException {
		if (hasNature(project))
			removeNature(project);
		else
			addNature(project);
	}

	public static boolean hasNature(IProject project) throws CoreException {

		String[] ids = project.getDescription().getNatureIds();
		if ( ids != null )
			for ( int i= 0; i < ids.length; ++i )
				if ( OpenrulesPlugin.OPENRULES_NATURE_ID.equals(ids[i] ) )
					return true;

		return false;
	}

	public static void addNature(IProject project) throws CoreException {
		IProjectDescription description = project.getDescription();

		List natures = new ArrayList(Arrays.asList(description.getNatureIds()));
		if (!natures.contains(OpenrulesPlugin.OPENRULES_NATURE_ID)) {
			natures.add(0, OpenrulesPlugin.OPENRULES_NATURE_ID);
			description.setNatureIds((String[]) natures
					.toArray(new String[natures.size()]));
		}

		project.setDescription(description, null);
		
	}

	public static void removeNature(IProject project) throws CoreException {
		IProjectDescription description = project.getDescription();

		List natures = new ArrayList(Arrays.asList(description.getNatureIds()));
		if (natures.contains(OpenrulesPlugin.OPENRULES_NATURE_ID)) {
			natures.remove(OpenrulesPlugin.OPENRULES_NATURE_ID);
			description.setNatureIds((String[]) natures
					.toArray(new String[natures.size()]));
		}

		project.setDescription(description, null);

		ErrorHelper.removeMarkers(project);
	}

	public static void addBuilder( IProject project ) throws CoreException {
		IProjectDescription description = project.getDescription();
		ICommand command = description.newCommand();
		command.setBuilderName(OpenrulesPlugin.OPENRULES_BUILDER_NAME);
		List specs = new ArrayList(Arrays.asList(description.getBuildSpec()));
		if (!specs.contains(command)) {
			// the last builder
			specs.add(command);
			description.setBuildSpec((ICommand[]) specs
					.toArray(new ICommand[specs.size()]));
		}
		project.setDescription(description, null);
	}

	public static void removeBuilder( IProject project ) throws CoreException {
		IProjectDescription description = project.getDescription();

		ICommand command = description.newCommand();
		command.setBuilderName(OpenrulesPlugin.OPENRULES_BUILDER_NAME);
		List specs = new ArrayList(Arrays.asList(description.getBuildSpec()));
		if (specs.contains(command)) {
			specs.remove(command);
			description.setBuildSpec((ICommand[]) specs
					.toArray(new ICommand[specs.size()]));
		}
		project.setDescription(description, null);
		
	}
	

}