package com.openrules.eclipse.util;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;

/**
 * @author smesh
 */
public class ExceptionUtil {

	static public CoreException wrapToCoreException(Throwable t) {

		if (t instanceof CoreException)
			return (CoreException) t;

		return new CoreException(new Status(IStatus.ERROR,
				getDefaultBundleId(), -1, getMessage(t), t));
	}

	static public void handleException(Throwable t, boolean showInMessageBox) {
		CoreException coreException = wrapToCoreException(t);

		Platform.getLog(Platform.getBundle(getDefaultBundleId())).log(
				coreException.getStatus());

		if (showInMessageBox)
			MessageBoxUtil.showError(getMessage(t));
	}

	static public String getDefaultBundleId() {
		return "com.openrules.base";
	}

	static public String getMessage(Throwable t) {
		String message = t != null ? unwrap(t).getMessage() : null;
		return message != null && message.length() > 0 ? message : "NO_MESSAGE";
	}

	static public Throwable unwrap(Throwable t) {
		Throwable cause = t.getCause();

		if (t instanceof CoreException) {
			CoreException coreException = (CoreException) t;
			IStatus status = coreException.getStatus();
			cause = status.getException();
		}

		if (cause != null && cause != t)
			return unwrap(cause);

		return t;
	}
}