package com.openrules.eclipse.wizard;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.IExecutableExtension;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;

import com.openrules.eclipse.util.AntUtil;
import com.openrules.eclipse.util.ExceptionUtil;
import com.openrules.eclipse.util.ProjectUtil;
import com.openrules.eclipse.util.UrlUtil;

/**
 * @author smesh
 */
public class NewProjectFromTemplateWizard extends BasicNewResourceWizard
		implements INewWizard, IExecutableExtension {

	static public final String ANT_PROP_SRC_DIR = "src.dir";

	static public final String ANT_PROP_DST_DIR = "dst.dir";

	static public final String ANT_PROP_DST_PROJECT_NAME = "dst.project.name";

	static public final String KEY_WINDOW_TITLE = "windowTitle";

	static public final String KEY_MAIN_PAGE_NAME = "mainPageName";

	static public final String KEY_MAIN_PAGE_TITLE = "mainPageTitle";

	static public final String KEY_MAIN_PAGE_DESCRIPTION = "mainPageDescription";

	static public final String KEY_INITIAL_PROJECT_NAME = "initialProjectName";

	static public final String KEY_TEMPLATE_PROJECT_PLUGIN_ID = "templateProjectpluginId";

	static public final String KEY_TEMPLATE_PROJECT_BUILD_FILE = "buildFile";

	static public final String KEY_TEMPLATE_PROJECT_DIR = "templateProjectDir";

	private WizardNewProjectCreationPage mainPage;

	private Map wizardProperties = new HashMap();

	public NewProjectFromTemplateWizard() {
	}

	public void init(IWorkbench workbench, IStructuredSelection selection) {
		super.init(workbench, selection);
		setNeedsProgressMonitor(true);
	}

	/**
	 * Extract values declared in the plugin manifest
	 */
	public void setInitializationData(IConfigurationElement config,
			String propertyName, Object data) throws CoreException {
		// TODO read javadoc for IConfigurationElement.setInitializationData()
		if (data instanceof Map) {
			wizardProperties = (Map) data;
		}
	}

	public void addPages() {
		try {
			setWindowTitle(getString(KEY_WINDOW_TITLE));

			mainPage = new WizardNewProjectCreationPage(KEY_MAIN_PAGE_NAME);

			mainPage.setTitle(getString(KEY_MAIN_PAGE_TITLE));
			mainPage.setDescription(getString(KEY_MAIN_PAGE_DESCRIPTION));

			String initialProjectName = getString(KEY_INITIAL_PROJECT_NAME,
					null);
			if (initialProjectName != null)
				mainPage.setInitialProjectName(initialProjectName);

			this.addPage(mainPage);

		} catch (Throwable t) {
			ExceptionUtil.handleException(t, true);
		}

	}

	public boolean performFinish() {
		IProject newProject = createNewProject();

		if (newProject == null)
			//			return false;
			return true;

		//    updatePerspective();
		selectAndReveal(newProject);

		return true;
	}


	private IProject createNewProject() {
		// get a project handle
		final IProject newProject = mainPage.getProjectHandle();

		// create the new project operation
		WorkspaceModifyOperation op = new WorkspaceModifyOperation() {
			protected void execute(IProgressMonitor monitor)
					throws CoreException {

				try {
					// get a project descriptor
					final IProjectDescription description = ResourcesPlugin
							.getWorkspace().newProjectDescription(
									newProject.getName());

					// set location for linked project
					Display display = mainPage.getControl().getDisplay();
					display.syncExec( new Runnable() {
						public void run() {
							if (!mainPage.useDefaults()) {
								description.setLocation(mainPage.getLocationPath());
							}
						}
					});
					
					

					ProjectUtil.createProject(description, newProject, monitor);

					AntUtil.runAnt(getAntBuildFileLocation(),
							getAntBuildFileProperties(newProject), monitor);

					// refresh workspace project
					newProject.refreshLocal(IResource.DEPTH_INFINITE, monitor);
				} catch (Throwable t) {
					throw ExceptionUtil.wrapToCoreException(t);
				}
			}
		};

		try {
			getContainer().run(true, true, op);

			// helps for "*.launch" to appear in the Favorites
			newProject.close(null);
			newProject.open(null);

		} catch (Throwable t) {
			ExceptionUtil.handleException(t, true);
			return null;
		}

		return newProject;
	}

	private String getString(String key) {
		return getString(key, key);
	}

	private String getString(String key, String defaultValue) {
		String value = (String) wizardProperties.get(key);
		return value != null ? value : defaultValue;
	}

	private String getTemplateProjectDir() throws Exception {
		return UrlUtil.getPathForFileUrl(UrlUtil.getCanonicalUrl(
				getString(KEY_TEMPLATE_PROJECT_PLUGIN_ID),
				getString(KEY_TEMPLATE_PROJECT_DIR)));
	}

	private String getAntBuildFileLocation() throws Exception {
		return UrlUtil.getPathForFileUrl(UrlUtil.getCanonicalUrl(
				getString(KEY_TEMPLATE_PROJECT_PLUGIN_ID),
				getString(KEY_TEMPLATE_PROJECT_BUILD_FILE)));
	}

	private Properties getAntBuildFileProperties(IProject newProject)
			throws Exception {

		Properties antBuildFileProperties = new Properties();

		antBuildFileProperties.setProperty(ANT_PROP_SRC_DIR,
				getTemplateProjectDir());
		antBuildFileProperties.setProperty(ANT_PROP_DST_DIR, newProject
				.getLocation().toOSString());
		antBuildFileProperties.setProperty(ANT_PROP_DST_PROJECT_NAME,
				newProject.getName());

		return antBuildFileProperties;
	}

}