package com.openrules.jsr94.rules;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.rules.InvalidRuleSessionException;
import javax.rules.ObjectFilter;
import javax.rules.RuleExecutionSetMetadata;
import javax.rules.RuleExecutionSetNotFoundException;
import javax.rules.RuleRuntime;
import javax.rules.RuleSession;
import javax.rules.StatefulRuleSession;
import javax.rules.StatelessRuleSession;
import javax.rules.admin.RuleExecutionSet;

import com.openrules.jsr94.rules.admin.RuleExecutionSetImpl;
import com.openrules.jsr94.rules.admin.RuleExecutionSetRepository;

abstract class ARuleSession implements RuleSession {

	private ObjectFilter defaultObjectFilter;

	protected RuleExecutionSetImpl ruleExecutionSet;

	protected Map properties;

	public ARuleSession(String bindUri, Map properties)
			throws RuleExecutionSetNotFoundException {
		this.properties = properties;
		this.ruleExecutionSet = getRuleSet(bindUri);
	}

	public Map getProperties() {
		return properties;
	}


	public RuleExecutionSetMetadata getRuleExecutionSetMetadata() {
		// find uor bind uri
		RuleExecutionSetRepository repository = RuleExecutionSetRepository
				.getInstance();
		String theBindUri = null;
		for (Iterator i = repository.getRegistrations().iterator(); i.hasNext();) {
			String aBindUri = (String) i.next();
			RuleExecutionSet aRuleSet = repository
					.getRuleExecutionSet(aBindUri);
			if (aRuleSet == ruleExecutionSet) {
				theBindUri = aBindUri;
				break;
			}
		}

		return new RuleExecutionSetMetadataImpl(theBindUri, ruleExecutionSet
				.getName(), ruleExecutionSet.getDescription());
	}

	public int getType() throws InvalidRuleSessionException {
		if (this instanceof StatelessRuleSession)
			return RuleRuntime.STATELESS_SESSION_TYPE;
		else if (this instanceof StatefulRuleSession)
			return RuleRuntime.STATEFUL_SESSION_TYPE;
		else
			throw new InvalidRuleSessionException("");
	}

	public void release() {
		ruleExecutionSet = null;
	}

	public void reset() {
	}

	static RuleExecutionSetImpl getRuleSet(String bindUri)
			throws RuleExecutionSetNotFoundException {

		RuleExecutionSetRepository repository = RuleExecutionSetRepository
				.getInstance();

		RuleExecutionSetImpl ruleExecutionSet = (RuleExecutionSetImpl) repository
				.getRuleExecutionSet(bindUri);

		if (ruleExecutionSet == null) {
			throw new RuleExecutionSetNotFoundException(
					"no execution set bound to: " + bindUri);
		}

		return ruleExecutionSet;
	}

	protected synchronized ObjectFilter getDefaultObjectFilter() {
		if (defaultObjectFilter == null) {
			if (ruleExecutionSet.getDefaultObjectFilter() != null) {
				defaultObjectFilter = (ObjectFilter) Util
						.newInstance(ruleExecutionSet.getDefaultObjectFilter());
			}
		}
		return defaultObjectFilter;
	}

	static protected List applyFilter(List objects, ObjectFilter filter) {
		if (filter == null)
			return objects;

		List result = new ArrayList(objects.size());
		for (Iterator it = objects.iterator(); it.hasNext();) {
			result.add(filter.filter(it.next()));
		}
		return result;
	}

}