package com.openrules.tools;

/*
 * Copyright (C) 2006-2011 OpenRules, Inc. 
 */

import java.io.PrintStream;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;

import org.apache.commons.beanutils.ResultSetDynaClass;
import org.openl.util.Log;

public class Database {

	public String driver, dsn, user_name, password;
	Connection connection;

	public Database(String dbName) {
		this(dbName, null, null, null);
	}

	public Database(String dbName, String user, String pwd, String driver) {
		this.driver = driver;
		this.dsn = dbName;
		this.user_name = user;
		this.password = pwd;
		initConnection();
	}

	public synchronized ResultSet executeQuery(String sql) throws Exception {
		Statement stmt = getConnection().createStatement();
		ResultSet rs = stmt.executeQuery(sql);
		return rs;
	}

	protected synchronized void initConnection() {
		String driverName = driver;

		if (driverName == null)
			driverName = "sun.jdbc.odbc.JdbcOdbcDriver";

		try {
			Class.forName(driverName); // loads the driver
			String dsnName = dsn;
			String usrName = user_name;
			String usrPwd = password;

			out().println(
					"Connecting to '" + dsnName + "' using '" + driverName
							+ "' ...");
			
//			Class driverClass = Class.forName(driverName);
//			DriverManager.registerDriver((Driver) driverClass.newInstance());

			connection = DriverManager.getConnection("jdbc:odbc:" + dsnName,
					usrName, usrPwd);
			connection.setAutoCommit(false);

			out().println("Connected");
		} catch (Exception ex) {
			Log.error(ex.getMessage());
			//ex.printStackTrace(System.err);
			throw new RuntimeException();
		}
	}

	public synchronized void freeConnection() throws Exception {
		out().println("Free Connection ...");
		try {
			if (connection != null) {
				out().println("Close Connection");
				connection.close();
				connection = null;
			}
		} catch (Exception ex) {
			//ex.printStackTrace(System.err);
			Log.error(ex.getMessage());
			throw new RuntimeException();
		}
		out().println("Free Connection ...Done");
	}

	public synchronized Connection getConnection() throws Exception {
		if (connection == null) {
			initConnection();
		}
		return connection;
	}

	PrintStream out() {
		return System.out;
	}

	public synchronized ResultSet createAndExecuteBriefQuery(String tableName) {
		try {
			String sql = "SELECT * from " + tableName;
			ResultSet rs = executeQuery(sql);
			System.out.println("OK for createAndExecuteBriefQuery");
			return rs;

		} catch (Exception e) {
			throw new RuntimeException(
					"Database: SQL error in createAndExecuteBriefQuery: " + e);
		}
	}

	// //Slow
	// public synchronized RowSetDynaClass readDataInstances(String tableName) {
	// try {
	// ResultSet rs = createAndExecuteBriefQuery(tableName);
	// System.out.println("Reading from DB to RowSetDynaClass...");
	// RowSetDynaClass rsdc = new RowSetDynaClass(rs);
	// System.out.println("RowSetDynaClass has been created");
	// //rs.close();
	// List<DynaBean> rows = rsdc.getRows();
	// Iterator<DynaBean> iter = rows.iterator();
	// int n = 0;
	// while(iter.hasNext()) {
	// n++;
	// if ((n > 0 && n % 100 == 0)) // || i > 11000)
	// System.out.print("\nRead " + n + " records from " + env.dsn + " table " +
	// tableName);
	// //+ ": " + instance);
	// DynaBean bean = iter.next();
	// //System.out.println(bean);
	// }
	// System.out.print("\nRead total " + n + " records");
	// return rsdc;
	// } catch (Exception e) {
	// System.err.println(e);
	// e.printStackTrace();
	// throw new RuntimeException("ERROR in Database.readDataInstances");
	// }
	// }

	public synchronized ResultSetDynaClass readDataInstances(String tableName) {
		try {
			ResultSet rs = createAndExecuteBriefQuery(tableName);
			System.out.println("Reading from DB to ResultSetDynaClass...");
			ResultSetDynaClass rows = new ResultSetDynaClass(rs);
			// rs.close();
			return rows;
		} catch (Exception e) {
			Log.error(e);
			//e.printStackTrace();
			throw new RuntimeException("ERROR in Database.readDataInstances1");
		}

	}

}