/*
 * Copyright (c) 2005 OpenRules, Inc.
 */
package com.openrules.forms.gui.impl;

import com.openrules.forms.gui.IAttributeList;
import com.openrules.forms.gui.IComponentList;
import com.openrules.forms.gui.IGUIComponent;
import com.openrules.forms.gui.IGUIConstants;
import com.openrules.forms.gui.ILayoutComponent;

public class TableLayout extends GUIComponent   implements ILayoutComponent
{

	public TableLayout()
	{
		super();
	}
	
	public TableLayout(TableLayout igc)
	{
		this.name = igc.getName();
		this.children = igc.getChildren();
		this.dataModelAdaptor = igc.getDataModelAdaptor();
		if (igc.getAttributes() != null)
		{
			this.attributes = new AttributeList(igc.getAttributes());
		}
	}

	public String getName()
	{
		return IGUIConstants.TABLE_CT;
	}
	
	public static void enumerateRows(IComponentList rows)
	{
		int len = rows.getLength();
		for (int i = 0; i < len; i++)
		{
			IGUIComponent gc = rows.getComponent(i);
			gc.setAttribute("class", i % 2 == 0 ? "even" : "odd");
		}		
	}
	
	public static IGUIComponent normalizeTable(IGUIComponent tt)
	{
		IComponentList rows = tt.getChildren();
		if (rows == null)
		  return null;
		
		int len = rows.getLength();
		
		if (len > 1)
		{
			enumerateRows(rows);
			return tt;
		}
		if (len == 0)
		  return null;  
					
		IComponentList cells = rows.getComponent(0).getChildren();
		if (cells == null)
		  return null;
		
		len = cells.getLength();

		if (len > 1)
		  return tt;
		if (len == 0)
		  return null;  

		IComponentList gc_list = cells.getComponent(0).getChildren();
		
		if (gc_list == null)
		  return null;
		  
		len = gc_list.getLength();

		if (len > 1)
		  return tt;
		if (len == 0)
		  return null;  
		
		
		IGUIComponent gc = gc_list.getComponent(0);
		
		IAttributeList my_attrs = tt.getAttributes();
		
		if (my_attrs == null)
		  return gc;
		  
		GUIComponent gc2 = new GUIComponent(gc);
		gc2.setAttributes(AttributeList.merge(gc.getAttributes(), my_attrs));  
		    
		
		return gc2;							  
		
	} 
	
	static public TableLayout merge(TableLayout g1, TableLayout g2)
	{
		if (g1 == null)
		  return g2;
		if (g2 == null)
		  return g1;  
		
		TableLayout g = new TableLayout(g1);
		
		g.setChildren(ComponentList.merge(g1.getChildren(), g2.getChildren()));
		g.setAttributes(AttributeList.merge(g1.getAttributes(), g2.getAttributes()));
		return g;
	}
	
	


}
