/*
 * Copyright (c) 2005-2009 OpenRules, Inc.
 */
package com.openrules.forms.gui.jsp;

import java.io.PrintWriter;
import java.util.Map;

import org.openl.util.Log;

import com.openrules.forms.IView;
import com.openrules.forms.gui.IAttribute;
import com.openrules.forms.gui.IAttributeList;
import com.openrules.forms.gui.IComponentList;
import com.openrules.forms.gui.IDataModelAdaptor;
import com.openrules.forms.gui.IDomainAdaptor;
import com.openrules.forms.gui.IGUIComponent;
import com.openrules.forms.gui.IGUIConstants;
import com.openrules.forms.gui.impl.ComponentList;
import com.openrules.forms.gui.impl.GUIComponent;
import com.openrules.forms.gui.impl.TableLayout;
import com.openrules.forms.impl.ValueAdaptor;
import org.openl.util.Log;

/**
 * 
 * JF: 01/13/2006 added TEXTAREA implementation
 */
public class JspRenderer
{
	/**
	 * @param view
	 * @param pw
	 */
	public void render(IView view, PrintWriter pw, Map inputMapping)
	{
		render(view.getTopComponent(), pw, inputMapping);
	}

	void renderRadioComponentList(
		IDomainAdaptor da,
		String name,
		String selectedValue,
		PrintWriter pw,
		Map inputMapping,
		IGUIComponent separator,
		boolean submit)
	{
		int len = da.getLength();

		for (int i = 0; i < len; i++)
		{
			String value = da.getValue(i);

			GUIComponent gc = new GUIComponent();
			gc.setName(IGUIConstants.INPUT_CT);
			gc.setAttribute(IGUIConstants.VALUE_AT, value);
			gc.setAttribute(IGUIConstants.TYPE_AT, IGUIConstants.RADIO_AV);
			gc.setAttribute(IGUIConstants.NAME_AT, name);

			if (value.equals(selectedValue))
				gc.setAttribute(IGUIConstants.CHECKED_AT, "true");
			
			if (submit) 
				gc.setAttribute("onclick","this.form.submit()");

			renderSimpleTag(gc, pw, inputMapping);

			pw.print(da.getDisplayValue(i));
			if (separator != null)
				renderSimpleTag(separator, pw, inputMapping);
		}
	}
	
	void renderCheckboxTag(
		IGUIComponent gc,
		IDomainAdaptor da,
		String value,
		PrintWriter pw,
		Map inputMapping)
	{
		if (value.equals("true"))
			gc.setAttribute(IGUIConstants.CHECKED_AT, "true");
		renderSimpleTag(gc, pw, inputMapping);
		//String domainValue = da.getValue(0);
		String domainValue = (da.getLength() == 0) ? null : da.getValue(0); // JF
		//Log.info("CHECKBOX domainValue:"+domainValue);
		if (domainValue != null)
		{
			pw.print(domainValue);
		}
	}

	IComponentList makeComboBoxComponentList(
		IDomainAdaptor da,
		String selectedValue)
	{
		int len = da.getLength();
		ComponentList list = new ComponentList(len);

		for (int i = 0; i < len; i++)
		{
			String value = da.getValue(i);
			if (value == null)
				continue;  // added by JF
			GUIComponent gc = new GUIComponent();
			gc.setName(IGUIConstants.OPTION_CT);
			gc.setAttribute(IGUIConstants.VALUE_AT, value);
			if (value.equals(selectedValue))
				gc.setAttribute(
					IGUIConstants.SELECTED_AT,
					IGUIConstants.SELECTED_AV);
			gc.setDataModelAdaptor(new ValueAdaptor(da.getDisplayValue(i)));

			list.addComponent(gc);
		}

		return list;

	}

	void renderATag(IGUIComponent gc, PrintWriter pw, Map inputMapping)
	{

		GUIComponent gc2 = new GUIComponent(gc);
		gc2.setName(IGUIConstants.A_CT);
		gc2.setChildren(gc.getChildren());
		//		gc2.setDataModelAdaptor(null);

		String name = newUniqueName();
		gc2.setAttribute(IGUIConstants.HREF_AT, "?" + name + "=submit");

		IDataModelAdaptor model = gc.getDataModelAdaptor();

		inputMapping.put(name, gc);

		renderSimpleTag(gc2, pw, inputMapping);
	}

	void renderInputTag(IGUIComponent gc, PrintWriter pw, Map inputMapping)
	{
		IAttributeList gcAttrs = gc.getAttributes();
		String type = null;

		String iname = null;
		
		if (gcAttrs != null)
		{
			type = gcAttrs.get(IGUIConstants.TYPE_AT);

			if (IGUIConstants.HREF_AT.equals(type))
			{
				renderATag(gc, pw, inputMapping);
				return;
			}
			
			iname = gc.getAttributes().get(IGUIConstants.NAME_AT);
		}

		GUIComponent gc2 = new GUIComponent(gc);
		gc2.setDataModelAdaptor(null);

		IDataModelAdaptor model = gc.getDataModelAdaptor();

		if (iname == null)
		{
			iname = newUniqueName();
			gc2.setAttribute(IGUIConstants.NAME_AT, iname);
		}

		inputMapping.put(iname, gc);

		String ivalue = null;

		if (gc.getAttributes() == null
			|| ((ivalue = (String) gc.getAttributes().get(IGUIConstants.VALUE_AT))
				== null))
		{
			ivalue = model == null ? null : model.getValue();
		}
		
		if (IGUIConstants.TEXTAREA_AV.equals(type))
		{
			renderTextareaTag(gc2, ivalue, pw);
			return;
		}
		
		IDomainAdaptor domain = model == null ? null : model.getDomainAdaptor();

		
		if (domain != null)
		{
			if (IGUIConstants.RADIO_AV.equals(type))
			{
				String orientation = null;
				boolean submit = false;
				if (gc.getAttributes() != null)
				{
					orientation =
						gc.getAttributes().get(IGUIConstants.ORIENTATION_PA);
					submit = gcAttrs.get(IGUIConstants.SUBMIT) != null;
				}
				if (orientation == null)
				  orientation = IGUIConstants.ORIENTATION_DEFAULT_PV;
				
				IGUIComponent separator = 
					!IGUIConstants.ORIENTATION_HORIZONTAL_PV.equals(orientation)
					? new GUIComponent(IGUIConstants.BR_CT)
					: null;
					
				renderRadioComponentList(
					domain,
					iname,
					ivalue,
					pw,
					inputMapping,
					separator,
					submit);
				return;
			}
			
			if (IGUIConstants.CHECKBOX_AV.equals(type))
			{
				renderCheckboxTag(gc2, domain, ivalue, pw, inputMapping);
				return;
			}
			
			gc2.setChildren(makeComboBoxComponentList(domain, ivalue));
			gc2.setName(IGUIConstants.COMBOBOX_CT);
		}
		else 
		if (ivalue != null)
			gc2.setAttribute(IGUIConstants.VALUE_AT, ivalue);

		renderSimpleTag(gc2, pw, inputMapping);
	}

	int id = 0;

	/**
	 * @return
	 */
	private String newUniqueName()
	{
		return ("input_field_" + (++id)).intern();
	}

	void render(IGUIComponent gc, PrintWriter pw, Map inputMapping)
	{
		String type = gc.getName();
		if (IGUIConstants.TABLE_CT.equals(type))
		{
			gc = TableLayout.normalizeTable(gc);
			if (gc == null)
			  return;
			type = gc.getName();  
		}

		if (IGUIConstants.INPUT_CT.equals(type))
		{
			renderInputTag(gc, pw, inputMapping);
			return;
		}

		//		if (IGUIConstants.A_CT.equals(type))
		//		{
		//			renderATag(gc, pw, inputMapping);
		//			return;
		//		}


//		if (gc != null)
			renderSimpleTag(gc, pw, inputMapping);
	}

	void renderSimpleTag(IGUIComponent gc, PrintWriter pw, Map inputMapping)
	{
		String tag = getComponentTag(gc);
		boolean tagClosed = true;
		if (tag != null)
		{
			pw.print("<" + tag);
			tagClosed = false;
			renderAttributes(gc,pw);
		} 
		//text
		String text = getText(gc);
		if (text != null)
		{

			if (!tagClosed)
			{
				pw.println(">");
				tagClosed = true;
			}

			pw.print(text);
		}
		//children
		IComponentList children = gc.getChildren();
		if (children != null)
		{
			if (!tagClosed)
			{
				pw.println(">");
				tagClosed = true;
			}

			for (int i = 0; i < children.getLength(); i++)
			{
				IGUIComponent c = children.getComponent(i);
				if (c != null)
					render(c, pw, inputMapping);
			}
		} 
		//close
		if (tag != null)
		{
			if (tagClosed)
				pw.println("</" + tag + ">");
			else
				pw.println("/>");
		}

	} 
	
	void renderTextareaTag(IGUIComponent gc, String value, PrintWriter pw)
	{
		String tag = IGUIConstants.TEXTAREA_CT;
		pw.print("<" + tag);
		renderAttributes(gc,pw);
		pw.println(">");
		if (value != null)
		{
			pw.print(value);
		}
		pw.println("</" + tag + ">");
	}
	
	
	void renderAttributes(IGUIComponent gc, PrintWriter pw)
	{
		IAttributeList httpAttributes = getHttpAttributes(gc);
		if (httpAttributes != null)
		{
			int length = httpAttributes.getLength();
			for (int i = 0; i < length; ++i)
			{
				IAttribute attr = httpAttributes.getAttribute(i);
				if (!pseudoAttribute(attr))
					pw.print(" "
							+ attr.getName()
							+ "=\""
							+ httpString(attr.getValue())
							+ "\"");
			}
		}
	} 
	
	/**
	 * @param attr
	 * @return
	 */
	private boolean pseudoAttribute(IAttribute attr)
	{
		return attr.getName().startsWith("_");
	}

	/**
	 * @param gc
	 * @return
	 */
	protected String getText(IGUIComponent gc)
	{
		//		IDataModelAdaptor dataAdaptor = gc.getDataModelAdaptor();
		//		return dataAdaptor == null ? null : dataAdaptor.getValue();
		return gc.getStringValue();
	} 
	
	/**
	 * @param object
	 * @return
	 */
	protected String httpString(Object object)
	{
		// TODO Auto-generated method stub
		return (String) object;
	} 
	/**
	 * @param gc
	 * @return
	 */
	protected IAttributeList getHttpAttributes(IGUIComponent gc)
	{ // TODO add style-conversion
		return gc.getAttributes();
	}

	/**
	 * @param gc
	 * @return
	 */
	protected String getComponentTag(IGUIComponent gc)
	{
		//TODO fix
		//		if (gc.getPresentationComponent() != null)
		//			return gc.getPresentationComponent().getName();
		return gc.getName();
	}

}
