/*
 * Copyright (c) 2005-2009 OpenRules, Inc.
 */
package com.openrules.forms.gui.jsp;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.openl.util.Log;
import org.openl.util.OpenIterator;

import com.openrules.forms.IAction;
import com.openrules.forms.IFormsRequest;
import com.openrules.forms.IInputValues;
import com.openrules.forms.gui.IDataModelAdaptor;
import com.openrules.forms.gui.IGUIComponent;

public class JspRequest implements IFormsRequest, IInputValues, IAction
{

	HttpServletRequest httpRequest;
	Map inputMap;

	/**
	 * @param httpRequest
	 */
	public JspRequest(HttpServletRequest httpRequest, Map inputMap)
	{
		this.httpRequest = httpRequest;
		this.inputMap = inputMap;
	}

	/**
	 *
	 */

	public IAction getAction()
	{
		return this;
	}

	/**
	 *
	 */

	public IInputValues getInputValues()
	{
		return this;
	}

	/**
	 *
	 */

	public Iterator allInputNames()
	{
		return OpenIterator.fromEnumeration(httpRequest.getParameterNames());
	}

	/**
	 *
	 */

	public String getInputValue(String name)
	{
		String result = httpRequest.getParameter(name);
		//System.out.println("request parameter: " + name + "=" + result);
		return result;
	}

	/**
	 *
	 */

	public IGUIComponent getComponent(String name)
	{
		return inputMap == null ? null : (IGUIComponent) inputMap.get(name);
	}
	
	Map validateInput()
	{
		Map errorMap = null;

		for (Iterator iter = allInputNames(); iter.hasNext();)
		{
			String key = (String) iter.next();

			int len = key.length();

			String param = null;

			if (len >= 2 &&  // Added by JF
				key.charAt(len - 2) == '.') //.x, .y
			{
				if (key.charAt(len - 1) == 'y')
					continue;

				String realKey = key.substring(0, len - 2);
				param = getInputValue(realKey);
				if (param != null)
					continue;
				param = "EU"; //Explorer - undefined
				key = realKey;  

			}

			if (param == null)
				param = getInputValue(key);

			IGUIComponent igc = getComponent(key);

			if (igc == null)
			{
				Log.warn("GUIComponent " + key + " not found");
				continue;
			}

			IDataModelAdaptor ma = igc.getDataModelAdaptor();

			if (ma == null)
			{
				Log.warn("GUIComponent " + key + " does not have data model");
				continue;
			}

			try
			{
				ma.acceptInput(param);
			}
			catch (Throwable t)
			{
				if (errorMap == null)
					errorMap = new HashMap();
				errorMap.put(igc, t);
			}

		}

		return errorMap;

	}

}
