/*
 * Copyright (c) 2005-2009 OpenRules, Inc.
 */
package com.openrules.forms.gui.jsp;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import org.openl.util.Log;

import com.openrules.forms.gui.impl.TableLayout;

public class Question extends DialogComponent {
	String type;
	String[] possibleAnswers;
	String defaultAnswer;
	String domain;
	String answer;
	String showAs;
	TableLayout layout;
	String[] validation; // like { "Range", "1", "100" }
	int size;
	int rows, cols;
	int number;
	boolean answerValid;
	boolean askable;
	boolean applicable;
	boolean required;
	Set<Question> children;

	public Question(Dialog d, String id, String name) {
		super(d, id, name);
		answer = null;
		defaultAnswer = null;
		possibleAnswers = new String[] { "Select" };
		children = new HashSet<Question>();
		layout = null;
		showAs = "";
		validation = null;
		size = 0;
		rows = 0;
		cols = 0;
		askable = true;
		applicable = true;
		required = false;
	}

	public boolean isRequired() {
		return required;
	}

	public void setRequired(boolean required) {
		this.required = required;
	}

	public final boolean isApplicable() {
		return applicable;
	}

	public final void setApplicable(boolean applicable) {
		this.applicable = applicable;
	}

	public final boolean isAskable() {
		return askable;
	}

	public final void setAskable(boolean askable) {
		this.askable = askable;
	}

	public final boolean isAnswerValid() {
		return answerValid;
	}

	public final void setAnswerValid(boolean answerValid) {
		this.answerValid = answerValid;
	}

	public String getDomain() {
		return domain;
	}

	public void setDomain(String domain) {
		this.domain = domain;
	}

	public final int getNumber() {
		return number;
	}

	public final void setNumber(int number) {
		this.number = number;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String[] getPossibleAnswers() {
		return possibleAnswers;
	}

	public void setPossibleAnswers(String[] possibleAnswers) {
		this.possibleAnswers = possibleAnswers;
	}

	public String getAnswer() {
		return answer;
	}

	public void setAnswer(String answer) {
		this.answer = answer;
		setAnswerValid(true);
		if (type.equals("QuestionBoolean")) {
			if (isTrue(answer))
				setAnswerTrue();
			else
				setAnswerFalse();
		}
	}

	public boolean compare(String s) {
		if (!isAnswerDefined()) {
			if (s == null)
				return true; // JF added this check in July-2014
			else
				return false;
		}
		if (answer == null)
			if (s == null)
				return true;
			else
				return false;
		if (!type.equals("QuestionBoolean"))
			return answer.equals(s);
		// boolean question
		if (Question.isTrue(s) && isAnswerTrue())
			return true;
		if (!Question.isTrue(s) && !isAnswerTrue())
			return true;
		return false;
	}

	static boolean isTrue(String s) {
		if ("true".equals(s) || "TRUE".equals(s) || "True".equals(s)
				|| "yes".equals(s) || "YES".equals(s) || "Yes".equals(s)) {
			// Log.info(s + " recognized as true");
			return true;
		}
		// Log.info(s + " recognized as false");
		return false;
	}

	public boolean isAnswerTrue() {
		return Question.isTrue(answer);
	}

	public boolean isAnswerFalse() {
		return !isAnswerTrue();
	}

	public void setAnswerTrue() {
		// Log.info("setAnswerTrue");
		setAnswer("true");
	}

	public void setAnswerFalse() {
		// Log.info("setAnswerFalse");
		setAnswer("false");
	}

	public String getShowAs() {
		return showAs;
	}

	public void setShowAs(String showAs) {
		this.showAs = showAs;
	}

	public TableLayout getLayout() {
		return layout;
	}

	public void setLayout(TableLayout layout) {
		this.layout = layout;
	}

	public void setAnswers(Answers[] questionAnswers) {
		for (int i = 0; i < questionAnswers.length; i++) {
			Answers qa = questionAnswers[i];
			if (qa.question.equals(this.getId())) {
				String a = qa.getDefaultAnswer();
				setAnswer(a);
				setDefaultAnswer(a);
				if (qa.getPossibleAnswers() != null)
					this.possibleAnswers = qa.getPossibleAnswers().getChoices();
			}
		}
	}

	public void setAnswers(String defaultAnswer, String[] possibleAnswers) {
		setDefaultAnswer(defaultAnswer);
		setPossibleAnswers(possibleAnswers);
	}

	public void setAnswers(String defaultAnswer, String pointerToPossibleAnswers) {
		setDefaultAnswer(defaultAnswer);
		if (pointerToPossibleAnswers == null)
			return;
		try {
			PossibleAnswers[] possibleAnswers = (PossibleAnswers[]) (getDialog().getEngine().run("getPossibleAnswers"));
			if (possibleAnswers == null)
				return;
			for (PossibleAnswers pa : possibleAnswers) {
				if (pointerToPossibleAnswers.equals(pa.getId())) {
					setPossibleAnswers(pa.getChoices());
					return;
				}
			}
			
		} catch (Exception e) {
			Log.error("The method \"getPossibleAnswers\" is not defined in Excel");
		}
		
	}

	public final String[] getValidation() {
		return validation;
	}

	public final void setValidation(String[] validation) {
		this.validation = validation;
	}

	public final int getSize() {
		return size;
	}

	public final void setSize(int size) {
		this.size = size;
	}

	public int getRows() {
		return rows;
	}

	public void setRows(int rows) {
		this.rows = rows;
	}

	public int getCols() {
		return cols;
	}

	public void setCols(int cols) {
		this.cols = cols;
	}

	public boolean isValidationPattern(String pattern) {
		if (validation != null && validation[0].equals(pattern))
			return true;
		return false;
	}

	public boolean isAnswerDefined() {
		if (answer == null || answer.trim().length() == 0)
			return false;
		return true;
	}

	public String validateIntegerAnswer() {
		String OK = "";
		if (!isAnswerDefined())
			return OK; // do not validate empty input

		int answer;
		try {
			answer = Integer.parseInt(getAnswer());
		} catch (Exception e) {
			return "Invalid integer answer";
		}

		if (validation != null && validation.length == 3
				&& validation[0].equals("Range"))
			try {
				int min = Integer.parseInt(getValidation()[1]);
				int max = Integer.parseInt(getValidation()[2]);
				if (answer >= min && answer <= max)
					return OK;
				else
					return "The answer should be within " + min + " and " + max;
			} catch (Exception e) {
				return "Invalid validation parameters";
			}
		return OK;
	}

	public String validateDoubleAnswer() {
		String OK = "";
		if (answer == null || answer.trim().length() == 0)
			return OK; // do not validate an empty input

		double answer;
		try {
			answer = Double.parseDouble(getAnswer());
		} catch (Exception e) {
			return "Invalid numeric answer";
		}

		if (validation != null && validation.length == 3
				&& validation[0].equals("Range"))
			try {
				double min = Double.parseDouble(getValidation()[1]);
				double max = Double.parseDouble(getValidation()[2]);
				if (answer >= min && answer <= max)
					return OK;
				else
					return "The answer should be within " + min + " and " + max;
			} catch (Exception e) {
				return "Invalid validation parameters";
			}
		return OK;
	}

	public String validateRange() {
		String OK = "";
		if (answer == null || answer.trim().length() == 0)
			return OK; // do not validate an empty input

		double answer;
		try {
			answer = Double.parseDouble(getAnswer());
		} catch (Exception e) {
			return "Invalid numeric answer";
		}

		if (validation != null && validation.length == 3
				&& validation[0].equals("Range"))
			try {
				double min = Double.parseDouble(getValidation()[1]);
				double max = Double.parseDouble(getValidation()[2]);
				if (answer >= min && answer <= max)
					return OK;
				else {
					getDialog().addError();
					return "ERROR: The answer should be within "
							+ getValidation()[1] + " and " + getValidation()[2];
				}
			} catch (Exception e) {
				return "Invalid validation parameters";
			}
		return OK;
	}

	void removeChar(char c) {
		if (answer == null)
			return;
		String r = "";
		for (int i = 0; i < answer.length(); i++) {
			if (answer.charAt(i) != c)
				r += answer.charAt(i);
		}
		answer = r;
	}

	public double getDoubleAnswer() {
		try {
			removeChar(',');
			return Double.parseDouble(getAnswer());
		} catch (Exception e) {
			Log.error("Invalid double answer: " + answer);
			return Double.NaN;
		}
	}

	public int getIntAnswer() {
		try {
			removeChar(',');
			return Integer.parseInt(getAnswer());
		} catch (Exception e) {
			Log.error("Invalid integer answer: " + answer);
			return Integer.MIN_VALUE;
		}
	}

	public void setIntAnswer(int ans) {
		setAnswer("" + ans);
	}

	public void setDoubleAnswer(double ans) {
		setAnswer("" + ans);
	}

	public String getDefaultAnswer() {
		return defaultAnswer;
	}

	public void setDefaultAnswer(String defaultAnswer) {
		this.defaultAnswer = defaultAnswer;
	}

	public Set<Question> getChildren() {
		return children;
	}

	public void setParent(String questionId) {
		Question parent = getDialog().getQuestion(questionId);
		if (parent != null) {
			// TO DO make sure that there are no loops
			// Log.info("Add child " + getId() + " for " + parent.getId());
			parent.getChildren().add(this);
		}
	}

	public void resetDefaultAnswer() {
		// Log.info("resetDefaultAnswer for "+getId());
		this.setAnswer(getDefaultAnswer());
		resetChildren();
	}

	public void resetChildren() {
		for (Question child : children) {
			child.resetDefaultAnswer();
		}
	}

	/**
	 * Hides this question and all its children
	 */
	public void hide() {
		// Log.info("hide "+getId());
		setHidden(true);
		hideChildren();
	}

	/**
	 * Hides all children of this question (if any) but not the question itself
	 */
	public void hideChildren() {
		for (Question child : children) {
			child.hide();
		}
	}

	/**
	 * Shows the questions (but not its children)
	 */
	public void show() {
		setHidden(false);
	}

	/**
	 * Shows immediate children of the question
	 */
	public void showChildren() {
		for (Question child : children) {
			child.show();
		}
	}

	/**
	 * Shows immediate children of the question
	 */
	public void showWithAllChildren() {
		setHidden(false);
		for (Question child : children) {
			child.showWithAllChildren();
		}
	}

	@Override
	public String toString() {
		return "Question [id=" + id + ", type=" + type + ", defaultAnswer="
				+ defaultAnswer + ", domain=" + domain + ", answer=" + answer
				+ ", hidden=" + hidden + "]";
	}

}
