/*
 * Copyright (c) 2005-2006 OpenRules, Inc.
 */
package com.openrules.forms.xls;

import java.util.Vector;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenlToolAdaptor;
import org.openl.syntax.impl.StringSourceCodeModule;
import org.openl.types.IOpenMethod;
import org.openl.types.impl.CompositeMethod;
import org.openl.util.StringTool;
import org.openl.vm.IRuntimeEnv;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

import com.openrules.forms.gui.IAttribute;
import com.openrules.forms.gui.IAttributeList;
import com.openrules.forms.gui.impl.AttributeList;

public class AttributeListHolder
{

	IAttributeList srcList;

	IOpenMethod[] methods;

	IAttributeList makeAttributes(
		Object target,
		Object[] params,
		IRuntimeEnv env)
	{
		if (methods == null)
			return srcList;

		AttributeList list = new AttributeList();

		for (int i = 0; i < methods.length; i++)
		{
			if (methods[i] == null)
				list.add(srcList.getAttribute(i));
			else
			{
				Object o = methods[i].invoke(target, params, env);
				if (o != null)
					list.put(
						srcList.getAttribute(i).getName(),
						String.valueOf(o));
			}

		}
		return list;
	}
	private static final char START_BRACE = '{', END_BRACE = '}';

	static public AttributeListHolder makeAttributes(
		Node node,
		IOpenSourceCodeModule src,
		OpenlToolAdaptor op)
	{
		NamedNodeMap nnmap = node.getAttributes();
		if (nnmap == null)
			return null;

		int len = nnmap.getLength();
		if (len == 0)
			return null;

		IOpenMethod[] methods = null;

		AttributeList list = new AttributeList();
		boolean hasMethods = false;

		for (int i = 0; i < len; i++)
		{
			Node n = nnmap.item(i);
			String nodeValue = n.getNodeValue();
			list.put(n.getNodeName(), nodeValue);

			if (nodeValue.length() > 0 && nodeValue.charAt(0) == START_BRACE)
			{
				String[] codes =
					StringTool.openBrackets(
						nodeValue,
						START_BRACE,
						END_BRACE,
						"");

				StringSourceCodeModule src0 =
					new StringSourceCodeModule(codes[0], src.getUri(0));

				CompositeMethod getMethod = op.makeMethod(src0);

				if (methods == null)
					methods = new IOpenMethod[len];
				methods[i] = getMethod;

			}
		}

		return new AttributeListHolder(list, methods);
	}

	/**
	 * @param list
	 * @param methods
	 */
	public AttributeListHolder(AttributeList list, IOpenMethod[] methods)
	{
		this.srcList = list;
		this.methods = methods;

	}

	/**
	 * @param attributes
	 * @param sources
	 * @param op
	 * @return
	 */
	public static AttributeListHolder makeAttributes(
		AttributeList list,
		Vector sources,
		OpenlToolAdaptor op)
	{
		IOpenMethod[] methods = null;

		boolean hasMethods = false;
		int len = list.size();

		for (int i = 0; i < len; i++)
		{
			String nodeValue = list.getAttribute(i).getValue();

			if (nodeValue.length() > 0 && nodeValue.charAt(0) == START_BRACE)
			{
				String[] codes =
					StringTool.openBrackets(
						nodeValue,
						START_BRACE,
						END_BRACE,
						"");

				StringSourceCodeModule src0 =
					new StringSourceCodeModule(
						codes[0],
						((IOpenSourceCodeModule) sources.get(i)).getUri(0));

				CompositeMethod getMethod = op.makeMethod(src0);

				if (methods == null)
					methods = new IOpenMethod[len];
				methods[i] = getMethod;

			}
		}

		return new AttributeListHolder(list, methods);
	}

	/**
	 * @param holder
	 * @param extProperties
	 * @return
	 */
	public static AttributeListHolder merge(
		AttributeListHolder h1,
		AttributeListHolder h2)
	{
		if (h1 == null)
			return h2;

		if (h2 == null)
			return h1;

		AttributeList newList =
			new AttributeList(h1.srcList.size() + h2.srcList.size());

		Vector methods = new Vector();
		boolean hasMethods = false;
		for (int i = 0; i < h1.srcList.size(); i++)
		{
			IAttribute attr = h1.srcList.getAttribute(i);
			newList.put(attr.getName(), attr.getValue());
			IOpenMethod m = h1.methods == null ? null : h1.methods[i];
			hasMethods = hasMethods || m != null;
			methods.add(m);
		}

		for (int i = 0; i < h2.srcList.size(); i++)
		{
			IAttribute attr = h2.srcList.getAttribute(i);

			int idx = newList.findIndex(attr.getName());

			if (idx < 0)
			{
				newList.put(attr.getName(), attr.getValue());
				IOpenMethod m = h2.methods == null ? null : h2.methods[i];
				hasMethods = hasMethods || m != null;
				methods.add(m);
			}
		}
		
		IOpenMethod[] om = hasMethods ?  (IOpenMethod[])methods.toArray(new IOpenMethod[methods.size()]) : null;
		
		return new AttributeListHolder(newList, om);
	}

}
