/*
 * Copyright (c) 2005 OpenRules, Inc.
 */
package com.openrules.forms.xls;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenlToolAdaptor;
import org.openl.binding.impl.BoundError;
import org.openl.syntax.impl.StringSourceCodeModule;
import org.openl.syntax.impl.SyntaxError;
import org.openl.vm.IRuntimeEnv;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

import com.openrules.forms.gui.IGUIComponent;
import com.openrules.forms.gui.impl.CellLayoutComponent;
import com.openrules.forms.gui.impl.ComponentList;
import com.openrules.table.ILogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

public class CellBuilder
{
	int colspan = 1, rowspan = 1;

	String halign = null;

	ICellGUIComponentBuilder igcb;

	CellLayoutComponent buildCell(
		Object target,
		Object[] params,
		IRuntimeEnv env)
	{
		CellLayoutComponent clc = new CellLayoutComponent();

		clc.setColspan(colspan);
		clc.setRowspan(rowspan);
		clc.setHalign(halign);

		IGUIComponent gc = igcb.buildComponent(target, params, env);

		if (clc.getName().equals(gc.getName()))
		{
			clc.setAttributes(gc.getAttributes());
			clc.setChildren(gc.getChildren());
		}
		else
		{
			ComponentList cl = new ComponentList();
			cl.add(gc);
			clc.setChildren(cl);

			//			if (gc.getName() == null)
			//				clc.setAttributes(gc.getAttributes());

		}
		return clc;
	}

	/**
	 * @param table
	 * @param op
	 */
	public void addComponent(ILogicalTable table, OpenlToolAdaptor op, AttributeListHolder extProperties)
		throws SyntaxError
	{

		String text = table.getGridTable().getStringValue(0, 0);

		if (text == null)
			text = "";
		else
			text = text.trim();

		GridCellSourceCodeModule src =
			new GridCellSourceCodeModule(table.getGridTable());

		if (text.startsWith("<"))
		{
			Node node = parseTaggedCell(src);

			igcb = makeStructuredlBuilder(node, src, op, extProperties);
		}
		else if (text.startsWith("["))
		{
			StringSourceCodeModule src2 =
				new StringSourceCodeModule(text, src.getUri(1));
			igcb = FieldCellBuilder.makeBuilder(src2, op, extProperties);
		}
		else
		{
			igcb = MethodCellBuilder.makeMethodBuilder(src, op, extProperties);
		}

		/*
				else if (text.startsWith(XlsLayoutBuilder.FIELD_PREFIX))
				{
					igcb = FieldCellBuilder.makeBuilder(table, op);
				}
				else
					igcb = new XlsLayoutBuilder.TextGUIComponentBuilder(text);
		*/
	}

	/**
	 * @param node
	 * @param src
	 * @return
	 */
	private ICellGUIComponentBuilder makeStructuredlBuilder(
		Node node,
		IOpenSourceCodeModule src,
		OpenlToolAdaptor op, AttributeListHolder extProperties)
		throws SyntaxError
	{
		return StructuredBuilder.makeStructuredlBuilder(node, src, op, extProperties);
	}

	Node parseTaggedCell(IOpenSourceCodeModule src) throws BoundError
	{

		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		Document doc = null;

		try
		{

			DocumentBuilder dp = factory.newDocumentBuilder();

			InputSource is = new InputSource(src.getCharacterStream());

			doc = dp.parse(is);
		}
		catch (Exception ex)
		{
			throw new BoundError(null, null, ex, src);
		}

		return doc.getFirstChild();

	}

}