/*
 * Copyright (c) 2005 OpenRules, Inc.
 */
package com.openrules.forms.xls;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenlToolAdaptor;
import org.openl.syntax.SyntaxErrorException;
import org.openl.syntax.impl.StringSourceCodeModule;
import org.openl.syntax.impl.SyntaxError;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethod;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.IParameterDeclaration;
import org.openl.types.impl.CompositeMethod;
import org.openl.types.impl.MethodSignature;
import org.openl.types.impl.OpenMethodHeader;
import org.openl.types.impl.ParameterDeclaration;
import org.openl.types.java.JavaOpenClass;
import org.openl.util.ArrayTool;
import org.openl.util.StringTool;
import org.openl.vm.IRuntimeEnv;

import com.openrules.data.IString2DataConvertor;
import com.openrules.data.String2DataConvertorFactory;
import com.openrules.forms.gui.IAttributeList;
import com.openrules.forms.gui.IDataModelAdaptor;
import com.openrules.forms.gui.IDomainAdaptor;
import com.openrules.forms.gui.IGUIComponent;
import com.openrules.forms.gui.IGUIConstants;
import com.openrules.forms.gui.impl.DomainAdaptors;
import com.openrules.forms.gui.impl.GUIComponent;

class FieldCellBuilder implements ICellGUIComponentBuilder
{

	private static final char START_BRACKET = '[', END_BRACKET = ']';
	private static final char START_BRACE = '{', END_BRACE = '}';

	IOpenMethod getMethod;
	IOpenMethod setMethod;
	IString2DataConvertor convertor;
	AttributeListHolder attrs;
	IOpenMethod domainAdaptorMethod;

//	String format;

	/**
	*
	*/

	public IGUIComponent buildComponent(
		Object target,
		Object[] params,
		IRuntimeEnv env)
	{

		GUIComponent gc = new GUIComponent();
		gc.setName(IGUIConstants.INPUT_CT);
		IAttributeList attrList = attrs == null ? null : attrs.makeAttributes(target, params, env);
		gc.setAttributes(attrList);
			
		String format = attrList == null ? null : attrList.get(IGUIConstants.FORMAT_PA);	
		gc.setDataModelAdaptor(new FieldValueModel(target, params, env, format));
		return gc;

	}

	class FieldValueModel implements IDataModelAdaptor
	{

		Object target;
		Object[] params;
		IRuntimeEnv env;
		String format;

		IDomainAdaptor domainAdaptor;

		public IDomainAdaptor getDomainAdaptor()
		{
			if (domainAdaptor != null)
				return domainAdaptor;

			if (domainAdaptorMethod == null)
				return null;

			Object res = domainAdaptorMethod.invoke(target, params, env);

			if (res instanceof IDomainAdaptor)
			{
				domainAdaptor = (IDomainAdaptor) res;
			}

			int stringArray = ArrayTool.dimensionOfArray(res, String.class);

			switch (stringArray)
			{
				case 0 :
					{
						String s = (String) res;
						if (s.indexOf(';') < 0)
						{
							String[] ary = StringTool.tokenize(s, ",");
							domainAdaptor = DomainAdaptors.fromArray(ary);
						}
						else
						{
							String[][] ary = StringTool.tokenize2(s, ";", ",");
							domainAdaptor = DomainAdaptors.fromArray2(ary);
						}
					}
					break;
				case 1 :
					domainAdaptor = DomainAdaptors.fromArray((String[]) res);
					break;
				case 2 :
					domainAdaptor = DomainAdaptors.fromArray2((String[][]) res);
					break;
			}

			return domainAdaptor;
		}

		/**
		 * @param adaptor
		 */
		public void setDomainAdaptor(IDomainAdaptor adaptor)
		{
			domainAdaptor = adaptor;
		}

		public void acceptInput(String inputValue)
		{
			Object[] p2 =
				(Object[]) ArrayTool.add(params, inputParam(inputValue));

			setMethod.invoke(target, p2, env);
		}

		/**
		 * @param inputValue
		 * @return
		 */
		protected Object inputParam(String inputValue)
		{
			return convertor == null
				? inputValue
				: convertor.parse(inputValue, format);
		}

		/**
		* @param target
		* @param params
		* @param env
		*/
		public FieldValueModel(Object target, Object[] params, IRuntimeEnv env, String format)
		{
			this.target = target;
			this.params = params;
			this.env = env;
			this.format = format;
		}

		public String getValue()
		{
			Object res = getMethod.invoke(target, params, env);
			if (res == null)
				return null;
			return convertor == null
				? String.valueOf(res)
				: convertor.format(res, format);
		}

		/**
		 *
		 */

		public void validateInput(String inputValue)
		{
			// TODO Auto-generated method stub
		}

	}

	static final String __tmpvar = "p0";

	static public FieldCellBuilder makeBuilder(
		IOpenSourceCodeModule src,
		OpenlToolAdaptor op,
		AttributeListHolder attrs
		
		)
		throws SyntaxError
	{

		String code = src.getCode().trim();

		String[] codes = null;

		if (code.charAt(0) == START_BRACKET)
		{
			codes =
				StringTool.openBrackets(code, START_BRACKET, END_BRACKET, " ");
		}
		else if (code.charAt(0) == START_BRACE)
		{
			codes = StringTool.openBrackets(code, START_BRACE, END_BRACE, " ");
		}
		else
			codes = new String[] { code, "", "" };

		StringSourceCodeModule src0 =
			new StringSourceCodeModule(codes[0], src.getUri(0));

		CompositeMethod getMethod = op.makeMethod(src0);

		CompositeMethod domainMethod = null;
		if (codes.length > 1 && codes[1].length() > 0)
		{
			StringSourceCodeModule src1 =
				new StringSourceCodeModule(codes[1], src.getUri(0));

			domainMethod = op.makeMethod(src1);
		}

		CompositeMethod setMethod = null;
		String src2 = null;
		IOpenClass fieldType = null;
		IString2DataConvertor convertor = null;

		if (codes.length > 2 && codes[2].length() > 0)
		{
			src2 = codes[2];
			fieldType = JavaOpenClass.STRING;
		}
		else
		{
			fieldType = getMethod.getMethodBodyBoundNode().getType();

			Class c = fieldType.getInstanceClass();

			convertor =
				c == String.class
					? null
					: String2DataConvertorFactory.getConvertor(c);
			src2 = codes[0];

			src2 += " = " + __tmpvar;
		}

		IOpenMethodHeader header = op.getHeader();

		MethodSignature ms = new MethodSignature(header.getSignature());

		MethodSignature ms2 =
			ms.merge(
				new IParameterDeclaration[] {
					 new ParameterDeclaration(fieldType, __tmpvar)});

		IOpenMethodHeader header2 =
			new OpenMethodHeader(
				header.getName(),
				JavaOpenClass.VOID,
				ms2,
				header.getDeclaringClass());

		StringSourceCodeModule s2 =
			new StringSourceCodeModule(src2, src.getUri(0));

		try
		{
			setMethod = op.makeMethod(s2, header2);
		}
		catch (SyntaxErrorException se)
		{
			//TODO subst src	
			throw se;
		}

		return new FieldCellBuilder(
			getMethod,
			setMethod,
			convertor,
			domainMethod, attrs);

	}
	/**
	 * @param getMethod
	 * @param setMethod
	 */
	public FieldCellBuilder(
		IOpenMethod getMethod,
		IOpenMethod setMethod,
		IString2DataConvertor convertor,
		IOpenMethod domainAdaptorMethod,
		AttributeListHolder attrs)
	{
		this.getMethod = getMethod;
		this.setMethod = setMethod;
		this.convertor = convertor;
		this.domainAdaptorMethod = domainAdaptorMethod;
		this.attrs = attrs;

	}

}