/*
 * Copyright (C) 2004-2006 OpenRules, Inc.
 */

package com.openrules.forms.xls;

import java.util.Vector;

import org.openl.OpenL;
import org.openl.OpenlToolAdaptor;
import org.openl.binding.IBindingContext;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.IBoundNode;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.OpenLRuntimeException;
import org.openl.binding.impl.ABoundNode;
import org.openl.binding.impl.BoundError;
import org.openl.binding.impl.module.ModuleOpenClass;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.impl.AMethod;
import org.openl.vm.IRuntimeEnv;

import com.openrules.forms.gui.impl.AttributeList;
import com.openrules.forms.gui.impl.TableLayout;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;
import com.openrules.table.IGridTable;
import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;
import com.openrules.table.openl.GridTableSourceCodeModule;

public class LayoutTableBoundNode
	extends ABoundNode
	implements IMemberBoundNode
{

	static final String PROPERTIES_HEADER = "properties",
		LAYOUT_HEADER = "layout";

	OpenL openl;
	IOpenMethodHeader header;

	LayoutMethod method;

	ModuleOpenClass module;

	AttributeListHolder properties = null;

	/**
	 * @param syntaxNode
	 * @param children
	 */
	public LayoutTableBoundNode(
		XlsTableSyntaxNode methodNode,
		OpenL openl,
		IOpenMethodHeader header,
		ModuleOpenClass module)
	{
		super(methodNode, IBoundNode.EMPTY);
		this.header = header;
		this.openl = openl;
		this.module = module;
	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IMemberBoundNode#addTo(org.openl.binding.impl.module.ModuleOpenClass)
	 */
	public void addTo(ModuleOpenClass openClass)
	{
		openClass.addMethod(method = new LayoutMethod(header));
	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IMemberBoundNode#finalizeBind(org.openl.binding.IBindingContext)
	 */
	public void finalizeBind(IBindingContext cxt) throws Exception
	{

		XlsTableSyntaxNode dtNode = (XlsTableSyntaxNode) syntaxNode;

		IGridTable table = dtNode.getTable();

		ILogicalTable lt = new LogicalTable(table);

		if (lt.getLogicalHeight() < 2)
			throw new BoundError(
				null,
				"Layout table must have at least 2 rows: {header},  [properties] , {layout}",
				null,
				new GridTableSourceCodeModule(table));

		OpenlToolAdaptor op =
			new OpenlToolAdaptor(openl, (IBindingContextDelegator) cxt);
		op.setHeader(header);

		ILogicalTable p_l = lt.rows(1);

		String propertiesX = p_l.getGridTable().getStringValue(0, 0);

		if (PROPERTIES_HEADER.equals(propertiesX))
		{
			loadProperties(p_l.getLogicalRow(0).columns(1), op);
			if (lt.getLogicalHeight() < 3)
				throw new BoundError(
					null,
					"Layout table must have at least 3 rows: {header},  {properties} , {layout}",
					null,
					new GridTableSourceCodeModule(table));
			p_l = p_l.rows(1);
		}

		ILogicalTable layoutTable = p_l;

		xlb = new XlsLayoutBuilder();

		xlb.buildRows(layoutTable, op, properties);

	}

	/**
	 * @param propTable
	 */
	protected void loadProperties(ILogicalTable propTable, OpenlToolAdaptor op)
		throws Exception
	{
		ILogicalTable propValues = propTable;

		AttributeList attributes = new AttributeList();
		Vector sources = new Vector();

		int h = propValues.getLogicalHeight();
		for (int i = 0; i < h; i++)
		{
			ILogicalTable row = propValues.getLogicalRow(i);

			if (row.getLogicalWidth() < 2)
			{
				throw new BoundError(
					null,
					"Property table must have structure: [property_name] [property_value]",
					null,
					new GridTableSourceCodeModule(row.getGridTable()));
			}

			String propertyName = row.getGridTable().getStringValue(0, 0);
			if (propertyName == null || propertyName.trim().length() == 0)
				continue;
			String propertyValue =
				row.getLogicalColumn(1).getGridTable().getStringValue(0, 0);
			if (propertyValue == null || propertyValue.trim().length() == 0)
				continue;
			validateProperty(propertyName, propertyValue, row);

			attributes.put(propertyName, propertyValue);
			sources.add(
				new GridCellSourceCodeModule(
					row.getLogicalColumn(1).getGridTable()));
		}

		if (sources.size() > 0)
			properties =
				AttributeListHolder.makeAttributes(attributes, sources, op);

	}

	/**
	 * @param propertyName
	 * @param propertyValue
	 * @param row
	 */
	private void validateProperty(
		String propertyName,
		String propertyValue,
		ILogicalTable row)
	{
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IBoundNode#evaluateRuntime(org.openl.vm.IRuntimeEnv)
	 */
	public Object evaluateRuntime(IRuntimeEnv env) throws OpenLRuntimeException
	{
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IBoundNode#getType()
	 */
	public IOpenClass getType()
	{
		return header.getType();
	}

	public class LayoutMethod extends AMethod
	{
		public LayoutMethod(IOpenMethodHeader header)
		{
			super(header);
		}

		/**
		 *
		 */

		public Object invoke(Object target, Object[] params, IRuntimeEnv env)
		{
			TableLayout t = xlb.buildTable(target, params, env);
			
			if (!xlb.isSingleCell &&  properties != null)
				t.setAttributes(properties.makeAttributes(target, params, env));

			return t;
		}

	}

	XlsLayoutBuilder xlb;

}
