/*
 * Copyright (c) 2005-2006 OpenRules, Inc.
 */
package com.openrules.forms.xls;

import java.util.Vector;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenlToolAdaptor;
import org.openl.syntax.impl.StringSourceCodeModule;
import org.openl.syntax.impl.SyntaxError;
import org.openl.vm.IRuntimeEnv;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.openrules.forms.gui.IGUIComponent;
import com.openrules.forms.gui.impl.ComponentList;
import com.openrules.forms.gui.impl.GUIComponent;

public class StructuredBuilder implements ICellGUIComponentBuilder
{

	String name;

	AttributeListHolder attrs;

	Vector builders;

	public StructuredBuilder(String name)
	{
		this.name = name;
	}

	void addBuilder(ICellGUIComponentBuilder b)
	{
		if (builders == null)
			builders = new Vector();
		builders.add(b);
	}

	public ICellGUIComponentBuilder buildTextNode(
	  Node node,
		Node textNode,
		IOpenSourceCodeModule src,
		OpenlToolAdaptor op)
		throws SyntaxError
	{
		return new NodeBuilder(textNode);
	}

	/**
	 *
	 */

	public IGUIComponent buildComponent(
		Object target,
		Object[] params,
		IRuntimeEnv env)
	{
		GUIComponent gc = new GUIComponent();
		gc.setName(name);
		if (attrs != null)
			gc.setAttributes(attrs.makeAttributes(target, params, env));

		ComponentList cl = new ComponentList();

		if (builders == null)
			return gc;

		int len = builders.size();

		for (int i = 0; i < len; i++)
		{
			ICellGUIComponentBuilder b =
				(ICellGUIComponentBuilder) builders.get(i);
			cl.addComponent(b.buildComponent(target, params, env));
		}

		gc.setChildren(cl);

		return gc;
	}

	static public ICellGUIComponentBuilder makeStructuredlBuilder(
		Node node,
		IOpenSourceCodeModule src,
		OpenlToolAdaptor op, AttributeListHolder extProperties)
		throws SyntaxError
	{
//		if (!hasFormsElements(node))
//			return new NodeBuilder(node);

		String tag = node.getNodeName();

		StructuredBuilder sb = null;
		if (FIELD_TAG.equals(tag))
		{
			sb = new StructuredFieldBuilder(node);
		}
		else if (CODE_TAG.equals(tag))
		{
			sb = new StructuredCodeBuilder();
		}
		else
			sb = new StructuredBuilder(tag);

		sb.attrs = AttributeListHolder.makeAttributes(node, src, op);
		
		sb.attrs = AttributeListHolder.merge(sb.attrs, extProperties);

		NodeList nl = node.getChildNodes();

		if (nl == null)
			return sb;

		for (int i = 0; i < nl.getLength(); i++)
		{
			Node xn = nl.item(i);

			if (xn.getNodeType() == Node.ELEMENT_NODE)
				sb.addBuilder(makeStructuredlBuilder(nl.item(i), src, op, null));
			else
			{
				String text = xn.getNodeValue();
				if (text != null && text.trim().length() > 0)
					sb.addBuilder(sb.buildTextNode(node, xn, src, op));
			}
		}

		return sb;
	}

	static boolean hasFormsElements(Node node)
	{
		String tag = node.getNodeName();
		if (isTagSpecial(tag))
			return true;

		NodeList nl = node.getChildNodes();
		if (nl == null)
			return false;
		for (int i = 0; i < nl.getLength(); i++)
		{
			if (hasFormsElements(nl.item(i)))
				return true;
		}
		return false;
	}

	/**
	 * @param tag
	 * @return
	 */
	private static boolean isTagSpecial(String tag)
	{
		for (int i = 0; i < specialTags.length; i++)
		{
			if (specialTags[i].equals(tag))
				return true;
		}
		return false;
	}

	static public final String CODE_TAG = "C", FIELD_TAG = "F";

	static final String[] specialTags = { CODE_TAG, FIELD_TAG };

	static class StructuredFieldBuilder extends StructuredBuilder
	{
		Node fieldNode;
		/**
		* @param name
		*/
		public StructuredFieldBuilder(Node fieldNode)
		{
			super(null);
			this.fieldNode = fieldNode;
		}

		/**
		 *
		 */

		public ICellGUIComponentBuilder buildTextNode(
			Node node,
			Node textNode,
			IOpenSourceCodeModule src,
			OpenlToolAdaptor op)
			throws SyntaxError
		{
			StringSourceCodeModule src2 =
				new StringSourceCodeModule(
					textNode.getNodeValue(),
					src.getUri(0));

			//			String format = null;
			//			NamedNodeMap nnmap = fieldNode.getAttributes();
			//			if (nnmap != null)
			//			{
			//				Node fnode = nnmap.getNamedItem(IGUIConstants.FORMAT_PA);
			//				if (fnode != null)
			//					format = fnode.getNodeValue(); 
			//			}
			//

			FieldCellBuilder fb = FieldCellBuilder.makeBuilder(src2, op, null);

			fb.attrs = AttributeListHolder.makeAttributes(node, src, op);

			return fb;
		}

	}

	static class StructuredCodeBuilder extends StructuredBuilder
	{

		/**
		* @param name
		*/
		public StructuredCodeBuilder()
		{
			super(null);
		}

		/**
		 *
		 */

		public ICellGUIComponentBuilder buildTextNode(
		  Node node,
			Node textNode,
			IOpenSourceCodeModule src,
			OpenlToolAdaptor op)
		{

			StringSourceCodeModule src2 =
				new StringSourceCodeModule(
					textNode.getNodeValue(),
					src.getUri(0));
			MethodCellBuilder mb =
				MethodCellBuilder.makeMethodBuilder(src2, op, null);

			mb.attrs = attrs;

			String[][] xx = new String[1][];

			return mb;
		}

	}

}
