/*
 * Copyright (c) 2005 OpenRules, Inc.
 */
package com.openrules.forms.xls;

import org.openl.OpenlToolAdaptor;
import org.openl.syntax.impl.SyntaxError;
import org.openl.vm.IRuntimeEnv;

import com.openrules.forms.gui.IGUIConstants;
import com.openrules.forms.gui.impl.ComponentList;
import com.openrules.forms.gui.impl.RowLayoutComponent;
import com.openrules.forms.gui.impl.TableLayout;
import com.openrules.table.IGridTable;
import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.ui.ICellStyle;

public class XlsLayoutBuilder
{

	RowBuilder[] rows;

	int hn = 0, wn = 0;



	void normalizeGrid(IGridTable grid)
	{
		int h = grid.getGridHeight();

		int w = grid.getGridWidth();
		
		
		for (int i = 0; i < h; i++)
		{
			boolean empty = true;
			
			for (int j = 0; j < w; j++)
			{
				String text = grid.getStringValue(j, i);
				if (text != null && text.trim().length() != 0)
				{
					empty = false;
					wn = Math.max(wn, j + 1);
				}
				
			}
			if (!empty)
			  hn = i + 1;
		}
		
		
	}
	
	boolean isSingleCell = false;

	public void buildRows(ILogicalTable layout, OpenlToolAdaptor op, AttributeListHolder properties)
		throws SyntaxError
	{
		IGridTable grid = layout.getGridTable();
		normalizeGrid(grid);
		
		LogicalTable lt =  new LogicalTable( grid.getLogicalRegion(0, 0, wn, hn).getGridTable());
		
		isSingleCell = lt.getLogicalHeight() == 1 && lt.getLogicalWidth() == 1;

		rows = new RowBuilder[hn];

		for (int i = 0; i < hn; i++)
		{
			rows[i] = new RowBuilder();

			CellBuilder[] clc = new CellBuilder[wn];

			rows[i].cells = clc;

			for (int j = 0; j < wn; j++)
			{
				if (grid.isPartOfTheMergedRegion(j, i))
				{
					//						we are going to produce CellComponent only for the top left cell
					boolean topLeft = false;

					int ch = 0, cw = 0;
					if ((ch = grid.getCellHeight(j, i)) > 1)
					{
						topLeft = true;
					}

					if ((cw = grid.getCellWidth(j, i)) > 1)
					{
						topLeft = true;
					}

					if (!topLeft)
						continue;
						
					clc[j] = new CellBuilder();
						
					
					clc[j].colspan = cw;
					clc[j].rowspan = ch;
				}
				else
				{
//					String str = grid.getStringValue(j, i);
//					if (str == null || str.trim().length() == 0)
//					  continue;
					clc[j] = new CellBuilder();  
				}

				ICellStyle style = grid.getCellStyle(j, i); 
					
				if (style != null)
				{
					switch(style.getAlignment())
					{
						case ICellStyle.ALIGN_RIGHT:
						  clc[j].halign = IGUIConstants.ALIGN_RIGHT_AV;
						  break;
						case ICellStyle.ALIGN_CENTER:  
							clc[j].halign = IGUIConstants.ALIGN_CENTER_AV;
							break;
					}
				}


				clc[j].addComponent(grid.getLogicalRegion(j, i, 1, 1), op, isSingleCell ? properties : null);
			}

		}

	}

	public TableLayout buildTable(
		Object target,
		Object[] params,
		IRuntimeEnv env)
	{
		TableLayout tlc = new TableLayout();

		ComponentList rlc = new ComponentList();

		tlc.setChildren(rlc);

		for (int i = 0; i < rows.length; i++)
		{
			rlc.addComponent(rows[i].buildRow(target, params, env));
		}
		return tlc;
	}

	static class RowBuilder
	{
		CellBuilder[] cells;

		RowLayoutComponent buildRow(
			Object target,
			Object[] params,
			IRuntimeEnv env)
		{
			RowLayoutComponent rlc = new RowLayoutComponent();

			ComponentList clc = new ComponentList();
			//CellLayoutComponent[cells.length];

			rlc.setChildren(clc);

			for (int i = 0; i < cells.length; i++)
			{
				if (cells[i] != null)
					clc.addComponent(cells[i].buildCell(target, params, env));
			}
			return rlc;
		}
	}

	/**
	 * @return
	 */
	public boolean isSingleCell()
	{
		return isSingleCell;
	}

}
