package com.openrules.java.ant;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 */

import java.lang.reflect.Method;

import org.apache.tools.ant.AntClassLoader;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;

import com.openrules.ruleengine.OpenRulesEngine;

/**
 * This class implements a syntax evaluator for xls-files that can be launched as
 * an Ant target
 * 
 * @author IK
 *
 */

public class EvalXlsTask extends Task {

	private static String LANG_ID = "com.openrules.lang.xls"; //$NON-NLS-1$

	private String source;

	private Path classpath ;
	
	public String getSource() {
		return source;
	}

	public void setSource(String sourceUrl) {
		this.source = sourceUrl;
	}

	public Path createClasspath() {
		if ( getClasspath() != null )
			throw new BuildException(Messages.getString("classpath_set_once")); //$NON-NLS-1$
		setClasspath( new Path( getProject() ) );
		return getClasspath();
	}

	public Path getClasspath() {
		return classpath;
	}

	public void setClasspath(Path classpath) {
		this.classpath = classpath;
	}

	public void execute() throws BuildException {
		String mode = System.getProperty(OpenRulesEngine.OPENRULES_MODE);
		if (mode == null) {
			mode = OpenRulesEngine.OPENRULES_MODE_EXECUTE;
			System.setProperty(OpenRulesEngine.OPENRULES_MODE, mode);
		}
		try {
			compile();
		} catch ( BuildException e ) {
			throw e;
		} catch (Exception e) {
			throw new BuildException(e);
		}
	}

	protected void validate() {
		if ( getSource() == null) 
			throw new BuildException(Messages.getString("source_url_not_set")); //$NON-NLS-1$
		if ( classpath.size() == 0 )
			throw new BuildException(Messages.getString("class_path_not_set")); //$NON-NLS-1$
			
	}

	private static String[] builderExceptions = new String[] {
		"org.openl.OpenConfigurationException", //$NON-NLS-1$
		"org.openl.syntax.SyntaxErrorException" //$NON-NLS-1$
	};
	
	protected boolean isBuilderException( Throwable t ) {
		String cn = t.getClass().getName(); 
		for ( int i = 0; i < builderExceptions.length; ++i ) {
			if ( cn.equals( builderExceptions[i] ) )
				return true;
		}
		return false;
	}
	
	protected void compile() throws Exception {
		validate();
		out(Messages.getString("compiling_source", getSource() ) ); //$NON-NLS-1$
		AntClassLoader cl = new AntClassLoader( getProject(), getClasspath() );
		
		Class comp = cl.loadClass("com.openrules.java.OpenRulesLCompiler"); //$NON-NLS-1$
		Class[] params = new Class[] { String.class, String.class,
				String.class };
		Method compile = comp.getMethod("compile", params); //$NON-NLS-1$
		try {
			compile.invoke(comp.newInstance(), new Object[] { getSource(),
					LANG_ID, null });
		} catch (Throwable e) {
			Throwable t = e;
			while ( t != null && t.getMessage() == null ) {
				t = t.getCause();
			}
			e = ( t == null ? e : t );
			if ( ! isBuilderException(e) )
				throw new BuildException(e);
			
			out( e.getMessage() );
			throw new BuildException(Messages.getString("syntax_errors")); //$NON-NLS-1$
		}

		out(Messages.getString("finished", getSource() ) ); //$NON-NLS-1$

	}

	protected void out(String string) {
		handleOutput(string);
	}

}
