package com.openrules.java;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 * Created on Aug 4, 2006 by IK
 * 
 */

import org.openl.OpenConfigurationException;
import org.openl.OpenL;
import org.openl.binding.impl.Binder;
import org.openl.conf.IConfigurableResourceContext;
import org.openl.conf.IOpenLBuilder;
import org.openl.conf.IOpenLConfiguration;
import org.openl.conf.IUserContext;
import org.openl.conf.OpenLConfiguration;
import org.openl.syntax.impl.Parser;
import org.openl.vm.SimpleVM;

import com.openrules.datasource.DataSourceFactory;
import com.openrules.datasource.IDataSource;
import com.openrules.datasource.IDataSourceFactory;

class JavaOpenLBuilder implements IOpenLBuilder {

	public static final String LANG_ID = "openrules.java";

	public static String DEFAULT_CONFIG_PATH = "classpath:/com/openrules/java/openrules.system.xml";

	private String userConfigPath = "classpath:/openrules.user.xml";

	// private IConfigurableResourceContext configurableResourceContext;
	// private IUserContext ucxt;

	private IDataSourceFactory dataSourceFactory;

	public JavaOpenLBuilder() {
		// configurableResourceContext = new ConfigurableResourceContext(null);
	}

	public String getUserConfigPath(String category) {
		//using default
		return userConfigPath;
	}

	public void setUserConfigPath(String path) {
		this.userConfigPath = path;
	}

	protected IDataSource getConfig(String path) {
		try {
			return dataSourceFactory.fromString(path);
		} catch (Exception e) {
			throw new OpenConfigurationException("Invalid configuration file "
					+ path, null, e);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.conf.IOpenLBuilder#build(java.lang.String)
	 */
	public OpenL build(String category) throws OpenConfigurationException {
		OpenL op = new OpenL();

		IOpenLConfiguration conf = OpenLConfiguration.getInstance(category,
				dataSourceFactory.getUserContext());

		if (conf == null) {
			String path = getCoreConfigPath(category);
			IDataSource coreConfig = getConfig(path);
			IDataSource userConfig = getConfig( getUserConfigPath(category) );
			try {
				conf = ConfigurationHelper.makeConfig(dataSourceFactory,
						coreConfig, userConfig);
				OpenLConfiguration.register(category, dataSourceFactory.getUserContext(), conf, true);
			} catch (Exception e) {
				throw new OpenConfigurationException(
						"Error processing config file: " + path, null, e);
			}
		}

		op.setParser(new Parser(conf));

		op.setBinder(new Binder(conf, conf, conf, conf, conf));

		op.setVm(new SimpleVM());
		return op;
	}

	private String getCoreConfigPath(String category) {
		if ( LANG_ID.equals( category) )
			return DEFAULT_CONFIG_PATH;
		throw new OpenConfigurationException("Unsupported language category " + category, null, null );
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.conf.IOpenLBuilder#setConfigurableResourceContext(org.openl.conf.IConfigurableResourceContext)
	 */
	public void setConfigurableResourceContext(
			IConfigurableResourceContext cxt, IUserContext ucxt) {
		// this.configurableResourceContext = cxt;
		// this.ucxt = ucxt;
		this.dataSourceFactory = new DataSourceFactory(cxt, ucxt);
	}

}