package com.openrules.java;

/*
 * Copyright (C) 2005 OpenRules, Inc.
 * 
 * Created on Aug 4, 2006 by IK
 * 
 */

/**
 * OpenRules adaptor for JavaOpenLBuilder:
 * defined openrules.java as the default language
 */

import org.openl.OpenConfigurationException;
import org.openl.OpenL;
import org.openl.binding.impl.Binder;
import org.openl.conf.IConfigurableResourceContext;
import org.openl.conf.IOpenLBuilder;
import org.openl.conf.IOpenLConfiguration;
import org.openl.conf.IUserContext;
import org.openl.conf.OpenLConfiguration;
import org.openl.syntax.impl.Parser;
import org.openl.util.Log;
import org.openl.vm.SimpleVM;

import com.openrules.forms.xls.XlsFormsOpenLBuilder;
import com.openrules.lang.xls.XlsOpenLBuilder;

public class OpenRulesLBuilder implements IOpenLBuilder {
	
	private IConfigurableResourceContext configurableResourceContext;
	private IUserContext ucxt;

	private XlsFormsOpenLBuilder xlsBuilder = null;
	private JavaOpenLBuilder javaBuilder = null;
	
	protected JavaOpenLBuilder getJavaBuilder() {
		if (javaBuilder == null) {
			javaBuilder = new JavaOpenLBuilder();
			javaBuilder.setConfigurableResourceContext(configurableResourceContext,ucxt);
		}
		return javaBuilder;
	}

	protected XlsFormsOpenLBuilder getXlsBuilder() {
		if (xlsBuilder == null) {
			xlsBuilder = new XlsFormsOpenLBuilder();
			xlsBuilder.setConfigurableResourceContext(configurableResourceContext,ucxt);
		}
		return xlsBuilder;
	}

	public OpenL build(String langId ) throws OpenConfigurationException {
		//Log.debug("LANG: "+langId);
		
		if ( XlsOpenLBuilder.LANG_ID.equals(langId) ) {
			return getXlsBuilder().build(null); 
		}
		if  ( JavaOpenLBuilder.LANG_ID.equals(langId) )
			return getJavaBuilder().build(langId);

		//if configuration in cache then just build OpenL for it
		IOpenLConfiguration conf = OpenLConfiguration.getInstance( langId, ucxt );
		if ( conf != null ) {
			OpenL openl = new OpenL();
			openl.setParser(new Parser(conf));
			openl.setBinder(new Binder(conf, conf, conf, conf, conf));
			openl.setVm(new SimpleVM());
			return openl;
		}
		
		throw new OpenConfigurationException("Invalid language ID: "+ langId, null, null ); 
	}

	public void setConfigurableResourceContext(IConfigurableResourceContext cxt, IUserContext ucxt) {
		this.configurableResourceContext = cxt;
		this.ucxt = ucxt;
	}


}
