package com.openrules.ruleengine;

/**
 * Business maps provide as a dynamic interface for decision when instead of 
 * Java objects a user may simply fill out a map of attributes with known names, types and values.
 * A decision may use the method "executeWithBusinessMaps" that fills out unknown
 * attributes inside the maps.
 */

import java.util.HashMap;

public interface BusinessMap {
	
	/**
	 * @return String with a name of the business concept for this map
	 */
	public String getBusinessConcept();
	
	/**
	 * @return a hash map with names and types of the map's attributes
	 */
	public HashMap<String, String> getNamedTypes();

	/**
	 * Sets a value for the attribute converting a string to the proper type.
	 * @param name a string
	 * @param value a string
	 */
	public void setAttribute(String name, String value);
	
	/**
	 * @param name
	 * @return a string value of the attribute with a given name
	 */
	public String getAttribute(String name);
	
	/**
	 * @param name a string
	 * @return a type of the attribute with a given name
	 */
	public String getType(String name);
	
	/**
	 * @return an underlying decision object of the class with name getBusinesConcept()
	 */
	public DecisionObject getDecisionObject();
	
	/**
	 * 
	 * @return a string with the map's attributes and their types
	 */
	public String showTypes();

}
