/**
 * 
 */
package com.openrules.ruleengine;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

public class BusinessMapImpl implements BusinessMap {

	String businessConcept;
	DecisionObject decisionObject;
	HashMap<String, String> namedTypes;


	public BusinessMapImpl(String businessConcept, DecisionObject decisionObject) {
		this.businessConcept = businessConcept;
		this.decisionObject = decisionObject;
		namedTypes = new HashMap<String, String>();
	}

	/* (non-Javadoc)
	 * @see com.openrules.ruleengine.BusinessMap#getBusinessConcept()
	 */
	@Override
	public String getBusinessConcept() {
		return businessConcept;
	}

	/* (non-Javadoc)
	 * @see com.openrules.ruleengine.BusinessMap#getNamedTypes()
	 */
	@Override
	public HashMap<String, String> getNamedTypes() {
		return namedTypes;
	}

	/* (non-Javadoc)
	 * @see com.openrules.ruleengine.BusinessMap#setAttribute(java.lang.String, java.lang.String)
	 */
	@Override
	public void setAttribute(String name, String value) {
		decisionObject.setAttribute(name, value);
	}

	/* (non-Javadoc)
	 * @see com.openrules.ruleengine.BusinessMap#getAttribute(java.lang.String)
	 */
	@Override
	public String getAttribute(String name) {
		return decisionObject.getAttribute(name);
	}

	/* (non-Javadoc)
	 * @see com.openrules.ruleengine.BusinessMap#getType(java.lang.String)
	 */
	@Override
	public String getType(String name) {
		return namedTypes.get(name);
	}

	/* (non-Javadoc)
	 * @see com.openrules.ruleengine.BusinessMap#getDecisionObject()
	 */
	@Override
	public DecisionObject getDecisionObject() {
		return decisionObject;
	}

	@Override
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("BusinessMap for " + businessConcept);
		HashMap<String,String> attributes = decisionObject.getAttributes();
		for (Map.Entry<String, String> entry : attributes.entrySet()) {
		    String name = entry.getKey();
		    String value = decisionObject.getAttribute(name); //entry.getValue();
		    buf.append("\n\t" + name + " = " + value);
		}
		return buf.toString();
	}
	
	public String showTypes() {
		StringBuffer buf = new StringBuffer();
		buf.append("BusinessMap for " + businessConcept);
		buf.append("\n\t[Attribute Name]: [Attribute Type]");
		HashMap<String,String> attributes = decisionObject.getAttributes();
		for (Map.Entry<String, String> entry : attributes.entrySet()) {
		    String name = entry.getKey();
		    String type = entry.getValue();
		    buf.append("\n\t" + name + ": " + type);
		}
		return buf.toString();
	}
	
	public static String getArrayType(Object object) {
		if (object instanceof java.lang.String[])
			return "String[]";
		else 
		if (object instanceof int[])
			return "int[]";
		else 
		if (object instanceof long[])
			return "long[]";
		else 
		if (object instanceof double[])
			return "double[]";
		else 
		if (object instanceof boolean[])
			return "boolean[]";
		else 
		if (object instanceof java.util.Date[])
			return "Date[]";
		return null;
	}

}
