package com.openrules.ruleengine;

import java.math.BigDecimal;
import java.util.Map;

import com.openrules.types.OperName;

public class CollectionHandler {
	
	Decision decision;
	
	public CollectionHandler(Decision decision) {
		this.decision = decision;
	}
	
	
	public void assignIntArrayFeature(String variableName, String oper, String arrayName) {
		
		int[] array = decision.getIntArray(arrayName);
		if (array.length == 0) {
			decision.log("Array " + arrayName + " is empty. Nothing to assign.");
			return;
		}
		if (OperName.isMin(oper)) {
			int min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min > array[i])
					min = array[i];
			}
			decision.getGlossary().assign(variableName, ""+min);
			return;
		}
		if (OperName.isMax(oper)) {
			int max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max < array[i])
					max = array[i];
			}
			decision.getGlossary().assign(variableName, ""+max);
			return;
		}
		if (OperName.isAverage(oper)) {
			int sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			double average = sum/array.length;
			decision.getGlossary().assign(variableName, ""+average);
			return;
		}
		if (OperName.isSum(oper)) {
			int sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			decision.getGlossary().assign(variableName, ""+sum);
			return;
		}
		if (OperName.isProduct(oper)) {
			double product = array[0];
			for(int i=1; i < array.length; i++) {
				product *= array[i];
			}
			decision.getGlossary().assign(variableName, ""+product);
			return;
		}
		if (OperName.isCount(oper)) {
			decision.getGlossary().assign(variableName, ""+array.length);
			return;
		}
		
		throw new RuntimeException("Unknown assignment operator '" + oper + "' for int[]");
	}
	
	public void assignLongArrayFeature(String variableName, String oper, String arrayName) {
		
		long[] array = decision.getLongArray(arrayName);
		if (array.length == 0) {
			decision.log("Array " + arrayName + " is empty. Nothing to assign.");
			return;
		}
		if (OperName.isMin(oper)) {
			long min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min > array[i])
					min = array[i];
			}
			decision.getGlossary().assign(variableName, ""+min);
			return;
		}
		if (OperName.isMax(oper)) {
			long max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max < array[i])
					max = array[i];
			}
			decision.getGlossary().assign(variableName, ""+max);
			return;
		}
		if (OperName.isAverage(oper)) {
			long sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			double average = sum/array.length;
			decision.getGlossary().assign(variableName, ""+average);
			return;
		}
		if (OperName.isSum(oper)) {
			long sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			decision.getGlossary().assign(variableName, ""+sum);
			return;
		}
		if (OperName.isProduct(oper)) {
			double product = array[0];
			for(int i=1; i < array.length; i++) {
				product *= array[i];
			}
			decision.getGlossary().assign(variableName, ""+product);
			return;
		}
		if (OperName.isCount(oper)) {
			decision.getGlossary().assign(variableName, ""+array.length);
			return;
		}
		
		throw new RuntimeException("Unknown assignment operator '" + oper + "' for long[]");
	}
	
	public void assignDoubleArrayFeature(String variableName, String oper, String arrayName) {
		
		double[] array = decision.getDoubleArray(arrayName);
		if (array.length == 0) {
			decision.log("Array " + arrayName + " is empty. Nothing to assign.");
			return;
		}
		if (OperName.isMin(oper)) {
			double min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min > array[i])
					min = array[i];
			}
			decision.getGlossary().assign(variableName, ""+min);
			return;
		}
		if (OperName.isMax(oper)) {
			double max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max < array[i])
					max = array[i];
			}
			decision.getGlossary().assign(variableName, ""+max);
			return;
		}
		if (OperName.isAverage(oper)) {
			double sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			double average = sum/array.length;
			decision.getGlossary().assign(variableName, ""+average);
			return;
		}
		if (OperName.isSum(oper)) {
			double sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			decision.getGlossary().assign(variableName, ""+sum);
			return;
		}
		if (OperName.isProduct(oper)) {
			double product = array[0];
			for(int i=1; i < array.length; i++) {
				product *= array[i];
			}
			decision.getGlossary().assign(variableName, ""+product);
			return;
		}
		if (OperName.isCount(oper)) {
			decision.getGlossary().assign(variableName, ""+array.length);
			return;
		}
		
		throw new RuntimeException("Unknown assignment operator '" + oper + "' for double[]");
	}
	
	public void assignBigDecimalArrayFeature(String variableName, String oper, String arrayName) {
		
		BigDecimal[] array = decision.getBigDecimalArray(arrayName);
		if (array.length == 0) {
			decision.log("Array " + arrayName + " is empty. Nothing to assign.");
			return;
		}
		if (OperName.isMin(oper)) {
			BigDecimal min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min.compareTo(array[i]) > 0)
					min = array[i];
			}
			decision.getGlossary().assign(variableName, ""+min);
			return;
		}
		if (OperName.isMax(oper)) {
			BigDecimal max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max.compareTo(array[i]) < 0)
					max = array[i];
			}
			decision.getGlossary().assign(variableName, ""+max);
			return;
		}
		if (OperName.isAverage(oper)) {
			BigDecimal sum = new BigDecimal(0);
			for(int i=1; i < array.length; i++) {
				sum.add(array[i]);
			}
			BigDecimal count = new BigDecimal(array.length);
			BigDecimal average = sum.divide(count);
			decision.getGlossary().assign(variableName, ""+average);
			return;
		}
		if (OperName.isSum(oper)) {
			BigDecimal sum = new BigDecimal(0);
			for(int i=1; i < array.length; i++) {
				sum.add(array[i]);
			}
			decision.getGlossary().assign(variableName, ""+sum);
			return;
		}
		if (OperName.isProduct(oper)) {
			BigDecimal product = array[0];
			for(int i=1; i < array.length; i++) {
				product.multiply(array[i]);
			}
			decision.getGlossary().assign(variableName, ""+product);
			return;
		}
		if (OperName.isCount(oper)) {
			decision.getGlossary().assign(variableName, ""+array.length);
			return;
		}
		
		throw new RuntimeException("Unknown operator '" + oper + "' for evaluation of BigDecimal[]");
	}
	

	
	/**
	 * If oper is "All Elements", then it checks if all elements in the array "arrayName" 
	 * satisfy the condition
	 * If oper is "One Element", then it checks if at least one element in the array "arrayName" 
	 * satisfies the condition
	 * @param arrayName
	 * @param oper
	 * @param expression
	 * @return
	 */
	public boolean evaluateIntArray(String arrayName, String oper, String condition) {
		int[] array = decision.getIntArray(arrayName);
		if (array.length == 0) {
			decision.log("Cannot evaluate an empty array " + arrayName + ". Condition failed.");
			return false;
		}
		if (OperName.isOneElement(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareIntValueWithExpression(array[i], condition);
				if (result == true)
				   return true; // at least one element satisfies the condition
			}
			return false;
		}
		if (OperName.isAllElements(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareIntValueWithExpression(array[i], condition);
				if (result == false)
				   return false; // at least one element violates the condition
			}
			return false;
		}
		if (OperName.isMin(oper)) {
			int min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min > array[i])
					min = array[i];
			}
			return decision.glossary.compareIntValueWithExpression(min, condition);
		}
		if (OperName.isMax(oper)) {
			int max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max < array[i])
					max = array[i];
			}
			return decision.glossary.compareIntValueWithExpression(max, condition);
		}
		if (OperName.isAverage(oper)) {
			int sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			double average = sum/array.length;
			return decision.glossary.compareDoubleValueWithExpression(average, condition);
		}
		if (OperName.isSum(oper)) {
			int sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			return decision.glossary.compareIntValueWithExpression(sum, condition);
		}
		if (OperName.isProduct(oper)) {
			double product = array[0];
			for(int i=1; i < array.length; i++) {
				product *= array[i];
			}
			return decision.glossary.compareDoubleValueWithExpression(product, condition);
		}
		if (OperName.isCount(oper)) {
			return decision.glossary.compareIntValueWithExpression(array.length, condition);
		}
		
		throw new RuntimeException("Unknown operator '" + oper + "' for evaluation of int[]");
	}
	
	/**
	 * If oper is "All Elements", then it checks if all elements in the array "arrayName" 
	 * satisfy the condition
	 * If oper is "One Element", then it checks if at least one element in the array "arrayName" 
	 * satisfies the condition
	 * @param arrayName
	 * @param oper
	 * @param expression
	 * @return
	 */
	public boolean evaluateLongArray(String arrayName, String oper, String condition) {
		long[] array = decision.getLongArray(arrayName);
		if (array.length == 0) {
			decision.log("Cannot evaluate an empty array " + arrayName + ". Condition failed.");
			return false;
		}
		if (OperName.isOneElement(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareLongValueWithExpression(array[i], condition);
				if (result == true)
				   return true; // at least one element satisfies the condition
			}
			return false;
		}
		if (OperName.isAllElements(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareLongValueWithExpression(array[i], condition);
				if (result == false)
				   return false; // at least one element violates the condition
			}
			return false;
		}
		if (OperName.isMin(oper)) {
			long min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min > array[i])
					min = array[i];
			}
			return decision.glossary.compareLongValueWithExpression(min, condition);
		}
		if (OperName.isMax(oper)) {
			long max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max < array[i])
					max = array[i];
			}
			return decision.glossary.compareLongValueWithExpression(max, condition);
		}
		if (OperName.isAverage(oper)) {
			long sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			double average = sum/array.length;
			return decision.glossary.compareDoubleValueWithExpression(average, condition);
		}
		if (OperName.isSum(oper)) {
			long sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			return decision.glossary.compareLongValueWithExpression(sum, condition);
		}
		if (OperName.isProduct(oper)) {
			double product = array[0];
			for(int i=1; i < array.length; i++) {
				product *= array[i];
			}
			return decision.glossary.compareDoubleValueWithExpression(product, condition);
		}
		if (OperName.isCount(oper)) {
			return decision.glossary.compareIntValueWithExpression(array.length, condition);
		}
		
		throw new RuntimeException("Unknown operator '" + oper + "' for evaluation of long[]");
	}
	
	/**
	 * If oper is "All Elements", then it checks if all elements in the array "arrayName" 
	 * satisfy the condition
	 * If oper is "One Element", then it checks if at least one element in the array "arrayName" 
	 * satisfies the condition
	 * @param arrayName
	 * @param oper
	 * @param expression
	 * @return
	 */
	/**
	 * If oper is "All Elements", then it checks if all elements in the array "arrayName" 
	 * satisfy the condition
	 * If oper is "One Element", then it checks if at least one element in the array "arrayName" 
	 * satisfies the condition
	 * @param arrayName
	 * @param oper
	 * @param expression
	 * @return
	 */
	public boolean evaluateDoubleArray(String arrayName, String oper, String condition) {
		double[] array = decision.getDoubleArray(arrayName);
		if (array.length == 0) {
			decision.log("Cannot evaluate an empty array " + arrayName + ". Condition failed.");
			return false;
		}
		if (OperName.isOneElement(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareDoubleValueWithExpression(array[i], condition);
				if (result == true)
				   return true; // at least one element satisfies the condition
			}
			return false;
		}
		if (OperName.isAllElements(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareDoubleValueWithExpression(array[i], condition);
				if (result == false)
				   return false; // at least one element violates the condition
			}
			return false;
		}
		if (OperName.isMin(oper)) {
			double min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min > array[i])
					min = array[i];
			}
			return decision.glossary.compareDoubleValueWithExpression(min, condition);
		}
		if (OperName.isMax(oper)) {
			double max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max < array[i])
					max = array[i];
			}
			return decision.glossary.compareDoubleValueWithExpression(max, condition);
		}
		if (OperName.isAverage(oper)) {
			double sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			double average = sum/array.length;
			return decision.glossary.compareDoubleValueWithExpression(average, condition);
		}
		if (OperName.isSum(oper)) {
			double sum = 0;
			for(int i=1; i < array.length; i++) {
				sum += array[i];
			}
			return decision.glossary.compareDoubleValueWithExpression(sum, condition);
		}
		if (OperName.isProduct(oper)) {
			double product = array[0];
			for(int i=1; i < array.length; i++) {
				product *= array[i];
			}
			return decision.glossary.compareDoubleValueWithExpression(product, condition);
		}
		if (OperName.isCount(oper)) {
			return decision.glossary.compareIntValueWithExpression(array.length, condition);
		}
		
		throw new RuntimeException("Unknown operator '" + oper + "' for evaluation of double[]");
	}
	
	/**
	 * If oper is "All Elements", then it checks if all elements in the array "arrayName" 
	 * satisfy the condition
	 * If oper is "One Element", then it checks if at least one element in the array "arrayName" 
	 * satisfies the condition
	 * @param arrayName
	 * @param oper
	 * @param expression
	 * @return
	 */
	public boolean evaluateBigDecimalArray(String arrayName, String oper, String condition) {
		BigDecimal[] array = decision.getBigDecimalArray(arrayName);
		if (array.length == 0) {
			decision.log("Cannot evaluate an empty array " + arrayName + ". Condition failed.");
			return false;
		}
		if (OperName.isOneElement(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareBigDecimalValueWithExpression(array[i], condition);
				if (result == true)
				   return true; // at least one element satisfies the condition
			}
			return false;
		}
		if (OperName.isAllElements(oper)) {
			for(int i=0; i < array.length; i++) {
				boolean result = decision.glossary.compareBigDecimalValueWithExpression(array[i], condition);
				if (result == false)
				   return false; // at least one element violates the condition
			}
			return false;
		}
		if (OperName.isMin(oper)) {
			BigDecimal min = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (min.compareTo(array[i]) > 0)
					min = array[i];
			}
			return decision.glossary.compareBigDecimalValueWithExpression(min, condition);
		}
		if (OperName.isMax(oper)) {
			BigDecimal max = array[0];
			if (array.length > 0)
			for(int i=1; i < array.length; i++) {
				if (max.compareTo(array[i]) < 0)
					max = array[i];
			}
			return decision.glossary.compareBigDecimalValueWithExpression(max, condition);
		}
		if (OperName.isAverage(oper)) {
			BigDecimal sum = new BigDecimal(0);
			for(int i=1; i < array.length; i++) {
				sum.add(array[i]);
			}
			BigDecimal count = new BigDecimal(array.length);
			BigDecimal average = sum.divide(count);
			return decision.glossary.compareBigDecimalValueWithExpression(average, condition);
		}
		if (OperName.isSum(oper)) {
			BigDecimal sum = new BigDecimal(0);
			for(int i=1; i < array.length; i++) {
				sum.add(array[i]);
			}
			return decision.glossary.compareBigDecimalValueWithExpression(sum, condition);
		}
		if (OperName.isProduct(oper)) {
			BigDecimal product = array[0];
			for(int i=1; i < array.length; i++) {
				product.multiply(array[i]);
			}
			return decision.glossary.compareBigDecimalValueWithExpression(product, condition);
		}
		if (OperName.isCount(oper)) {
			return decision.glossary.compareIntValueWithExpression(array.length, condition);
		}
		
		throw new RuntimeException("Unknown operator '" + oper + "' for evaluation of int[]");
	}
	
	/**
	 * Finds a map associated in the glossary with the name "mapName" and checks
	 * if it contains the same value as "value" for the key "key"
	 * 
	 * @param mapName
	 * @param key
	 * @param value
	 * @return true if yes; false if not or if there are no "mapName" found or
	 *         there is no value for the "key"
	 */
	public boolean evaluateMap(String mapName, String key, String value) {
		Map map = (Map) decision.getGlossary().getGetMap(mapName).getObjectValue();
		if (key != null)
			key = key.trim();
		if (value != null)
			value = value.trim();
		if (key == null || key == "")
			return true;
		String mapValue = (String) map.get(key);
		if (mapValue == null || mapValue == "") {
			if (value == null || value == "")
				return true;
			else
				return false;
		} else {
			if (mapValue.equals(value))
				return true;
			else
				return false;
		}
	}

	/**
	 * Puts "value" with the "key"into the map"mapName"
	 * 
	 * @param mapName
	 * @param key
	 * @param value
	 */
	public void assignMap(String mapName, String key, String value) {
		Map<String, Object> map = (Map<String, Object>) decision.getGlossary()
				.getGetMap(mapName).getObjectValue();
		if (key != null)
			key = key.trim();
		if (value != null)
			value = value.trim();
		map.put(key, value);
	}


}
