package com.openrules.ruleengine;

/**
 * This class is used by the batch file "build.bat" in the standard workspace "openrules.medels"to build 
 * execution paths for all goals and save them in the file Goals.xls.
 *
 */

public class DecisionBuilder { //extends DecisionTest {

	/**
	 * Builds an execution path for the "decision" using "goalNames", and saving it the 
	 * Excel file "goalFile". If additionalGoals=true it also builds additional
	 * tables with execution paths for all other goals
	 * paths 
	 * @param decision a Decision
	 * @param goalName a String
	 * @param goalFile a String
	 * @param additionalGoals a boolean
	 * @return
	 */
	static public boolean build(Decision decision, String goalName, String goalFile, boolean additionalGoals) {
		if (decision.getGlossary().isContainsIntersectingNames()) {
			decision.log("WARNING: The glossary contains intersecing names: there could be problem during automatic decision build.");
		}
		try {
			decision.setExecutionPath(goalName);
			
			String generatedDecisionName = decision.getDecisionNameUsingGoal(goalName);
		//	decision.log("\nDEPLOY Execution Path for Goal '" + goalName + "'"
		//			+ " to the file '" + goalFile + "' using the name '" + generatedDecisionName + "'");
			decision.buildExecutionPath(generatedDecisionName,goalFile,additionalGoals);
			return true;
		} catch (Exception e) {
			return false;
		}
	}
	
	/**
	 * A simple Java launcher to build and save execution paths for standard decision models
	 * in the workspace "openrules.models"
	 */
	public static void main(String[] args) {
		String goalName = "Unknown goalName";
		String modelFile = "Unknown modelFile";
		String goalFile = "Unknown goalFile";
		boolean additionalGoals = false;
		if (args.length >= 3) {
			modelFile = args[0];
			goalName = args[1];
			goalFile = args[2];
			if (args.length == 4)
				additionalGoals = Boolean.parseBoolean(args[3]);
		}
		else {
			System.out.println("DecisionBuilder expects three or 4 parameters: modelFile goalName goalFile [false]");
			System.exit(-1);
		}
		
        System.out.println("GOAL_NAME=" + goalName);
        System.out.println("MODEL_FILE=" + modelFile);
        System.out.println("GOAL_FILE=" + goalFile);
		
		Decision decision = new Decision(goalName,modelFile);
		
		DecisionBuilder.build(decision,goalName,goalFile, additionalGoals);
//		if (decision.getGlossary().isContainsIntersectingNames()) {
//			decision.log("Resolve intersecting names problem(s) to use an automatic build.");
//			return;
//		}
//		decision.setExecutionPath(goalName);
//		
//		String generatedDecisionName = decision.getDecisionNameUsingGoal(goalName);
//		decision.log("\nDEPLOY Execution Path for Goal '" + goalName + "'"
//				+ " to the file '" + goalFile + "' using the name '" + generatedDecisionName + "'");
//		decision.buildExecutionPath(generatedDecisionName,goalFile);
	}

}
