package com.openrules.ruleengine;

import java.util.ArrayList;

/*
 * Copyright (C) 2018 OpenRules, Inc.
 * 
 */

/**
 * A placeholder for OpenRulesEngine to access Excel-based data tables. 
 *
 */

public class DecisionData {
	
	OpenRulesEngine engine;
	String xlsDataFileName;
	
	/**
	 * Creates an instance of OpenRulesEngine
	 * @param xlsDataFileName
	 */
	public DecisionData(String xlsDataFileName) {
		 engine = new OpenRulesEngine(xlsDataFileName);
		 this.xlsDataFileName = xlsDataFileName;
	}
		
	/**
	 * Returns an object defined in Excel Data table or a Variable table "name".
	 * This object can be an array of objects (e.g. "employees") or a single variable (e.g. "employee")  
	 * @param name a string with the name of the Excel Data table (e.g. "employees") 
	 * @return an objects that should be cast to already known Java type such as Employee[] or Employee
	 */
	public Object get(String name) {
		try {
			return (Object[]) engine.getData(name);
		} catch (Exception e) {
			return engine.getVariable(name);
		}
	}
	
	/**
	 * Returns an array of objects defined in Excel Data table "name"
	 * @param name a string with the name of the Excel Data table (e.g. "employees") 
	 * @return an array of objects that should be cast to already known Java type such as Employee[] 
	 */
	public Object[] getArray(String name) {
		return engine.getData(name);
	}

	/**
	 * Returns an object defined in Excel Variable table "name"
	 * @param name a string with the name of the Excel Variable table (e.g. "employee") 
	 * @return an object that should be cast to already known Java type such as Employee 
	 */
	public Object getVariable(String name) {
		return engine.getVariable(name);
	}
	
	/**
	 * Reads an array of test cases from Excel table of the type "DecisionTableTest" called "decisionTests"
	 * @param decisionTests a string that represents a name of Excel table of the type "DecisionTableTest"
	 * @return an array of test cases of teh type ArrayList<DecisionTest>
	 */
	public ArrayList<DecisionTest> readTestCases(String decisionTests) {
		
		Decision testDecision = new Decision(engine);
		engine.run(decisionTests, testDecision); // creates tests based on the Excel table "decisionTests"
		ArrayList<DecisionTest> tests = testDecision.getTests();
		return tests;
	}

}
