package com.openrules.ruleengine;

import java.io.File;
import java.io.PrintStream;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.openl.types.IOpenClass;
import org.openl.types.impl.DynamicObject;
import org.openl.types.impl.DynamicObjectField;
import org.openl.util.Log;

import com.openrules.types.BusinessObjectArray;

public class DecisionInterfaceGenerator {
	String path;
	Decision decision;

	public DecisionInterfaceGenerator(Decision decision,String path) {
		this.path = path;
		this.decision = decision;
	}
	
	public void generateJavaClass(Decision decision, String className, String packageName) {

		DynamicObject dynamicObject = null;
		try {
			dynamicObject = (DynamicObject) decision.execute("get" + className, decision);
		} catch (Exception e) {
			dynamicObject = (DynamicObject) decision.execute("get" + className);
		}
				
		if (dynamicObject == null) {
			String msg = "Decision's generateJavaClass cannot find the xls-method <get"
					+ className + ">";
			Log.error(msg);
			throw new RuntimeException(msg);
		}
		generateJavaClass(dynamicObject,className,packageName);
	}
	
	public void generateJavaClass(DynamicObject dynamicObject, String className, String packageName) {
		File dir = new File(path);
		if(!dir.exists()){
			Log.info("Create directory: " + path);
			dir.mkdirs();
		}

		String filename = path + className + ".java";
		Log.info("Generate " + filename + "...");
		PrintStream out = null;
		try {
			out = new PrintStream(filename, "UTF-8");
			
		} catch (Exception e) {
			String msg = "Cannot create output file " + filename;
			Log.error(msg);
			throw new RuntimeException(msg);
		}
		
		//Prolog
		out.println("package " + packageName + ";");
		out.println("//");
		out.println("// This is the generated interface for the decision object "+ className);
		out.println("//");
		out.println("import com.openrules.ruleengine.DecisionObject;");
		out.println("import com.openrules.types.DateUtil;");
		out.println("import java.util.HashMap;");
		out.println("import com.openrules.ruleengine.BusinessMap;");
		out.println("import com.openrules.ruleengine.BusinessMapImpl;");
		
		out.println("\npublic class " + className + " implements DecisionObject {");
		out.println("\n\tstatic final char div = ',';");
		
		IOpenClass openClass = dynamicObject.getType();
		HashMap fields = dynamicObject.getFields();
		
		// Generate Getters/Setters
		Iterator it = fields.entrySet().iterator();
		int n = 0;
		while (it.hasNext()) {
			Map.Entry pair = (Map.Entry) it.next();
			String name = (String) pair.getKey();
			DynamicObjectField field = (DynamicObjectField) openClass.getField(name);
			String type = field.getType().getName();
			
			String arrayType = BusinessObjectArray.getType(pair.getValue());
			if (arrayType != null)
				type = arrayType;
			
			out.println("\n\tprotected " + type + " " + name +";");
			String capitalizedName = name.substring(0, 1).toUpperCase() + name.substring(1);
			// Getter
			out.println("\tpublic " + type + " get" + capitalizedName + "() {");
			out.println("\t\treturn " + name + ";");
			out.println("\t}");
			// Setter
			out.println("\tpublic void set" + capitalizedName + "(" + type + " " + name + ") {");
			out.println("\t\tthis." + name + " = " + name + ";");
			out.println("\t}");
			n++;
		}
		
		// Generate getAttribute(String name) 
		out.println("\n\tpublic String getAttribute(String name) {");
		it = fields.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry pair = (Map.Entry) it.next();
			String name = (String) pair.getKey();
			String capitalizedName = name.substring(0, 1).toUpperCase() + name.substring(1);
			DynamicObjectField field = (DynamicObjectField) openClass.getField(name);
			
			out.println("\t\tif (\"" + name + "\".equals(name)) {");
			String arrayType = BusinessObjectArray.getType(pair.getValue());
			if (arrayType != null) {
				out.println("\t\t\t" + arrayType + " array = get"+ capitalizedName + "();");
				out.println("\t\t\tif (array == null) return null;");
				out.println("\t\t\tif (array.length == 1) return \"\" + array[0];");
				out.println("\t\t\tStringBuffer buf = new StringBuffer();");
				out.println("\t\t\tfor (int i = 0; i < array.length; i++) {");
				out.println("\t\t\t\tif (i > 0) buf.append(div);");
				out.println("\t\t\t\tbuf.append(array[i]);");
				out.println("\t\t\t}");
				out.println("\t\t\treturn buf.toString();");
				out.println("\t\t}");
				continue;
			}
			String type = field.getType().getName();
			String returnValue;
			if (type.equals("java.lang.String"))
				returnValue = "get" + capitalizedName + "();";
			else if (type.equals("java.util.Date"))
				returnValue = "DateUtil.getStringFromDate(get" + capitalizedName + "());";
			else
				returnValue = "\"\" + get" + capitalizedName + "();";
			out.println("\t\t\treturn " + returnValue);
			out.println("\t\t}");
		}
		out.println("\t\tthrow new RuntimeException(\"DecisionObject.getAttribute(\" + name + \") failed - unknown attribute\");");
		out.println("\t}");
		
		// Generate setAttribute(String name, String value) 
		out.println("\n\tpublic void setAttribute(String name, String value) {");
		it = fields.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry pair = (Map.Entry) it.next();
			String name = (String) pair.getKey();
			String capitalizedName = name.substring(0, 1).toUpperCase() + name.substring(1);
			DynamicObjectField field = (DynamicObjectField) openClass.getField(name);
			String type = field.getType().getName();
			
			out.println("\t\tif (\"" + name + "\".equals(name)) {");
			
			String arrayType = BusinessObjectArray.getType(pair.getValue());
			if (arrayType != null) {
				out.println("\t\t\tString[] values = value.split(\",\");"); 
				if (arrayType.equals("String[]"))
					out.println("\t\t\tset" + capitalizedName + "(values); return;");
				else if (arrayType.equals("int[]")) {
					out.println("\t\t\tint[] numbers = new int[values.length];"); 
					out.println("\t\t\tfor (int i = 0; i < numbers.length; i++)"); 
					out.println("\t\t\t\tnumbers[i] = Integer.parseInt(values[i]);"); 
					out.println("\t\t\tset" + capitalizedName + "(numbers); return;");
				}
				else if (arrayType.equals("double[]")) {
					out.println("\t\t\tdouble[] numbers = new double[values.length];"); 
					out.println("\t\t\tfor (int i = 0; i < numbers.length; i++)"); 
					out.println("\t\t\t\tnumbers[i] = Double.parseDouble(values[i]);"); 
					out.println("\t\t\tset" + capitalizedName + "(numbers); return;");
				}
				else if (arrayType.equals("boolean[]")) {
					out.println("\t\t\tboolean[] numbers = new boolean[values.length];"); 
					out.println("\t\t\tfor (int i = 0; i < numbers.length; i++)"); 
					out.println("\t\t\t\tnumbers[i] = Boolean.parseBoolean(values[i]);"); 
					out.println("\t\t\tset" + capitalizedName + "(numbers); return;");
				}
				else if (arrayType.equals("Date[]")) {
					out.println("\t\t\tboolean[] dates = new Date[values.length];"); 
					out.println("\t\t\tfor (int i = 0; i < dates.length; i++)"); 
					out.println("\t\t\t\tdates[i] = DateUtil.getDateFromString(values[i]);"); 
					out.println("\t\t\tset" + capitalizedName + "(dates); return;");
				}
				out.println("\t\t}");
				continue;
			}
						
			String setValue = null;
			if (type.equals("int"))
				setValue = "Integer.parseInt(value)";
			else if (type.equals("long"))
				setValue = "Long.parseLong(value)";
			else if (type.equals("double"))
				setValue = "Double.parseDouble(value)";
			else if (type.equals("float"))
				setValue = "Float.parseFloat(value)";
			else if (type.equals("boolean"))
				setValue = "Boolean.parseBoolean(value)";
			else if (type.equals("java.util.Date"))
				setValue = "DateUtil.getDateFromString(value)";
			else
				setValue = "value";
			out.println("\t\t\tset" + capitalizedName + "("+ setValue +"); return;");
			out.println("\t\t}");
		}
		out.println("\t\tthrow new RuntimeException(\"DecisionObject.setAttribute(\"	+ name + \",\" + value + \") failed - unknown attribute\");");
		out.println("\t}");
		
		// Generate toString() 
		out.println("\n\tpublic String toString() {");
		out.println("\n\t\treturn \"" + className + ":\"");
		it = fields.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry pair = (Map.Entry) it.next();
			String name = (String) pair.getKey();
			String capitalizedName = name.substring(0, 1).toUpperCase() + name.substring(1);
			DynamicObjectField field = (DynamicObjectField) openClass.getField(name);
			String type = field.getType().getName();
			out.println("\t\t\t + \" " + name + "=\" + " + name);
		}
		out.println("\t\t;");
		out.println("\t}");
		
		
		// Generate createDecisionObject() 
		out.println("\n\tstatic public DecisionObject createDecisionObject() {");
		out.println("\t\treturn new " + className + "();");
		out.println("\t}");
		
		// Generate HashMap of attribute names and types
		out.println("\n\tprivate static final HashMap<String, String> attributes;");
		out.println("\tstatic {");
		out.println("\t\tattributes = new HashMap<String, String>();");
		it = fields.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry pair = (Map.Entry) it.next();
			String name = (String) pair.getKey();
			String capitalizedName = name.substring(0, 1).toUpperCase() + name.substring(1);
			DynamicObjectField field = (DynamicObjectField) openClass.getField(name);
			String type = field.getType().getName();
			String arrayType = BusinessObjectArray.getType(pair.getValue());
			if (arrayType != null)
				type = arrayType;
			out.println("\t\tattributes.put(\"" + name + "\",\"" + type + "\");");
		}
		out.println("\t}");
		out.println("\tpublic HashMap getAttributes() {");
		out.println("\t\treturn attributes;");
		out.println("\t}");
		
		// Generate getClassName() 
		out.println("\n\tstatic public String getClassName() {");
		out.println("\t\treturn \"" + className + "\";");
		out.println("\t}");
		
		// Generate createBusinessMap()
		out.println("\n\tstatic public BusinessMap createBusinessMap() {");
		out.println("\t\tString className = " + className + ".getClassName();");
		out.println("\t\tBusinessMap map = new BusinessMapImpl(className, new " + className + "());" );
		out.println("\t\treturn map;");
		out.println("\t}");
		
		// Epilog
		out.println("}");
		out.close();
		
		Log.info("JavaBean " + filename + " with " + n +" attributes has been generated.");
	}
	
}
