package com.openrules.ruleengine;

import java.util.Iterator;
import java.util.Set;

/*
 * Copyright (C) 2018 OpenRules, Inc.
 * 
 */

/**
 * A placeholder for OpenRulesEngine. Can create different decisions based on the same engine
 *
 */

public class DecisionModel {
	
	OpenRulesEngine engine;
	String xlsMainFileName;
	Decision decision;
	
	/**
	 * Creates an instance of OpenRulesEngine
	 * @param mainDecisionName
	 * @param xlsMainFileName
	 */
	public DecisionModel(String mainDecisionName, String xlsMainFileName) {
		 engine = new OpenRulesEngine(xlsMainFileName);
		 this.xlsMainFileName = xlsMainFileName;
		 this.decision = null;
	}
	
	/**
     * Creates an instance of OpenRulesEngine
     * @param xlsMainFileName
     */
    public DecisionModel(String xlsMainFileName) {
         engine = new OpenRulesEngine(xlsMainFileName);
         this.xlsMainFileName = xlsMainFileName;
         this.decision = new Decision(engine);
    }
	
	public OpenRulesEngine getEngine() {
		return engine;
	}
	
	public Glossary getGlossary() {
	    return decision.getGlossary();
	}
	
	/**
	 * Creates and returns an instance of Decision based on the existing engine
	 * @param a decision's name - can be a goal or a name a table of the type Decision
	 * @return an instance of Decision
	 */
	public Decision createDecision(String name) {
		return new Decision(name,engine);
	}
	
	/**
	 * Creates and returns an instance of Goal defined in this decision model
	 * @param a goal's name  
	 * @return an instance of Goal that is a subclass of Decision
	 */
	public Goal createGoal(String goalName) {
		return new Goal(goalName,this);
	}
	
	public void log(String text) {
		getEngine().log(text);
	}
	
	/**
     * Returns an array of objects defined in Excel Data table "name"
     * @param name a string with the name of the Excel Data table (e.g. "employees") 
     * @return an array of objects that should be cast to already known Java type such as Employee[] 
     */
    public Object[] getData(String dataArrayName) {
        return decision.getData(dataArrayName);
    }
    
    /**
     * Returns an object defined in Excel Variable table "name"
     * @param name a string with the name of the Excel Variable table (e.g. "employee") 
     * @return an object that should be cast to already known Java type such as Employee 
     */
    public Object getVariable(String variableName) {
        return decision.getVariable(variableName);
    }
	
    /**
     * Generates java interfaces for business objects that will be passed to the decision model
     * @param outputFolder - the name of the source folder for the generated Java packages and classes, e.g. "src" 
     * @param packageName - the name of the Java package that will be created in the "outputFolder" and in which 
     * the generated Java classes will be placed
     */
    public void generateJavaInterfaces(String goalName, String outputFolder, String packageName) {
        Goal goal = createGoal(goalName);
        goal.put("FEEL", "On");
        goal.test(1);
        JavaInterfaceGenerator generator = new JavaInterfaceGenerator(goal);
        generator.generateJavaInterfaces(outputFolder,packageName);
    }

}
