package com.openrules.ruleengine;

import java.util.ArrayList;
import java.util.HashMap;

/*
 * Copyright (C) 2009-2014 OpenRules, Inc.
 */

/**
 * Represents one decision test including related business objects and expected
 * results
 */

public class DecisionTest {

    String id;
    HashMap<String, Object> decisionObjects;
    HashMap<String, String> expectations;
    ArrayList<DecisionMismatch> mismatches;

    public DecisionTest(String id) {
        super();
        this.id = id;
        decisionObjects = new HashMap<String, Object>();
        expectations = new HashMap<String, String>();
        mismatches = new ArrayList<DecisionMismatch>();
    }

    public String getId() {
        return id;
    }

    public void addDecisionObject(String businessConcept, Object object) {
        decisionObjects.put(businessConcept, object);
    }

    public HashMap<String, Object> getDecisionObjects() {
        return decisionObjects;
    }

    public void setDecisionObjects(HashMap<String, Object> decisionObjects) {
        this.decisionObjects = decisionObjects;
    }

    public void addExpectation(String variableName, String value) {
        expectations.put(variableName, value);
    }

    public HashMap<String, String> getExpectations() {
        return expectations;
    }

    public void setExpectations(HashMap<String, String> expectations) {
        this.expectations = expectations;
    }

    public DecisionMismatch addMismatch(String variable, Object actualValue, Object expectedValue) {
        DecisionMismatch mismatch = new DecisionMismatch(this, variable, actualValue, expectedValue);
        mismatches.add(mismatch);
        return mismatch;
    }

    public ArrayList<DecisionMismatch> getMismatches() {
        return mismatches;
    }

    public void setId(String id) {
        this.id = id;
    }

    /**
     * A simple Java launcher to test DMN Decision Models
     */
    public static void main(String[] args) {
        String testFile = "Unknown testFile";
        String goalName = "Unknown decisionName";
        String feel = "On";
        if (args.length > 1) {
            testFile = args[0];
            goalName = args[1];
            if (args.length > 2) {
                feel = args[2];
            }
        } else {
            System.out.println("Usage: com.openrules.ruleengine.DecisionTest testFile goalName [feel]");
            System.exit(-1);
        }
        long startTime = System.currentTimeMillis();
        
        System.out.println("TEST_FILE=" + testFile);
        System.out.println("GOAL_NAME=" + goalName);
        System.out.println("FEEL=" + feel);

        
        DecisionModel model = new DecisionModel(testFile);
        
        Goal goal = model.createGoal(goalName);
        goal.put("FEEL", feel);
        goal.put("Report", "On");

        goal.test("testCases");
        
        long endTime = System.currentTimeMillis();
        model.log("Elapsed time: " + (endTime - startTime +1) + " ms");
    }

}
