package com.openrules.ruleengine;

/**
 * @author JF
 */

import java.util.ArrayList;
import java.util.List;

import com.openrules.table.external.IRuleTable;
import com.openrules.table.external.Objects;
import com.openrules.table.external.RuleProvider;
import com.openrules.table.external.RuleProviderForObjects;
import com.openrules.table.external.RuleTable;


public class ExternalRules {
	
	private boolean modified;
	private final List ruleTables; // expected type is IRuleTable
	
	public ExternalRules() {
		modified = false;
		ruleTables = new ArrayList();
	}

	public ExternalRules( List ruleTables ) {
		modified = false;
		this.ruleTables = ruleTables; 
	}
	
	public final boolean isModified() {
		return modified;
	}
	public final void setModified(boolean modified) {
		this.modified = modified;
	}

	public final List getRuleTables() {
		return ruleTables;
	}

	public final IRuleTable addRuleTable(IRuleTable table) {
		ruleTables.add(table);
		return table;
	}

	public final IRuleTable addRuleTable(String name, String templateName, Object[][] rules) {
		return addRuleTable(name,templateName,null,rules);
	}

	public final IRuleTable addRuleTable(String name, String templateName, String[] labels, Object[][] rules) {
		RuleTable ruleTable = new RuleTable(name,templateName, new RuleProvider(labels, rules) );
		return addRuleTable(ruleTable);
	}
	
	// added by JF to support decision templates (Nov-2012) 
	public final IRuleTable addRuleTable(String name, String templateName, String[] labels, 
			                        String[] descriptions, String[][] rules) {
		RuleTable ruleTable = new RuleTable(name,templateName,  
				                  new RuleProvider(labels, descriptions, rules) );
		return addRuleTable(ruleTable);
	}
		
	public final IRuleTable addRuleTable(String name, String templateName, Objects[] rules) {
		return addRuleTable(name,templateName,null,rules);
	}

	public final IRuleTable addRuleTable(String name, String templateName, String[] labels, Objects[] rules) {
		RuleTable ruleTable = new RuleTable(name,templateName, new RuleProviderForObjects(labels, rules) );
		return addRuleTable(ruleTable);
	}
	

}
