package com.openrules.ruleengine;

import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;

import org.apache.commons.beanutils.PropertyUtils;
import org.openl.types.impl.DynamicObject;
import org.openl.util.Log;
import org.openl.util.StringTool;

import com.openrules.types.DateUtil;
import com.openrules.types.Oper;

public class GlossaryArrayHandler {
	
	Glossary glossary;
	
	public GlossaryArrayHandler(Glossary glossary) {
		super();
		this.glossary = glossary;
	}
	
	public boolean compareArrays(String variableName, Oper op, String[] compareValues) {
		boolean allowUndefinedValue = false;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		Object object = ge.getBusinessObject();
		Object array = ge.getProperty();
//		try {
//			if (object instanceof DynamicObject) {
//				array = ((DynamicObject) object).getFieldValue(variableName);
//			} else {
//				array = PropertyUtils.getProperty(object, variableName);
//			}
//		} catch (Exception e) 
		if (array == null) {
			String err = "ERROR in Glossary: cannot find an array " + variableName + " within object " + object.toString();
			Log.error(err);
			//e.printStackTrace();
			throw new RuntimeException(err);
		}
		if (array instanceof int[]) {
			int[] array1 = (int[]) array;
			int[] array2 = new int[compareValues.length];
			for (int i = 0; i < compareValues.length; i++) {
				try {
					array2[i] = Integer.parseInt(compareValues[i]);
				} catch (Exception e) {
					// not integer, try it as a name of the array
					array2 = getIntArray(compareValues[i]);
					break;
				}
			}
			return op.compare(array1,array2);
		} else if (array instanceof long[]) {
			long[] array1 = (long[]) array;
			long[] array2 = new long[compareValues.length];
			for (int i = 0; i < compareValues.length; i++) {
				try {
					array2[i] = Long.parseLong(compareValues[i]);
				} catch (Exception e) {
					// try it as a name of the array
					array2 = getLongArray(compareValues[i]);
					break;
				}
			}
			return op.compare(array1,array2);
		} else if (array instanceof double[]) {
			double[] array1 = (double[]) array;
			double[] array2 = new double[compareValues.length];
			for (int i = 0; i < compareValues.length; i++) {
				try {
					array2[i] = Double.parseDouble(compareValues[i]);
				} catch (Exception e) {
					// try it as a name of the array
					array2 = getDoubleArray(compareValues[i]);
					break;
				}
			}
			return op.compare(array1,array2);
		} else if (array instanceof BigDecimal[]) {
			BigDecimal[] array1 = (BigDecimal[]) array;
			BigDecimal[] array2 = new BigDecimal[compareValues.length];
			for (int i = 0; i < compareValues.length; i++) {
				try {
					array2[i] = new BigDecimal(compareValues[i]);
				} catch (Exception e) {
					// try it as a name of the array
					array2 = getBigDecimalArray(compareValues[i]);
					break;
				}
			}
			return op.compare(array1,array2);
		} else if (array instanceof Comparable[]) { // covers Date[], String[], Object[]
			Comparable[] array1 = (Comparable[]) array;
			Comparable[] array2 = (Comparable[]) compareValues;
			return op.compare(array1,array2);
		} else {
			String err = "ERROR: Uncomparable arrays " + variableName + 
					" and " + Arrays.toString(compareValues);
			Log.error(err);
			throw new RuntimeException(err);
		}
	}
	
	public final Comparable[] getArray(Object object, String name) {
		try {
			Comparable[] array = null;
			if (object instanceof DynamicObject) {
				array = (Comparable[]) ((DynamicObject) object).getFieldValue(name);
			} else {
				array = (Comparable[]) PropertyUtils.getProperty(object, name);
			}
			return array;
		} catch (Exception e) {
			String err = "ERROR in Glossary: cannot find an array " + name + " within object " + object.toString();
			Log.error(err);
			e.printStackTrace();
			throw new RuntimeException(err);
		}
	}
	
	public String[] getStringArray(String variableName) {
		boolean allowUndefinedValue = false;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		Object object = ge.getBusinessObject();
		try {
			String[] array = null;
			if (object instanceof DynamicObject) {
				array = (String[]) ((DynamicObject) object).getFieldValue(ge.attribute);
			} else {
				array = (String[]) PropertyUtils.getProperty(object, ge.attribute);
			}
			return array;
		} catch (Exception e) {
			String err = "ERROR in Glossary: cannot find a String array " + variableName + " within object "
					+ object.toString();
			// Log.error(err);
			// e.printStackTrace();
			throw new RuntimeException(err);
		}
	}

	public double[] getDoubleArray(String variableName) {
		boolean allowUndefinedValue = false;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		Object object = ge.getBusinessObject();
		try {
			double[] array = null;
			if (object instanceof DynamicObject) {
				array = (double[]) ((DynamicObject) object).getFieldValue(ge.attribute);
			} else {
				array = (double[]) PropertyUtils.getProperty(object, ge.attribute);
			}
			return array;
		} catch (Exception e) {
			String err = "ERROR in Glossary: cannot find a double array " + variableName + " within object "
					+ object.toString();
			// Log.error(err);
			// e.printStackTrace();
			throw new RuntimeException(err);
		}
	}

	public int[] getIntArray(String variableName) {
		boolean allowUndefinedValue = false;
		int[] array = null;
		try {
			GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
			Object object = ge.getBusinessObject();
			try {
				if (object instanceof DynamicObject) {
					array = (int[]) ((DynamicObject) object).getFieldValue(ge.attribute);
				} else {
					array = (int[]) PropertyUtils.getProperty(object, ge.attribute);
				}
				return array;
			} catch (Exception e) {
				String err = "ERROR in Glossary: cannot find an int array " + variableName + " within object "
						+ object.toString();
				// Log.error(err);
				// e.printStackTrace();
				throw new RuntimeException(err);
			}
			
		} catch (Exception e) {
			// consider a list: int1, int2, ...
			String[] strings = variableName.split(",");
			array = new int[strings.length];
			for (int i = 0; i < strings.length; i++) {
				array[i] = Integer.parseInt(strings[i].trim());
			}
			return array;
		}
	}

	public long[] getLongArray(String variableName) {
		boolean allowUndefinedValue = false;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		Object object = ge.getBusinessObject();
		try {
			long[] array = null;
			if (object instanceof DynamicObject) {
				array = (long[]) ((DynamicObject) object).getFieldValue(ge.attribute);
			} else {
				array = (long[]) PropertyUtils.getProperty(object, ge.attribute);
			}
			return array;
		} catch (Exception e) {
			String err = "ERROR in Glossary: cannot find a long array " + variableName + " within object "
					+ object.toString();
			// Log.error(err);
			// e.printStackTrace();
			throw new RuntimeException(err);
		}
	}

	public BigDecimal[] getBigDecimalArray(String variableName) {
		boolean allowUndefinedValue = false;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		Object object = ge.getBusinessObject();
		try {
			BigDecimal[] array = null;
			if (object instanceof DynamicObject) {
				array = (BigDecimal[]) ((DynamicObject) object).getFieldValue(ge.attribute);
			} else {
				array = (BigDecimal[]) PropertyUtils.getProperty(object, ge.attribute);
			}
			return array;
		} catch (Exception e) {
			String err = "ERROR in Glossary: cannot find an BigDecimal array " + variableName + " within object "
					+ object.toString();
			// Log.error(err);
			// e.printStackTrace();
			throw new RuntimeException(err);
		}
	}


//	public final void addIntegers(List<Integer> list, String[] values) {
//		String name = values[0];
//		if (name.startsWith("$")) {
//			if (name.startsWith("$"))
//				name = name.substring(1);
//			Object value = glossary.getObjectByName(name);
//			if (value instanceof int[]) {
//				for(int v : (int[])value)
//					list.add(v);
//			}
//			else { // single object
//				list.add(new Integer((String)value));
//			}
//		}
//	}
	
	public final Object arrayIntToAdd(int[] oldArray, String[] values, String oper) {
		if (values == null || values.length == 0)
			return oldArray;
		int[] addArray = null;
		String name = values[0];
		if (name.startsWith("$")) {
			if (name.startsWith("$"))
				name = name.substring(1);
			Object value = glossary.getObjectByName(name);
			if (value instanceof int[]) {
				addArray = (int[])value;
			}
			else { // single object
				addArray = new int[] { Integer.parseInt((String)value) };
			}
		}
		else {				
			addArray = new int[values.length];
			for (int i = 0; i < values.length; i++) {
				addArray[i] = Integer.parseInt(values[i]);
			}
		}
		if (oldArray == null || Oper.ARE.equals(oper))
			oldArray = new int[0];
		int[] newArray = new int[oldArray.length + addArray.length];
		int n = 0;
		for (int i = 0; i < oldArray.length; i++) {
			newArray[n++] = oldArray[i];
		}
		for (int i = 0; i < addArray.length; i++) {
			newArray[n++] = addArray[i];
		}
		return (Object)newArray;
	}
	
	public final Object arrayLongToAdd(long[] oldArray, String[] values, String oper) {
		if (values == null || values.length == 0)
			return oldArray;
		long[] addArray = null;
		String name = values[0];
		if (name.startsWith("$")) {
			if (name.startsWith("$"))
				name = name.substring(1);
			Object value = glossary.getObjectByName(name);
			if (value instanceof long[]) {
				addArray = (long[])value;
			}
			else { // single object
				addArray = new long[] { Long.parseLong((String)value) };
			}
		}
		else {				
			addArray = new long[values.length];
			for (int i = 0; i < values.length; i++) {
				addArray[i] = Long.parseLong(values[i]);
			}
		}
		if (oldArray == null || Oper.ARE.equals(oper))
			oldArray = new long[0];
		long[] newArray = new long[oldArray.length + addArray.length];
		int n = 0;
		for (int i = 0; i < oldArray.length; i++) {
			newArray[n++] = oldArray[i];
		}
		for (int i = 0; i < addArray.length; i++) {
			newArray[n++] = addArray[i];
		}
		return (Object)newArray;
	}
	
	public final Object arrayDoubleToAdd(double[] oldArray, String[] values, String oper) {
		if (values == null || values.length == 0)
			return oldArray;
		double[] addArray = null;
		String name = values[0];
		if (name.startsWith("$")) {
			if (name.startsWith("$"))
				name = name.substring(1);
			Object value = glossary.getObjectByName(name);
			if (value instanceof double[]) {
				addArray = (double[])value;
			}
			else { // single object
				addArray = new double[] { Double.parseDouble((String)value) };
			}
		}
		else {				
			addArray = new double[values.length];
			for (int i = 0; i < values.length; i++) {
				addArray[i] = Double.parseDouble(values[i]);
			}
		}
		if (oldArray == null || Oper.ARE.equals(oper))
			oldArray = new double[0];
		double[] newArray = new double[oldArray.length + addArray.length];
		int n = 0;
		for (int i = 0; i < oldArray.length; i++) {
			newArray[n++] = oldArray[i];
		}
		for (int i = 0; i < addArray.length; i++) {
			newArray[n++] = addArray[i];
		}
		return (Object)newArray;
	}
	
	public final Object arrayStringToAdd(String[] oldArray, String[] values, String oper) {
		if (values == null || values.length == 0)
			return oldArray;
		String[] addArray = null;
		String name = values[0];
		if (name.startsWith("$")) {
			if (name.startsWith("$"))
				name = name.substring(1);
			Object value = glossary.getObjectByName(name);
			if (value instanceof String[]) {
				addArray = (String[])value;
			}
			else { // single object
				addArray = new String[] { (String)value };
			}
		}
		else {				
			addArray = (String[]) values;
		}
		
		if (oldArray == null || Oper.ARE.equals(oper))
			oldArray = new String[0];
		String[] newArray = new String[oldArray.length + addArray.length];
		int n = 0;
		for (int i = 0; i < oldArray.length; i++) {
			newArray[n++] = oldArray[i];
		}
		for (int i = 0; i < addArray.length; i++) {
			newArray[n++] = addArray[i];
		}
		return (Object)newArray;
	}
	
	public final Object arrayDateToAdd(Date[] oldArray, String[] values, String oper) {
		if (values == null || values.length == 0)
			return oldArray;
		Date[] addArray = null;
		if (values[0].startsWith("$")) {
			Object value = glossary.getObjectByName(values[0].substring(1));
			if (value instanceof Date[]) {
				addArray = (Date[])value;
			}
			else { // single object
				addArray = new Date[] { (Date) DateUtil.parse((String)value) };
			}
		}
		else {				
			addArray = new Date[values.length];
			for (int i = 0; i < values.length; i++) {
				addArray[i] = (Date) DateUtil.parse(values[i]);
			}
		}
		
		if (oldArray == null || Oper.ARE.equals(oper))
			oldArray = new Date[0];
		Date[] newArray = new Date[oldArray.length + addArray.length];
		int n = 0;
		for (int i = 0; i < oldArray.length; i++) {
			newArray[n++] = oldArray[i];
		}
		for (int i = 0; i < addArray.length; i++) {
			newArray[n++] = addArray[i];
		}
		return (Object)newArray;
	}
	
	public final Object arrayBigDecimalToAdd(BigDecimal[] oldArray, String[] values, String oper) {
		if (values == null || values.length == 0)
			return oldArray;
		BigDecimal[] addArray = null;
		if (values[0].startsWith("$")) {
			Object value = glossary.getObjectByName(values[0].substring(1));
			if (value instanceof Date[]) {
				addArray = (BigDecimal[])value;
			}
			else { // single object
				addArray = new BigDecimal[] { new BigDecimal((String)value) };
			}
		}
		else {				
			addArray = new BigDecimal[values.length];
			for (int i = 0; i < values.length; i++) {
				addArray[i] = new BigDecimal(values[i]);
			}
		}
		
		if (oldArray == null || Oper.ARE.equals(oper))
			oldArray = new BigDecimal[0];
		BigDecimal[] newArray = new BigDecimal[oldArray.length + addArray.length];
		int n = 0;
		for (int i = 0; i < oldArray.length; i++) {
			newArray[n++] = oldArray[i];
		}
		for (int i = 0; i < addArray.length; i++) {
			newArray[n++] = addArray[i];
		}
		return (Object)newArray;
	}
	
	public final Object arrayObjectToAdd(Object[] oldArray,	String name, String oper) {
		if (name == null)
			return oldArray;
		//Object[] addArray = new Object[0];
		Object[] addArray;
		if (name.startsWith("$"))
			name = name.substring(1);
		Object value = glossary.getObjectByName(name);
		if (value == null)
			return null;
		if (value instanceof Object[]) {
			addArray = (Object[])value;
		}
		else { // single object
			//addArray = new Object[] { value };
			addArray = (Object[]) Array.newInstance(oldArray.getClass().getComponentType(), 1);
			addArray[0] = value;
		}
		if (oldArray == null || Oper.ARE.equals(oper)) {
			//oldArray = new Object[0];
			oldArray = null;
		}
		int oldLength = ( oldArray == null ? 0 : oldArray.length );
		//Object[] newArray = new Object[oldArray.length + addArray.length];
		int newLength = oldArray.length + addArray.length;
		Object[] newArray = (Object[]) Array.newInstance(oldArray.getClass().getComponentType(), newLength );
		int n = 0;
		for (int i = 0; i < oldArray.length; i++) {
			newArray[n++] = oldArray[i];
		}
		for (int i = 0; i < addArray.length; i++) {
			newArray[n++] = addArray[i];
		}
		return (Object)newArray;
	}
	
	public final Object addObject(List list, String name, String oper) {
		if (name.startsWith("$"))
			name = name.substring(1);
		Object value = glossary.getObjectByName(name);
		if (Oper.ARE.equals(oper)) 
			list.clear();
		list.add(value);
		return list;
	}
		
	public final void addArray(String variableName, GlossaryElement ge,	String[] values, String oper) {
		//String type = ge.getType();
		Object object = ge.getBusinessObject();
		Object property = ge.getProperty();
		if (property == null) { // it means this is a list defined in Excel and not initialized 
			property = new ArrayList();
			ge.setProperty(property);
		}
		Object array = null;
		try {
			if (property instanceof String[]) {
				array = arrayStringToAdd((String[]) ge.getProperty(),values, oper);
			} else if (property instanceof int[]) {
				array = arrayIntToAdd((int[]) ge.getProperty(), values, oper);
			} else if (property instanceof long[]) {
				array = arrayLongToAdd((long[]) ge.getProperty(), values, oper);
			} else if (property instanceof double[]) {
				array = arrayDoubleToAdd((double[]) ge.getProperty(),values, oper);
			} else if (property instanceof BigDecimal[]) {
				array = arrayBigDecimalToAdd((BigDecimal[]) ge.getProperty(),values, oper);
			} else if (property instanceof Date[]) {
				array = arrayDateToAdd((Date[]) ge.getProperty(),values, oper);
			} else if (property instanceof Object[]) {
				array = arrayObjectToAdd((Object[]) ge.getProperty(),values[0], oper);
//			} else if (property instanceof List<Integer>) {
//				addIntegers((List<Integer>) ge.getProperty(),values);
//				return;
			} else if (property instanceof List) {
				addObject((List) ge.getProperty(),values[0], oper);
				return;
			} else {
				String err = "ERROR trying to Add/Assign values/object to array " + variableName + 
						": make sure the array is not null and values have the expected type";
				Log.error(err);
				throw new RuntimeException(err);
			}
			String attribute = ge.getAttribute();
			if (ge.isDynamic())
				((DynamicObject) object).setFieldValue(attribute, array);
			else {
				PropertyUtils.setProperty(object, attribute, array);
			}
		} catch (Exception e) {
			String err = "ERROR in Glossary trying to assign an array to "
					+ ge.getAttribute();
			Log.error(err);
			throw new RuntimeException(err);
		}
	}


}
