package com.openrules.ruleengine;

import java.math.BigDecimal;
import java.util.Date;

import org.apache.commons.beanutils.PropertyUtils;
import org.openl.types.impl.DynamicObject;
import org.openl.util.Log;
import org.openl.util.StringTool;

import com.openrules.feel.FeelEvaluator;
import com.openrules.feel.OpenRulesFeelEvaluator;
import com.openrules.types.CompareToDate;
import com.openrules.types.CompareToDouble;
import com.openrules.types.CompareToString;
import com.openrules.types.DataMap;
import com.openrules.types.DateUtil;
import com.openrules.types.FromToBigDecimal;
import com.openrules.types.FromToDouble;
import com.openrules.types.FromToInt;
import com.openrules.types.FromToLong;
import com.openrules.types.Oper;

public class GlossaryCompareAssign {
	
	Glossary glossary;
	FeelEvaluator feelEvaluator;

	public GlossaryCompareAssign(Glossary glossary) {
		this.glossary = glossary;
		feelEvaluator = null;
	}
	
	/**
	 * This method is used by conditions of the type "ConditionVarOperValues"
	 * 
	 * @param variableName
	 * @param op
	 * @param value
	 * @return
	 */
	public final boolean compare(String variableName, Oper op, String compareValue) {
//		if (IRELEVANT.equals(compareValue))
//			return true;
		if (op == null) {
			String err = "ERROR: Missing Oper for " + variableName;
			Log.error(err);
			throw new RuntimeException(err);
		}
		if (compareValue == null) {
			String err = "ERROR: Missing Value after operator " + op.getId()
					+ " for " + variableName;
			Log.error(err);
			throw new RuntimeException(err);
		}
		boolean allowUndefinedValue = false;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		Object object = ge.getBusinessObject();
		if (op.isCompareTwoArrays()) {
			String delim = ",";
			String[] compareValues = StringTool.tokenize(compareValue, delim); // compareValue.split(delim);
			for (int i = 0; i < compareValues.length; i++) {
				String element = compareValues[i].trim();
				compareValues[i] = glossary.resolveValue(element);
			}
			return glossary.arrayHandler.compareArrays(variableName, op, compareValues);
//			Comparable[] array1 = glossary.arrayHandler.getArray(object, ge.attribute);
//			String delim = ",";
//			String[] strings = StringTool.tokenize(compareValue, delim); // compareValue.split(delim);
//			for (int i = 0; i < strings.length; i++) {
//				String element = strings[i].trim();
//				strings[i] = glossary.resolveValue(element);
//			}
//			Comparable[] array2 = (Comparable[]) strings;
//			return op.compare(array1, array2);
		}

		String type = ge.type;
		if (DataMap.TYPE_UNDEFINED.equals(type)) { // when value is unknown
			if (!op.isEmpty()) {
				Log.info("Warning: property <" + ge.attribute
						+ "> has undefined type. Condition failed.");
				return false;
			} 
			else
				type = DataMap.TYPE_STRING;
		}
		Object property = ge.property;
		String value = glossary.resolveValue(compareValue);
		try {
			if (type.equals(DataMap.TYPE_BASE_DOUBLE)
					|| type.equals(DataMap.TYPE_DOUBLE)) {
				try {
					double propertyValue = ((Double) property).doubleValue();
					double doubleValue;
					try {
						doubleValue = Double.parseDouble(value);
						return op.compare(propertyValue, doubleValue);
					} catch (Exception e) {
						// try to evaluate as a FEEL expression
						if (glossary.getDecision().isFEELOn() && startsWithCompareOperator(op.getId())) {
							try {
								String expression = variableName + " " + op.getId() + " " + value;
								return getFeelEvaluator().evaluateAsBoolean(expression);
							} catch (Exception e2) {
								// keep going
							}
						}
						throw new RuntimeException(e);
					}
					
				} catch (Exception e) { // try to compare with Oper such as WITHIN
					return op.compare((Double) property, value);
				}
			} else if (type.equals(DataMap.TYPE_BIG_DECIMAL)) {
				BigDecimal bd1 = (BigDecimal) property;
				BigDecimal bd2 = new BigDecimal(value);
				return op.compare(bd1,bd2);
			} else if (type.equals(DataMap.TYPE_STRING)) {
				if (glossary.getDecision().isFEELOn()) {
					String evaluatedValue = evaluateString(value);
					if (evaluatedValue != null)
						value = evaluatedValue;
//					String stringVariableName = (String) value;
//					try {
//						value = glossary.getString(stringVariableName);
//					} catch (Exception e) {
//						//keep going
//					}
				}
				String propertyValue = (String) property;
				return op.compare(propertyValue, value);
			} else if (type.equals(DataMap.TYPE_BASE_INT)
					|| type.equals(DataMap.TYPE_INT)) {
				try {
					int propertyValue = ((Integer) property).intValue();
					int intValue;
					try {
						intValue = Integer.parseInt(value);
						return op.compare(propertyValue, intValue);
					} catch (Exception e) {
						// try to evaluate as a FEEL expression
						if (glossary.getDecision().isFEELOn() && startsWithCompareOperator(op.getId())) {
							try {
								String expression = variableName + " " + op.getId() + " " + value;
								return getFeelEvaluator().evaluateAsBoolean(expression);
							} catch (Exception e2) {
								// keep going
							}
						}
						throw new RuntimeException(e);
					}
					
				} catch (Exception e) { // try to compare with Oper such as WITHIN
					return op.compare((Integer) property, value);
				}
			} else if (type.equals(DataMap.TYPE_BASE_LONG)
					|| type.equals(DataMap.TYPE_LONG)) {
				try {
					long propertyValue = ((Long) property).longValue();
					long longValue;
					try {
						longValue = Long.parseLong(value);
						return op.compare(propertyValue, longValue);
					} catch (Exception e) {
						// try to evaluate as a FEEL expression
						if (glossary.getDecision().isFEELOn() && startsWithCompareOperator(op.getId())) {
							try {
								String expression = variableName + " " + op.getId() + " " + value;
								return getFeelEvaluator().evaluateAsBoolean(expression);
							} catch (Exception e2) {
								// keep going
							}
						}
						throw new RuntimeException(e);
					}
					
				} catch (Exception e) { // try to compare with Oper such as WITHIN
					return op.compare((Long) property, value);
				}
			} else if (type.equals(DataMap.TYPE_BASE_BOOL)
					|| type.equals(DataMap.TYPE_BOOL)) {
				boolean propertyValue = ((Boolean) property).booleanValue();
				boolean boolValue = false;
				if (value.equalsIgnoreCase("TRUE")
						|| value.equalsIgnoreCase("YES"))
					boolValue = true;
				//boolean boolValue = Boolean.parseBoolean(value); 
				return op.compare(propertyValue, boolValue);
			} else if (type.equals(DataMap.TYPE_DATE)) {
				Date propertyValue = (Date) property;
				Date dateValue = DateUtil.getDateFromString(value);
				// Log.debug("Date Compare " + propertyValue + " " +
				// op.toString() + " " + dateValue);
				return op.compare(propertyValue, dateValue);
			} else {
				String err = "ERROR in Glossary: unsupported type <" + type
						+ "> for property <" + variableName + ">";
				Log.error(err);
				throw new RuntimeException(err);
			}
		} catch (Exception e) {
			String err = "ERROR in Glossary trying to compare a value " +property + " of <"
					+ variableName + "> with <" + value + "> using oper "
					+ op.toString() + " for type="+type 
					+ "\n" + e.toString();
			//Log.error(err);
			throw new RuntimeException(err);
		}
	}
	
	/**
	 * This method is used by conditions of the type "If" without operators
	 * 
	 * @param variableName
	 * @param value
	 * @return
	 */
	public final boolean compare(String variableName, String value) {
		boolean allowUndefinedValue = false;
		if (Glossary.IRRELEVANT.equals(value)) // hyphen is irrelevant character
			return true;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		Object property = ge.property;
		String type = ge.type;
		if (type.equals(DataMap.TYPE_UNDEFINED)) { // when value is unknown
			Log.info("Warning: property <" + ge.attribute
					+ "> has undefined type. Condition failed.");
			return false;
		}
		Object object = ge.getBusinessObject();
		try {
			if (type.equals(DataMap.TYPE_BASE_DOUBLE)
					|| type.equals(DataMap.TYPE_DOUBLE)) {
				double propertyValue = ((Double) property).doubleValue();
				try {
					double doubleValue = Double.parseDouble(value); // may fail
					//return Double.compare(propertyValue,doubleValue) == 0;
					return Math.abs(propertyValue - doubleValue) < CompareToDouble.precision;
					
				} catch (Exception e) {
					// Try as a FEEL expression
					if (glossary.getDecision().isFEELOn()) {
						try {
							String operator = "";
							if (!startsWithCompareOperator(value))
								operator = "=";
							String expression = variableName + operator + value;
							return getFeelEvaluator().evaluateAsBoolean(expression,false);
						} catch (Exception e3) {
							// FEEL failed - keep going
						}
					}
					try {
						// try FromToDouble interval
						FromToDouble interval = new FromToDouble(value);
						return interval.contains(propertyValue);
					} catch (Exception e2) {
						throw e;
					}
				}
			} else if (type.equals(DataMap.TYPE_STRING)) {
				if (glossary.getDecision().isFEELOn()) {
					String evaluatedValue = evaluateString(value);
					if (evaluatedValue != null)
						value = evaluatedValue;
//					String stringVariableName = (String) value;
//					try {
//						value = glossary.getString(stringVariableName);
//					} catch (Exception e) {
//						//keep going
//					}
				}
				String propertyValue = (String) property;
				CompareToString expression = new CompareToString(value);
				return expression.compare(propertyValue);
			} else if (type.equals(DataMap.TYPE_BASE_INT)
					|| type.equals(DataMap.TYPE_INT)) {
				int propertyValue = ((Integer) property).intValue();
				try {
					int intValue = Integer.parseInt(value); // may fail
					return propertyValue == intValue;
				} catch (Exception e) {
					// try FEEL
					if (glossary.getDecision().isFEELOn()) {
						try {
							String operator = "";
							if (!startsWithCompareOperator(value))
								operator = "=";
							String expression = variableName + operator + value;
							return getFeelEvaluator().evaluateAsBoolean(expression,false);
						} catch (Exception e3) {
							// keep going
						}
					}
					try {
						FromToInt interval = new FromToInt(value);
						return interval.contains(propertyValue);
					} catch (Exception e2) {
						throw e;						
					}
				}
			} else if (type.equals(DataMap.TYPE_BASE_LONG)
					|| type.equals(DataMap.TYPE_LONG)) {
				long propertyValue = ((Long) property).longValue();
				try {
					long longValue = Long.parseLong(value); // may fail
					return propertyValue == longValue;
				} catch (Exception e) {
					// try FEEL
					if (glossary.getDecision().isFEELOn()) {
						try {
							String operator = "";
							if (!startsWithCompareOperator(value))
								operator = "=";
							String expression = variableName + operator + value;
							return getFeelEvaluator().evaluateAsBoolean(expression,false);
						} catch (Exception e3) {
							// keep going
						}
					}
					try {
						FromToInt interval = new FromToInt(value);
						int intPropertyValue = (int) propertyValue; // this is a shortcut - we don't expect long intervals
						return interval.contains(intPropertyValue);
					} catch (Exception e2) {
						throw e;						
					}
				}
			} else if (type.equals(DataMap.TYPE_BIG_DECIMAL)) {
				BigDecimal propertyValue = (BigDecimal) property;
				FromToBigDecimal interval = new FromToBigDecimal(value);
				return interval.contains(propertyValue);
			} else if (type.equals(DataMap.TYPE_BASE_BOOL)
					|| type.equals(DataMap.TYPE_BOOL)) {
				boolean propertyValue = ((Boolean) property).booleanValue();
				boolean compareValue = false;
				if (value.equalsIgnoreCase("TRUE")
						|| value.equalsIgnoreCase("YES"))
					compareValue = true;
				return propertyValue == compareValue;
			} else if (type.equals(DataMap.TYPE_DATE)) {
				Date propertyValue = (Date) property;
				CompareToDate expression = new CompareToDate(value);
				return expression.compare(propertyValue);
			} else {
				String err = "ERROR in Glossary: unsupported type <" + type
						+ "> for object <" + object.toString() + " - see "
						+ ge.attribute;
				Log.error(err);
				throw new RuntimeException(err);
			}
		} catch (Exception e) {
			String err = "ERROR in Glossary trying to compare a value of <"
					+ variableName + "> with " + value;
			Log.error(err);
			throw new RuntimeException(err);
		}
	}
	
	public boolean compareIntValueWithExpression(int value, String expression) {
		try {
			int expValue = Integer.parseInt(expression); // may fail
			return value == expValue;
		} catch (Exception e) {
			// try FEEL
			if (glossary.getDecision().isFEELOn()) {
				try {
					String operator = "";
					if (!startsWithCompareOperator(expression))
						operator = "=";
					String feelExpression = value + operator + expression;
					return getFeelEvaluator().evaluateAsBoolean(feelExpression,false);
				} catch (Exception e3) {
					// keep going
				}
			}
			try {
				FromToInt interval = new FromToInt(expression);
				return interval.contains(value);
			} catch (Exception e2) {
				throw new RuntimeException(e2);						
			}
		}
	}
	
	public boolean compareLongValueWithExpression(long value, String expression) {
		try {
			long expValue = Long.parseLong(expression); // may fail
			return value == expValue;
		} catch (Exception e) {
			// try FEEL
			if (glossary.getDecision().isFEELOn()) {
				try {
					String operator = "";
					if (!startsWithCompareOperator(expression))
						operator = "=";
					String feelExpression = value + operator + expression;
					return getFeelEvaluator().evaluateAsBoolean(feelExpression,false);
				} catch (Exception e3) {
					// keep going
				}
			}
			try {
				FromToLong interval = new FromToLong(expression);
				return interval.contains(value);
			} catch (Exception e2) {
				throw new RuntimeException(e2);						
			}
		}
	}
	
	public boolean compareBigDecimalValueWithExpression(BigDecimal value, String expression) {
		try {
			BigDecimal expValue = new BigDecimal(expression); // may fail
			return value.compareTo(expValue) == 0;
		} catch (Exception e) {
			// try FEEL
			if (glossary.getDecision().isFEELOn()) {
				try {
					String operator = "";
					if (!startsWithCompareOperator(expression))
						operator = "=";
					String feelExpression = value + operator + expression;
					return getFeelEvaluator().evaluateAsBoolean(feelExpression,false);
				} catch (Exception e3) {
					// keep going
				}
			}
			try {
				FromToBigDecimal interval = new FromToBigDecimal(expression);
				return interval.contains(value);
			} catch (Exception e2) {
				throw new RuntimeException(e2);						
			}
		}
	}
	
	public boolean compareDoubleValueWithExpression(double value, String expression) {
		try {
			double expValue = Double.parseDouble(expression); // may fail
			return Math.abs(value - expValue) < CompareToDouble.precision;
		} catch (Exception e) {
			// try FEEL
			if (glossary.getDecision().isFEELOn()) {
				try {
					String operator = "";
					if (!startsWithCompareOperator(expression))
						operator = "=";
					String feelExpression = value + operator + expression;
					return getFeelEvaluator().evaluateAsBoolean(feelExpression,false);
				} catch (Exception e3) {
					// keep going
				}
			}
			try {
				FromToDouble interval = new FromToDouble(expression);
				return interval.contains(value);
			} catch (Exception e2) {
				throw new RuntimeException(e2);						
			}
		}
	}

	
	public final void assign(String variableName, Oper op, String[] assignValues) {
		
		if (assignValues == null)
			return;
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < assignValues.length; i++) {
			if (i > 0)
				buf.append(", ");
			buf.append(assignValues[i].trim());
		}
		assign(variableName, op, buf.toString());
	}

	public final void assign(String variableName, Oper op, String assignValue) {
		if (Glossary.IRRELEVANT.equals(assignValue))
			return; // do nothing
		if (op.isArrayOperator()) {
			if (glossary.getDecision().assignArrayFeature(variableName, op.getId(), assignValue) == true)
				return;
		}
		boolean allowUndefinedValue = true;
		GlossaryElement ge = glossary.getProperty(variableName, allowUndefinedValue);
		String type = ge.getType();
		Object object = ge.getBusinessObject();
		try {
//			if (op.getId().equals(Oper.ARE)) { // toString().equalsIgnoreCase("are"))
//												// {
//				String delim = op.getDelimiter();
//				String[] strings = StringTool.tokenize(assignValue, delim);// assignValue.split(delim);
//				for (int i = 0; i < strings.length; i++) {
//					String element = strings[i].trim();
//					strings[i] = glossary.resolveValue(element);
//				}
//				Comparable[] array = (Comparable[]) strings;
//				// PropertyUtils.setProperty(object, ge.attribute, array);
//				if (ge.isDynamic())
//					((DynamicObject) object).setFieldValue(ge.getAttribute(), array);
//				else
//					PropertyUtils.setProperty(object, ge.getAttribute(), array);
//				return;
//			}
			//String value = resolveValue(assignValue);
//			if (op.getId().equals(Oper.ADD)) {
//				String delim = op.getDelimiter();
//				glossary.addArray(variableName, ge, assignValue, delim);
//				return;
//			}
//			String value = resolveValue(assignValue);
			
			String value = glossary.resolveValue(assignValue);
			if (op.getId().equals(Oper.ADD) || op.getId().equals(Oper.ARE)) {
				String delim = op.getDelimiter();
				// String[] strings = value.split(delim);
				String[] strings = StringTool.tokenize(value, delim);
				for (int i = 0; i < strings.length; i++) {
					strings[i] = strings[i].trim();
				}
				glossary.arrayHandler.addArray(variableName, ge, strings,op.getId());
				return;
			}
			
			
			// Calculate object "newValue"
			Object newValue;
			if (type.equals(DataMap.TYPE_BASE_DOUBLE)
					|| type.equals(DataMap.TYPE_DOUBLE)) {
				//newValue = new Double(value);
				double propertyValue = ((Double) ge.getProperty()).doubleValue();
				double doubleValue;
				try {
					doubleValue = Double.parseDouble(value);
				} catch (Exception e) {
					// try to evaluate as a FEEL expression
					if (glossary.getDecision().isFEELOn()) {
						try {
							doubleValue = getFeelEvaluator().evaluateAsDouble(value);
						} catch (Exception e2) {
							throw new RuntimeException(e);
						}
					}
					else
						throw new RuntimeException(e);
				}
				newValue = new Double(doubleValue);
				if (op.isCompoundAssignment()) {
					if (op.isAssignPlus())
						propertyValue += doubleValue;
					else if (op.isAssignMinus())
						propertyValue -= doubleValue;
					else if (op.isAssignMultiply())
						propertyValue *= doubleValue;
					else if (op.isAssignDivide())
						propertyValue /= doubleValue;
					newValue = new Double(propertyValue);
				}
				if (op.isMax()) {
					newValue = (propertyValue > doubleValue) ? propertyValue
							: doubleValue;
				}
				if (op.isMin()) {
					newValue = (propertyValue < doubleValue) ? propertyValue
							: doubleValue;
				}
			} else if (type.equals(DataMap.TYPE_STRING)) {
				newValue = (String) value;
				if (glossary.getDecision().isFEELOn()) {
					if ("\"\"".equals(value)) // allow to use "" to assign an empty string
						newValue = "";
					else {
						String evaluatedValue = evaluateString(value);
						if (evaluatedValue != null)
							newValue = evaluatedValue;
//						try {
//							String stringVariableName = value;
//							newValue = glossary.getString(stringVariableName);
//						} catch (Exception e) {
//							//keep going
//						}
					}
				}
			} else if (type.equals(DataMap.TYPE_BASE_INT)
					|| type.equals(DataMap.TYPE_INT)) {
				//newValue = new Integer(value);
				int propertyValue = ((Integer) ge.getProperty()).intValue();
				int intValue;
				try {
					intValue = Integer.parseInt(value);
				} catch (Exception e) {
					// try to evaluate as a FEEL expression
					if (glossary.getDecision().isFEELOn()) {
						try {
							intValue = getFeelEvaluator().evaluateAsInt(value);
						} catch (Exception e2) {
							throw new RuntimeException(e);
						}
					}
					else
						throw new RuntimeException(e);
				}
				newValue = new Integer(intValue);
				if (op.isCompoundAssignment()) {
					if (op.isAssignPlus())
						propertyValue += intValue;
					else if (op.isAssignMinus())
						propertyValue -= intValue;
					else if (op.isAssignMultiply())
						propertyValue *= intValue;
					else if (op.isAssignDivide())
						propertyValue /= intValue;
					newValue = new Integer(propertyValue);
				}
				if (op.isMax()) {
					newValue = (propertyValue > intValue) ? propertyValue
							: intValue;
				}
				if (op.isMin()) {
					newValue = (propertyValue < intValue) ? propertyValue
							: intValue;
				}
			} else if (type.equals(DataMap.TYPE_BASE_LONG)
					|| type.equals(DataMap.TYPE_LONG)) {
				//newValue = new Integer(value);
				long propertyValue = ((Long) ge.getProperty()).longValue();
				long longValue;
				try {
					longValue = Long.parseLong(value);
				} catch (Exception e) {
					// try to evaluate as a FEEL expression
					if (glossary.getDecision().isFEELOn()) {
						try {
							longValue = getFeelEvaluator().evaluateAsLong(value);
						} catch (Exception e2) {
							throw new RuntimeException(e);
						}
					}
					else
						throw new RuntimeException(e);
				}
				newValue = new Long(longValue);
				if (op.isCompoundAssignment()) {
					if (op.isAssignPlus())
						propertyValue += longValue;
					else if (op.isAssignMinus())
						propertyValue -= longValue;
					else if (op.isAssignMultiply())
						propertyValue *= longValue;
					else if (op.isAssignDivide())
						propertyValue /= longValue;
					newValue = new Long(propertyValue);
				}
				if (op.isMax()) {
					newValue = (propertyValue > longValue) ? propertyValue
							: longValue;
				}
				if (op.isMin()) {
					newValue = (propertyValue < longValue) ? propertyValue
							: longValue;
				}
			} else if (type.equals(DataMap.TYPE_BIG_DECIMAL)) {
				newValue = new BigDecimal(value);
				BigDecimal propertyValue = (BigDecimal) ge.getProperty();
				BigDecimal bdValue = new BigDecimal(value);
				if (op.isCompoundAssignment()) {
					if (op.isAssignPlus())
						propertyValue = propertyValue.add(bdValue); 
					else if (op.isAssignMinus())
						propertyValue = propertyValue.subtract(bdValue);
					else if (op.isAssignMultiply())
						propertyValue = propertyValue.multiply(bdValue); 
					else if (op.isAssignDivide())
						propertyValue = propertyValue.divide(bdValue); 
					newValue = propertyValue;
				}
				if (op.isMax()) {
					newValue = (propertyValue.compareTo(bdValue) > 0) ? propertyValue
							: bdValue;
				}
				if (op.isMin()) {
					newValue = (propertyValue.compareTo(bdValue) < 0) ? propertyValue
							: bdValue;
				}
			} else if (type.equals(DataMap.TYPE_BASE_BOOL)
					|| type.equals(DataMap.TYPE_BOOL)) {
				newValue = new Boolean(value);
			} else if (type.equals(DataMap.TYPE_DATE)) {
				newValue = DateUtil.parse(value);
				// try {
				// newValue = simpleDateFormat.parse(value);
				// } catch (Exception e) {
				// //newValue = dateFormat.parse(value);
				// newValue = getDateFromString(value);
				// }
			} else {
				String err = "ERROR in Glossary: unsupported type <" + type
						+ "> for property <" + variableName;
				Log.error(err);
				throw new RuntimeException(err);
			}

			// Make actual assignment
			if (ge.isDynamic() && object instanceof DynamicObject) 
				((DynamicObject) object).setFieldValue(ge.getAttribute(), newValue);
			else {
				PropertyUtils.setSimpleProperty(object, ge.getAttribute(), newValue);
			}
			ge.property = newValue; // JF added on Nov 14, 2015
			
			//glossary.getDecision().updateReportedRule(); // current rule's text is not completed yet

		} catch (Exception e) {
			String err = "ERROR in Glossary trying to assign a value "
					+ assignValue + " to " + variableName;
			Log.error(err);
			throw new RuntimeException(err);
		}
	}
		
	public FeelEvaluator getFeelEvaluator() {
		if (feelEvaluator == null) 
			feelEvaluator = new OpenRulesFeelEvaluator(glossary);
		return feelEvaluator;
	}

	private final boolean startsWithCompareOperator(String expression) {
		String expr = expression.trim();
		Character s = expr.charAt(0);
		if (s == '=' || s=='>' || s=='<')
			return true;
		return false;
	}
	
//	public String resolveValue(String text) {
//		return glossary.resolveValue(text);
//	}
	
	public String getVariableValue(String variableName) {
		String value = null;
		String name = variableName;
		if (name.startsWith("'") && name.endsWith("'") )  {
			name = name.substring(1, name.length()-1);
			name = name.replaceAll("''","'");
		}
		Object object = glossary.getVarValue(name);
		if (object == null)
			return null;
		return object.toString();
	}
	
	/**
	 * Evaluates if "string" is a String decision variable, and if yes returns its value.
	 * If not, evaluates if "string" is a FEEL concatenation, and if yes returns 
	 * a concatenated value.
	 * Otherwise, returns null
	 * @param string
	 * @return string or null
	 */
	public String evaluateString(String string) {
		if (string == null || string.isEmpty())
			return null;
		String stringVariableName = (String) string;
		String value = null;
		try {
			value = glossary.getString(stringVariableName);
			return value;
		} catch (Exception e) {
			//keep going
		}
		// check for concatenation
		if (string.indexOf("\"") < 0) 
			return null;
		if (string.indexOf("+") < 0) 
			return null;
		String feelExpression = string;
		String[] elements = feelExpression.split("\\+");
		if (elements.length == 1) {
			return null;
		}
		
		StringBuffer buffer = new StringBuffer();
		for (int i = 0; i < elements.length; i++) {
			String element = elements[i];
			element = element.trim();
			if (element.startsWith("\"") && element.endsWith("\"") )  {
				buffer.append(element.substring(1, element.length()-1));
				continue;
			}
			value = getVariableValue(element);
			if (value == null)
				return null;
			buffer.append(value);
		}
		return buffer.toString();
	}

}
