package com.openrules.ruleengine;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/*
 * Copyright (C) 2005-2009 OpenRules, Inc.
 * 
 * Feb-2009: IK replaced parameters String src to IOpenSourceCodeModule src 
 * June-2009: JF/IK made RuntimeEnv a local variable for every run to support unsynchronized threads 
 * IRuntimeEnv env = vm.getRuntimeEnv();
 * Object instance = openClass.newInstance(env);
 * Now setMethod(..) is always called for a new run.
 * 
 */

import org.openl.IOpenSourceCodeModule;
import org.openl.IOpenVM;
import org.openl.OpenConfigurationException;
import org.openl.OpenL;
import org.openl.conf.IUserContext;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenField;
import org.openl.types.IOpenMethod;
import org.openl.util.Log;
import org.openl.vm.IRuntimeEnv;

import com.openrules.dt.DecisionTable;
import com.openrules.dt.IDecisionRow;
import com.openrules.dt.ProxyDecisionTable;
import com.openrules.dt.ProxyFunctionalRow;

/**
 * @author Jacob
 *
 */
public class RuleEngine {

	private final OpenL openl;
	private final IOpenClass openClass;
	private final IOpenVM vm;

	class RuleMethodImpl implements RuleMethod {
		private final IRuntimeEnv env;
		private final Object instance;
		private final IOpenMethod method;
		private final String name;

		public RuleMethodImpl(String name) {
			this.name = name;
			this.env = vm.getRuntimeEnv();
			this.instance = openClass.newInstance(env);
			this.method = openClass.getMatchingMethod(name, null);
			if (this.method == null)
				throw new RuntimeException("Method " + name + " not found");
		}

		public synchronized Object run(Object[] objects) {
			return method.invoke(instance, objects, env);
		}

		public String getName() {
			return name;
		}

		public IRuntimeEnv getEnv() {
			return env;
		}

	}

	// public RuleEngine(String openlName, IOpenSourceCodeModule src,
	public RuleEngine(IOpenSourceCodeModule src, IUserContext ucxt) {
		if (ucxt != null)
			// openl = OpenL.getInstance(openlName, ucxt);
			openl = OpenL.getInstance(ucxt);
		else
			openl = OpenL.getInstance();

		vm = openl.getVm();
		openClass = openl.compileModule(src);
	}

	public RuleMethod getMethod(String name) {
		RuleMethod method = new RuleMethodImpl(name);
		return method;
	}

	OpenL getOpenL() {
		return openl;
	}

	public Object automaticRun(Decision decision) {
		return null;
	}

//	public void print(String string) {
//		System.out.print(string);
//	}
//	
//	public void println(String string) {
//		System.out.println(string);
//	}
//	
//	public void println() {
//		println("");
//	}
	
	public Iterator getMethodIterator() {
//		IRuntimeEnv env = vm.getRuntimeEnv();
//		Object instance = openClass.newInstance(env);
		return openClass.methods();
	}

	public Object getField(String fieldName) {
		
		final IOpenField field = openClass.getField( fieldName );
		if ( field == null ) {
			throw new RuntimeException("Field " + fieldName + " not found");
		}

		final IRuntimeEnv env = vm.getRuntimeEnv();
		final Object instance = openClass.newInstance(env);

		return field.get(instance, env);
	
	}
}