/*
 * Created on Oct 29, 2003
 *
 *  Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.table;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.openl.syntax.impl.FileSourceCodeModule;

import com.openrules.table.xls.XlsSheetGridModel;

/**
 * @author snshor
 *
 */
public class TablesTest extends TestCase
{

	/**
	 * Constructor for TablesTest.
	 * @param name
	 */
	public TablesTest(String name)
	{
		super(name);
	}

	public void testSplitter() throws Exception
	{
		String testXls = "tst/com/openrules/lang/xls/Test2.xls";

		FileSourceCodeModule source = new FileSourceCodeModule(testXls, null);

		POIFSFileSystem fs = new POIFSFileSystem(source.getByteStream());

		HSSFWorkbook wb = new HSSFWorkbook(fs);

		int nsheets = wb.getNumberOfSheets();

		for (int i = 0; i < nsheets; i++)
		{
			HSSFSheet sheet = wb.getSheetAt(i);
			String name = wb.getSheetName(i);
			XlsSheetGridModel xsGrid =
				new XlsSheetGridModel(sheet, name, source);

			IGridTable[] tables = new GridSplitter(xsGrid).split();

			if (name.equals("ABC"))
			{
				for (int j = 0; j < tables.length; j++)
				{
					System.out.println(
						XlsSheetGridModel.xlsCellPresentation(
							tables[j].getGridColumn(0, 0),
							tables[j].getGridRow(0, 0)));
				}

				Assert.assertEquals(15, xsGrid.getNumberOfMergedRegions());

				Assert.assertEquals(9, tables.length);

				LogicalTable lt = new LogicalTable(tables[5]);
				
				subtestRegion(lt.rows(1));
				

				Assert.assertEquals(6, lt.getLogicalHeight());
				Assert.assertEquals(1, lt.getLogicalWidth());

				ILogicalTable row1 = lt.getLogicalRow(0);

				Assert.assertEquals(1, row1.getLogicalHeight());
				Assert.assertEquals(1, row1.getLogicalWidth());

				Assert.assertEquals(2, row1.getGridTable().getLogicalHeight());
				Assert.assertEquals(4, row1.getGridTable().getLogicalWidth());


				ILogicalTable row2 = lt.getLogicalRow(1);

				Assert.assertEquals(1, row2.getLogicalHeight());
				Assert.assertEquals(2, row2.getLogicalWidth());

				Assert.assertEquals(2, row2.getGridTable().getLogicalHeight());
				Assert.assertEquals(4, row2.getGridTable().getLogicalWidth());


				ILogicalTable col22 = row2.columns(1, 1);

				Assert.assertEquals(2, col22.getLogicalHeight());
				Assert.assertEquals(1, col22.getLogicalWidth());

				Assert.assertEquals(2, col22.getGridTable().getLogicalHeight());
				Assert.assertEquals(3, col22.getGridTable().getLogicalWidth());




				ILogicalTable row222 = col22.rows(1, 1);

				Assert.assertEquals(1, row222.getLogicalHeight());
				Assert.assertEquals(3, row222.getLogicalWidth());

				
				LogicalTable invRow2 = new LogicalTable(new TransposedGridTable(row2.getGridTable()));
				
				Assert.assertEquals(2, invRow2.getLogicalHeight());
				Assert.assertEquals(1, invRow2.getLogicalWidth());

				ILogicalTable invCol22 = invRow2.getLogicalRow(1);

				Assert.assertEquals(1, invCol22.getLogicalHeight());
				Assert.assertEquals(2, invCol22.getLogicalWidth());
				
				ILogicalTable invRow222 = invCol22.columns(1, 1);

				Assert.assertEquals(3, invRow222.getLogicalHeight());
				Assert.assertEquals(1, invRow222.getLogicalWidth());

				

			}

		}

	}

	void subtestRegion(ILogicalTable testHeader1) throws Exception
	{
		ILogicalTable bb = testHeader1.getLogicalRegion(1, 0, 1, 1);
		
		Assert.assertEquals(2, bb.getLogicalHeight());

		Assert.assertEquals(5, testHeader1.getLogicalRegion(1, 3, 1, 2).getLogicalHeight());
		Assert.assertEquals(5, testHeader1.transpose().getLogicalRegion(3, 1, 2, 1).getLogicalWidth());

	}


}
