/*
 * Created on Oct 23, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.data.impl;

import java.lang.reflect.Array;
import java.sql.Timestamp;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.openl.binding.impl.BoundError;
import org.openl.syntax.SyntaxErrorException;
import org.openl.util.Log;

import com.openrules.data.DuplicatedTableException;
import com.openrules.data.IColumnDescriptor;
import com.openrules.data.IDataBase;
import com.openrules.data.IDataTableModel;
import com.openrules.data.ITable;
import com.openrules.table.ILogicalTable;

/**
 * @author snshor
 *
 */
public class DataBase implements IDataBase
{

	boolean validationOccured = false;

	HashMap tables = new HashMap();

	public DataBase()
	{
	}

	public synchronized ITable addTable(
		IDataTableModel dataModel,
		ILogicalTable data)
		throws Exception
	{
		if (validationOccured)
			throw new RuntimeException("Trying to add table after validation");
		Table t = (Table) tables.get(dataModel.getName());
		if (t != null)
			throw new DuplicatedTableException(t.dataModel, dataModel);
		t = new Table(dataModel, data);	
		tables.put(dataModel.getName(), t);
		t.preLoad();
		return t;
	}

	/**
	 *
	 */

	public synchronized void validate() throws Exception
	{
		if (validationOccured)
			return;

		validateInternal();
	}

	protected void validateInternal() throws Exception
	{
		Log.debug("Validating and Populating Data tables..." + Log.timestamp());
		Vector errors = new Vector();
		Collection<String> values = tables.values();
		
		for (Iterator iter = values.iterator(); iter.hasNext();)
		{
			Table t = (Table) iter.next();
			Log.debug("Populating data for table " + t.getDataModel().getName() + " - " + Log.timestamp());
				try
				{
					t.populate(this);
				}
				catch (BoundError e)
				{
					errors.add(e);
				}
		}
		Log.debug("Data tables validated " + Log.timestamp());
		validationOccured = true;
		if (errors.size() > 0)
		{
			BoundError[] ee = (BoundError[])errors.toArray(new BoundError[0]);
			throw new SyntaxErrorException(null, ee);
		}
		
	}
	


		

	/**
	 *
	 */

	public synchronized ITable getTable(String name)
	{
//		validate();
		Table t = (Table) tables.get(name);
//		if (t == null)
//			throw new Exception("Data table " + name + " is not found");
		return t;
	}

	static class Table implements ITable
	{
		ILogicalTable data;
		IDataTableModel dataModel;

		Object ary;
		
		
		/**
		 * 
		 */
		public Table(IDataTableModel dataModel, ILogicalTable data)
		{
			this.dataModel = dataModel;
			this.data = data;
		}
		
		
		void preLoad() throws Exception
		{
			int rows = data.getLogicalHeight();
			int startRow = 1;
			
			ary = Array.newInstance(dataModel.getInstanceClass(), rows - startRow);
			
			if (dataModel.getInstanceClass().isPrimitive())
			  return;
			
			
			for (int i = startRow; i < rows; i++)
			{
				Object target = dataModel.newInstance();
				Array.set(ary, i- startRow, target);	

				int columns = data.getLogicalWidth();
				
				for (int j = 0; j < columns; j++)
				{
					IColumnDescriptor cd = dataModel.getDescriptor()[j];
					if (cd != null)
					  cd.populateLiteral(target, data.getLogicalRegion(j, i , 1, 1));					
				}
						
			} 
			
		}
		
		
		void populate(IDataBase db) throws Exception
		{
			int rows = data.getLogicalHeight();
			int columns = data.getLogicalWidth();

			int startRow = 1;
			
			for (int i = startRow; i < rows; i++)
			{
				Object target = Array.get(ary, i - startRow);
				
				for (int j = 0; j < columns; j++)
				{
					IColumnDescriptor cd = dataModel.getDescriptor()[j];
					if (cd != null)
					  cd.populateLink(target, data.getLogicalRegion(j, i, 1, 1), db);					
				}
			}
			
			
		}
		

		/**
		 *
		 */

		public Object getData(int row)
		{
			return Array.get(ary, row);
		}

		/**
		 *
		 */

		public Object getDataArray()
		{
			return ary;
		}

		/**
		 *
		 */

		public Object getFirst(Object primaryKey)
		{
			// TODO Auto-generated method stub
			return null;
		}

		/**
		 *
		 */

		public int getSize()
		{
			return Array.getLength(ary);
		}

		/**
		 *
		 */

		public IDataTableModel getDataModel()
		{
			return dataModel;
		}

		/**
		 *
		 */

		public Object findObject(int columnIndex, String skey)
		{
			int len = getSize();
			IColumnDescriptor descr = dataModel.getDescriptor()[columnIndex];
			Object key = descr.getConvertor().parse(skey, null);
			
			for (int i = 0; i < len; i++)
			{
				Object target = Array.get(ary, i);
				if (descr == null)
				  return null;
				Object test = descr.getColumnValue(target);
				if (key.equals(test))
				  return target;
			}
			
			return null;
		}

		/**
		 *
		 */

		public int getColumnIndex(String columnName)
		{
			IColumnDescriptor[] dd = dataModel.getDescriptor();
			for (int i = 0; i < dd.length; i++)
			{
				if (dd[i] == null)
				  continue;
				if (dd[i].getName().equals(columnName))
				  return i;
			}
			return -1;
		}

	}

}
