/*
 * Created on Oct 23, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.data.impl;

import java.lang.reflect.Array;
import java.util.Vector;

import org.openl.OpenL;
import org.openl.binding.impl.BoundError;
import org.openl.syntax.impl.IdentifierNode;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenField;
import org.openl.vm.IRuntimeEnv;

import com.openrules.data.IColumnDescriptor;
import com.openrules.data.IDataBase;
import com.openrules.data.IString2DataConvertor;
import com.openrules.data.ITable;
import com.openrules.dt.FunctionalRow;
import com.openrules.table.ALogicalTable;
import com.openrules.table.ILogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

/**
 * @author snshor
 *
 */
public class OpenlBasedColumnDescriptor implements IColumnDescriptor {

	IOpenField field;
	IdentifierNode indexTable;
	IdentifierNode indexKey;
	IString2DataConvertor convertor;

	OpenL openl;

	public OpenlBasedColumnDescriptor(
		IOpenField field,
		IdentifierNode indexTable,
		IdentifierNode indexKey,
		IString2DataConvertor convertor,
		OpenL openl) {
		this.field = field;
		this.indexTable = indexTable;
		this.indexKey = indexKey;
		this.convertor = convertor;
		this.openl = openl;
	}

	/**
	 *
	 */

	public String getName() {
		return field.getName();
	}

	/**
	 *
	 */

	public void populateLink(Object target, ILogicalTable values, IDataBase db)
		throws Exception {

		if (indexTable == null)
			return;

		String tablename = indexTable.getIdentifier();
		ITable t = db.getTable(tablename);
		if (t == null) {
			BoundError err =
				new BoundError(
					indexTable,
					"Table " + tablename + " not found",
					null);
			throw err;
		}

		int foreignKeyIndex = 0;
		String columnName = "<not_initialized>";
		if (indexKey != null) {
			columnName = indexKey.getIdentifier();

			foreignKeyIndex = t.getColumnIndex(columnName);
		}

		if (foreignKeyIndex == -1) {
			BoundError err =
				new BoundError(
					indexKey,
					"Column " + columnName + " not found",
					null);
			throw err;
		}

		//		Class c = field.getType().getInstanceClass();
		//
		//		Class cc = c.isArray() ? c.getComponentType() : c;

		IOpenClass fieldType = field.getType();

		boolean isArray = fieldType.getAggregateInfo().isAggregate(fieldType);

		if (!isArray) {
			String s = values.getGridTable().getStringValue(0, 0);
			if (s != null) {

				s = s.trim();
				if (s.length() > 0) {
					Object res = t.findObject(foreignKeyIndex, s);
					if (res == null)
						throw new BoundError(
							null,
							"Index Key " + s + " not found",
							null,
							new GridCellSourceCodeModule(
								values.getGridTable()));
					field.set(target, res, getRuntimeEnv());
				}
			}
		} else {
			Vector v = new Vector();
			for (int i = 0; i < values.getLogicalHeight(); i++) {
				String s = values.getGridTable().getStringValue(0, i);
				
				if (s != null)
				{
					s = s.trim();
				}
				else break;
				  
				if (s.length() == 0)
					break;

				Object res = t.findObject(foreignKeyIndex, s);
				if (res == null)
					throw new BoundError(
						null,
						"Index Key " + s + " not found",
						null,
						new GridCellSourceCodeModule(
							values.getLogicalRow(i).getGridTable()));
				v.add(res);
			}

			//			Object ary = Array.newInstance(cc, v.size());
			IOpenClass componentType =
				fieldType.getAggregateInfo().getComponentType(fieldType);
			Object ary =
				fieldType.getAggregateInfo().makeIndexedAggregate(
					componentType,
					new int[] { v.size()});

			for (int i = 0; i < v.size(); i++) {
				Array.set(ary, i, v.get(i));
			}

			field.set(target, ary, getRuntimeEnv());

		}

	}

	public void populateLiteral(Object target, ILogicalTable values)
		throws Exception {
		if (indexTable != null) {
			return;
		}

		//		Class c = field.getType().getInstanceClass();
		//
		//		Class cc = c.isArray() ? c.getComponentType() : c;

		IOpenClass paramType = field.getType();
		boolean indexed = paramType.getAggregateInfo().isAggregate(paramType);

		if (indexed)
			paramType =
				paramType.getAggregateInfo().getComponentType(paramType);

		values = ALogicalTable.make1ColumnTable(values);

		if (!indexed) {
			Object res = FunctionalRow.loadSingleParam(paramType, values, null);
			if (res != null)
				field.set(target, res, getRuntimeEnv());
		} else {
			Vector v = new Vector();
			int h = values.getLogicalHeight();
			for (int i = 0; i < h; i++) {
				Object res =
					FunctionalRow.loadSingleParam(
						paramType,
						values.getLogicalRow(i),
						null);
				if (res == null)
					break;

				v.add(res);
			}

			Object ary =
				paramType.getAggregateInfo().makeIndexedAggregate(
					paramType,
					new int[] { v.size()});

			//Array.newInstance(cc, v.size());

			for (int i = 0; i < v.size(); i++) {
				Array.set(ary, i, v.get(i));
			}

			field.set(target, ary, getRuntimeEnv());

		}
	}

	public void populateLiteral2(Object target, ILogicalTable values)
		throws Exception {
		if (indexTable != null) {
			return;
		}

		Class c = field.getType().getInstanceClass();

		Class cc = c.isArray() ? c.getComponentType() : c;

		values = ALogicalTable.make1ColumnTable(values);

		if (!c.isArray()) {
			String s = values.getGridTable().getStringValue(0, 0);
			if (s != null && s.trim().length() > 0) {
				Object res = convertor.parse(s, null);
				field.set(target, res, getRuntimeEnv());
			}
		} else {
			Vector v = new Vector();
			for (int i = 0; i < values.getLogicalHeight(); i++) {
				String s = values.getGridTable().getStringValue(0, i);
				if (s == null || s.trim().length() == 0)
					break;

				Object res = convertor.parse(s, null);
				v.add(res);
			}

			Object ary = Array.newInstance(cc, v.size());

			for (int i = 0; i < v.size(); i++) {
				Array.set(ary, i, v.get(i));
			}

			field.set(target, ary, getRuntimeEnv());

		}
	}

	IRuntimeEnv getRuntimeEnv() {
		return openl.getVm().getRuntimeEnv();
	}

	/**
	 *
	 */

	public Object getColumnValue(Object target) {
		return field.get(target, getRuntimeEnv());
	}

	/**
	 * @return
	 */
	public IString2DataConvertor getConvertor() {
		return convertor;
	}

}
