/*
 * Created on Sep 10, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */
package com.openrules.dt;
import java.util.ArrayList;
import java.util.List;

import org.openl.OpenL;
import org.openl.binding.IBindingContextDelegator;
import org.openl.types.IMemberMetaInfo;
import org.openl.types.IMethodSignature;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethod;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.java.JavaOpenClass;
import org.openl.vm.IRuntimeEnv;

import com.openrules.lang.xls.IXlsTableNames;
import com.openrules.table.ILogicalTable;
import com.openrules.types.IDecision;
/**
 * 
 * This class is converted to abstract with two abstract methods:
 * invoke() and defineActionReturnType(int)
 * @see DecisionTableMultiHit
 * @see DecisionTableSingleHit
 * 
 * @author snshor 
 * modified by JF to define a return type
 * modified by IK to add rule templates (invocationStrategy)
 * Apr-2011: JF added option calculateConditionsFirst
 */
public class DecisionTable implements IDecisionTable, IDecisionTableConstants {

	static final int UNDEFINED = 0; 
	static final int FALSE = 1; 
	static final int TRUE = 2;
	static final int NA = 3;
	static final int SPECIAL_FALSE = 4;
	static final int SPECIAL_TRUE = 5;
	static final int COLUMN_MODE = 0;
	static final int ROW_MODE = 1;
	
	IOpenMethodHeader header;
	ILogicalTable table;
	IDecisionRow[] functionRows;
	IDecisionRow[] conditionRows;
	IDecisionRow[] actionRows;
	
	IInvocationStrategy invocationStrategy;
	boolean calculateConditionsFirst;
	String direction;

	int columns;
	
	IDecision 	decision; // added by JF to support decision-specific reports

	public DecisionTable(IOpenMethodHeader header) {
		this.header = header;
		calculateConditionsFirst = true;
		direction = IXlsTableNames.DIRECTION_UNKNOWN;
		setDecision(null);
	}

	public IInvocationStrategy getInvocationStrategy() {
		return invocationStrategy;
	}
	public void setInvocationStrategy(IInvocationStrategy invocationStrategy) {
		this.invocationStrategy = invocationStrategy;
	}
	
	public final boolean isCalculateConditionsFirst() {
		return calculateConditionsFirst;
	}

	public final void setCalculateConditionsFirst(boolean calculateConditionsFirst) {
		this.calculateConditionsFirst = calculateConditionsFirst;
	}

	public String getDirection() {
		return direction;
	}

	public void setDirection(String direction) {
		this.direction = direction;
	}

	public void bindTable(ILogicalTable table, IDecisionRow[] functionRows,
//			conditionRows,
//			IDecisionRow[] actionRows, 
			//		IDecisionTableStructure structure,
			OpenL openl, IOpenClass module, IBindingContextDelegator cxtd,
			int columns) throws Exception {
		this.table = table;
		this.functionRows = functionRows;
		
		List tmpCond = new ArrayList();
		List tmpAct = new ArrayList();
		for( int i = 0; i < functionRows.length; ++i) {
			IDecisionRowMetaInfo meta = functionRows[i].getMetaInfo(); 
			if ( meta.isCondition() )
				tmpCond.add( functionRows[i] );
			else 
				tmpAct.add( functionRows[i] );
		}
		this.conditionRows = (IDecisionRow[]) tmpCond.toArray( new IDecisionRow[ tmpCond.size() ] );
		this.actionRows = (IDecisionRow[]) tmpAct.toArray( new IDecisionRow[ tmpAct.size() ] );
		
		//		this.structure = structure;
		this.columns = columns;
		prepare(header, openl, module, cxtd);
	}
	//	public Object getResult(int row, int col)
	//	{
	//	}
	
	public int getColumns() {
		return columns;
	}

	public void prepare(IOpenMethodHeader header, OpenL openl,
			IOpenClass dtModule, IBindingContextDelegator cxtd)
			throws Exception {
		
		IMethodSignature signature = header.getSignature();
		
		for (int i = 0; i < conditionRows.length; i++) {
			conditionRows[i].prepare(JavaOpenClass.BOOLEAN, signature, openl,
					dtModule, cxtd);
		}
		
		for (int i = 0; i < actionRows.length; i++) {
			IOpenClass methodType = invocationStrategy.defineActionReturnType(i, this ); // JF
			actionRows[i].prepare(methodType, signature, openl,
					dtModule, cxtd);
		}
	}
	
	public Object invoke(Object target, Object[] params, IRuntimeEnv env) {
		return invocationStrategy.invoke(target, params, env, this);
	}
	
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IMethodCaller#invoke(java.lang.Object,
	 *      java.lang.Object[], org.openl.vm.IRuntimeEnv)
	 */
	//abstract public Object invoke(Object target, Object[] params, IRuntimeEnv env);

	/**
	 * @return
	 */
	public IOpenClass getDeclaringClass() {
		return header.getDeclaringClass();
	}
	/**
	 * @return
	 */
	public IMemberMetaInfo getInfo() {
		return header.getInfo();
	}
	/**
	 * @return
	 */
	public String getName() {
		return header.getName();
	}
	/**
	 * @return
	 */
	public IMethodSignature getSignature() {
		return header.getSignature();
	}
	/**
	 * @return
	 */
	public IOpenClass getType() {
		return header.getType();
	}
	/**
	 * @return
	 */
	public boolean isStatic() {
		return header.isStatic();
	}
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IMethodCaller#getMethod()
	 */
	public IOpenMethod getMethod() {
		return this;
	}
	
	public ILogicalTable getLogicalTable() {
		return table;
	}

	public IDecisionRow[] getActions() {
		return actionRows;
	}

	public IDecisionRow[] getConditions() {
		return conditionRows;
	}
	
	public IDecisionRow[] getFunctionRows() {
		return conditionRows;
	}
	
	public IDecision getDecision() {
		return decision;
	}
	
	public void setDecision(IDecision decision) {
		this.decision = decision;
	}
	

}
