package com.openrules.dt;

import org.openl.types.IOpenClass;
import org.openl.types.java.JavaOpenClass;
import org.openl.util.Log;
import org.openl.util.Report;
import org.openl.vm.IRuntimeEnv;

import com.openrules.types.IDecision;

/*
 * Copyright 2007,2008 OpenRules, Inc. 
 */

/**
 * @author JF
 * modified by IK - invocationStrategy
 * Apr-2011: JF added option "RuleSet" to process DT row-by-row 
 *                   (not all conditions being evaluated first)
 */


public class DecisionTableMultiHit extends DecisionTableReport {

	public IOpenClass defineActionReturnType(int action, IDecisionTable dTable ) {
		return JavaOpenClass.VOID;
	}
	
	public boolean calculateConditionsFirst() {
		return true;
	}
	
	/*
	 * This methods is extracted from the old DecisionTable
	 */
	public Object invoke(Object target, Object[] params, IRuntimeEnv env, IDecisionTable dTable ) {
		
		if (Log.isDebugEnabled()) {
			Log.debug("Invoke " + dTable.getName());
		}
		
		if (params.length == 1 && params[0] instanceof IDecision)
			dTable.setDecision((IDecision)params[0]);
		
		IDecisionRow[] conditions = dTable.getConditions();
		
		DecisionTableAlgorithm algorithm = new DecisionTableAlgorithm( conditions.length,
					dTable.getColumns(), 
					conditions, target, params, env);

		IDecisionRow[] actions = dTable.getActions();
		String name = dTable.getName();
		Object returnObject = null;		
		if (dTable.isCalculateConditionsFirst()) {
			boolean[] mask = algorithm.calculateTable(); // all conditions are checked BEFORE action execution
			
			if (Log.isDebugEnabled()) {
				//Log.debug( dTable.getName());
				Log.debug( DecisionTableAlgorithm.printMask(mask) );        
			}
			
			reportStartDecisionTable(dTable, "Multi-Hit");
			for (int i = 0; i < mask.length; i++) {
				if (!mask[i])
					continue;
				reportRuleConditions(dTable,i);
				for (int j = 0; j < actions.length; j++) {
					IDecisionRow action = actions[j];
					action.executeAction(i, target, params, env);
					reportRuleAction(dTable, name, action, i, j); 
				}
				reportUpdateCurrentRule(dTable);
			}
		}
		else { // JF: check conditions AFTER previous rulesInMemory are executed
			reportStartDecisionTable(dTable, "Rule Sequence");
			for (int i = 0; i < dTable.getColumns(); i++) {
				boolean mask = algorithm.calcColumn(i);
				if (!mask)
					continue;
				reportRuleConditions(dTable,i);
				for (int j = 0; j < actions.length; j++) {
					IDecisionRow action = actions[j];
					action.executeAction(i, target, params, env);
					reportRuleAction(dTable, name, action, i, j); 
				}
			}
		}
		reportEndDecisionTable(dTable);
		return returnObject;
	}
	
}
