package com.openrules.dt;

import org.openl.util.IReport;
import org.openl.util.Report;

import com.openrules.types.IDecision;

/*
 * Copyright 2007-2013 OpenRules, Inc. 
 */

/**
 * @author JF
 */

abstract public class DecisionTableReport implements IInvocationStrategy {
	
	public IReport getReport(IDecisionTable dTable) {
		IDecision decision = dTable.getDecision();
		if (decision == null)
			return null;
		return decision.getReport();
	}
	
	public void reportStartDecisionTable(IDecisionTable dTable, String type) {
		IReport report = getReport(dTable);
		if (report == null)
			return;
		if (report.isOn() && dTable.getColumns() > 0) {
			IDecisionRow[] conditions = dTable.getConditions();
			IDecisionRow[] actions = dTable.getActions();
			String name = dTable.getName();
			String[] conditionLabels = new String[conditions.length];
			String[] conditionDescriptions = new String[conditions.length];
			for (int i = 0; i < conditions.length; i++) {
				conditionLabels[i] = conditions[i].getMetaInfo().getLabel();
				conditionDescriptions[i] = conditions[i].getMetaInfo().getDescription();
			}
			String[] actionLabels = new String[actions.length];
			String[] actionDescriptions = new String[actions.length];
			for (int i = 0; i < actions.length; i++) {
				actionLabels[i] = actions[i].getMetaInfo().getLabel();
				actionDescriptions[i] = actions[i].getMetaInfo().getDescription();
			}
			report.startDecisionTable(name, type, conditionLabels, conditionDescriptions,
				 	 actionLabels, actionDescriptions);
		}
	}
	
	public void reportRuleConditions(IDecisionTable dTable, int row) {
		IReport report = getReport(dTable);
		if (report == null)
			return;
		if (report.isOn()) { // Report rule conditions
			IDecisionRow[] conditions = dTable.getConditions();
			//IDecisionRow[] actions = dTable.getActions();
			String tableName = dTable.getName();
			report.addRule(tableName,row);
			for (int c = 0; c < conditions.length; c++) {
				String value = conditions[c].getColumnValue(row);
				report.addRuleCondition(tableName,value,c);
			}
		}
	}
	
	public void reportRuleAction(IDecisionTable dTable, String name, IDecisionRow action, int columnNumber, int actionNumber) {
		IReport report = getReport(dTable);
		if (report == null)
			return;
		if (report.isOn()) { // Report rule actions
			String value = action.getColumnValue(columnNumber);
			report.addRuleAction(name,value,actionNumber);
		}
	}
	
	public void reportEndDecisionTable(IDecisionTable dTable) {
		IReport report = getReport(dTable);
		if (report == null)
			return;
		if (report.isOn() && dTable.getColumns() > 0)
			report.endDecisionTable(dTable.getName());
	}
	
	public void reportUpdateCurrentRule(IDecisionTable dTable) {
		IReport report = getReport(dTable);
		if (report == null)
			return;
		if (report.isOn()) {
			IDecision decision = dTable.getDecision();
			decision.updateReportedRule();
		}
	}
		
}
