package com.openrules.dt;

import java.util.ArrayList;

import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.java.JavaOpenClass;
import org.openl.util.Log;
import org.openl.util.Report;
import org.openl.vm.IRuntimeEnv;

import com.openrules.types.IDecision;

/*
 * Copyright 2007,2008 OpenRules, Inc. 
 */

/**
 * @author JF
 * modified by IK - invocationStrategy
 * JF, Nov-2010: defineActionReturnType always returns dTable.getType()
 * because for optional actions the lastAction is not always
 * the last one from the template
 */

public class DecisionTableSingleHit extends DecisionTableReport {

	public IOpenClass defineActionReturnType(int action, IDecisionTable dTable ) {
//		boolean lastAction = (action == dTable.getActions().length-1); 
//		if (lastAction )
			return dTable.getType();
//		else
//			return JavaOpenClass.VOID;
	}
	
	public Object invoke(Object target, Object[] params, IRuntimeEnv env, IDecisionTable dTable ){
		
		if (Log.isDebugEnabled()) {
			Log.debug("Invoke " + dTable.getName());
		}
		
		if (params.length == 1 && params[0] instanceof IDecision)
			dTable.setDecision((IDecision)params[0]);
		
		int numberOfColumns = dTable.getColumns();
		IDecisionRow[] conditions = dTable.getConditions();
		IDecisionRow[] actions = dTable.getActions();
		String name = dTable.getName();
		
		DecisionTableAlgorithm algorithm = new DecisionTableAlgorithm( conditions.length,
				numberOfColumns, 
				conditions, 
				target, params, env);
		
		reportStartDecisionTable(dTable, "Single-Hit");
		
		Object returnObject = InvocationStrategy.NO_CONDITIONS_SATISFIED; 
		// no actions have been executed yet
		StringBuffer buf = new StringBuffer();
		buf.append( "[" ); 
		for (int i = 0; i < numberOfColumns; i++) {
			if (!algorithm.calcColumn(i)) {
				buf.append( "f" );        
				continue;
			}
			buf.append( "T]" ); 
			if (Log.isDebugEnabled())
				Log.debug(buf.toString());
			reportRuleConditions(dTable,i);
			returnObject = InvocationStrategy.SOME_CONDITIONS_SATISFIED;
			boolean stopRulesExecution = false;
			for (int j = 0; j < actions.length; j++) {
				IDecisionRow action = actions[j];
				Object actionReturnObject = action.executeAction(i, target, params, env);
				reportRuleAction(dTable,name, action, i, j); 
				if (actionReturnObject != null)
					returnObject = actionReturnObject;
				boolean lastAction = (j == actions.length-1); 
				if (lastAction)  { 
					stopRulesExecution = true;
					break;
					// return returnObject;
				}
			}
			reportUpdateCurrentRule(dTable);
			if (stopRulesExecution)
				break;
		}
		reportEndDecisionTable(dTable);
//		 JF: to satisfy a customer request not to return a string like "NO_CONDITIONS_SATISFIED" but rather null
//		This was a mistake as it prohibit execution of the default action for SingleHit tables - moved to ProxyDecisionTable 
//		if (returnObject == InvocationStrategy.NO_CONDITIONS_SATISFIED || 
//			returnObject == InvocationStrategy.SOME_CONDITIONS_SATISFIED)
//			returnObject = null;
		return returnObject;
	}

}
