/*
 * Copyright, 2009 OpenRules Inc.
 */


package com.openrules.dt;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.openl.OpenL;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.impl.module.ModuleOpenClass;

import com.openrules.lang.xls.XlsLoader.RuleTableStub;
import com.openrules.table.external.IRuleTable;

/**
 *
 * Generates conditions and actions for a ExternalDecisionTable instance. Validates
 * structural integrity of the table. Required meta information is taken from
 * the template reference. <code>ExternalDecisionTable.getTemplate()</code>
 *
 * @see ExternalFunctionalRow
 * @see ProxyDTLoader
 *
 * @author IK
 *
 */
public class ExternalDTLoader implements IDecisionTableConstants {

	class LocalMapping {

		List rows = new ArrayList();
		boolean looksLikeMapping = true;

		public LocalMapping(IRuleTable iRuleTable, DecisionTable base ) throws Exception {

			// list of labels for the proxy table
			String[] labels = iRuleTable.getRules().getLabels();
			if ( labels != null  ) {

				// map labels to row for the base table
				Map baseRows = new HashMap();
				for (int i = 0; i < base.functionRows.length; ++i) {
					IDecisionRow row = base.functionRows[i];
					baseRows.put( row.getMetaInfo().getLabel(), row);
				}
				
				//get template function rows for the provided labels
				for (int i = 0; i < labels.length; ++i ) {
					FunctionalRow functionRow = (FunctionalRow) baseRows.get(labels[i]);
					if (functionRow == null) {
						throw new Exception("Function label "+labels[i]+" is not found in template table");
					} else {
						rows.add(functionRow);
					}
				}
				
			} else { 
				//no mapping headers
				//get all template function rows except optional
				for (int i = 0; i < base.functionRows.length; ++i) {
					IDecisionRow row = base.functionRows[i];
					if ( ! row.getMetaInfo().isOptional() )
						rows.add( row );
				}
			}
		}
	}

	public IDecisionTable load( IRuleTable ruleTable, ProxyDecisionTable dt,
			OpenL openl, ModuleOpenClass module, IBindingContextDelegator cxtd)
			throws Exception {

		DecisionTable base = (DecisionTable) dt.getTemplate();

		LocalMapping localMapping = new LocalMapping(ruleTable, base);
		
		String[] titles = ruleTable.getRules().getDescriptions();
		
		List proxyRows = new ArrayList();
		int templateHeight = 0;
		for (int i = 0; i < localMapping.rows.size(); ++i ) {
			FunctionalRow templateRow = (FunctionalRow) localMapping.rows.get(i);
			IDecisionRowMetaInfo meta = templateRow.getMetaInfo();
			int adjustment = i - meta.getPos();
			String title = ( titles == null ? templateRow.getDescription() : titles[i] );  
			ExternalFunctionalRow proxy = new ExternalFunctionalRow(ruleTable, templateRow, adjustment, templateHeight, title );
			proxyRows.add(proxy);
			templateHeight += templateRow.getCellHeight();
		}
		
		final ExternalFunctionalRow[] proxyArray = (ExternalFunctionalRow[]) proxyRows.toArray( new ExternalFunctionalRow[proxyRows.size()]);
		
		boolean isDeclarationStub = ruleTable instanceof RuleTableStub ;
		
		if ( !isDeclarationStub && templateHeight != ruleTable.getRules().getNumberOfColumns() )
			throw new Exception("Invalid structure of external decision table '"
					+ ruleTable.getTableName()
					+ "'. It must be the same as the structure of '"
					+ ruleTable.getTemplateName() + "' except 'optional' parameters");
		
		int columns = ruleTable.getRules().getNumberOfRows();  // decision table columns == rows in vertical array
		
		dt.bindTable( null,	proxyArray,	openl, module, cxtd,columns);
		
		return dt;
	}
}