/*
 * Copyright, 2009 OpenRules Inc.
 */

package com.openrules.dt;

import org.openl.OpenL;
import org.openl.OpenlToolAdaptor;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.impl.BoundError;
import org.openl.types.IMethodSignature;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.IParameterDeclaration;
import org.openl.types.impl.OpenMethodHeader;

import com.openrules.data.IString2DataConvertor;
import com.openrules.data.String2DataConvertorFactory;
import com.openrules.table.external.ExternalDataLocation;
import com.openrules.table.external.ExternalDataSourceCodeModule;
import com.openrules.table.external.IRuleTable;


public class ExternalFunctionalRow extends ProxyFunctionalRow {

	private final IRuleTable ruleTable;
	private int startIdx;

	public ExternalFunctionalRow( IRuleTable ruleTable, FunctionalRow templateRow, int adjustment, int startIdx , String title) {
		super(null,templateRow,adjustment, title, "external" ); 
		this.ruleTable = ruleTable;
		this.label = "$$"+templateRow.label;
		this.startIdx = startIdx;
	}

	public boolean isExternal() {
		return true;
	}


	int nValues() {
		return ruleTable.getRules().getNumberOfRows();
	}

	int numberOfRules() {
		return nValues();
	}

//	// JF: this method was commented out as it is exactly the same as in FunctionalRow but was
//	//     missing: method = generateMethod(signature, openl, cxtd, dtModule, methodType);
//	public void prepare(IOpenClass methodType, IMethodSignature signature,
//			OpenL openl, IOpenClass dtModule, IBindingContextDelegator cxtd)
//			throws Exception {
//		
//		OpenlToolAdaptor ota = new OpenlToolAdaptor(openl, cxtd);
//		IOpenMethodHeader h = new OpenMethodHeader("dt_header", null, signature, dtModule);
//		ota.setHeader(h);
//
//		paramValues = prepareParamValues(ota);
//
//	}

	Object[][] prepareParamValues(OpenlToolAdaptor ota) throws Exception {
		int len = nValues();
		Object[][] values = new Object[len][];

		IParameterDeclaration[] params = getParams(ota.getOpenl());

		for (int col = 0; col < len; col++) {
			Object[] valueAry = new Object[params.length];

			boolean notEmpty = false;

			for (int j = 0; j < params.length; j++) {
				Object v = loadSingleParam( params[j].getType(), col, j, ota);

				if (v != null) {
					notEmpty = true;
				}
				valueAry[j] = v;
			}

			if (notEmpty)
				values[col] = valueAry;
		}

		return values;
	}
	
	public Object loadSingleParam(IOpenClass paramType, int row, int paramIdx, OpenlToolAdaptor ota) throws BoundError {

		//int column = this.getPos() + paramIdx;
		int column = startIdx + paramIdx;
		Object val = ruleTable.getRules().getValue(row,column);
		
		if ( !(val instanceof String) )
			return val;
		
		String src = ((String) val).trim();
		if ( src.length() == 0)
			return null;

		IString2DataConvertor conv = String2DataConvertorFactory
				.getConvertor(paramType.getInstanceClass());

		try {
			return conv.parse(src, null);
		} catch (Throwable t) {
			//TODO enrich error message
			throw new BoundError( new ExternalDataLocation(row,column), null, t, 
					new ExternalDataSourceCodeModule( ruleTable ));
		}
	}
	
}
