/*
 * Created on Sep 11, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.dt;

import java.lang.reflect.Array;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenL;
import org.openl.OpenlTool;
import org.openl.OpenlToolAdaptor;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.impl.BoundError;
import org.openl.syntax.impl.IdentifierNode;
import org.openl.syntax.impl.SubTextSourceCodeModule;
import org.openl.syntax.impl.TokenizerParser;
import org.openl.types.IMethodSignature;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethod;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.IParameterDeclaration;
import org.openl.types.impl.MethodSignature;
import org.openl.types.impl.OpenMethodHeader;
import org.openl.types.impl.ParameterDeclaration;
import org.openl.types.java.JavaOpenClass;
import org.openl.vm.IRuntimeEnv;

import com.openrules.data.IString2DataConvertor;
import com.openrules.data.String2DataConvertorFactory;
import com.openrules.types.Oper;
import com.openrules.table.ALogicalTable;
import com.openrules.table.IGridTable;
import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

/*
 * @author snshor 
 * Apr-2007: JF replaced "=" to ":=" 
 * May-2007: JF replaced return
 * type void to Object for executeAction()
 * 11/16/2010 - added snippet preprocessor
 * Dec-2010: JF added '::= formula' that is equivalent to ':= "" + formula'
 */
public class FunctionalRow implements IDecisionRowMetaInfo, IDecisionRow, IDecisionTableConstants {

	protected int row;
	protected int type;
	protected String label;
	protected String description;
	protected boolean optional;
	protected int cellHeight;
	protected String tableName;

	protected ILogicalTable decisionTable;
	protected ILogicalTable paramsTable;
	protected ILogicalTable codeTable;

	protected FunctionalRow() {
	}

	// factory method
	public static FunctionalRow make(int row, ILogicalTable dt) throws Exception {
		
		OpenL openl = OpenL.getInstance(); // JF added to support decision inference

		String rawLabel = dt.getLogicalRow(row).getGridTable().getStringValue(INFO_COLUMN, 0);
		
		int type = IDecisionRowMetaInfo.Helper.labelType(rawLabel);
		if ( type == TYPE_UNDEFINED )
			return null;

		FunctionalRow tmp = new FunctionalRow();
		tmp.row = row;
		tmp.cellHeight = dt.getLogicalRow(row).getGridTable().getGridHeight();
		tmp.type = type;
		tmp.label = IDecisionRowMetaInfo.Helper.unwrapLabel(rawLabel);
		tmp.optional = IDecisionRowMetaInfo.Helper.isOptional(rawLabel);
		tmp.decisionTable = dt;
		tmp.paramsTable = dt.getLogicalRegion(PARAM_COLUMN, row, 1,1);
		tmp.codeTable = dt.getLogicalRegion(CODE_COLUMN, row, 1, 1);
		tmp.description = dt.getLogicalRegion(PRESENT_COLUMN, row, 1, 1).getGridTable().getStringValue(0,0);
		
		return tmp;
	}

	public boolean isCondition() {
		return type == TYPE_CONDITION;
	}

	public boolean isAction() {
		return type == TYPE_ACTION;
	}

	public boolean isOptional() {
		return optional;
	}

	public boolean isProxy() {
		return false;
	}

	public boolean isExternal() {
		return false;
	}

	public String getLabel() {
		return label;
	}

	public String getDescription() {
		return description;
	}

	public int getPos() {
		return row;
	}
	
	public int getCellHeight() {
		return cellHeight;
	}

	public String getTableName() {
		return tableName;
	}

	public IDecisionRowMetaInfo getMetaInfo() {
		return this;
	}

	
	public void prepare(IOpenClass methodType, IMethodSignature signature,
			OpenL openl, IOpenClass dtModule, IBindingContextDelegator cxtd)
			throws Exception {
		method = generateMethod(signature, openl, cxtd, dtModule, methodType);

		
		IOpenMethodHeader h = new OpenMethodHeader("dt_header", null,
				signature, dtModule);
		OpenlToolAdaptor ota = new OpenlToolAdaptor(openl, cxtd);
		ota.setHeader(h);

		paramValues = prepareParamValues(ota);

	}

	int nValues() {
		return decisionTable.getLogicalWidth() - DATA_COLUMN;
	}

	// JF
	int numberOfRules() {
		return decisionTable.getGridTable().getGridWidth() - DATA_COLUMN;
	}

	ILogicalTable getValueCell(int col) {
		return decisionTable.getLogicalRegion(col + DATA_COLUMN, row, 1, 1);
	}

	Object[][] prepareParamValues(OpenlToolAdaptor ota) throws Exception {
		
		int len = nValues();
		Object[][] values = new Object[len][];

		IParameterDeclaration[] params = getParams(ota.getOpenl());

		for (int col = 0; col < len; col++) {
			ILogicalTable valueCell = getValueCell(col);
			IGridTable paramGridColumn = valueCell.getGridTable();

			Object[] valueAry = new Object[params.length];

			int fromHeight = 0;
			boolean notEmpty = false;

			for (int j = 0; j < params.length; j++) {

				int gridHeight = paramsTable.getLogicalRow(j).getGridTable()
						.getGridHeight();
				IGridTable singleParamGridTable = (IGridTable) paramGridColumn
						.rows(fromHeight, fromHeight + gridHeight - 1);
				
				Object v = loadParam(new LogicalTable(singleParamGridTable),
						params[j].getType(), ota);

				if (v != null) {
					notEmpty = true;
				}
				valueAry[j] = v;

				fromHeight += gridHeight;
			}

			if (notEmpty)
				values[col] = valueAry;
		}

		return values;
	}
	

	Object loadParam(ILogicalTable dataTable, IOpenClass paramType,
			OpenlToolAdaptor ota) throws BoundError {
		boolean indexed = paramType.getAggregateInfo().isAggregate(paramType);

		if (indexed)
			paramType = paramType.getAggregateInfo()
					.getComponentType(paramType);

		if (!indexed)
			return loadSingleParam(paramType, dataTable, ota);

		dataTable = ALogicalTable.make1ColumnTable(dataTable);

		int h = calcHeight(dataTable);

		if (h == 0)
			return null;

		Object ary = paramType.getAggregateInfo().makeIndexedAggregate(
				paramType, new int[] { h });

		for (int i = 0; i < h; i++) {
			ILogicalTable cell = dataTable.getLogicalRow(i);
			Array.set(ary, i, loadSingleParam(paramType, cell, ota));
		}

		return ary;

	}

	int calcHeight(ILogicalTable table) {
		int h = table.getLogicalHeight();

		for (int i = 0; i < h; i++) {
			String src = table.getLogicalRow(i).getGridTable().getStringValue(
					0, 0);
			if (src == null || src.trim().length() == 0)
				return i;

		}

		return h;
	}

	static public Object loadSingleParam(IOpenClass paramType,
			ILogicalTable cell, OpenlToolAdaptor ota) throws BoundError {

		String src = cell.getGridTable().getStringValue(0, 0);
		if (src == null || (src = src.trim()).length() == 0)
			return null;

		//if (src.startsWith("{") || src.startsWith("::=")) {
		if (src.startsWith("::=")) {
			//GridCellSourceCodeModule srcCode = new GridCellSourceCodeModule(cell.getGridTable());
			GridCellSourceCodeModule srcCode = new SnippetPreprocessor(cell.getGridTable()); // JF
			return ota.makeMethod(srcCode);
		}

		if (src.startsWith(":=")) {
			IOpenSourceCodeModule srcCode = new SubTextSourceCodeModule(
					//new GridCellSourceCodeModule(cell.getGridTable()), 2);
					new SnippetPreprocessor(cell.getGridTable()), 2); // JF
			return ota.makeMethod(srcCode);
		}
		
		if (src.startsWith("{")) {
			IOpenSourceCodeModule srcCode = new SnippetPreprocessor(cell.getGridTable()); // JF
			return ota.makeMethod(srcCode);
		}
		
		IString2DataConvertor conv = String2DataConvertorFactory
				.getConvertor(paramType.getInstanceClass());

		try {
			return conv.parse(src, null);
		} catch (Throwable t) {
			throw new BoundError(null, null, t, new GridCellSourceCodeModule(
					cell.getGridTable()));
		}
	}

	IParameterDeclaration[] params;

	static public IOpenClass getType(String typeCode, OpenL openl)
			throws Exception {
		if (typeCode.endsWith("[]")) {
			String baseCode = typeCode.substring(0, typeCode.length() - 2);
			IOpenClass baseType = OpenlTool.getType(baseCode, openl);
			if (baseType == null)
				return null;
			return baseType.getAggregateInfo().getIndexedAggregateType(
					baseType, 1);
		}

		IOpenClass type = OpenlTool.getType(typeCode, openl);
		// if (type == null)
		// throw new Exception("Type " + typeCode + " not found");

		return type;
	}

	static final IParameterDeclaration NO_PARAMETER = new ParameterDeclaration(
			JavaOpenClass.STRING, null);

	IParameterDeclaration[] getParams(OpenL openl) throws Exception {
		if (params == null) {
			int len = paramsTable.getLogicalHeight();
			params = new IParameterDeclaration[len];

			for (int i = 0; i < len; i++) {

				ILogicalTable paramTable = paramsTable.getLogicalRow(i);
				IOpenSourceCodeModule src = new GridCellSourceCodeModule(
						paramTable.getGridTable());
				IdentifierNode[] nodes = TokenizerParser.tokenize(src, " \n\r");
				if (nodes.length == 0) {
					// no parameters
					params[0] = NO_PARAMETER;
					return params;
				}

				String errMsg;

				if (nodes.length != 2) {
					errMsg = "Parameter Cell format: <type> <name>";
					BoundError err = new BoundError(null, errMsg, null, src);
					throw err;
				}

				String typeCode = nodes[0].getIdentifier();
				IOpenClass ptype = getType(typeCode, openl);
				if (ptype == null)
					throw new BoundError(nodes[0], "Type not found: "
							+ typeCode);

				params[i] = new ParameterDeclaration(ptype, nodes[1]
						.getIdentifier());

			}

		}
		return params;
	}

	IOpenMethod generateMethod(IMethodSignature signature, OpenL openl,
			IBindingContextDelegator cxt, IOpenClass declaringClass,
			IOpenClass methodType) throws Exception {

		//IOpenSourceCodeModule src = new GridCellSourceCodeModule(codeTable
		//		.getGridTable());

		IOpenSourceCodeModule src = new SnippetPreprocessorTemplate( codeTable.getGridTable(), this );
		
		IParameterDeclaration[] mparams = getParams(openl);

		IMethodSignature newSignature = hasNoParams() ? signature
				: ((MethodSignature) signature).merge(mparams);

		OpenMethodHeader methodHeader = new OpenMethodHeader(null, methodType,
				newSignature, declaringClass);

		//ContextPropagation contextPropagation = new ContextPropagation(cxt);
		//ILocalVar var = contextPropagation.addVar("org.openl.this", "$TITLE", JavaOpenClass.STRING );
		//return OpenlTool.makeMethod(src, openl, methodHeader, contextPropagation);

		return OpenlTool.makeMethod(src, openl, methodHeader, cxt);
	}

	boolean hasNoParams() {
		return params[0] == NO_PARAMETER;
	}

	static Object notNull(Object x) {
		return x == null ? "" : x;
	}

	//////////////////////////////////////////////////////////////////
	
	boolean addSignEqual(String label) { // to show default assignment operator in WHY-analyzer
		if ("$If".equalsIgnoreCase(label) || 
			"$Action".equalsIgnoreCase(label) || 
			"$Then".equalsIgnoreCase(label)) 
			return true;
		return false;
	}
	
	boolean startsWithOper(String s) {
		String[] startChars = {":","<",">","="};
		for (int i = 0; i < startChars.length; i++) {
			if (s.startsWith(startChars[i]))
				return true;
		}
		return false;
	}

	IOpenMethod method;
	Object[][] paramValues; 
	
	public String getColumnValue(int col) { // for reporting only
		Object value = paramValues[col];
		if (value == null)
			return null;
		
		if (value instanceof Object[]) {
			Object[] values = (Object[]) value;
			StringBuffer buf = new StringBuffer();
			for (int i = 0; i < values.length; i++) {
				if (values[i] instanceof String)
					buf.append(values[i]+" ");
				else
				if (values[i] instanceof Oper) {
					Oper oper = (Oper) values[i];
					buf.append(oper.getName()+" ");
				}
				else {
					String src = getColumnSource(col,i);
					if (src == null)
						src ="-";
					buf.append(src);
				}
			}
			
			String result = buf.toString();
			if (addSignEqual(label) && !(startsWithOper(result)))
				result = "= " + result;
			return result;
		}
		
		return "-";
	}
	
	public String getColumnSource(int col, int paramCol) { // JF for reporting only
		
		ILogicalTable valueCell = getValueCell(col);
		IGridTable paramGridColumn = valueCell.getGridTable();
		IGridTable singleParamGridTable = (IGridTable) paramGridColumn.rows(0,row);
		return singleParamGridTable.getStringValue(0,paramCol); // JF
	}

	public IDecisionValue calculateCondition(int col, Object target,
			Object[] dtParams, IRuntimeEnv env) {
		Object value = paramValues[col];
		if (value == null)
			return IDecisionValue.NxA;
		if (value instanceof IDecisionValue)
			return (IDecisionValue) value;

		Object[] params = hasNoParams() ? dtParams : mergeParams(target,
				dtParams, env, (Object[]) value);
		// (Object[]) ArrayTool.merge(dtParams, value);
		// dtParams.length == method.getSignature().getParameterTypes().length ?
		// dtParams :
		// (Object[]) ArrayTool.merge(dtParams, value);
		
		Boolean res = (Boolean) method.invoke(target, params, env);

		return res == null || res.booleanValue() ? IDecisionValue.True
				: IDecisionValue.False;

	}

	static public Object[] mergeParams(Object target, Object[] dtParams,
			IRuntimeEnv env, Object[] params) {
		Object[] newParams = new Object[dtParams.length + params.length];

		System.arraycopy(dtParams, 0, newParams, 0, dtParams.length);

		for (int i = 0; i < params.length; i++) {
			Object x = params[i];

			if (x instanceof IOpenMethod) {
				x = ((IOpenMethod) x).invoke(target, dtParams, env);
			}

			newParams[i + dtParams.length] = x;
		}

		return newParams;
	}

	/*
	 * JF replaced return type from void to Object
	 *
	 * @see com.openrules.dt.IDecisionRow#executeAction(int, java.lang.Object,
	 *      java.lang.Object[], org.openl.vm.IRuntimeEnv)
	 */
	public Object executeAction(int col, Object target, Object[] dtParams,
			IRuntimeEnv env) {
		
		Object value = paramValues[col];
		if (value == null)
			return null;
		
		if (hasNoParams())
			return method.invoke(target, dtParams, env);
		else
			return method.invoke(target, mergeParams(target, dtParams, env,
					(Object[]) value),

			// (Object[]) ArrayTool.merge(dtParams, value),
					env);

		// return res.booleanValue() ? IDecisionValue.True :
		// IDecisionValue.False;

	}
	
	public Object[][] getParamValues() {
		return paramValues;
	}

	public String toString() {
		return this.label;
	}
}


//class ContextPropagation extends BindingContextDelegator  {
//
//	LocalFrameBuilder localFrame = new LocalFrameBuilder();
//
//	public ContextPropagation(IBindingContext delegate) {
//		super(delegate);
//	    localFrame.pushLocalVarContext();
//	}
//
//	public ILocalVar addVar(String namespace, String name, IOpenClass type)
//	    throws DuplicatedVarException   {
//	    return localFrame.addVar(namespace, name, type);
//	  }
//
//	public IOpenField findVar(String namespace, String name)
//	    throws AmbiguousVarException
//	  {
//	  	IOpenField var = localFrame.findLocalVar(namespace, name);
//	  	
//	    return var != null ? var : delegate.findVar(namespace, name);
//	  }
//	
//}

