package com.openrules.dt;


public interface IDecisionRowMetaInfo {

	public static final int TYPE_UNDEFINED = 0;
	public static final int TYPE_CONDITION = 1;
	public static final int TYPE_ACTION = 2;

	public boolean isCondition();

	public boolean isAction();

	public boolean isOptional();

	public boolean isProxy();

	public boolean isExternal();

	public String getLabel();

	public int getPos();

	public String getDescription();
	public String getTableName();
	
	public static class Helper {
		public static String unwrapLabel( String rawLabel ) {
			if ( rawLabel == null )
				return null;
			if ( isOptional(rawLabel) )
				return rawLabel.substring(1, rawLabel.length()-1 );
			return rawLabel;
		}
		
		// modified by JF adding If/Then indicators
		public static int labelType(String rawLabel) {
			//String label = unwrapLabel(rawLabel);
			if (rawLabel != null && rawLabel.length() >= 1) {
				String label = rawLabel.trim();
				//if (label.charAt(0) == 'C' && !label.startsWith("Conclusion"))
				if (isCondition(label))
					return TYPE_CONDITION;
				//if (label.charAt(0) == 'A' || label.startsWith("Conclusion"))
				if (isAction(label))
					return TYPE_ACTION;
			}
			return TYPE_UNDEFINED;
		}
		
		public static boolean isCondition(String rawLabel) {
			String label = unwrapLabel(rawLabel);
			if  (label == null || label.length()==0 
					|| label.indexOf(" ")>0)  // one word only!
				return false;
			if (label.startsWith("Conclusion"))
				return false;
			if (label.startsWith("Condition"))
				return true;
			if (label.startsWith("IF") || label.startsWith("If"))
				return true;
			if (label.charAt(0) == 'C' && 
			    (label.length() >= 2 && Character.isDigit(label.charAt(1))))
			    return true;
			if (label.charAt(0) == 'C') // everything that starts with 'C'
				return true;
			if (label.equals("OnOff")) {
				String mode = System.getProperty("OPENRULES_MODE");
				if (!"Solve".equals(mode))
					return true;
			}
			return false;
				
		}
		
		public static boolean isAction(String rawLabel) {
			String label = unwrapLabel(rawLabel);
			if  (label == null || label.length()==0
					|| label.indexOf(" ")>0)  // one word only!)
				return false;
			if (label.startsWith("AND"))
				return false;
			if (label.startsWith("Conclusion"))
				return true;
			if (label.startsWith("Message"))
				return true;
			if (label.equals("#"))
				return true;
			if (label.startsWith("Action"))
				return true;
			if (label.startsWith("Then") || label.startsWith("THEN"))
				return true;
			if (label.charAt(0) == 'A' && 
				(label.length() >= 2 && Character.isDigit(label.charAt(1))))
				return true;
			if (label.charAt(0) == 'A') // everything that starts with 'A'
				return true;
			if (label.equals("OnOff")) {
				String mode = System.getProperty("OPENRULES_MODE");
				if ("Solve".equals(mode))
					return true;
			}
			return false;
		}
		
		public static boolean isOptional(String label) {
			if ( label == null || label.length() < 3 )
				return false;
			return ( label.charAt(0) == '[' && label.charAt(label.length() - 1 ) == ']' );
		}
		
		public static boolean isFunctionLabel(String s) {
			return labelType(s) != TYPE_UNDEFINED;
		}
	}
}
