package com.openrules.dt;

/*
 * Copyright, 2008 OpenRules Inc.
 */

import org.openl.OpenL;
import org.openl.OpenlTool;
import org.openl.OpenlToolAdaptor;
import org.openl.binding.IBindingContextDelegator;
import org.openl.types.IMethodSignature;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethod;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.IParameterDeclaration;
import org.openl.types.impl.MethodSignature;
import org.openl.types.impl.OpenMethodHeader;

import com.openrules.table.ILogicalTable;

/**
 * Links data section of a decision table to code and parameters sections of a
 * template table.
 *
 * @see FunctionalRow
 * @see ProxyDecisionTable
 *
 * @author IK
 */

public class ProxyFunctionalRow extends FunctionalRow {

	final FunctionalRow templateRow;

	public ProxyFunctionalRow( ILogicalTable decisionTable, FunctionalRow templateRow, int rowAdjustment, String description, String tableName ) {
		this.decisionTable = decisionTable;
		this.templateRow = templateRow;
		this.row = templateRow.row + rowAdjustment;
		this.cellHeight = templateRow.cellHeight;
		this.type = templateRow.type;
		this.paramsTable = templateRow.paramsTable;
		this.codeTable = templateRow.codeTable;
		this.method = templateRow.method;
		this.params = templateRow.params;
		this.label = "$"+templateRow.label;
		this.description = description;
		this.optional = templateRow.optional;
		this.tableName = tableName;
	}

	public FunctionalRow getTemplateRow() {
		return templateRow;
	}

	public boolean isProxy() {
		return true;
	}

	int nValues() {
		return decisionTable.getLogicalWidth() - PROXY_DATA_COLUMN;
	}

	int numberOfRules() {
		return decisionTable.getGridTable().getGridWidth() - PROXY_DATA_COLUMN;
	}

	ILogicalTable getValueCell(int col) {
		return decisionTable.getLogicalRegion(col + PROXY_DATA_COLUMN,row,1,1);
	}

	public void prepare(IOpenClass methodType, IMethodSignature signature,
			OpenL openl, IOpenClass dtModule, IBindingContextDelegator cxtd)
			throws Exception {

		IOpenMethod myMethod = generateMethod(signature, openl, cxtd, dtModule, methodType);
		if ( myMethod != null ) { 
			//there is code customization, we have our own method now
			this.method = myMethod;
		}
		
		OpenlToolAdaptor ota = new OpenlToolAdaptor(openl, cxtd);
		IOpenMethodHeader h = new OpenMethodHeader("dt_header", null, signature, dtModule);
		ota.setHeader(h);

		paramValues = prepareParamValues(ota);


	}

	//generate method only if there is some customization for this proxy ( like macro substitution )
	IOpenMethod generateMethod(IMethodSignature signature, OpenL openl,
			IBindingContextDelegator cxt, IOpenClass declaringClass,
			IOpenClass methodType) throws Exception {

		SnippetPreprocessorTemplate src = new SnippetPreprocessorTemplate( codeTable.getGridTable(), this );
		if ( !src.isModified() ) {
			return null;
		}
		
		IParameterDeclaration[] mparams = getParams(openl);

		IMethodSignature newSignature = hasNoParams() ? signature
				: ((MethodSignature) signature).merge(mparams);

		OpenMethodHeader methodHeader = new OpenMethodHeader(null, methodType,
				newSignature, declaringClass);

		return OpenlTool.makeMethod(src, openl, methodHeader, cxt);
	}
	

}
