package com.openrules.dt;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.openl.util.Log;

import com.openrules.table.IGridTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

/**
 * Added since the version 6.3.0
 * 
 * @author JF
 * 
 */
public class SnippetPreprocessor extends GridCellSourceCodeModule {

	static final String[][] pairs = {
			// backward compatibility
//			 { "decision\\(\\).", "decision." },
//			 { "getDecisionObject\\(", "decision.getDecisionObject(" },
//			 { "getGlossary\\(", "decision.getGlossary(" },
//			 { "getInt\\(", "decision.getInt(" },
//			 { "getReal\\(", "decision.getReal(" }, // [^\\.]?
//			 { "getString\\(", "decision.getString(" },
//			 { "getBool\\(", "decision.getBool(" },
//			 { "getDate\\(", "decision.getDate(" },
//			 { "setInt\\(", "decision.setInt(" },
//			 { "setReal\\(", "decision.setReal(" },
//			 { "setString\\(", "decision.setString(" },
//			 { "setBool\\(", "decision.setBool(" },
//			 { "setDate\\(", "decision.setDate(" },
//			 { "compareInt\\(", "decision.compareInt(" },
//			 { "compareReal\\(", "decision.compareReal(" },
//			 { "compareString\\(", "decision.compareString(" },
//			 { "compareBool\\(", "decision.compareBool(" },
//			 { "compareDate\\(", "decision.compareDate(" },
			// new macros
			{ "\\$\\{([^\\})]+)\\}", "decision.getString(\"$1\"\\)" },
			{ "\\$I\\{([^\\})]+)\\}", "decision.getInt(\"$1\"\\)" },
			{ "\\$L\\{([^\\})]+)\\}", "decision.getLong(\"$1\"\\)" },
			{ "\\$R\\{([^\\})]+)\\}", "decision.getReal(\"$1\"\\)" },
			{ "\\(\\$G\\{([[+-]?\\d+(\\.\\d{1,2})?]+)\\}\\)", "(new BigDecimal($1\\),MathContext.DECIMAL128)" },
			{ "\\$G\\{([[+-]?\\d+(\\.\\d{1,2})?]+)\\}", "new BigDecimal($1\\)" },
			{ "\\$G\\{([^\\})]+)\\}", "decision.getBigDecimal(\"$1\"\\)" },
			{ "\\$D\\{([^\\})]+)\\}", "decision.getDate(\"$1\"\\)" },
			{ "\\$B\\{([^\\})]+)\\}", "decision.getBool(\"$1\"\\)" },
			{ "\\$O\\{([^\\})]+)\\}", "(($1)decision.getBusinessObject(\"$1\"\\))" },
			{ "\\$V\\{([^\\})]+)\\}", "getVar(decision,\"$1\"\\)" }

	};

	static Pattern[] patterns;

	static {
		patterns = new Pattern[pairs.length];
		for (int i = 0; i < patterns.length; i++) {
			String pattern = pairs[i][0];
			try {
				patterns[i] = Pattern.compile(pattern);
			} catch (Exception e) {
				Log.error("SnippetPreprocessor invalid pattern:" + pattern);
				throw new RuntimeException(e);
			}
		}
	}

	private String parsedCode = null;
	private boolean modified = false;

	private String macro(Pattern p, String src, String replacement) {
		Matcher m = p.matcher(src);
		if (m.find()) {
			modified = true;
			return m.replaceAll(replacement);
		}
		return src;
	}

	public SnippetPreprocessor(IGridTable table) {
		super(table);
		String code = super.getCode();
		if (code != null && code.indexOf('$') >= 0) {
			for (int i = 0; i < patterns.length; i++) {
				code = macro(patterns[i], code, pairs[i][1]);
			}
		}
//		if (modified)
//		    System.out.println("\nREPLACED: " + super.getCode() +"\n=> " + code);
		this.parsedCode = code;
	}

	static public String process(String input) {
		String output = input;
		for (int i = 0; i < patterns.length; i++) {
			// output = macro(patterns[i], output, pairs[i][1]);
			Pattern p = patterns[i];
			String replacement = pairs[i][1];
			Matcher m = p.matcher(output);
			if (m.find()) {
				// modified = true;
				output = m.replaceAll(replacement);
			}
		}
		return output;
	}

	public String getCode() {
		return parsedCode;
	}

	public boolean isModified() {
		return modified;
	}

	public static void main(String[] args) {

		String input = "REAL(Tax Withheld) + REAL(Earned Income Credit)";
		String pattern = "REAL\\(([^\\)]+)\\)";
		String replacement = "decision.getReal(\"$1\"\\)";
		// String regex = Pattern.quote(pattern);
		// Pattern p = Pattern.compile(regex);
		Pattern p = Pattern.compile(pattern);
		Matcher m = p.matcher(input);
		System.out.println("Input: " + input);
		if (m.find()) {
			System.out.println("Output: " + m.replaceAll(replacement));
		}

		input = " REAL(Tax Withheld) + REAL(Earned Income Credit)";
		pattern = "REAL\\(([^\\))]+)\\)";
		// pattern = "\\QREAL\\(([^\\))]+)\\)\\E";
		replacement = "decision.getReal(\"$1\"\\)";
		p = Pattern.compile(pattern);
		m = p.matcher(input);
		System.out.println("Input: " + input);
		if (m.find()) {
			System.out.println("Output: " + m.replaceAll(replacement));
		}

		input = "$'Tax Withheld' + $'Earned Income Credit'";
		pattern = "\\$'([^')]+)'";
		replacement = "decision.getReal(\"$1\"\\)";
		p = Pattern.compile(pattern);
		m = p.matcher(input);
		System.out.println("Input: " + input);
		if (m.find()) {
			System.out.println("Output: " + m.replaceAll(replacement));
		}

		String test = "$R{Tax Withheld} + $R{Earned Income Credit}+ $I{Extra} + decision.get(something)";
		System.out.println("\nInput: " + test);
		test = SnippetPreprocessor.process(test);
		System.out.println("Output: " + test);
		
		test = "$L{Tax Withheld} + $L{Earned Income Credit}+ $I{Extra} + decision.get(something)";
		System.out.println("\nInput: " + test);
		test = SnippetPreprocessor.process(test);
		System.out.println("Output: " + test);
		
		test = "$G{Cost}.divide($G{13.0001})";
		System.out.println("\nInput: " + test);
		test = SnippetPreprocessor.process(test);
		System.out.println("Output: " + test);
		
		test = "$G{-16.0001}";
		System.out.println("\nInput: " + test);
		test = SnippetPreprocessor.process(test);
		System.out.println("Output: " + test);
		
		test = "log(\"FT  \" + $V{Mon FT}.getValue() +  \"  \"  + $V{Tue FT}.getValue());";
		System.out.println("\nInput: " + test);
		test = SnippetPreprocessor.process(test);
		System.out.println("Output: " + test);

	}

}