package com.openrules.dt;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.openrules.lang.xls.IXlsTableNames;
import com.openrules.table.IGridTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

public class SnippetPreprocessorTemplate extends GridCellSourceCodeModule {

	public static 	Pattern columnTitleMacroPattern;
	public static 	Pattern tableTitleMacroPattern;
	public static 	Pattern columnNumberMacroPattern;
	
	static {
		String regex;
		regex = Pattern.quote( IXlsTableNames.FUNCTIONAL_ROW_MACRO_COLUMN_TITLE); //since 1.5
		//String regex = "\\Q" + IXlsTableNames.FUNCTIONAL_ROW_MACRO_COLUMN_TITLE + "\\E";
		columnTitleMacroPattern = Pattern.compile(regex);
		regex = Pattern.quote( IXlsTableNames.FUNCTIONAL_ROW_MACRO_TABLE_TITLE );
		//regex = "\\Q" + IXlsTableNames.FUNCTIONAL_ROW_MACRO_TABLE_TITLE + "\\E";
		tableTitleMacroPattern = Pattern.compile(regex);
		regex = Pattern.quote( IXlsTableNames.FUNCTIONAL_ROW_MACRO_COLUMN_NUMBER );
		columnNumberMacroPattern = Pattern.compile(regex);
	}

	private String parsedCode = null;
	private boolean modified = false;

	private String macro(Pattern p, String src, String replacement) {
		Matcher m = p.matcher(src);
		if ( m.find() ) {
			modified = true;
			return m.replaceAll( replacement );
		}
		return src;
	}
		
	public SnippetPreprocessorTemplate(IGridTable table, FunctionalRow fr ) {
		super(table);
		String description = "\""+fr.getDescription()+"\"";
		String tableName = "\""+fr.getTableName()+"\"";
		String position = "\"" + fr.getPos()+"\"";
		String code = super.getCode();
		code = macro( columnTitleMacroPattern, code, description );
		code = macro( tableTitleMacroPattern, code, tableName );
		code = macro( columnNumberMacroPattern, code, position );
		this.parsedCode = code;
	}

	public String getCode()	{
		return parsedCode;
	}

	public boolean isModified() {
		return modified;
	}
}