/*
 * Created on Oct 2, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 * Modified by JF on 4/19/2005
 * Added VariableNodeBinder
 * 
 * Modified by IK on 08/12/2006 to support for imports directly
 * from xls Environment tables
 * 
 * March-2009: IK have implemented two-pass xls binder to allow
 * rulesInMemory to be defined before template based on which they are created
 */

package com.openrules.lang.xls;

import java.util.HashMap;
import java.util.Map;

import org.openl.IOpenBinder;
import org.openl.OpenConfigurationException;
import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.IBoundCode;
import org.openl.binding.IBoundError;
import org.openl.binding.IBoundNode;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.impl.BoundCode;
import org.openl.binding.impl.BoundError;
import org.openl.binding.impl.TooManyErrorsError;
import org.openl.binding.impl.module.ModuleBindingContext;
import org.openl.binding.impl.module.ModuleNode;
import org.openl.conf.IUserContext;
import org.openl.syntax.IParsedCode;
import org.openl.syntax.ISyntaxError;
import org.openl.syntax.ISyntaxNode;
import org.openl.syntax.SyntaxErrorException;
import org.openl.util.ASelector;
import org.openl.util.AStringConvertor;
import org.openl.util.ISelector;
import org.openl.util.RuntimeExceptionWrapper;

import com.openrules.lang.xls.binding.AXlsTableBinder;
import com.openrules.lang.xls.binding.DTNodeBinder;
import com.openrules.lang.xls.binding.DataNodeBinder;
import com.openrules.lang.xls.binding.DatatypeNodeBinder;
import com.openrules.lang.xls.binding.ExternalDTNodeBinder;
import com.openrules.lang.xls.binding.MethodTableNodeBinder;
import com.openrules.lang.xls.binding.ProxyDTNodeBinder;
import com.openrules.lang.xls.binding.UnresolvedDependenciesBoundNode;
import com.openrules.lang.xls.binding.VariableNodeBinder;
import com.openrules.lang.xls.binding.XlsModuleOpenClass;
import com.openrules.lang.xls.syntax.XlsModuleSyntaxNode;

/**
 * @author snshor, ik
 * 
 */
public class XlsBinder implements IOpenBinder, IXlsNodeTypes {

	IUserContext ucxt;

	/**
	 * 
	 */
	public XlsBinder(IUserContext ucxt) {
		this.ucxt = ucxt;
	}

	public IBoundCode bind(IParsedCode parsedCode) {
		return bind(parsedCode, null);
	}

	public IBoundCode bind(IParsedCode parsedCode,
			IBindingContextDelegator delegator) {
		XlsModuleSyntaxNode moduleNode = (XlsModuleSyntaxNode) parsedCode
				.getTopNode();

		OpenL openl = null;
		try {
			openl = OpenL.getInstance(moduleNode.getModule().getUri(0),
			// moduleNode.getOpenlNode().getOpenlName(), // IK
					ucxt);

		} catch (OpenConfigurationException e) {
			return new BoundCode(parsedCode, null,
					new IBoundError[] { new BoundError(moduleNode
							.getOpenlNode(), null, e) }, 0);
		}

		IOpenBinder openlBinder = openl.getBinder();

		IBindingContext context = openlBinder.makeBindingContext();
		if (delegator != null) {
			delegator.setTopDelegate(context);
			context = delegator;
		}

		IBoundNode topNode = null;
		try {
			topNode = bind(moduleNode, openl, context);
		} catch (TooManyErrorsError err) {
		}

		return new BoundCode(parsedCode, topNode, context.getError(), 0);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.IOpenBinder#bind(org.openl.syntax.IParsedCode)
	 */
	public IBoundNode bind(ISyntaxNode moduleNode, OpenL openl,
			IBindingContext cxt) {
		// TODO fix schema, name
		XlsModuleOpenClass module = new XlsModuleOpenClass(null,
				"UndefinedXlsType");

		// int nchildren = moduleNode.getNumberOfChildren();

		// IMemberBoundNode[] children = new IMemberBoundNode[nchildren];
		ModuleBindingContext moduleContext = new ModuleBindingContext(cxt,
				module);

		ASelector dataTypeSelector = new ASelector.StringValueSelector(
				IXlsNodeTypes.XLS_DATATYPE, new SyntaxConvertor());

		bindInternal(moduleNode, openl, moduleContext, module, dataTypeSelector);

		return bindInternal(moduleNode, openl, moduleContext, module,
				dataTypeSelector.not());
	}

	static class SyntaxConvertor extends AStringConvertor {

		public String getStringValue(Object test) {
			return ((ISyntaxNode) test).getType();
		}
	}
	
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.IOpenBinder#bind(org.openl.syntax.IParsedCode)
	 */
	protected IBoundNode bindInternal(ISyntaxNode moduleNode, OpenL openl,
			ModuleBindingContext moduleContext, XlsModuleOpenClass module,
			ISelector childSelector) {

		int nchildren = moduleNode.getNumberOfChildren();
		IMemberBoundNode[] children = new IMemberBoundNode[nchildren];

		int pass = 0;
		boolean done = false;
		
		while( !done ) {
			done = true;
			++pass;
			for (int i = 0; i < nchildren; i++) {
				try {
					if ( pass == 1 ) { //first pass
						ISyntaxNode childNode = moduleNode.getChild(i);
						if (childSelector.select(childNode)) {
							children[i] = preBindXlsNode(childNode,	openl,moduleContext,module);
							
							if ( children[i] !=null ) {
								if ( children[i] instanceof UnresolvedDependenciesBoundNode )
									done = false;
								else
									children[i].addTo(module);
							}
						}
					} else {
						//second pass
						//process nodes unresolved on the first pass
						if ( children[i] instanceof UnresolvedDependenciesBoundNode ) {
							IMemberBoundNode node = ((UnresolvedDependenciesBoundNode)children[i]).getNode();
							node.addTo(module);
						}
						
					}
					
				} catch (BoundError err) {
					moduleContext.addError(err);
				} catch (SyntaxErrorException se) {
					ISyntaxError[] ee = se.getSyntaxErrors();
	
					for (int j = 0; j < ee.length; j++) {
						moduleContext.addError(ee[j]);
					}
	
				}
	
				catch (Exception e) {
					moduleContext.addError(new BoundError(moduleNode.getChild(i),
							null, e));
				}
			}
		}
		
		if (moduleContext.getNumberOfErrors() == 0) {
			pass = 0;
			done = false; 
			while ( !done ) {
				done = true;
				++pass;
				for (int i = 0; i < children.length; i++) {
					if (children[i] != null) {
						try {
							if ( pass == 1 ) {
								if ( children[i] instanceof UnresolvedDependenciesBoundNode )
									done = false;
								else 
									((IMemberBoundNode) children[i]).finalizeBind(moduleContext);
							} else {
								//second pass, process nodes not finalized on the first pass  
								if ( children[i] instanceof UnresolvedDependenciesBoundNode ) {
									IMemberBoundNode node = ((UnresolvedDependenciesBoundNode)children[i]).getNode();
									node.finalizeBind(moduleContext);
								}
							}
						} catch (BoundError be) {
							moduleContext.addError(be);
						} catch (SyntaxErrorException se) {
							ISyntaxError[] ee = se.getSyntaxErrors();
	
							for (int j = 0; j < ee.length; j++) {
								moduleContext.addError(ee[j]);
							}
	
						} catch (Exception e) {
							moduleContext.addError(new BoundError(moduleNode
									.getChild(i), null, e));
						}
					}
				}
			}
		}
		return new ModuleNode(moduleNode, moduleContext.getModule());

	}

	IMemberBoundNode preBindXlsNode(ISyntaxNode syntaxNode, OpenL openl,
			IBindingContext cxt, XlsModuleOpenClass module) throws Exception {
		String type = syntaxNode.getType();

		AXlsTableBinder binder = (AXlsTableBinder) binderFactory().get(type);

		// if (XLS_DT.equals(type))
		// return preBindDT(syntaxNode, openl, cxt, module);
		//
		// if (XLS_DATA.equals(type))
		// return preBindData(syntaxNode, openl, cxt, module);
		//
		// if (XLS_DATATYPE.equals(type))
		// return preBindDatatype(syntaxNode, openl, cxt, module);
		//
		// if (XLS_METHOD.equals(type))
		// return preBindMethod(syntaxNode, openl, cxt, module);

		if (binder == null)
			throw new Exception("Unknown Table Type: " + type);

		return binder.preBind(syntaxNode, openl, cxt, module);
	}

	/**
	 * @param syntaxNode
	 * @param openl
	 * @param cxt
	 * @param module
	 * @return
	 */
	IMemberBoundNode preBindDatatype(ISyntaxNode syntaxNode, OpenL openl,
			IBindingContext cxt, XlsModuleOpenClass module) throws Exception {
		return new DatatypeNodeBinder().preBind(syntaxNode, openl, cxt, module);
	}

	IMemberBoundNode preBindDT(ISyntaxNode syntaxNode, OpenL openl,
			IBindingContext cxt, XlsModuleOpenClass module) throws Exception {
		return new DTNodeBinder().preBind(syntaxNode, openl, cxt, module);
	}

	IMemberBoundNode preBindMethod(ISyntaxNode syntaxNode, OpenL openl,
			IBindingContext cxt, XlsModuleOpenClass module) throws Exception {
		return new MethodTableNodeBinder().preBind(syntaxNode, openl, cxt,
				module);
	}

	IMemberBoundNode preBindData(ISyntaxNode syntaxNode, OpenL openl,
			IBindingContext cxt, XlsModuleOpenClass module) throws Exception {
		return new DataNodeBinder().preBind(syntaxNode, openl, cxt, module);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.IOpenBinder#makeBindingContext()
	 */
	public IBindingContext makeBindingContext() {
		throw new UnsupportedOperationException("XlsBinder is top level Binder");
	}

	static Map binderFactory;

	static synchronized public Map binderFactory() {
		if (binderFactory == null) {
			binderFactory = new HashMap();

			for (int i = 0; i < binders.length; i++) {
				try {
					binderFactory.put(binders[i][0], Class.forName(
							binders[i][1]).newInstance());
				} catch (Exception ex) {
					throw RuntimeExceptionWrapper.wrap(ex);
				}
			}
		}
		return binderFactory;
	}

	static final String[][] binders = {
			{ XLS_DATA, DataNodeBinder.class.getName() },
			{ XLS_VARIABLE, VariableNodeBinder.class.getName() }, // JF
			{ XLS_DATATYPE, DatatypeNodeBinder.class.getName() },
			{ XLS_DT, DTNodeBinder.class.getName() },
			{ XLS_DT_PROXY, ProxyDTNodeBinder.class.getName() },
			{ XLS_DT_EXTERNAL, ExternalDTNodeBinder.class.getName() },
			{ XLS_METHOD, MethodTableNodeBinder.class.getName() }, };

}
