package com.openrules.lang.xls;

import org.apache.commons.lang.StringUtils;
import org.openl.syntax.impl.IdentifierNode;

public class XlsHelpers {

	public static boolean isProxyTable(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 4 )
			return false;

		String header = parsedHeader[0].getIdentifier();
		String proxyLiteral = parsedHeader[2].getIdentifier();
		boolean proxy =  
			( IXlsTableNames.RULES.equals(header) ||
			  IXlsTableNames.RULE_SEQUENCE.equals(header) ||
			  IXlsTableNames.RULES_EXTERNAL.equals(header) ) 
			&& ( "template".equals(proxyLiteral) ||
				 "extends".equals(proxyLiteral) || 
				 "extend".equals(proxyLiteral) || 
				 "implements".equals(proxyLiteral) || 
				 "implement".equals(proxyLiteral) 
			    ) ;
		return proxy;
	}

	// DecisionTable <name>
	public static boolean isDecisionTable(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return IXlsTableNames.DECISION_TABLE.equals(header) || "DT".equals(header) 
				|| "RuleFamily".equals(header) || "DecisionTableSingleHit".equals(header); 
	}
	
	// DecisionTable1 <name>
	public static boolean isDecisionTable1(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return IXlsTableNames.DECISION_TABLE1.equals(header) || "DT1".equals(header) 
				|| "RuleFamily1".equals(header) || "DecisionTableMultiHit".equals(header);
	}
	
	// DecisionTable2 <name>
	public static boolean isDecisionTable2(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return IXlsTableNames.DECISION_TABLE2.equals(header) || "DT2".equals(header) 
				 || "DecisionTableSequence".equals(header) || "RuleFamily2".equals(header); 
	}
	
	// DecisionTable<Custom> <name>
	public static boolean isDecisionTableCustom(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return header.startsWith(IXlsTableNames.DECISION_TABLE); 
	}
	
	// ImportedDecisions importedDecisions
	public static boolean isDecisionImport(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return IXlsTableNames.DECISION_IMPORT.equals(header); 
	}
	
	// Glossary <name>
	public static boolean isGlossary(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return IXlsTableNames.GLOSSARY.equals(header); 
	}
	
	// Decision <name>
	public static boolean isDecisionModel(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return IXlsTableNames.DECISION_MODEL.equals(header) ||
			   IXlsTableNames.DECISION.equals(header); 
	}
	
	// DecisionObject <name>
	public static boolean isDecisionObject(IdentifierNode[] parsedHeader) {
		if ( parsedHeader.length != 2 )
			return false;
		String header = parsedHeader[0].getIdentifier();
		return IXlsTableNames.DECISION_OBJECT.equals(header.trim()); 
	}
}
