/*
 * Created on Sep 23, 2003
 *
 * Developed by Intelligent ChoicePoint Inc./OpenRules, Inc. 2003-2006
 * Modified by IK on Aug-Sep 2006: added Import_JAVA, IMPORT_STATIC, IMPORT_SCHEMA
 * Modified by JF on 4/19/2005: added XLS_VARIABLE for future use
 * JF on 08/17/2005: introduced the default language "openrules.java" that
 * will be used that there are no language property defined in any Env table
 * JF on 08/05/2006: default language is "com.openrules.lang.xls"
 * Ignore language define in the Environment tables: always use the default
 * Modified by IK on 08/12/2006: add support for imports directly
 * from xls Environment tables. Now XlsLoader creates OpenLConfiguration based
 * on the Environment tables
 *
 */

package com.openrules.lang.xls;

import java.io.File;
import java.io.FilenameFilter;
import java.io.InputStream;
import java.io.Reader;
import java.net.MalformedURLException;
import java.net.URI;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Vector;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.openl.IOpenSourceCodeModule;
import org.openl.OpenConfigurationException;
import org.openl.OpenL;
import org.openl.conf.ConfigurableResourceContext;
import org.openl.conf.IConfigurableResourceContext;
import org.openl.conf.IOpenLConfiguration;
import org.openl.conf.IUserContext;
import org.openl.conf.JavaImportTypeConfiguration;
import org.openl.conf.JavaImportTypeConfiguration.StringHolder;
import org.openl.conf.JavaLibraryConfiguration;
import org.openl.conf.LibraryFactoryConfiguration;
import org.openl.conf.NameSpacedLibraryConfiguration;
import org.openl.conf.NameSpacedTypeConfiguration;
import org.openl.conf.OpenLConfiguration;
import org.openl.conf.OpenSchemaConfiguration;
import org.openl.conf.TypeFactoryConfiguration;
import org.openl.syntax.IParsedCode;
import org.openl.syntax.ISyntaxError;
import org.openl.syntax.ISyntaxNode;
import org.openl.syntax.impl.IdentifierNode;
import org.openl.syntax.impl.ParsedCode;
import org.openl.syntax.impl.SyntaxError;
import org.openl.syntax.impl.TokenizerParser;
import org.openl.util.Log;
import org.openl.util.MessageException;
import org.openl.util.RuntimeExceptionWrapper;
import org.openl.util.StringTool;

import com.openrules.datasource.DataSourceCodeModule;
import com.openrules.datasource.DataSourceFactory;
import com.openrules.datasource.IDataSource;
import com.openrules.datasource.IDataSourceFactory;
import com.openrules.datasource.IDataSourceProtocolHandler;
import com.openrules.datasource.Utils;
import com.openrules.lang.xls.syntax.ExternalDTSyntaxNode;
import com.openrules.lang.xls.syntax.XlsModuleSyntaxNode;
import com.openrules.lang.xls.syntax.XlsOpenlSyntaxNode;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;
import com.openrules.table.GridSplitter;
import com.openrules.table.IGridTable;
import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.external.ExtraDataSourceCodeModule;
import com.openrules.table.external.IRuleTable;
import com.openrules.table.external.InvalidStructureException;
import com.openrules.table.external.RuleProvider;
import com.openrules.table.external.RuleTable;
import com.openrules.table.openl.GridCellSourceCodeModule;
import com.openrules.table.syntax.GridLocation;
import com.openrules.table.syntax.XlsURLConstants;
import com.openrules.table.xls.XlsSheetGridModel;

/**
 * @author snshor
 * @author IK
 * 
 *         IK: on Sep. 30, 2006 added
 *         OpenLConfiguration.unregister(source.getUri(0), ucxt); to avoid
 *         "already exists" error when the same main-xls is loaded again after
 *         modifications
 * 
 *         IK: Feb-2009 added support for external rule tables defined in Java
 *         JF: Dec-2010 added trim() for the keywords inside Environment table
 *         and produce warnings for unknown keywords
 *         Apr-2011: JF added option "RuleSet" to process DT row-by-row 
 *                   (not all conditions being evaluated first)
 */
public class XlsLoader implements IXlsTableNames, IXlsNodeTypes {

	private static final String DEFAULT_RULES_LANGUAGE = "openrules.java"; // IK
	private static final String OPENRULES_PROTOCOL_NAME = "openrules.protocol.name"; // JF
	private static final String OPENRULES_PROTOCOL_CLASS = "openrules.protocol.class"; // JF

	Vector nodesVector = new Vector();

	Vector errors = new Vector();

	XlsOpenlSyntaxNode openl = null;

	// IConfigurableResourceContext cxt;

	OpenLConfiguration conf; // IK

	// IUserContext ucxt;

	String searchPath;

	IDataSourceFactory dataSourceFactory;

	/**
	 * @param cxt
	 * @param ucxt
	 * @param string
	 */
	public XlsLoader(IConfigurableResourceContext cxt, IUserContext ucxt,
			String searchPath) {
		// this.ucxt = ucxt;
		// this.cxt = cxt;

		if (ucxt != null && "db".equals(ucxt.getProtocol()))
			this.dataSourceFactory = new DataSourceFactory(ucxt.getUserProperties(),cxt, ucxt); // JF
		else	
			this.dataSourceFactory = new DataSourceFactory(cxt, ucxt);

		this.searchPath = searchPath;

		this.conf = new OpenLConfiguration();
		conf.setConfigurationContext(new ConfigurableResourceContext(cxt
				.getClassLoader(), conf));
	}

	void setOpenl(XlsOpenlSyntaxNode openl) {
		if (this.openl == null)
			this.openl = openl;
		else
			addError(new SyntaxError(openl,
					"Only one openl statement is allowed", null));
	}

	HashSet preprocessedWorkBooks = new HashSet();

	void preprocessWorkbook(IOpenSourceCodeModule source) {
		String uri = source.getUri(0);
		if (preprocessedWorkBooks.contains(uri))
			return;
		preprocessedWorkBooks.add(uri);

		InputStream is = null;
		try {
			is = source.getByteStream();
			//POIFSFileSystem fs = new POIFSFileSystem(is);
			Workbook wb = null;
			try {
	            wb = WorkbookFactory.create(is);
	        } catch (Exception e) {
	        	String msg = "Cannot open workbook " + uri;
	            Log.error(msg, e);
	            throw e;
	        }
//			Workbook wb;
//		    int mid = uri.lastIndexOf(".");
//		    String ext = uri.substring(mid+1,uri.length());
//		    		    
//		    if ("xls".equals(ext)) {
//		    	POIFSFileSystem fs = new POIFSFileSystem(is);
//		    	//wb = new HSSFWorkbook(fs);
//		    	wb = WorkbookFactory.create(fs);
//		    }
//		    else { // xlsx 
//		    	//wb = new XSSFWorkbook(is);
//		    	//wb = WorkbookFactory.create(OPCPackage.open(is));
//		    	wb = WorkbookFactory.create(is);
//		    }
			
			// generateExtraSheets(source, wb);

			int nsheets = wb.getNumberOfSheets();

			for (int i = 0; i < nsheets; i++) {
				Sheet sheet = wb.getSheetAt(i);
				String sheetName = wb.getSheetName(i);

				XlsSheetSourceCodeModule sheetSource = new XlsSheetSourceCodeModule(
						sheetName, source);

				XlsSheetGridModel xlsGrid = new XlsSheetGridModel(sheet,
						sheetName, sheetSource);

				IGridTable[] tables = new GridSplitter(xlsGrid).split();

				for (int j = 0; j < tables.length; j++) {
					preprocessTable(tables[j], sheetSource);
				}
			}

			processExternal(source, wb);

		} catch (Exception e) {
			throw RuntimeExceptionWrapper.wrap(e);
		} finally {
			try {
				if (is != null)
					is.close();
			} catch (Throwable e) {
				Log.error("Error trying close input stream:", e);
			}
		}

	}

	private void processExternal(IOpenSourceCodeModule source, Workbook wb) {
		if (source instanceof ExtraDataSourceCodeModule) {
			Object[] data = ((ExtraDataSourceCodeModule) source).getExtraData();
			if (data != null)
				for (int i = 0; i < data.length; ++i) {
					if (data[i] instanceof RuleTable) {
						IRuleTable ruleTable = (IRuleTable) data[i];

						// lookup for a possible forward declaration and remove
						// it if found
						for (Iterator it = nodesVector.iterator(); it.hasNext();) {
							Object node = it.next();
							if (node instanceof ExternalDTSyntaxNode) {
								IRuleTable rt = ((ExternalDTSyntaxNode) node)
										.getRuleTable();
								if (rt.getTableName().equals(
										ruleTable.getTableName())) {
									it.remove();
									break;
								}
							}
						}

						addNode(new ExternalDTSyntaxNode(ruleTable, source));
					}
				}
		}
	}

	IParsedCode parse(IOpenSourceCodeModule source) {

		preprocessWorkbook(source);

		IUserContext ucxt = dataSourceFactory.getUserContext();

		// make sure that the core configuration exists. if not try to build
		OpenL.getInstance(DEFAULT_RULES_LANGUAGE, ucxt);

		// build a new language configuratioon based on the core configuration
		// and this xls environment settings
		IOpenLConfiguration base = OpenLConfiguration.getInstance(
				DEFAULT_RULES_LANGUAGE, ucxt);
		conf.setParent(base);

		try {
			conf.validate(conf.getConfigurationContext());
		} catch (OpenConfigurationException e) {
			addError(new SyntaxError(null, "Configuration Error", e, source));
		}

		// IK
		OpenLConfiguration.unregister(source.getUri(0), ucxt);
		OpenLConfiguration.register(source.getUri(0), ucxt, conf, true);

		if (openl == null) {
			// Log.info("No language has been defined. The default language is
			// openrules.java");
			// addError(new SyntaxError((ILocation) null, "No openl has been
			// defined",
			// null, source));
			// setOpenl(new XlsOpenlSyntaxNode("openrules.java", null, source));
			setOpenl(new XlsOpenlSyntaxNode(source.getUri(0), null, source));
		}

		ISyntaxNode[] nodes = (ISyntaxNode[]) nodesVector
				.toArray(new ISyntaxNode[0]);
		return new ParsedCode(new XlsModuleSyntaxNode(nodes, source, openl),
				source, (ISyntaxError[]) errors.toArray(new ISyntaxError[0]));
	}

	static class ParsedXls extends ParsedCode {

		/**
		 * @param topnode
		 * @param source
		 * @param syntaxErrors
		 */
		public ParsedXls(ISyntaxNode topnode, IOpenSourceCodeModule source,
				ISyntaxError[] syntaxErrors) {
			super(topnode, source, syntaxErrors);
		}

	}

	static Map tableHeaders;

	static final String[][] headerMapping = { 
		    //{ OLD_RULES, XLS_DT },
			{ RULES, XLS_DT },
			{ RULE_SEQUENCE, XLS_DT }, // JF
			{ RULES_PROXY, XLS_DT_PROXY },
			// { RULES_EXTERNAL, XLS_DT_EXTERNAL },
			{ DATA_TABLE, XLS_DATA },
			{ VARIABLE_TABLE, XLS_VARIABLE }, // JF
			{ DATATYPE_TABLE, XLS_DATATYPE }, 
			{ METHOD_TABLE, XLS_METHOD },
			{ METHOD_TABLE2, XLS_METHOD } };

	private static final String DEFAULT_OPENRULES_NAMESPACE = "org.openl.this";

	private static final String DEFAULT_XML_SCHEMA_FACTORY = "dom.factory";

	static public Map tableHeaders() {
		if (tableHeaders == null) {
			tableHeaders = new HashMap();
			// System.out.println("HEADERS:");
			for (int i = 0; i < headerMapping.length; i++) {
				tableHeaders.put(headerMapping[i][0], headerMapping[i][1]);
				// System.out.println(headerMapping[i][0] + ": " +
				// headerMapping[i][1]);
			}
		}
		return tableHeaders;
	}

	void preprocessTable(IGridTable table, XlsSheetSourceCodeModule source) {
		// String header = table.getStringValue(0, 0);

		IOpenSourceCodeModule src = new GridCellSourceCodeModule(table);

		IdentifierNode[] parsedHeader = TokenizerParser.tokenize(src, " \n\r");

		if (parsedHeader.length == 0)
			return;
				
		String header = null;
		// non standard "proxy rulesInMemory"
		if (XlsHelpers.isDecisionTable(parsedHeader)) {
			// for DecisionTable <name>
			header = RULES_PROXY;
		} else if (XlsHelpers.isDecisionTable1(parsedHeader)) {
			// for DecisionTable1 <name>
			header = RULES_PROXY;
		} else if (XlsHelpers.isDecisionTable2(parsedHeader)) {
			// for DecisionTable2 <name>
			header = RULES_PROXY;
		} else if (XlsHelpers.isDecisionTableCustom(parsedHeader)) {
			// for DecisionTable<Custom> <name>
			header = RULES_PROXY;
		} else if (XlsHelpers.isDecisionImport(parsedHeader)) {
			// for DecisionImport extends DecisionImportTemplate
			header = RULES_PROXY;
		} else if (XlsHelpers.isGlossary(parsedHeader)) {
			// for Rules Glossary extends GlossaryTemplate
			header = RULES_PROXY;
		} else if (XlsHelpers.isDecisionModel(parsedHeader)) {
			// for Rules <name> extends DecisionModelTemplate
			header = RULES_PROXY;
		} else if (XlsHelpers.isDecisionObject(parsedHeader)) {
			// for Rules <name> extends DecisionObjectTemplate
			header = RULES_PROXY;
		} else if (XlsHelpers.isProxyTable(parsedHeader)) {
			// for Rules <name> extends <template>
			header = RULES_PROXY;
		} else
			header = parsedHeader[0].getIdentifier();

		String xls_type = (String) tableHeaders().get(header);
		
		if (xls_type != null) {
			
			// JF - print table
			Log.debug("preprocessTable: " + table.getStringValue(0, 0) + " \t" + Log.timestamp());
			
			addNode(new XlsTableSyntaxNode(xls_type, new GridLocation(table),
					source, table, header));
			return;
		}


		if (header.equals(ENVIRONMENT_TABLE))
			preprocessEnvironmentTable(table, source);

		if (header.equals(RULES_EXTERNAL))
			preprocessExternalDeclarationTable(table, source);

		// else if (header.equals(INCLUDE_TABLE))
		// preprocessIncludeTable(table, source, parsedHeader

	}

	/**
	 * @param table
	 * @param source
	 * @param parsedHeader
	 */
	private void preprocessOpenlTable(IGridTable table,
			XlsSheetSourceCodeModule source) {
		// String openlName =
		// parsedHeader.length >= 2
		// ? parsedHeader[1].getIdentifier()
		// : "UNDEFINED_OPENL_NAME";

		String openlName = table.getStringValue(1, 0);
		/*
		 * Ignore language define in the Environment tables: always use the
		 * default "openrules.java"
		 */
		openlName = "openrules.java";
		setOpenl(new XlsOpenlSyntaxNode(openlName, new GridLocation(table),
				source));
	}

	/**
	 * process forward declaration of external rulesInMemory
	 * 
	 */
	private void preprocessExternalDeclarationTable(IGridTable table,
			XlsSheetSourceCodeModule source) {

		LogicalTable lt = new LogicalTable(table);

		int h = lt.getLogicalHeight();

		for (int i = 1; i < h; i++) {
			ILogicalTable row = lt.getLogicalRow(i);

			String tableName = row.getLogicalColumn(0).getGridTable()
					.getStringValue(0, 0);

			String templateName = row.getLogicalColumn(1).getGridTable()
					.getStringValue(0, 0);

			IRuleTable ruleTable = new RuleTableStub(tableName, templateName);
			addNode(new ExternalDTSyntaxNode(ruleTable, source));

		}

	}

	/*
	 * IK added Import_JAVA, IMPORT_STATIC, IMPORT_SCHEMA
	 */
	private void preprocessEnvironmentTable(IGridTable table,
			XlsSheetSourceCodeModule source) {

		LogicalTable lt = new LogicalTable(table);

		int h = lt.getLogicalHeight();

		for (int i = 1; i < h; i++) {
			ILogicalTable row = lt.getLogicalRow(i);

			String name = row.getLogicalColumn(0).getGridTable()
					.getStringValue(0, 0);
			name = name.trim(); // JF
			if (name.equals(LANG_PROPERTY))
				preprocessOpenlTable(row.getGridTable(), source);
			else if (name.equals(INCLUDE_TABLE))
				preprocessIncludeTable(row.getGridTable(), source);
			else if (name.equals(IMPORT_JAVA))
				preprocessJavaImportTable(row.getGridTable(), source);
			else if (name.equals(IMPORT_STATIC))
				preprocessStaticImportTable(row.getGridTable(), source);
			else if (name.equals(IMPORT_SCHEMA))
				preprocessSchemaImportTable(row.getGridTable(), source);
			else if (name.equals(INCLUDE_PATH))
				preprocessIncludePathTable(row.getGridTable(), source);
			else if (name.equals(DATASOURCE))
				preprocessDataSourceTable(row.getGridTable(), source);
			else {
				Log.warn("UNKNOWN TERM <" + name + "> in the Environment table");
			}
		}

	}

	/*
	 * Takes all cell values for a given import attribute and add s them to \ a
	 * resulting array. Cell value can have multiple values divided by ';'
	 */
	private String[] getStringLines(IGridTable gridTable,
			XlsSheetSourceCodeModule source) {
		int h = gridTable.getLogicalHeight();

		List list = new ArrayList();
		for (int i = 0; i < h; i++) {
			String str = gridTable.getStringValue(1, i);
			if (str == null)
				continue;
			str = str.trim();
			if (str.length() == 0)
				continue;
			String[] tmp = str.split(";");
			for (int j = 0; j < tmp.length; j++)
				list.add(str);
		}
		return (String[]) list.toArray(new String[list.size()]);

	}

	private NameSpacedTypeConfiguration getTypeConfiguration() {
		TypeFactoryConfiguration typeFactory = conf.getTypeFactory();
		if (typeFactory == null) {
			typeFactory = new TypeFactoryConfiguration();
			conf.setTypeFactory(typeFactory);
		}
		NameSpacedTypeConfiguration typeConfig = typeFactory
				.getConfiguredTypeLibrary(DEFAULT_OPENRULES_NAMESPACE);
		if (typeConfig == null) {
			typeConfig = new NameSpacedTypeConfiguration();
			typeConfig.setNamespace(DEFAULT_OPENRULES_NAMESPACE);
			conf.getTypeFactory().addConfiguredTypeLibrary(typeConfig);
		}
		return typeConfig;
	}

	private void preprocessSchemaImportTable(IGridTable gridTable,
			XlsSheetSourceCodeModule source) {
		String[] locations = getStringLines(gridTable, source);
		NameSpacedTypeConfiguration typeConfig = getTypeConfiguration();

		for (int i = 0; i < locations.length; ++i) {
			String location = locations[i];
			Log.info("IMPORT.SCHEMA=" + location);
			OpenSchemaConfiguration schema = new OpenSchemaConfiguration();
			schema.setFactory(DEFAULT_XML_SCHEMA_FACTORY);
			schema.setLocation(location);
			typeConfig.addSchema(schema);
		}

	}

	private void preprocessStaticImportTable(IGridTable gridTable,
			XlsSheetSourceCodeModule source) {
		String[] libClassnames = getStringLines(gridTable, source);

		LibraryFactoryConfiguration libFactory = conf.getMethodFactory();
		if (libFactory == null) {
			libFactory = new LibraryFactoryConfiguration();
			conf.setMethodFactory(libFactory);
		}

		NameSpacedLibraryConfiguration lib = libFactory
				.getConfiguredLibrary(DEFAULT_OPENRULES_NAMESPACE);
		if (lib == null) {
			lib = new NameSpacedLibraryConfiguration();
			lib.setNamespace(DEFAULT_OPENRULES_NAMESPACE);
			libFactory.addConfiguredLibrary(lib);
		}

		for (int i = 0; i < libClassnames.length; ++i) {
			String classname = libClassnames[i];
			Log.info("IMPORT.STATIC=" + classname);
			JavaLibraryConfiguration javalib = new JavaLibraryConfiguration();
			javalib.setClassName(classname);
			lib.addJavalib(javalib);
		}

	}

	private void preprocessJavaImportTable(IGridTable gridTable,
			XlsSheetSourceCodeModule source) {

		String[] imports = getStringLines(gridTable, source);
		NameSpacedTypeConfiguration typeConfig = getTypeConfiguration();

		for (int i = 0; i < imports.length; ++i) {
			String imp = imports[i];
			Log.info("IMPORT.JAVA=" + imp);
			JavaImportTypeConfiguration javaimport = new JavaImportTypeConfiguration();
			int pos = imp.indexOf(".*");
			if (pos > 0) {
				imp = imp.substring(0, pos);
				javaimport.setAll(imp);
			} else {
				StringHolder className = new StringHolder();
				className.addText(imp);
				javaimport.addConfiguredClassName(className);
			}
			typeConfig.addJavaImport(javaimport);
		}
	}

	private void preprocessIncludePathTable(IGridTable gridTable,
			XlsSheetSourceCodeModule source) {
		String[] paths = getStringLines(gridTable, source);
		for (int i = 0; i < paths.length; i++) {
			Log.info("INCLUDE.PATH=" + paths[i]);
			// validation
			// search path has to be an absolute path or a fragment ( no schema
			// prefix )
			try {
				URI p = new URI(paths[i]);
				if (p.isOpaque())
					throw new MessageException(
							"Include path has to be an absolute path or fragment(i.e no schema prefix)");
			} catch (Throwable t) {
				addError(new SyntaxError(null, "Invalid include.path '"
						+ paths[i] + "'", t, new GridCellSourceCodeModule(
						gridTable.getLogicalRegion(1, i, 1, 1).getGridTable())));
				continue;
			}

		}
		if (paths.length > 0)
			this.searchPath = StringUtils.join(paths, ";");

	}

	private void preprocessIncludeTable(IGridTable table,
			XlsSheetSourceCodeModule source) {

		int h = table.getLogicalHeight();

		HashSet processedURIs = new HashSet();
		for (int i = 0; i < h; i++) {
			String pattern = table.getStringValue(1, i);
			if (pattern == null)
				continue;
			pattern = pattern.trim();
			if (pattern.length() == 0)
				continue;
			Log.info("INCLUDE=" + pattern);
			List includes;
			String where = source.workbook.getUri(0);
			try {
				includes = findIncludes(where, pattern);
			} catch (Throwable t) {
				addError(new SyntaxError(null, "Include " + pattern
						+ " not found", t, new GridCellSourceCodeModule(table
						.getLogicalRegion(1, i, 1, 1).getGridTable())));
				throw new RuntimeException("Include " + pattern + " not found");
			}
			if (includes.size() == 0) {
				Log.info("No files with a pattern " + pattern
						+ " have been found");
				continue;
			}
			Iterator it = includes.iterator();
			while (it.hasNext()) {
				IDataSource src = null;
				String includeFile = null;
				try {
					src = (IDataSource) it.next();
					includeFile = src.getURI().toString();
					if (processedURIs.contains(src.getURI()))
						Log.warn("Include " + includeFile + " is already been processed. Ignoring.");
					else {
						Log.info("Processing " + includeFile);
						processedURIs.add(src.getURI());
						DataSourceCodeModule code = new DataSourceCodeModule(
								src);
						preprocessWorkbook(code);
					}
				} catch (Throwable t) {
					String msg = "Failed to process include-file: " + includeFile;
					//String msg = "Failure processing " + pattern;
					addError(new SyntaxError(null, msg, 
							t, new GridCellSourceCodeModule(
							table.getLogicalRegion(1, i, 1, 1).getGridTable())));
					throw new RuntimeException(msg,t);
				}
			}

		}
	}
	
	private void preprocessDataSourceTable(IGridTable table,
			XlsSheetSourceCodeModule source) {
		String[] lines = getStringLines(table, source);
		for (int i = 0; i < lines.length; i++) {
			String url = lines[i].trim();
			Log.info("DataSource {" + url + "}");
			try {
				IDataSource src = this.dataSourceFactory.fromString(url);
				if (src == null) {
					throw new RuntimeException("Property file not found '"
							+ url + "'");
				}
				Properties options = Utils.loadProperties(src);

				String protocolName = options
						.getProperty(OPENRULES_PROTOCOL_NAME);
				if (protocolName == null) {
					protocolName = "db";
					Log.info("Property '"
							+ OPENRULES_PROTOCOL_NAME + "' is not found in '"
							+ url + "'. Use the default name " + protocolName);
				}

				String protocolClass = options
						.getProperty(OPENRULES_PROTOCOL_CLASS);
				if (protocolClass == null) {
					protocolClass = "com.openrules.db.DbDataSourceHandler";
					Log.info("Property '"
							+ OPENRULES_PROTOCOL_CLASS + "' is not found in '"
							+ url + "'. Use the default class " + protocolClass);					
				}

				Class c = Class.forName(protocolClass.trim());
				IDataSourceProtocolHandler dsh = (IDataSourceProtocolHandler) c
						.newInstance();

				// initialization
				dsh.initialize(options);

				// symbol '*' defines default datasource handler
				if ("*".equals(protocolName))
					this.dataSourceFactory.setDefaultHandler(dsh);
				else
					this.dataSourceFactory.register(protocolName, dsh);

			} catch (Throwable t) {
				addError(new SyntaxError(null, "Invalid datasource '"
						+ lines[i] + "'", t, new GridCellSourceCodeModule(table
						.getLogicalRegion(1, i, 1, 1).getGridTable())));
				continue;
			}
		}
	}

	/**
	 * @param include
	 * @return
	 * @throws Exception
	 */
	public List findIncludes(String sourceURI, String includePath)
			throws Exception {
		if (!includePath.startsWith("<"))
			return includesFor(sourceURI, includePath);

		List includes = new ArrayList();

		// lookup by searchPath
		if (searchPath != null) {
			Log.debug("Trying to find " + includePath
					+ " in the include.path [" + searchPath + "]");
			String include = StringTool.openBrackets(includePath, '<', '>', "")[0];

			String[] path = StringTool.tokenize(searchPath, ";");
			for (int i = 0; i < path.length; i++) {
				// if search path is not absolute it will be resolved
				// against sourceURI
				URI parent = dataSourceFactory.fromString(sourceURI, path[i])
						.getURI();
				String parentURI = parent.toASCIIString();
				IDataSource ds = dataSourceFactory.fromString(parentURI,
						include);
				if (ds != null) {
					includes.addAll(includesFor(ds.getURI()));
				}
			}
		}
		return includes;

	}

	private List includesFor(String sourceURI, String includePath)
			throws MalformedURLException {
		Log.debug("Serching for " + includePath + " in " + sourceURI);
		IDataSource ds = this.dataSourceFactory.fromString(sourceURI,
				includePath);
		if (ds == null)
			return Collections.EMPTY_LIST;
		return includesFor(ds.getURI());
	}

	private List includesFor(URI targetURI) throws MalformedURLException {
		List includes = new ArrayList();

		// is it file pattern?
		String scheme = targetURI.getScheme();
		if (null == scheme || "file".equals(scheme)) {
			// file
			String path = targetURI.getPath();
			File f = new File(path);
			String fileName = f.getName();
			// is it file pattern?
			if (fileName.indexOf("*") != -1 || 
				fileName.indexOf("?") != -1 ) {
				String pattern = fileName.replaceAll("\\.", "\\.")
						.replaceAll("\\*", "\\.\\*")
						.replaceAll("\\?", "[^\\.]");

				final Pattern filePattern = Pattern.compile(pattern);

				File[] files = f.getParentFile().listFiles(
						new FilenameFilter() {
							public boolean accept(File dir, String name) {
								return filePattern.matcher(name).matches();
							}
						});

				for (int i = 0; i < files.length; ++i) {
					IDataSource ds = this.dataSourceFactory.fromString(files[i]
							.toURI().toString());
					if (ds != null)
						includes.add(ds);
				}
				return includes;
			}
		}

		// not a file pattern
		IDataSource ds = this.dataSourceFactory
				.fromString(targetURI.toString());
		if (ds != null)
			includes.add(ds);

		return includes;
	}

	void addNode(ISyntaxNode node) {
		nodesVector.add(node);
	}

	void addError(ISyntaxError error) {
		errors.add(error);
	}

	public static class XlsSheetSourceCodeModule implements
			IOpenSourceCodeModule {
		String sheetName;

		IOpenSourceCodeModule workbook;

		public XlsSheetSourceCodeModule(String sheetName,
				IOpenSourceCodeModule workbook) {
			this.sheetName = sheetName;
			this.workbook = workbook;
		}

		public InputStream getByteStream() {
			throw new UnsupportedOperationException();
		}

		public Reader getCharacterStream() {
			throw new UnsupportedOperationException();
		}

		public String getCode() {
			throw new UnsupportedOperationException();
		}

		public int getTabSize() {
			throw new UnsupportedOperationException();
		}

		public String getUri(int textpos) {
			return workbook.getUri(0) + "?" + XlsURLConstants.SHEET + "="
					+ sheetName;
		}

		public int getStartPosition() {
			return 0;
		}

	}

	public static class RuleTableStub extends RuleTable {
		public RuleTableStub(String name, String templateName) {
			super(name, templateName, new RuleProvider(null) {
				public void validateStructure()
						throws InvalidStructureException {
					// nothing to do
				};
			});
		}
	}

}