/*
 * Created on Oct 3, 2003
 * 
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.lang.xls.binding;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.impl.BoundError;
import org.openl.syntax.ISyntaxNode;
import org.openl.syntax.impl.ISyntaxConstants;
import org.openl.syntax.impl.IdentifierNode;
import org.openl.syntax.impl.TokenizerParser;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenField;
import org.openl.types.impl.AOpenField;
import org.openl.util.Log;
import org.openl.vm.IRuntimeEnv;
import com.openrules.lang.xls.IXlsTableNames;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;

import com.openrules.data.IColumnDescriptor;
import com.openrules.data.IString2DataConvertor;
import com.openrules.data.ITable;
import com.openrules.data.String2DataConvertorFactory;
import com.openrules.data.impl.OpenlBasedColumnDescriptor;
import com.openrules.data.impl.OpenlBasedDataTableModel;
import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

/**
 * @author snshor
 *  
 */
public class DataNodeBinder extends AXlsTableBinder implements IXlsTableNames
{

	static final int TYPE_INDEX = 1, TABLE_NAME_INDEX = 2;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.lang.xls.binding.AXlsTableBinder#preBind(org.openl.syntax.ISyntaxNode,
	 *      org.openl.OpenL, org.openl.binding.IBindingContext)
	 */
	public IMemberBoundNode preBind(ISyntaxNode syntaxNode, OpenL openl,
			IBindingContext cxt, XlsModuleOpenClass module) throws Exception
	{

		XlsTableSyntaxNode xlsDataNode = (XlsTableSyntaxNode) syntaxNode;

		ILogicalTable table = new LogicalTable(xlsDataNode.getTable());
		
		IOpenSourceCodeModule src = new GridCellSourceCodeModule(table
				.getGridTable());

		IdentifierNode[] parsedHeader = TokenizerParser.tokenize(src, " \n\r");

		String errMsg;

		if (parsedHeader.length < 3)
		{
			errMsg = "Data table format: Data <typename> <tablename>";
			BoundError err = new BoundError(null, errMsg, null, src);
			throw err;
		}

		String typeName = parsedHeader[TYPE_INDEX].getIdentifier();

		String tableName = parsedHeader[TABLE_NAME_INDEX].getIdentifier();
		
		IOpenClass tableType = cxt.findType(ISyntaxConstants.THIS_NAMESPACE,
				typeName);

		if (tableType == null)
		{
			errMsg = "Type not found: " + typeName;

			BoundError err = new BoundError(parsedHeader[TYPE_INDEX], errMsg,
					null);
			throw err;
		}

		//		if (table.getLogicalHeight() == 1)
		//			table = table.transpose();

		//		if (table.getLogicalHeight() < 4)
		//		{
		//			errMsg = "Data table structure must be more than 3 rows";
		//			BoundError err =
		//				new BoundError(
		//					null,
		//					errMsg,
		//					null,
		//					new GridTableSourceCodeModule(table.getGridTable()));
		//			throw err;
		//		}

		ITable dataTable = makeTable(module, table, tableName, tableType, cxt,
				openl);

		return new DataTableMethodBoundNode(xlsDataNode, dataTable, module);
	}

	ITable makeTable(XlsModuleOpenClass xlsOpenClass, ILogicalTable table,
			String tableName, IOpenClass tableType, IBindingContext cxt,
			OpenL openl) throws Exception
	{
		ILogicalTable dataTable = findOrientation(table, tableType);
		
		Log.debug("DataNodeBinder: makeTable " + tableName 
				+ " " + dataTable.getLogicalHeight() 
				+ "x" + dataTable.getLogicalWidth()
				+ " - " + Log.timestamp());

		boolean hasIndexRow = checkIndexRow(dataTable);

		ILogicalTable descriptorRows = dataTable.rows(0, hasIndexRow ? 1 : 0);

		IColumnDescriptor[] descriptors = makeDescriptors(descriptorRows, cxt,
				tableType, openl, hasIndexRow);

		OpenlBasedDataTableModel dataModel = new OpenlBasedDataTableModel(
				tableName, tableType, openl, descriptors);

		ITable t = xlsOpenClass.getDataBase().addTable(dataModel,
				dataTable.rows(hasIndexRow ? 2 : 1));
		//Log.debug("made table of size: " + t.getSize() + " - " + Log.timestamp());
		return t;
	}

	/**
	 * @param dataTable
	 * @return
	 */
	private boolean checkIndexRow(ILogicalTable dataTable)
	{
		ILogicalTable indexRow = dataTable.rows(1, 1);

		int w = indexRow.getLogicalWidth();

		for (int i = 0; i < w; i++)
		{
			ILogicalTable cell = indexRow.getLogicalColumn(i);

			String res = cell.getGridTable().getStringValue(0, 0);

			if (res == null || res.trim().length() == 0)
				continue;
			return res.charAt(0) == '>';
		}

		return false;

	}

	/**
	 * @param table
	 * @param tableType
	 * @return
	 */
	protected ILogicalTable findOrientation(ILogicalTable table,
			IOpenClass tableType)
	{
		ILogicalTable dataTable = table.rows(1);

		//TODO optimize
		int cnt1 = countFields(dataTable, tableType);

		int cnt2 = countFields(dataTable.transpose(), tableType);

		return cnt1 < cnt2 ? dataTable.transpose() : dataTable;
	}

	/**
	 * @param dataTable
	 * @param tableType
	 * @return
	 */
	protected int countFields(ILogicalTable dataTable, IOpenClass tableType)
	{
		int cnt = 0;
		int w = dataTable.getLogicalWidth();
		for (int i = 0; i < w; ++i)
		{
			String fieldName = dataTable.getLogicalColumn(i).getGridTable()
					.getStringValue(0, 0);
			if (fieldName == null)
				continue;
			IOpenField of = tableType.getField(fieldName);
			if (of != null)
				++cnt;
		}
		return cnt;
	}

	IColumnDescriptor[] makeDescriptors(ILogicalTable descriptorRows,
			IBindingContext cxt, IOpenClass type, OpenL openl,
			boolean hasIndexRow) throws Exception
	{

		int width = descriptorRows.getLogicalWidth();
		IColumnDescriptor[] dd = new IColumnDescriptor[width];

		for (int i = 0; i < width; i++)
		{

			GridCellSourceCodeModule src = new GridCellSourceCodeModule(
					descriptorRows.getLogicalColumn(i).getGridTable());

			String s = src.getCode();

			if (s.length() == 0)
			{
				continue;
			}
			
			IdentifierNode[] parsedFields = TokenizerParser.tokenize(src, ". \n\r");
			
			IOpenField[] fields = new IOpenField[parsedFields.length];
			
			
			IOpenClass targetType = type;
			for (int j = 0; j < fields.length; j++)
			{
				String fieldName =  parsedFields[j].getIdentifier();
				IOpenField field = targetType.getField(fieldName);

				if (field == null)
				{
					BoundError err = new BoundError(parsedFields[j], "Field " + fieldName
							+ " not found in " + targetType.getName());
					throw err;
				}
				
				targetType = field.getType();
				fields[j] = field;
			}
			

			IOpenField field = fields.length == 1 ? fields[0] : new FieldChain("composite_field_chain", type, fields);			
			
			
			
			IdentifierNode indexTable = null;
			IdentifierNode indexKey = null;

			if (hasIndexRow)
			{

				src = new GridCellSourceCodeModule(descriptorRows
						.getLogicalRegion(i, 1, 1, 1).getGridTable());

				//      if (parsedCode.length < 1)
				//      {
				//        BoundError err = new BoundError(
				//            null,
				//            "Column descriptor format: <columnname> [foreign_table
				// [foreign_key]]",
				//            null, src);
				//        throw err;
				//
				//      }

				IdentifierNode[] parsedCode = TokenizerParser.tokenize(src,
						" >\n\r");

				indexTable = parsedCode.length > 0
						? parsedCode[0]
						: null;
				indexKey = parsedCode.length > 1
						? parsedCode[1]
						: null;

			}

			IString2DataConvertor conv = null;

			if (indexTable == null)
			{
				Class cc = field.getType().getInstanceClass();
				if (cc.isArray())
					cc = cc.getComponentType();

				try
				{
					conv = String2DataConvertorFactory.getConvertor(cc);
				} catch (Throwable t)
				{
					throw new BoundError(null, null, t, src);
				}

			}

			dd[i] = new OpenlBasedColumnDescriptor(field, indexTable, indexKey,
					conv, openl);

		}
		return dd;

	}
	
	
	
	static class FieldChain extends AOpenField implements IOpenField 
	{
		
		IOpenField[] fields; 
		
			/**
		 * @param name
		 * @param type
		 */
		public FieldChain(String name, IOpenClass type, IOpenField[] fields)
		{
			super(name, type);
			this.fields = fields;
		}
		
		
		
		

		/**
		 *
		 */

		public Object get(Object target, IRuntimeEnv env)
		{
			throw new UnsupportedOperationException();
		}

		/**
		 *
		 */

		public void set(Object target, Object value, IRuntimeEnv env)
		{
			//find last target, make if necessary
			for (int i = 0; i < fields.length - 1; i++)
			{
				Object newTarget = 	fields[i].get(target, env);
				if (newTarget == null)
				{
					newTarget = fields[i].getType().newInstance(env);
					fields[i].set(target, newTarget, env);
				}
				target = newTarget;
			}
			
			
			fields[fields.length - 1].set(target, value, env);
			
		}
		
		

		/**
		 *
		 */

		public IOpenClass getDeclaringClass()
		{
			return fields[0].getDeclaringClass();
		}

		/**
		 *
		 */

		public IOpenClass getType()
		{
			return fields[fields.length - 1].getType();
		}

}
	

}