/*
 * Created on Oct 3, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.lang.xls.binding;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.impl.BoundError;
import org.openl.binding.impl.module.ModuleOpenClass;
import org.openl.syntax.ISyntaxNode;
import org.openl.syntax.impl.ISyntaxConstants;
import org.openl.syntax.impl.IdentifierNode;
import org.openl.syntax.impl.TokenizerParser;
import com.openrules.lang.xls.IXlsTableNames;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;

import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

/**
 * @author snshor
 *
 */
public class DatatypeNodeBinder
	extends AXlsTableBinder
	implements IXlsTableNames
{

	static final int TYPE_INDEX = 1, TABLE_NAME_INDEX = 2;

	/* (non-Javadoc)
	 * @see com.openrules.lang.xls.binding.AXlsTableBinder#preBind(org.openl.syntax.ISyntaxNode, org.openl.OpenL, org.openl.binding.IBindingContext)
	 */
	public IMemberBoundNode preBind(
		ISyntaxNode syntaxNode,
		OpenL openl,
		IBindingContext cxt,
		XlsModuleOpenClass module)
		throws Exception
	{

		XlsTableSyntaxNode xlsDataNode = (XlsTableSyntaxNode) syntaxNode;

		ILogicalTable table = new LogicalTable(xlsDataNode.getTable());

		IOpenSourceCodeModule src =
			new GridCellSourceCodeModule(table.getGridTable());

		IdentifierNode[] parsedHeader = TokenizerParser.tokenize(src, " \n\r");

		String errMsg;

		if (parsedHeader.length < 2)
		{
			errMsg = "Datatype table format: Datatype <typename> [tablename]";
			BoundError err = new BoundError(null, errMsg, null, src);
			throw err;
		}

		String typeName = parsedHeader[TYPE_INDEX].getIdentifier();

		String tableName =
			parsedHeader.length > 2
				? parsedHeader[TABLE_NAME_INDEX].getIdentifier()
				: null;

		if (cxt.findType(ISyntaxConstants.THIS_NAMESPACE, typeName) != null)
		{
			errMsg = "Duplicated Type Definition: " + typeName;

			BoundError err =
				new BoundError(parsedHeader[TYPE_INDEX], errMsg, null);
			throw err;
		}

		ModuleOpenClass tableType =
			new ModuleOpenClass(module.getSchema(), typeName);

		cxt.addType(ISyntaxConstants.THIS_NAMESPACE, tableType);

		//		if (table.getLogicalHeight() == 1)
		//			table = table.transpose();

		//		if (table.getLogicalHeight() != 2)
		//		{
		//			errMsg = "Datatype table must have 2 rows";
		//			BoundError err =
		//				new BoundError(
		//					null,
		//					errMsg,
		//					null,
		//					new GridTableSourceCodeModule(table.getGridTable()));
		//			throw err;
		//		}

		return new DatatypeTableMethodBoundNode(
			xlsDataNode,
			(ModuleOpenClass) tableType,
			table,
			openl);
	}

}
