/*
 * Created on Oct 7, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.lang.xls.binding;

import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.IBoundNode;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.OpenLRuntimeException;
import org.openl.binding.impl.ABoundNode;
import org.openl.binding.impl.module.ModuleOpenClass;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.java.JavaOpenClass;
import org.openl.util.Log;
import org.openl.vm.IRuntimeEnv;

import com.openrules.dt.DTLoader;
import com.openrules.dt.DecisionTable;
import com.openrules.dt.InvocationStrategy;
import com.openrules.lang.xls.IXlsTableNames;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;
import com.openrules.table.IGridTable;
import com.openrules.table.LogicalTable;

/**
 * @author snshor
 * 
 * May,2007: JF replaced one DecisionTable to DecisionTableMultiHit and
 * DecisionTableSingleHit - see method "addTo"
 * Apr-2011: JF added option "RuleSequence" to process DT row-by-row 
 *           (not all conditions being evaluated first)
 * 
 */
public class DecisionTableMethodBoundNode extends ABoundNode implements
		IMemberBoundNode {

	OpenL openl;
	IOpenMethodHeader header;

	DecisionTable method;

	ModuleOpenClass module;

	/**
	 * @param syntaxNode
	 * @param children
	 */
	public DecisionTableMethodBoundNode(XlsTableSyntaxNode dtNode, OpenL openl,
			IOpenMethodHeader header, ModuleOpenClass module) {
		super(dtNode, IBoundNode.EMPTY);
		this.header = header;
		this.openl = openl;
		this.module = module;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IMemberBoundNode#addTo(org.openl.binding.impl.module.ModuleOpenClass)
	 */
	public void addTo(ModuleOpenClass openClass) {
		// openClass.addMethod(method = new DecisionTableMultiHit(header)); //
		// JF

		method = new DecisionTable(header);
		String keyword = ((XlsTableSyntaxNode)syntaxNode).getHeader();

		if (header.getType() == JavaOpenClass.VOID) {
			method.setInvocationStrategy(InvocationStrategy.MULTI_HIT);
		}
		else
			method.setInvocationStrategy(InvocationStrategy.SINGLE_HIT);
		
		if (IXlsTableNames.RULE_SEQUENCE.equals(keyword))
			method.setCalculateConditionsFirst(false); // Rules are evaluated row by row (previous actions may matter)
		
		if (IXlsTableNames.DECISION_TABLE.equals(keyword) || "DT".equals(keyword) || "RuleFamily".equals(keyword) || "DecisionTableSingleHit".equals(keyword) ||
			IXlsTableNames.DECISION_TABLE1.equals(keyword) || "DT1".equals(keyword) || "RuleFamily1".equals(keyword) || "DecisionTableMultiHit".equals(keyword) ||
			IXlsTableNames.DECISION_TABLE2.equals(keyword) || "DT2".equals(keyword) || "RuleFamily2".equals(keyword) || "DecisionTableSequence".equals(keyword) ||
			IXlsTableNames.DECISION_TABLE_TEMPLATE.equals(keyword) ||
			IXlsTableNames.DECISION_TABLE1_TEMPLATE.equals(keyword) ||
			IXlsTableNames.DECISION_TABLE2_TEMPLATE.equals(keyword) ||
			IXlsTableNames.RULE_SEQUENCE.equals(keyword) ||
			keyword.startsWith(IXlsTableNames.DECISION_TABLE)
			)
			method.setDirection(IXlsTableNames.DIRECTION_VERTICAL);

		openClass.addMethod(method);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IMemberBoundNode#finalizeBind(org.openl.binding.IBindingContext)
	 */
	public void finalizeBind(IBindingContext cxt) throws Exception {

		XlsTableSyntaxNode dtNode = (XlsTableSyntaxNode) syntaxNode;
		
		//Log.debug("finalizeBind for " + dtNode.getHeader() + " - " + Log.timestamp());

		IGridTable table = dtNode.getTable();
		
		new DTLoader().load(new LogicalTable(table), method, openl, module,	(IBindingContextDelegator) cxt);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IBoundNode#evaluateRuntime(org.openl.vm.IRuntimeEnv)
	 */
	public Object evaluateRuntime(IRuntimeEnv env) throws OpenLRuntimeException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IBoundNode#getType()
	 */
	public IOpenClass getType() {
		return header.getType();
	}

}
