/*
 * Copyright (C) 2009 OpenRules, Inc.
 * 
 */


package com.openrules.lang.xls.binding;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.impl.BoundError;
import org.openl.syntax.ISyntaxNode;
import org.openl.types.IOpenMethod;

import com.openrules.dt.IDecisionTable;
import com.openrules.lang.xls.IXlsTableNames;
import com.openrules.lang.xls.syntax.ExternalDTSyntaxNode;
import com.openrules.table.external.ExternalDataSourceCodeModule;
import com.openrules.table.external.IRuleTable;

public class ExternalDTNodeBinder extends AXlsTableBinder implements IXlsTableNames
{

	private static Pattern namePattern = Pattern.compile("^[_a-zA-Z][_a-zA-Z0-9]*$");

	private String validateName(String name, IOpenSourceCodeModule src) throws BoundError {
		Matcher m = namePattern.matcher( name );
		if ( !m.matches() ) {
			String errMsg = "Invalid method name format: " + name ;
			BoundError err = new BoundError(null, errMsg, null, src);
			throw err;
		}
		return name;
	}

	/* (non-Javadoc)
	 * @see com.openrules.lang.xls.binding.AXlsTableBinder#preBind(org.openl.syntax.ISyntaxNode, org.openl.OpenL, org.openl.binding.IBindingContext)
	 */
	public IMemberBoundNode preBind(
		ISyntaxNode syntaxNode,
		OpenL openl,
		IBindingContext cxt,
		XlsModuleOpenClass module) throws Exception
	{

		ExternalDTSyntaxNode dtNode = (ExternalDTSyntaxNode) syntaxNode;

		IRuleTable ruleTable = dtNode.getRuleTable();
		
		IOpenSourceCodeModule src = new ExternalDataSourceCodeModule( ruleTable );

		String methodName = validateName( ruleTable.getTableName(), src );
		String templateName = validateName( ruleTable.getTemplateName(), src );

		IMemberBoundNode node = new ExternalDataMethodBoundNode(dtNode, openl, module, templateName, methodName);
		IOpenMethod baseMethod = module.getMatchingMethod( ruleTable.getTemplateName(),	null);
		if ( baseMethod == null ) {
			//could not be bound at this time, template is not yet resolved
			return new UnresolvedDependenciesBoundNode(node); 
		}
		return node;
	}


}
