/*
 * Created on Oct 7, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.lang.xls.binding;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenL;
import org.openl.OpenlTool;
import org.openl.binding.IBindingContext;
import org.openl.binding.IBoundNode;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.OpenLRuntimeException;
import org.openl.binding.impl.ABoundNode;
import org.openl.binding.impl.BoundError;
import org.openl.binding.impl.module.ModuleOpenClass;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.impl.CompositeMethod;
import org.openl.vm.IRuntimeEnv;

import com.openrules.dt.SnippetPreprocessor;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;
import com.openrules.table.IGridTable;
import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;
import com.openrules.table.openl.GridTableSourceCodeModule;

/**
 * @author snshor
 *
 */
public class MethodTableBoundNode
	extends ABoundNode
	implements IMemberBoundNode
{

	OpenL openl;
	IOpenMethodHeader header;

	CompositeMethod method;

	ModuleOpenClass module;

	/**
	 * @param syntaxNode
	 * @param children
	 */
	public MethodTableBoundNode(
		XlsTableSyntaxNode methodNode,
		OpenL openl,
		IOpenMethodHeader header,
		ModuleOpenClass module)
	{
		super(methodNode, IBoundNode.EMPTY);
		this.header = header;
		this.openl = openl;
		this.module = module;
	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IMemberBoundNode#addTo(org.openl.binding.impl.module.ModuleOpenClass)
	 */
	public void addTo(ModuleOpenClass openClass)
	{
		openClass.addMethod(method = new CompositeMethod(header, null));
	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IMemberBoundNode#finalizeBind(org.openl.binding.IBindingContext)
	 */
	public void finalizeBind(IBindingContext cxt) throws Exception
	{

		XlsTableSyntaxNode dtNode = (XlsTableSyntaxNode) syntaxNode;

		IGridTable table = dtNode.getTable();

		ILogicalTable lt = new LogicalTable(table);

		if (lt.getLogicalHeight() != 2)
			throw new BoundError(
				null,
				"Method table must have 2 row cells: <header> and <body>",
				null,
				new GridTableSourceCodeModule(table));

		

		IOpenSourceCodeModule src =
			//new GridCellSourceCodeModule(lt.getLogicalRow(1).getGridTable());
			new SnippetPreprocessor(lt.getLogicalRow(1).getGridTable());


		CompositeMethod methodBody =
			OpenlTool.makeMethod(src, openl, header, cxt);
			
			
		method.setMethodBodyBoundNode(methodBody.getMethodBodyBoundNode());		

	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IBoundNode#evaluateRuntime(org.openl.vm.IRuntimeEnv)
	 */
	public Object evaluateRuntime(IRuntimeEnv env) throws OpenLRuntimeException
	{
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see org.openl.binding.IBoundNode#getType()
	 */
	public IOpenClass getType()
	{
		return header.getType();
	}

}
