package com.openrules.lang.xls.binding;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.impl.BoundError;
import org.openl.syntax.ISyntaxNode;
import org.openl.syntax.impl.IdentifierNode;
import org.openl.syntax.impl.TokenizerParser;
import org.openl.types.IOpenMethod;
import org.openl.util.Log;

import com.openrules.lang.xls.IXlsTableNames;
import com.openrules.lang.xls.XlsHelpers;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;
import com.openrules.table.IGridTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

public class ProxyDTNodeBinder extends AXlsTableBinder implements IXlsTableNames
{

	private static Pattern namePattern = Pattern.compile("^[_a-zA-Z][_a-zA-Z0-9]*$");

	private String validateName(String name, IOpenSourceCodeModule src) throws BoundError {
		Matcher m = namePattern.matcher( name );
		if ( !m.matches() ) {
			String errMsg = "Invalid method name format: " + name ;
			BoundError err = new BoundError(null, errMsg, null, src);
			throw err;
		}
		return name;
	}

	/* (non-Javadoc)
	 * @see com.openrules.lang.xls.binding.AXlsTableBinder#preBind(org.openl.syntax.ISyntaxNode, org.openl.OpenL, org.openl.binding.IBindingContext)
	 */
	public IMemberBoundNode preBind(
		ISyntaxNode syntaxNode,
		OpenL openl,
		IBindingContext cxt,
		XlsModuleOpenClass module) throws Exception
	{

		XlsTableSyntaxNode dtNode = (XlsTableSyntaxNode) syntaxNode;

		IGridTable table = dtNode.getTable();

		IOpenSourceCodeModule src = new GridCellSourceCodeModule(table);

		IdentifierNode[] parsedHeader = TokenizerParser.tokenize(src, " \n\r");

		String methodId = parsedHeader[1].getIdentifier(); 
		String templateId = null;
		
		//Log.debug("parsedHeader[0]="+parsedHeader[0]);
		
		if ( XlsHelpers.isDecisionTable(parsedHeader) ) {
			templateId = IXlsTableNames.DECISION_TABLE_TEMPLATE;
		} else if ( XlsHelpers.isDecisionTable1(parsedHeader) ) {
			templateId = IXlsTableNames.DECISION_TABLE1_TEMPLATE;
		} else if ( XlsHelpers.isDecisionTable2(parsedHeader) ) {
			templateId = IXlsTableNames.DECISION_TABLE2_TEMPLATE;
		} else if ( XlsHelpers.isDecisionImport(parsedHeader) ) {
			templateId = IXlsTableNames.DECISION_IMPORT_TEMPLATE;
		} else if ( XlsHelpers.isGlossary(parsedHeader) ) {
			templateId = IXlsTableNames.GLOSSARY_TEMPLATE;
		} else if ( XlsHelpers.isDecisionModel(parsedHeader) ) {
			templateId = IXlsTableNames.DECISION_TEMPLATE;
		} else if ( XlsHelpers.isDecisionObject(parsedHeader) ) {
			templateId = IXlsTableNames.DECISION_OBJECT_TEMPLATE;
		} else if ( XlsHelpers.isProxyTable(parsedHeader) ) {
			templateId = parsedHeader[3].getIdentifier();
		} else if ( XlsHelpers.isDecisionTableCustom(parsedHeader) ) {
			templateId = parsedHeader[0].getIdentifier()+"Template";
		} else {
			String errMsg = "Templetized Rules format: " + IXlsTableNames.RULES + " <methodName> implements <templateName>";
			BoundError err = new BoundError(null, errMsg, null, src);
			throw err;
		}

		String methodName = validateName( methodId, src );
		String templateName = validateName( templateId, src );

		IMemberBoundNode node = new ProxyDecisionTableMethodBoundNode(dtNode, openl, module, templateName, methodName);
		
		IOpenMethod baseMethod = module.getMatchingMethod( templateName,	null);
		if ( baseMethod == null ) {
			 //could not be bound at this time, template is not yet resolved			
			return new UnresolvedDependenciesBoundNode( node);
		}
		
		return node;
	}


}
