/*
 * Created on Oct 7, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.lang.xls.binding;

import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.IBoundNode;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.OpenLRuntimeException;
import org.openl.binding.impl.ABoundNode;
import org.openl.binding.impl.module.ModuleOpenClass;
import org.openl.syntax.ISyntaxNode;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethod;
import org.openl.vm.IRuntimeEnv;

import com.openrules.dt.IDecisionTable;
import com.openrules.dt.ProxyDTLoader;
import com.openrules.dt.ProxyDecisionTable;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;
import com.openrules.table.IGridTable;
import com.openrules.table.LogicalTable;

/**
 * @author snshor
 * 
 * May,2007: JF replaced one DecisionTable to DecisionTableMultiHit and
 * DecisionTableSingleHit - see method "addTo"
 * 
 */
public class ProxyDecisionTableMethodBoundNode extends ABoundNode implements
		IMemberBoundNode {

	OpenL openl;
	// IOpenMethodHeader header;

	ProxyDecisionTable method;
	
	final ModuleOpenClass module;
	final String templateName;
	final String methodName;

	/**
	 * @param syntaxNode
	 * @param children
	 */
	public ProxyDecisionTableMethodBoundNode(ISyntaxNode dtNode,
			OpenL openl, /* IOpenMethodHeader header, */
			ModuleOpenClass module,
			String temlateName, 
			String methodName) {
		super(dtNode, IBoundNode.EMPTY);
		this.templateName = temlateName;
		this.methodName = methodName;
		this.openl = openl;
		this.module = module;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IMemberBoundNode#addTo(org.openl.binding.impl.module.ModuleOpenClass)
	 */
	public void addTo(ModuleOpenClass openClass) {
		// openClass.addMethod(method = new DecisionTableMultiHit(header)); //
		// JF
		// if (header.getType() == JavaOpenClass.VOID)
		// method = new DecisionTableMultiHit(header);
		// else
		// method = new DecisionTableSingleHit(header);

		IOpenMethod baseMethod = openClass.getMatchingMethod(this.templateName,	null);
		
		String err = null;
		if ( baseMethod == null )
			err = "Template " + this.templateName + " could not be found";
		else if (!(baseMethod instanceof IDecisionTable))
			err = "Template " + this.templateName + " should be a Decision Table";

		if ( err != null )
			throw new RuntimeException(err);
			
		method = new ProxyDecisionTable(methodName, (IDecisionTable) baseMethod);
		
		IDecisionTable baseDecisionTable = (IDecisionTable)baseMethod;
		if (baseDecisionTable.isCalculateConditionsFirst() == false) 
			method.setCalculateConditionsFirst(false);
		
		method.setDirection(baseDecisionTable.getDirection());

		openClass.addMethod(method);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IMemberBoundNode#finalizeBind(org.openl.binding.IBindingContext)
	 */
	public void finalizeBind(IBindingContext cxt) throws Exception {

		XlsTableSyntaxNode dtNode = (XlsTableSyntaxNode) syntaxNode;

		IGridTable table = dtNode.getTable();

		new ProxyDTLoader().load(new LogicalTable(table), method, openl,
				module, (IBindingContextDelegator) cxt);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IBoundNode#evaluateRuntime(org.openl.vm.IRuntimeEnv)
	 */
	public Object evaluateRuntime(IRuntimeEnv env) throws OpenLRuntimeException {
		// TODO Auto-generated method stub
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IBoundNode#getType()
	 */
	public IOpenClass getType() {
		// return header.getType();
		throw new RuntimeException("Not implemented");
	}

}
