package com.openrules.lang.xls.binding;

import org.openl.binding.IBindingContext;
import org.openl.binding.IBoundNode;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.OpenLRuntimeException;
import org.openl.binding.impl.ABoundNode;
import org.openl.binding.impl.module.ModuleOpenClass;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenField;
import org.openl.types.impl.AOpenField;
import org.openl.types.impl.DynamicObject;
import org.openl.vm.IRuntimeEnv;

import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;

public class VariableBoundNode extends ABoundNode implements IMemberBoundNode {

	protected IOpenField field;

	public VariableBoundNode(XlsTableSyntaxNode dtNode, String name, IOpenClass type, Object value) {
		
		super(dtNode, IBoundNode.EMPTY);
		field = new VariableOpenField(name, type, value);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IMemberBoundNode#addTo(org.openl.binding.impl.module.ModuleOpenClass)
	 */
	public void addTo(ModuleOpenClass openClass) {
		openClass.addField(field);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IMemberBoundNode#finalizeBind(org.openl.binding.IBindingContext)
	 */
	public void finalizeBind(IBindingContext cxt) throws Exception {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IBoundNode#evaluateRuntime(org.openl.vm.IRuntimeEnv)
	 */
	public Object evaluateRuntime(IRuntimeEnv env) throws OpenLRuntimeException {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IBoundNode#getType()
	 */
	public IOpenClass getType() {
		return field.getType();
	}

	static class VariableOpenField extends AOpenField {
		protected Object initialValue;

		public VariableOpenField(String name, IOpenClass type, Object value) {
			super(name, type);
			this.initialValue = value;
		}

		public Object get(Object target, IRuntimeEnv env) {
			Object data = ((DynamicObject) target).getFieldValue(name);

			if (data == null) {
				data = initialValue;
				((DynamicObject) target).setFieldValue(name, data);
			}

			return data;
		}

		public void set(Object target, Object value, IRuntimeEnv env) {
			((DynamicObject) target).setFieldValue(name, value);
		}

		public boolean isWritable() {
			return true;
		}

	}

}
