
package com.openrules.lang.xls.binding;

import org.openl.IOpenSourceCodeModule;
import org.openl.OpenL;
import org.openl.binding.IBindingContext;
import org.openl.binding.IMemberBoundNode;
import org.openl.binding.impl.BoundError;
import org.openl.syntax.ISyntaxNode;
import org.openl.syntax.impl.ISyntaxConstants;
import org.openl.syntax.impl.IdentifierNode;
import org.openl.syntax.impl.TokenizerParser;
import org.openl.types.IOpenClass;

import com.openrules.data.ITable;
import com.openrules.lang.xls.syntax.XlsTableSyntaxNode;
import com.openrules.table.ILogicalTable;
import com.openrules.table.LogicalTable;
import com.openrules.table.openl.GridCellSourceCodeModule;

/**
 * Implements 'Variable' declaration.
 * 
 * It is a 'DataTable' structure which holds only a single object row.
 * If there is no object row in the table then the value of the 'Variable' resolves to null 
 * 
 * @see DataNodeBinder
 * @author IK
 */

public class VariableNodeBinder extends DataNodeBinder
{

	public IMemberBoundNode preBind(ISyntaxNode syntaxNode, OpenL openl,
			IBindingContext cxt, XlsModuleOpenClass module) throws Exception
	{

		XlsTableSyntaxNode xlsDataNode = (XlsTableSyntaxNode) syntaxNode;

		ILogicalTable table = new LogicalTable(xlsDataNode.getTable());

		IOpenSourceCodeModule src = new GridCellSourceCodeModule(table.getGridTable());

		IdentifierNode[] parsedHeader = TokenizerParser.tokenize(src, " \n\r");

		String errMsg;

		if (parsedHeader.length < 3) {
			errMsg = "Variable table format: Variable <typename> <variablename>";
			BoundError err = new BoundError(null, errMsg, null, src);
			throw err;
		}

		String typeName = parsedHeader[TYPE_INDEX].getIdentifier();

		String tableName = parsedHeader[TABLE_NAME_INDEX].getIdentifier();

		IOpenClass tableType = cxt.findType(ISyntaxConstants.THIS_NAMESPACE,typeName);

		if (tableType == null) {
			errMsg = "Type not found: " + typeName;
			BoundError err = new BoundError(parsedHeader[TYPE_INDEX], errMsg,null);
			throw err;
		}

		ITable dataTable = makeTable(module, table, tableName, tableType, cxt, openl);
		if ( dataTable.getSize() > 1 ) {
			errMsg = "Variable table must have only one object";
			BoundError err = new BoundError(parsedHeader[TYPE_INDEX], errMsg,null);
			throw err;
		}
			
		String name = dataTable.getDataModel().getName();
		IOpenClass type = dataTable.getDataModel().getType();
		Object value = ( dataTable.getSize() == 0 ? null : dataTable.getData(0) );
		
		return new VariableBoundNode(xlsDataNode, name, type, value);
	}

}