/*
 * Created on Oct 28, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */
 
package com.openrules.table;

import java.util.Vector;

/**
 * @author snshor
 * modified by IK
 *
 */
public class LogicalTable  extends ALogicalTable implements ILogicalTable
{
	IGridTable gridTable;
	

	ILogicalTable[] rows;
	
	ILogicalTable[] cols;	
	
	ILogicalTableStructure tableStructure;
	
	public LogicalTable(final IGridTable gridTable)
	{
		this.gridTable = gridTable;
		this.tableStructure = new ILogicalTableStructure() {
			public int getCellHeight(int col, int row) {
				return gridTable.getCellHeight(col,row);
			}
			public int getCellWidth(int col, int row) {
					return gridTable.getCellWidth(col,row);
			}
		};
	}

	public ILogicalTableStructure getTableStructure() {
		return tableStructure;
	}
	public void setTableStructure(ILogicalTableStructure tableStructure) {
		this.tableStructure = tableStructure;
	}



	synchronized ILogicalTable[] getRows()
	{
		if (rows == null)
		{
			Vector v = new Vector();
			int cellHeight;
			int gridHeight = gridTable.getGridHeight();
			for (int i = 0; i < gridHeight; i += cellHeight)
			{
				cellHeight = tableStructure.getCellHeight(0,i);
				int fromRow = i;
				int toRow = fromRow + cellHeight - 1;
				if (cellHeight == 0) 
					toRow++; // JF
				v.add( newInstance(new GridTableRows(getGridTable(), fromRow, toRow)));
				if (cellHeight == 0) 
					break; // JF - something is wrong,  probably an array like String[] values in a template
			}
			
			rows = (ILogicalTable[])v.toArray(new ILogicalTable[v.size()]);
		}
		
		return rows;
	}



	synchronized ILogicalTable[] getColumns()
	{
		if (cols == null)
		{
			Vector v = new Vector();
			int cellWidth;
			for (int i = 0; i < gridTable.getGridWidth(); i += cellWidth)
			{
				cellWidth = tableStructure.getCellWidth(i, 0);
				v.add(newInstance(new GridTableColumns(getGridTable(), i, i + cellWidth - 1)));
			}
			
			cols = (ILogicalTable[])v.toArray(new ILogicalTable[v.size()]);
		}
		
		return cols;
	}




	/**
	 *
	 */

	public ILogicalTable getLogicalColumn(int column)
	{
		return getColumns()[column];
	}

	/**
	 *
	 */

	public int getLogicalHeight()
	{
		return getRows().length;
	}

	/**
	 *
	 */

	public ILogicalTable getLogicalRow(int row)
	{
		return getRows()[row];
	}

	/**
	 *
	 */

	public int getLogicalWidth()
	{
		return getColumns().length;
	}

	/**
	 * @return
	 */
	public IGridTable getGridTable()
	{
		return gridTable;
	}

	/**
	 *
	 */

	public ILogicalTable transpose()
	{
		return newInstance( new TransposedGridTable(gridTable) );
	}


	protected ILogicalTable columnsInternal(int from, int to)
	{
		ILogicalTable[] cols = getColumns();
		
		int startWidth = 0;
		int endWidth = 0;
		for (int i = 0; i <= to; i++)
		{
			int width = cols[i].getGridTable().getGridWidth();
			if (i < from)
				startWidth += width;
			endWidth += width;
		}
		
		return newInstance((IGridTable) gridTable.columns(startWidth, endWidth -1));
	}

	/**
	 *
	 */

	protected ILogicalTable rowsInternal(int from, int to)
	{
		ILogicalTable[] rows = getRows();
		
		int startHeight = 0;
		int endHeight = 0;
		for (int i = 0; i <= to; i++)
		{
			int height = rows[i].getGridTable().getGridHeight();
			if (i < from)
				startHeight += height;
			endHeight += height;
		}
		
		return newInstance((IGridTable) gridTable.rows(startHeight, endHeight -1));
	}



	/**
	 *
	 */

	public ILogicalTable getLogicalRegionInternal(
		int column,
		int row,
		int width,
		int height)
	{
		int gridColumn = 0, gridRow = 0;
		int gridHeight = 0, gridWidth = 0;
		
		for (int i = 0; i < row; i++)
		{
			gridRow += getRows()[i].getGridTable().getGridHeight();
		}

		for (int i = 0; i < column; i++)
		{
			gridColumn += getColumns()[i].getGridTable().getGridWidth();
		}
		
		for (int i = 0; i < height; i++)
		{
			gridHeight += getRows()[i + row].getGridTable().getGridHeight();
		}
		
		for (int i = 0; i < width; i++)
		{
			gridWidth += getColumns()[i + column].getGridTable().getGridWidth();
		}
		

		
		return newInstance((IGridTable)gridTable.getLogicalRegion(gridColumn, gridRow, gridWidth, gridHeight));
	}
	
	ILogicalTable newInstance( IGridTable grid ) {
		return new LogicalTable(grid);
	}

}
