package com.openrules.table.external;

/**
 * This class is used to create Excel file and different decision constructs in ti this file.
 * Each new table is being added to a new worksheet.
 */

import java.io.FileOutputStream;
import java.util.HashMap;

import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Font;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.util.CellRangeAddress;
import org.openl.util.Log;

public class DecisionBook implements IDecisionBook {
	
	Workbook wb;
	int worksheetNumber;
	
	static int ZOOM_PERCENT = 85;
	static int RULE_ROW_HEIGHT = 18;
	static int CODE_ROW_HEIGHT = 30;
	static int COLUMN_WIDTH = 27;

	static CellStyle styleSignature = null;
	static CellStyle styleTitle = null;
	static CellStyle style = null;
	
	public DecisionBook() {
		wb = new HSSFWorkbook();
		setStyle();
		setStyleForSignature();
		setStyleForTitle();
		worksheetNumber = 0;
	}

	/**
	 * Write content placed in apostrophes into the cell (row,col) of the
	 * worksheet
	 */
	public void outcell(Row row, int col, String content, CellStyle style) {
		Cell cell = row.createCell(col);
		cell.setCellValue(content);
		cell.setCellStyle(style);
	}

	public void setStyle(Row row, int col, CellStyle style) {
		Cell cell = row.getCell(col);
		cell.setCellStyle(style);
	}

	public void setStyle() {
		style = (CellStyle) wb.createCellStyle();
		style.setBorderBottom(CellStyle.BORDER_THIN);
		style.setBorderLeft(CellStyle.BORDER_THIN);
		style.setBorderRight(CellStyle.BORDER_THIN);
		style.setBorderTop(CellStyle.BORDER_THIN);
		//style.setAlignment(HorizontalAlignment.FILL);
		//style.setWrapText(true);
	}

	public void setStyleForSignature() {
		styleSignature = (CellStyle) wb.createCellStyle();
		styleSignature.setBorderBottom(CellStyle.BORDER_THIN);
		styleSignature.setBorderLeft(CellStyle.BORDER_THIN);
		styleSignature.setBorderRight(CellStyle.BORDER_THIN);
		styleSignature.setBorderTop(CellStyle.BORDER_THIN);
		// style.setFillForegroundColor(HSSFColor.LIME.index);
		// style.setFillBackgroundColor(HSSFColor.RED.index);
		styleSignature.setFillForegroundColor(HSSFColor.BLACK.index);
		styleSignature.setFillPattern(CellStyle.SOLID_FOREGROUND);

		Font font = (Font) wb.createFont();
		font.setColor(HSSFColor.WHITE.index);
		styleSignature.setFont(font);
		//styleSignature.setWrapText(true);
	}
	
	public void setStyleForTitle() {
		styleTitle = (CellStyle) wb.createCellStyle();
		styleTitle.setBorderBottom(CellStyle.BORDER_THIN);
		styleTitle.setBorderLeft(CellStyle.BORDER_THIN);
		styleTitle.setBorderRight(CellStyle.BORDER_THIN);
		styleTitle.setBorderTop(CellStyle.BORDER_THIN);
		// styleTitle.setFillForegroundColor(HSSFColor.LIME.index);
		// styleTitle.setFillBackgroundColor(HSSFColor.RED.index);
		styleTitle.setFillForegroundColor(HSSFColor.GREY_25_PERCENT.index);
		styleTitle.setFillPattern(CellStyle.SOLID_FOREGROUND);

		Font font = (Font) wb.createFont();
		font.setColor(HSSFColor.BLACK.index);
		styleTitle.setFont(font);
		//styleTitle.setWrapText(true);
	}

	/**
	 * Adds a Decision Table to the DecisionBook
	 * @param tableName, e.g. "DefineGreeting"
	 * @param templateName, e.g. "DecisionTableTemplate"
	 * @param labels, e.g. new String[] { "If", "If", "Then" }
	 * @param descriptions, e.g. new String[] { "Current Hour", "Current Hour", "Result" }
	 * @param rulesInMemory, e.g. new String[] {">=0","<=11","Good Morning"},
	 *                    new String[] {">=12","<=17","Good Afternoon"},
	 *                    ...
	 */
	public void addDecisionTable(
			 String tableName,      // table name, e.g. "DefineGreeting"
			 String templateName,   // template name, e.g. "DecisionTableTemplate"
			 String[] labels,       // labels, e.g. new String[] { "If", "If", "Then" }
			 String[] descriptions, // descriptions/variables, e.g. new String[] { "Current Hour", "Current Hour", "Result" }
			 String[][] rules)       //  rulesInMemory, e.g. new String[] {">=0","<=11","Good Morning"},...
	{
		IRuleTable ruleTable = new RuleTable(tableName, templateName, labels, descriptions, rules);
		addDecisionTable(ruleTable);
	}
	
	/**
	 * Auto size the column widths of the sheet
	 * @param sheet
	 * @param numberOfColumns
	 */
	public void autoSize(Sheet sheet, int numberOfColumns) {
		for(int c = 0; c <= numberOfColumns; c++) {
		     sheet.autoSizeColumn(c);
		}
	}
	
	public void addDecision(
			 String decisionName,   
			 String[] decisionTableNames)  
	{	
		worksheetNumber++;
		String signature = "Decision "+ decisionName;
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("Decision"+worksheetNumber);
		// out("my $titleformat" + worksheetNumber +
		// " = $workbook->add_format( border => 1, align => 'left', bold => 1, bg_color => 'black', color => 'white');");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfRows = decisionTableNames.length;
		int numberOfColumns = 1;
		Row[] rows = new Row[1 + 1 + 1 + numberOfRows + start];
		
		// Signature row
		rows[row] = sheet.createRow(row); // signature
		outcell(rows[row], col, signature, styleSignature);
		row++;
		// Column type row
		rows[row] = sheet.createRow(row); // column type
		outcell(rows[row], col, "ActionExecute", styleTitle);
		row++;
		// Title row
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col,"Decision Tables", styleTitle);
		row++;		
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			int r = row + i;
			rows[r] = sheet.createRow(r);
			outcell(rows[r],col,decisionTableNames[i], style);
		}
		autoSize(sheet,numberOfColumns);
	}
	
	public void addDecision2(
			 String decisionName,   
			 String[] decisionTableNames)  
	{	
		worksheetNumber++;
		String signature = "Decision "+ decisionName;
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("Decision"+worksheetNumber);
		// out("my $titleformat" + worksheetNumber +
		// " = $workbook->add_format( border => 1, align => 'left', bold => 1, bg_color => 'black', color => 'white');");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfRows = decisionTableNames.length;
		int numberOfColumns = 2;
		Row[] rows = new Row[1 + 1 + numberOfRows + start];
		
		// Signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		// Second row
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col,"Decision Names", styleTitle);
		outcell(rows[row],col+1,"Decision Tables", styleTitle);
		row++;		
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			int r = row + i;
			rows[r] = sheet.createRow(r);
			outcell(rows[r],col,decisionTableNames[i], style);
			outcell(rows[r],col+1,decisionTableNames[i], style);
		}

		autoSize(sheet,numberOfColumns);

	}

	public void addDecisionTable(IRuleTable ruleTable) {
		worksheetNumber++;
		String tableName = ruleTable.getTableName();
		Log.info("Add worksheet " + worksheetNumber + ": " + tableName);
		Sheet sheet = wb.createSheet(tableName);
		// out("my $titleformat" + worksheetNumber +
		// " = $workbook->add_format( border => 1, align => 'left', bold => 1, bg_color => 'black', color => 'white');");
		short start = 1;
		short row = start;
		short col = 1;
		String type = ruleTable.getTableType(); // DecisionTable, DecisionTable1, ...
		String signature = type + " " + tableName;
		IRuleProvider rules = ruleTable.getRules();
		int numberOfRows = rules.getNumberOfRows();
		int signatureRows = 3;
		Row[] rows = new Row[signatureRows + numberOfRows + start];
		rows[row] = sheet.createRow(row); // signature
		rows[row + 1] = sheet.createRow(row + 1); // Is/Then
		rows[row + 2] = sheet.createRow(row + 2); // Fact Types
		for (int i = 0; i < numberOfRows; i++) {
			int r = signatureRows + i + start;
			rows[r] = sheet.createRow(r);
		}

		// signature row
		// outcell(rows[row], col, signature, style);
		// Merge all columns in the signature line
		int numberOfColumns = rules.getNumberOfColumns();
		if (numberOfColumns > 1) {
			int endcol = col + numberOfColumns - 1;
			sheet.addMergedRegion(new CellRangeAddress(row, // first row
															// (0-based)
					row, // last row (0-based)
					col, // first column (0-based)
					endcol // last column (0-based)
			));
			for (int c = col; c <= (endcol - col + 1); c++) {
				outcell(rows[row], c, signature, styleSignature);
			}
		} else
			outcell(rows[row], col, signature, styleSignature);
		// Add Labels and Variables
		int column = col;
		String[] labels = rules.getLabels();
		String[] variables = rules.getDescriptions();
		for (int i = 0; i < labels.length; i++) {
			// C1, C2, If, Then, ..
			String label = labels[i];
			String variable = variables[i];
			// One column
			if ("If".equalsIgnoreCase(label) 
				|| "Then".equalsIgnoreCase(label)
				|| "Action".equalsIgnoreCase(label)
				|| "ActionAny".equalsIgnoreCase(label)
				|| "ActionPrint".equalsIgnoreCase(label)
				|| "ActionExecute".equalsIgnoreCase(label)
				|| "#".equals(label)
				|| "Message".equalsIgnoreCase(label)) {
				outcell(rows[row + 1], column, label, styleTitle);
				outcell(rows[row + 2], column, variables[i], styleTitle);
				column += 1;
			} else
			// Two merged columns
			if ("Condition".equalsIgnoreCase(label)
					|| "ConditionMap".equalsIgnoreCase(label)
					|| "ConditionAny".equalsIgnoreCase(label)
					|| "Conclusion".equalsIgnoreCase(label)
					|| "ActionMap".equalsIgnoreCase(label)) {
				// merge label
				sheet.addMergedRegion( new CellRangeAddress(
						row + 1, // first row (0-based)
						row + 1, // last row (0-based)
						column, // first column (0-based)
						column + 1 // last column (0-based)
				));
				outcell(rows[row + 1], column, label, styleTitle);
				outcell(rows[row + 1], column+1, label, styleTitle);
				// merge variable
				sheet.addMergedRegion( new CellRangeAddress(
						row + 2, // first row (0-based)
						row + 2, // last row (0-based)
						column, // first column (0-based)
						column + 1 // last column (0-based)
				));
				outcell(rows[row + 2], column, variable, styleTitle);
				outcell(rows[row + 2], column+1, variable, styleTitle);
				column += 2;
			} else
			// TO DO: Three merged columns
			if ("Condition".equalsIgnoreCase(label)
					|| "ConditionVarOperValue".equalsIgnoreCase(label)
					|| "ConclusionVarOperValue".equalsIgnoreCase(label)
					|| "ActionRulesOnArray".equalsIgnoreCase(label)) {
				// merge label
				sheet.addMergedRegion( new CellRangeAddress(
						row + 1, // first row (0-based)
						row + 1, // last row (0-based)
						column, // first column (0-based)
						column + 2 // last column (0-based)
				));
				outcell(rows[row + 1], column, label, styleTitle);
				outcell(rows[row + 1], column+1, label, styleTitle);
				outcell(rows[row + 1], column+2, label, styleTitle);
				// merge variable
				sheet.addMergedRegion( new CellRangeAddress(
						row + 2, // first row (0-based)
						row + 2, // last row (0-based)
						column, // first column (0-based)
						column + 2 // last column (0-based)
				));
				outcell(rows[row + 2], column, variable, styleTitle);
				outcell(rows[row + 2], column+1, variable, styleTitle);
				outcell(rows[row + 2], column+2, variable, styleTitle);
				column += 3;
			}
			else {
				throw new RuntimeException("XlsGenerator: unrecognized template column");
			}
		}
		
		int n = 0;
		// Add Rule rows
		for (int i = 0; i < numberOfRows; i++) {
			int newrow = start + signatureRows + i;
			Row iRow = rows[newrow];
			// set row height
			// out("$worksheet" + ws + "->set_row(" + newrow + ", " +
			// RULE_ROW_HEIGHT + ");");
			// format all rule cells
			for (int j = 0; j < numberOfColumns; j++)
				outcell(iRow, col + j, " ", style);
			boolean[] cells = new boolean[numberOfColumns];
			for (int j = 0; j < numberOfColumns; j++)
				cells[j] = false;
			for (int j = 0; j < numberOfColumns; j++) {
				int newcol = col + j;
				// System.out.println("action: "+action.toString() + " column="
				// + newcol + " newrow=" + newrow);
				//String value = rulesInMemory.getValue(i, j).toString();
				Object val = rules.getValue(i, j);
				String value = null;
				if (val != null)
					value = val.toString();
				if (value != null && value.startsWith("="))
					value = value.substring(1);
				outcell(iRow, newcol, value, style);
				cells[newcol - col] = true;
			}
		}
		// Set the row height for rows with code
		// outlineRow(row + 1);
		// outlineRow(row + 2, CODE_ROW_HEIGHT);
		// outlineRow(row + 3);

		if (numberOfColumns > 15) {
			// The zoom is expressed as a fraction. For example to express a
			// zoom of 75% use 3 for the numerator and 4 for the denominator.
			sheet.setZoom(3, 4); // 75 percent magnification
		}
		
		autoSize(sheet,numberOfColumns);
	}
	
	public void addLayoutTable(
			 String tableName,      // layout name, e.g. "DefineGreeting"
			 HashMap<String,String> properties,   // {"border", "1"}, {"style",	"color:black"}...														
			 String[][] table)     //  table rows, e.g. new String[] {"Gender","Age","Greeting"},...
	{	
		worksheetNumber++;
		Log.info("Add worksheet " + worksheetNumber + ": " + tableName);
		Sheet sheet = wb.createSheet(tableName);
		// out("my $titleformat" + worksheetNumber +
		// " = $workbook->add_format( border => 1, align => 'left', bold => 1, bg_color => 'black', color => 'white');");
		short start = 1;
		short row = start;
		short col = 1;
		String signature = "Layout TableLayout " + tableName + "()";
		int numberOfRows = table.length;
		int numberOfColumns = table[0].length;
		int numberOfProperties = properties.size();
		Row[] rows = new Row[1 + numberOfProperties + numberOfRows + start];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		if (numberOfColumns > 1) {
			int endcol = col + numberOfColumns - 1;
			sheet.addMergedRegion(
				new CellRangeAddress(row, // first row (0-based)
				                     row, // last row (0-based)
				                     col, // first column (0-based)
				                     endcol) // last column (0-based)
			);
			for (int c = col; c <= (endcol - col + 1); c++) {
				outcell(rows[row], c, signature, styleSignature);
			}
		} else
			outcell(rows[row], col, signature, styleSignature);
		row++;
		// Properties
		if (numberOfProperties > 0) {
			for (int i = 0; i < numberOfProperties; i++) {
				int r = row + i;
				rows[r] = sheet.createRow(r);
			}
			// merge "properties" rows in the first column
			if (numberOfProperties > 1) {
				sheet.addMergedRegion( new CellRangeAddress(
						row, // first row (0-based)
						row + numberOfProperties - 1, // last row (0-based)
						col, // first column (0-based)
						col // last column (0-based)
				));
			}
			outcell(rows[row], col, "properties", styleTitle);
			// add property names in the second column and property values in the third column
			int r = row;
			for (String propertyName: properties.keySet()) {
				outcell(rows[r], col+1, propertyName, styleTitle);
				outcell(rows[r], col+2, properties.get(propertyName), styleTitle);
				r++;
			}
			// merge the third and other columns in the properties rows
			if (numberOfColumns > 3) {
				int lastColumn = col + 2 + (numberOfColumns - 3);
				r = row;
				for (String propertyName: properties.keySet()) {
					sheet.addMergedRegion( new CellRangeAddress(
							r, // first row (0-based)
							r, // last row (0-based)
							col+2, // first column (0-based)
							lastColumn // last column (0-based)
					));
					r++;
				}
			}
		}
		row += numberOfProperties;
		
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			int r = row + i;
			rows[r] = sheet.createRow(r);
		}
		
		int n = 0;
		// Add table rows
		for (int i = 0; i < numberOfRows; i++) {
			int newrow = row + i;
			Row iRow = rows[newrow];
			// set row height
			// out("$worksheet" + ws + "->set_row(" + newrow + ", " +
			// RULE_ROW_HEIGHT + ");");
			// format all rule cells
			for (int j = 0; j < numberOfColumns; j++)
				outcell(iRow, col + j, " ", style);
			boolean[] cells = new boolean[numberOfColumns];
			for (int j = 0; j < numberOfColumns; j++)
				cells[j] = false;
			for (int j = 0; j < numberOfColumns; j++) {
				int newcol = col + j;
				// System.out.println("action: "+action.toString() + " column="
				// + newcol + " newrow=" + newrow);
				String value = table[i][j];
//				if (value != null && value.startsWith("="))
//					value = value.substring(1);
				outcell(iRow, newcol, value, style);
				cells[newcol - col] = true;
			}
		}
		// Set the row height for rows with code
		// outlineRow(row + 1);
		// outlineRow(row + 2, CODE_ROW_HEIGHT);
		// outlineRow(row + 3);

		if (numberOfColumns > 15) {
			// The zoom is expressed as a fraction. For example to express a
			// zoom of 75% use 3 for the numerator and 4 for the denominator.
			sheet.setZoom(3, 4); // 75 percent magnification
		}

		autoSize(sheet,numberOfColumns);

	}
	
	public void addGlossary(
			 String[] variableNames,   
			 String businessConcept,   													
			 String[] attributeNames)  
	{	
		worksheetNumber++;
		String signature = "Glossary glossary";
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("Glossary");
		// out("my $titleformat" + worksheetNumber +
		// " = $workbook->add_format( border => 1, align => 'left', bold => 1, bg_color => 'black', color => 'white');");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfRows = variableNames.length;
		int numberOfColumns = 3;
		Row[] rows = new Row[1 + 1 + numberOfRows + start];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		// Second row
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col,"Decision Variables", styleTitle);
		outcell(rows[row],col+1,"Business Concept", styleTitle);
		outcell(rows[row],col+2,"Attributes", styleTitle);
		row++;		
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			int r = row + i;
			rows[r] = sheet.createRow(r);
			outcell(rows[r],col,variableNames[i], style);
			outcell(rows[r],col+1,"BusinessObject", style);
			outcell(rows[r],col+2,attributeNames[i], style);
		}

		autoSize(sheet,numberOfColumns);
	}
	
	//decisionBook.addDatatype(BUSINESS_OBJECT_NAME,types,names);
	public void addDatatype(
			 String businessConcept,
			 String[] types,   
			 String[] names)  
	{	
		worksheetNumber++;
		String signature = "Datatype " + businessConcept;
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("Datatype"+ businessConcept);
		// out("my $titleformat" + worksheetNumber +
		// " = $workbook->add_format( border => 1, align => 'left', bold => 1, bg_color => 'black', color => 'white');");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfRows = names.length;
		int numberOfColumns = 2;
		Row[] rows = new Row[1 + numberOfRows + start];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			int r = row + i;
			rows[r] = sheet.createRow(r);
			outcell(rows[r],col,types[i], style);
			outcell(rows[r],col+1,names[i], style);
		}

		autoSize(sheet,numberOfColumns);
	}
	
	public void addData(
			 String datatype,
			 String arrayName,
			 String[] attributeNames, 
			 String[] variableNames,   
			 String[][] values)  
	{	
		worksheetNumber++;
		String signature = "Data "+ datatype + " " + arrayName;
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("Data" + datatype);
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfColumns = attributeNames.length;
		int numberOfRows = values.length;
		int numberOfMethodRows = 4;
		Row[] rows = new Row[3 + numberOfRows + start + numberOfMethodRows];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		// Second row
		rows[row] = sheet.createRow(row); 
		for(int c = 0; c < numberOfColumns; c++) {
			outcell(rows[row],col+c,attributeNames[c], styleTitle);
		}
		row++;
		// Third row
		rows[row] = sheet.createRow(row); 
		for(int c = 0; c < numberOfColumns; c++) {
			outcell(rows[row],col+c,variableNames[c], styleTitle);
		}
		row++;
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			//int r = row + i;
			rows[row] = sheet.createRow(row);
			for(int c = 0; c < numberOfColumns; c++) {
				outcell(rows[row],col+c,values[i][c], style);
			}
			row++;
		}
		
		// Add a data access Method
		row++;
		rows[row] = sheet.createRow(row); 
		String methodSignature = "Method "+ datatype + "[] get" + 
		       arrayName.substring(0,1).toUpperCase() + arrayName.substring(1) + "()";
		outcell(rows[row], col, methodSignature, styleSignature);
		row++;
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col,"return " + arrayName + ";", style);
		
		autoSize(sheet,numberOfColumns);
	}
	
	public void addDataArrayString(
			 String worksheet,
			 String arrayName,
			 String[] values)  
	{	
		worksheetNumber++;
		String datatype = "ArrayString";
		String attributeName = "value";
		String signature = "Data "+ datatype + " " + arrayName;
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet(worksheet);
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfColumns = 1;
		int numberOfRows = values.length;
		int numberOfMethodRows = 4;
		Row[] rows = new Row[3 + numberOfRows + start + numberOfMethodRows];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// First row
		outcell(rows[row], col, signature, styleSignature);
		row++;
		// Second row
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col,attributeName, styleTitle);
		row++;
		// Third row
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col,arrayName, styleTitle);
		row++;
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			rows[row] = sheet.createRow(row);
			outcell(rows[row],col,values[i], style);
			row++;
		}
		
		// Add a data access Method
		row++;
		rows[row] = sheet.createRow(row); 
		String methodSignature = "Method String[] get" + 
		       arrayName.substring(0,1).toUpperCase() + arrayName.substring(1) + "()";
		outcell(rows[row], col, methodSignature, styleSignature);
		row++;
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col,"return ArrayString.getValues(" + arrayName + ");", style);
		
		autoSize(sheet,numberOfColumns);
	}
	
	public void addTestCases(String[] testIds, 
							 String	  objectName,
			 				 String[] dataObjects,   
			 				 String[] expectedVariableNames,
			 				 String[][] expectedValues)  
	{	
		worksheetNumber++;
		String signature = "DecisionTableTest testCases";
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("TestCases");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfColumns = 2 + expectedVariableNames.length;
		int numberOfRows = testIds.length;
		Row[] rows = new Row[3 + numberOfRows + start];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		// Second row
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col+0,"#", styleTitle);
		outcell(rows[row],col+1,"ActionUseObject", styleTitle);
		for(int c = 0; c < expectedVariableNames.length; c++) {
			outcell(rows[row],col+2+c,"ActionExpect", styleTitle);
		}
		row++;
		// Third row
		rows[row] = sheet.createRow(row); 
		outcell(rows[row],col+0,"Test ID", styleTitle);
		outcell(rows[row],col+1,objectName, styleTitle);
		for(int c = 0; c < expectedVariableNames.length; c++) {
			outcell(rows[row],col+2+c,expectedVariableNames[c], styleTitle);
		}
		row++;
		// Table rows
		for (int i = 0; i < numberOfRows; i++) {
			int r = row + i;
			rows[r] = sheet.createRow(r);
			outcell(rows[r],col+0,testIds[i], style);
			outcell(rows[r],col+1,dataObjects[i], style);
			for(int c = 0; c < expectedVariableNames.length; c++) {
				outcell(rows[r],col+2+c,expectedValues[i][c], style);
			}
		}
		
		autoSize(sheet,numberOfColumns);
	}
	
	public void addEnvironmentTable(String[] column1, String[] column2) {
		if (column1.length != column2.length)
			throw new RuntimeException("addEnvironmentTable: column1.length != column2.length");
		worksheetNumber++;
		String signature = "Environment";
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("Env");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfColumns = 2;
		Row[] rows = new Row[2+column1.length];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		// Table rows
		for (int i = 0; i < column1.length; i++) {
			rows[row] = sheet.createRow(row);
			outcell(rows[row],col,column1[i], style);
			outcell(rows[row],col+1,column2[i], style);
			row++;
		}

		autoSize(sheet,numberOfColumns);
	}
	
	public void addEnvironmentTable(String column1, String column2) {
		worksheetNumber++;
		String signature = "Environment";
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("Env");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfColumns = 2;
		Row[] rows = new Row[3];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		// Table rows
		rows[row] = sheet.createRow(row);
		outcell(rows[row],col,column1, style);
		outcell(rows[row],col+1,column2, style);
		
		autoSize(sheet,numberOfColumns);
	}
	
	public void addBusinessObjectsTable(String[] businessConcepts) {
		worksheetNumber++;
		String signature = "DecisionObject decisionObjects";
		Log.info("Add worksheet " + worksheetNumber + ": " + signature);
		Sheet sheet = wb.createSheet("BusinessObjects");
		short start = 1;
		short row = start;
		short col = 1;
		
		int numberOfColumns = 2;
		Row[] rows = new Row[3+businessConcepts.length];
		
		// signature row
		rows[row] = sheet.createRow(row); // signature
		// Merge all columns in the signature line
		int endcol = col + numberOfColumns - 1;
		sheet.addMergedRegion(
			new CellRangeAddress(row, // first row (0-based)
			                     row, // last row (0-based)
			                     col, // first column (0-based)
			                     endcol) // last column (0-based)
		);
		for (int c = col; c <= (endcol - col + 1); c++) {
			outcell(rows[row], c, signature, styleSignature);
		}
		row++;
		rows[row] = sheet.createRow(row);
		outcell(rows[row],col,"Business Concept", styleTitle);
		outcell(rows[row],col+1,"Business Object", styleTitle);
		row++;
		// Table rows
		for (int i = 0; i < businessConcepts.length; i++) {
			rows[row] = sheet.createRow(row);
			outcell(rows[row],col,businessConcepts[i], style);
			outcell(rows[row],col+1,":= decision.get(\"" + businessConcepts[i] + "\")", style);
			row++;
		}

		autoSize(sheet,numberOfColumns);
	}
	
	public int getNumberOfDecisionTables() {
		return worksheetNumber;
	}

	public void saveToFile(String xlsFile) {
		if (getNumberOfDecisionTables() == 0) {
			Log.info("No decision tables found");
			return;
		}
		try {
			FileOutputStream fileOut = new FileOutputStream(xlsFile);
			wb.write(fileOut);
			fileOut.close();
			Log.info("DecisionBook with " + getNumberOfDecisionTables() + " decision tables has been saved to "+xlsFile);
		} catch (Exception e) {
			throw new RuntimeException("Cannot create file " + xlsFile);
		}
	}
	
	public static void main(String[] args) {
		
		DecisionBook decisionBook = new DecisionBook();
		
		RuleTable ruleTable1 = new RuleTable( 
				"DefineGreeting", 	//table name 
				"DecisionTableTemplate", 	//template name
				new String[] { "If", "If", "Then" }, // labels
				new String[] { "Current Hour", "Current Hour", "Result" }, // descriptions/variables
				new String[][] {	//rulesInMemory
					new String[] {">=0","<=11","Good Morning"},
					new String[] {">=12","<=17","Good Afternoon"},
					new String[] {">=18","<=21","Good Evening"},
					new String[] {">=22","<=24","Good Night"}
				}
		);
		decisionBook.addDecisionTable(ruleTable1); 
		
		decisionBook.addDataArrayString("Goals","Goals", 
				new String[] { "Goal 1", "Goal 2", "Goal 3" });
		
		// One Column - no merge
//		RuleTable ruleTable2 = new RuleTable(  
//			"CheckGreeting", 	//table name 
//			"DecisionTableTemplate", 	//template name
//			new String[] { "If", "Message" }, // labels
//			new String[] { "Result", "Message" }, // descriptions/variables
//			new String[][] {	//rulesInMemory
//				new String[] {"Good Afternoon", "Good Result"},
//				new String[] {"", "Error: Expected Good Afternoon"}
//			}
//		);

		// Two columns to merge
//		RuleTable ruleTable2 = new RuleTable(  
//			"CheckGreeting", 	//table name 
//			"DecisionTableTemplate", 	//template name
//			new String[] { "Condition", "Message" }, // labels
//			new String[] { "Result", "Message" }, // descriptions/variables
//			new String[][] {	//rulesInMemory
//				new String[] {"Is", "Good Afternoon", "Good Result"},
//				new String[] {"Is Not", "Good Afternoon", "Error: Expected Good Afternoon"}
//			}
//		);
		
		// Three columns to merge
		RuleTable ruleTable2 = new RuleTable(  
			"CheckGreeting", 	//table name 
			"DecisionTableTemplate", 	//template name
			new String[] { "ConditionVarOperValue", "Message" }, // labels
			new String[] { "<Var> <Oper> <Value>", "Message" }, // descriptions/variables
			new String[][] {	//rulesInMemory
				new String[] {"Result","Is Not","Good Afternoon", "Error: Expected Good Afternoon"},
				new String[] {"Result","Is","Good Afternoon", "Good Result"}
			}
		);		
		decisionBook.addDecisionTable(ruleTable2);
		
		
		HashMap<String,String> properties = new HashMap<String,String>();
		properties.put("border", "1");
		properties.put("width", "100%");
		properties.put("style", "color:black;border:8px groove blue");
		properties.put("cellspacing", "2");
		
		decisionBook.addLayoutTable("Catalogue", properties, 
				new String[][] {	
				new String[] {"A11","A12","A13", "A14", "A15", "A16"},
				new String[] {"A21","A22","A23", "A24", "A25", "A26"},
				new String[] {"A31","A32","A33", "A34", "A35", "A36"}
				}
		);
		
		decisionBook.addGlossary( new String[] {"V1","V2","V3", "V4"}, 
				                  "Object", 
				                  new String[] {"v1","v2","v3", "v4"});
		
		decisionBook.saveToFile("c:/temp/DecisionBook.xls");
		
	}
	
	

}
