package com.openrules.table.external;

import java.util.HashMap;

public interface IDecisionBook {
	
	/**
	 * Adds a Decision Table to the DecisionBook
	 * @param tableName, e.g. "DefineGreeting"
	 * @param templateName, e.g. "DecisionTableTemplate"
	 * @param labels, e.g. new String[] { "If", "If", "Then" }
	 * @param descriptions, e.g. new String[] { "Current Hour", "Current Hour", "Result" }
	 * @param rulesInMemory, e.g. new String[] {">=0","<=11","Good Morning"},
	 *                    new String[] {">=12","<=17","Good Afternoon"},
	 *                    ...
	 */
	public void addDecisionTable(
			 String tableName,      // table name, e.g. "DefineGreeting"
			 String templateName,   // template name, e.g. "DecisionTableTemplate"
			 String[] labels,       // labels, e.g. new String[] { "If", "If", "Then" }
			 String[] descriptions, // descriptions/variables, e.g. new String[] { "Current Hour", "Current Hour", "Result" }
			 String[][] rules       //  rulesInMemory, e.g. new String[] {">=0","<=11","Good Morning"},...
	);
	
	public void addDecision(
			 String decisionName,   
			 String[] decisionTableNames);
	
	public void addDecisionTable(IRuleTable ruleTable);
	
	public void addGlossary(
			 String[] variableNames,   
			 String businessConcept,   													
			 String[] attributeNames);
	
	public void addDatatype(
			 String businessConcept,
			 String[] types,   
			 String[] names);
	
	public void addData(
			 String datatype,
			 String arrayName,
			 String[] attributeNames, 
			 String[] variableNames,   
			 String[][] values);
	
	public void addLayoutTable(
			 String tableName,      
			 HashMap<String,String> properties,  														
			 String[][] table);
	
	public void addTestCases(String[] testIds, 
			 String	  objectName,
			 String[] dataObjects,   
			 String[] expectedVariableNames,
			 String[][] expectedValues);
	
	public void addEnvironmentTable(String column1, String column2);
	
	/**
	 * Save the DecisionBook into an xls-file
	 * @param xlsFile
	 */
	public void saveToFile(String xlsFile);
	
	/**
	 * 
	 * @return a number of decision tables already added to this workbook
	 */
	public int getNumberOfDecisionTables();

}
