package com.openrules.table.external;

import com.openrules.table.external.IRuleProvider;
import com.openrules.table.external.InvalidStructureException;

/*
 * Copyright (C) 2009 OpenRules, Inc.
 * 
 * Created on Feb-2009 by IK to support external rule tables
 */

public class RuleProvider implements IRuleProvider {
	private final Object[][] data;
	private String[] labels;
	private String[] descriptions;

	public RuleProvider(Object[][] data) {
		this(null,data);
	}

	public RuleProvider(String[] labels, Object[][] data) {
		this(labels,null,data);
	}

	public RuleProvider(String[] labels, String[] descriptions, Object[][] data) {
		this.labels = labels;
		this.descriptions = descriptions;
		this.data = (data == null ? new Object[0][0] : data);
	}

	public int getNumberOfRows() {
		if (data.length == 0)
			return 0;
		return data.length;
	}

	public int getNumberOfColumns() {
		if (data.length == 0)
			return 0;
		return data[0].length;
	}


	public Object getValue(int row, int col) {
		return data[row][col];
	}

	public void validateStructure() throws InvalidStructureException {
		int rows = getNumberOfRows();
		int cols = getNumberOfColumns();
		for (int r = 0; r < rows; ++r)
			if (data[r].length != cols)
				throw new InvalidStructureException("Error at row ["+r+"]: length is not the same as the first row");
	}

	public String[] getLabels() {
		return labels;
	}

	public String[] getDescriptions() {
		return descriptions;
	}

}
