package com.openrules.table.external;

import com.openrules.table.external.InvalidStructureException;
import com.openrules.table.external.IRuleProvider;

/*
 * Copyright (C) 2009 OpenRules, Inc.
 * 
 */

public class RuleProviderForObjects implements IRuleProvider {
	private final Objects[] data;
	private String[] labels;
	private String[] descriptions;

	public RuleProviderForObjects(Objects[] data) {
		this(null,null,data);
	}

	public RuleProviderForObjects(String[] labels, Objects[] data) {
		this(labels,null,data);
	}

	public RuleProviderForObjects(String[] labels, String[] descriptions, Objects[] data) {
		this.labels = labels;
		this.descriptions = descriptions;
		this.data = (data == null ? new Objects[0] : data);
	}

	public int getNumberOfRows() {
		if (data.length == 0)
			return 0;
		return data.length;
	}

	public int getNumberOfColumns() {
		if (data.length == 0)
			return 0;
		return data[0].size();
	}


	public Object getValue(int row, int col) {
		return data[row].get(col);
	}

	public void validateStructure() throws InvalidStructureException {
		int rows = getNumberOfRows();
		int cols = getNumberOfColumns();
		for (int r = 0; r < rows; ++r)
			if (data[r].size() != cols)
				throw new InvalidStructureException("Error at row ["+r+"]: length is not the same as the first row");
	}

	public String[] getLabels() {
		return labels;
	}

	public String[] getDescriptions() {
		return descriptions;
	}

}
