package com.openrules.table.external;

import org.openl.util.Log;

import com.openrules.lang.xls.IXlsTableNames;

/*
 * Copyright (C) 2009 OpenRules, Inc.
 * 
 * Created on Feb-2009 by IK to support external rule tables
 */

public class RuleTable implements IRuleTable {
	private String tableName;
	private String templateName;
	private IRuleProvider rules;
	
	/**
	 * A constructor for a decision table
	 * @param tableName, e.g. "DefineGreeting"
	 * @param templateName, e.g. "DecisionTableTemplate"
	 * @param labels, e.g. new String[] { "If", "If", "Then" }
	 * @param descriptions, e.g. new String[] { "Current Hour", "Current Hour", "Result" }
	 * @param rulesInMemory, e.g. new String[] {">=0","<=11","Good Morning"},
	 *                    new String[] {">=12","<=17","Good Afternoon"},
	 *                    ...
	 */
	public RuleTable(
			 String tableName,      // table name, e.g. "DefineGreeting"
			 String templateName,   // template name, e.g. "DecisionTableTemplate"
			 String[] labels,       // labels, e.g. new String[] { "If", "If", "Then" }
			 String[] descriptions, // descriptions/variables, e.g. new String[] { "Current Hour", "Current Hour", "Result" }
			 String[][] rules)      //  rulesInMemory, e.g. new String[] {">=0","<=11","Good Morning"},...
	{
				
		this(tableName, templateName, new RuleProvider(labels, descriptions, rules));
	}

	public RuleTable(String name, String templateName, IRuleProvider rules) {
		this.tableName = name;
		this.templateName = templateName;
		this.rules = rules;
		try {
			rules.validateStructure();
		} catch (Exception e) {
			String signature = IXlsTableNames.RULES_EXTERNAL + " " + tableName
					+ " template " + templateName;
			throw new RuntimeException("Invalid structure for: " + signature, e);
		}
	}

	public String getTableName() {
		return tableName;
	}

	public String getTemplateName() {
		return templateName;
	}
	
	public String getTableType() {
		int end = templateName.indexOf("Template");
		if (end <=0)
			throw new RuntimeException("RuleTable: unsupported template name (<...>Template is expected");
		return templateName.substring(0, end);
	}

	public IRuleProvider getRules() {
		return rules;
	}

	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("ExternalRules " + getTableName() + " template "
				+ getTemplateName());
		buf.append("\n");
		IRuleProvider dp = getRules();
		int length = dp.getNumberOfRows();
		for (int i = 0; i < length; ++i) {
			for (int c = 0; c < dp.getNumberOfColumns(); ++c) {
				buf.append(dp.getValue(i, c));
				buf.append(" ");
			}
			buf.append("\n");
		}
		return buf.toString();
	}

}
