/*
 * Created on Sep 19, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.table.xls;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.util.CellRangeAddress;
//import org.apache.poi.hssf.util.CellRangeAddress;
import org.openl.IOpenSourceCodeModule;
import org.openl.util.Log;

import com.openrules.table.IGrid;
import com.openrules.table.IGridRegion;
import com.openrules.table.ui.ICellStyle;

/**
 * @author snshor
 * Jan. 11, 2008: JF expanded merged regions inside getStringCellValue()
 * Jan-2009: JF
 *     1) POI-3.2 deprecated the class Region. Switched the implementation to CellRangeAddress
 *     2) in getCellValue replaced deprecated cell.getStringCellValue() to 
 *        cell.getRichStringCellValue().getString()
 * June 2010: POI 3.6 deprecated org.apache.poi.hssf.util.CellRangeAddress. Switch to
 *            org.apache.poi.ss.util.CellRangeAddress
 *
 */
public class XlsSheetGridModel implements IGrid
{
	Sheet sheet;
	IOpenSourceCodeModule xlsModule;
	int numMergedRegions = -1;


	String name;

	public XlsSheetGridModel(
		Sheet sheet,
		String name,
		IOpenSourceCodeModule xlsModule)
	{
		this.sheet = sheet;
		this.name = name;
		this.xlsModule = xlsModule;
	}



	public Cell getCell(int x, int y)
	{
		Row row = sheet.getRow(y);
		if (row == null)
			return null;

		//return row.getCell((short) x); - param short deprecated in 3.2 // JF
//		if (x<0)
//			Log.debug("CELL Index < 0");
		return row.getCell(x);
	}

	public boolean isEmpty(int x, int y)
	{
		Row row = sheet.getRow(y);
		if (row == null)
			return true;

		//Cell cell = row.getCell((short) x); - param short deprecated in 3.2 // JF
		if (x<0) {
//			Log.debug("CELL Index < 0");
			return true;
		}
		Cell cell = row.getCell(x);
		if (cell == null)
			return true;

		return cell.getCellType() == Cell.CELL_TYPE_BLANK;

	}


	public boolean isPartOfTheMergedRegion(int x, int y)
	{
		return getRegionContaining(x, y) != null;
	}
	
//	public static boolean isTopLeftOfTheMergedRegion(int x, int y, Region reg)
//	{
//		return reg.getColumnFrom() == x && reg.getColumnFrom() == y; 
//	} 
	
	public static boolean isTopLeftOfTheMergedRegion(int x, int y, CellRangeAddress reg)
	{
		return reg.getFirstColumn() == x && reg.getFirstColumn() == y; 
	} 
	
	
//	public Region getRegionContaining(int x, int y)
//	{
//		int nregions = getNumberOfMergedRegions();
//		for (int i = 0; i < nregions; i++)
//		{
//			Region reg = sheet.getMergedRegionAt(i);
//			//if (reg.contains(y, (short) x))
//			// JF: this is in place implementation of the removed "contains"
//			int row = y;
//			short col = (short) x;
//			if ((reg.getRowFrom() <= row) && (reg.getRowTo() >= row)
//	                && (reg.getColumnFrom() <= col) && (reg.getColumnTo() >= col))
//				return reg;
//				return reg;
//		}
//		return null;
//	}
	public CellRangeAddress getRegionContaining(int x, int y)
	{
		int nregions = getNumberOfMergedRegions();
		for (int i = 0; i < nregions; i++)
		{
			CellRangeAddress reg = sheet.getMergedRegion(i);
			//if (reg.contains(y, (short) x))
			// JF: this is in place implementation of the removed "contains" for POI-3.2
			int row = y;
			short col = (short) x;
			if ((reg.getFirstRow() <= row) && (reg.getLastRow() >= row)
	                && (reg.getFirstColumn() <= col) && (reg.getLastColumn() >= col))
				return reg;
		}
		return null;
	}




	static Object getCellValue(Cell cell)
	{
		try {
			switch (cell.getCellType())
			{
				case Cell.CELL_TYPE_BLANK :
					return null;
				case Cell.CELL_TYPE_BOOLEAN :
					return new Boolean(cell.getBooleanCellValue());
				case Cell.CELL_TYPE_FORMULA :
				case Cell.CELL_TYPE_NUMERIC :
					double value = cell.getNumericCellValue();
					return value == (int) value
						? (Object) new Integer((int) value)
						: (Object) new Double(value);
				case Cell.CELL_TYPE_STRING :
					//return cell.getStringCellValue(); - deprecated
					return cell.getRichStringCellValue().getString(); // JF
				case Cell.CELL_TYPE_ERROR :
					return "Error cell type: " + cell.getCellType(); // JF
				default :
					return "unknown type: " + cell.getCellType();
			}
		} catch (java.lang.IllegalStateException e) { 
			// JF - to avoid "Cannot get a numeric value from a error formula cell"
			return "Error cell type: " + cell.getCellType();
		}
	}

	/**
	 * @return
	 */
	public String getName()
	{
		return name;
	}
	
	public String getUri()
	{
		String xlsUri = xlsModule == null ? "" : xlsModule.getUri(0);
		return xlsUri;// + "#" + name;
		
	}

	/**
	 *
	 */

//	public int getCellHeight(int column, int row)
//	{
//		
//		Region reg = getRegionContaining(column, row);
//		if (reg == null)
//		  return 1;
//		
//		if (reg.getColumnFrom() == column && reg.getRowFrom() == row)
//		  return reg.getRowTo() - reg.getRowFrom() + 1;  
//		return 1;
//	}
	public int getCellHeight(int column, int row)
	{
		
		CellRangeAddress reg = getRegionContaining(column, row);
		if (reg == null)
		  return 1;
		
		if (reg.getFirstColumn() == column && reg.getFirstRow() == row)
		  return reg.getLastRow() - reg.getFirstRow() + 1;  
		return 1;
	}

	/**
	 *
	 */

//	public int getCellWidth(int column, int row)
//	{
//		Region reg = getRegionContaining(column, row);
//		if (reg == null)
//		  return 1;
//		
//		if (reg.getColumnFrom() == column && reg.getRowFrom() == row)
//		  return reg.getColumnTo() - reg.getColumnFrom() + 1;  
//		return 1;
//	}
	public int getCellWidth(int column, int row)
	{
		CellRangeAddress reg = getRegionContaining(column, row);
		if (reg == null)
		  return 1;
		
		if (reg.getFirstColumn() == column && reg.getFirstRow() == row)
		  return reg.getLastColumn() - reg.getFirstColumn() + 1;  
		return 1;
	}

	/**
	 *
	 */

	public int getMaxColumnIndex(int rownum)
	{
		Row row = sheet.getRow(rownum);
		
		return row == null ? 0 : row.getLastCellNum();
	}

	/**
	 *
	 */

	public int getMaxRowIndex()
	{
		return sheet.getLastRowNum();
	}


	XlsGridRegion[] regions = null;

	public synchronized IGridRegion getMergedRegion(int i)
	{
		if (regions == null)
		{
			int n = getNumberOfMergedRegions();
			regions = new XlsGridRegion[n];
			for (int j = 0; j < n; j++)
			{
				regions[i] = new XlsGridRegion(sheet.getMergedRegion(i));
			} 
		}
		
		return regions[i];
	}
	
	
	
	static class XlsGridRegion implements IGridRegion
	{
		//Region poiXlsRegion;
		CellRangeAddress poiXlsRegion;
		
		XlsGridRegion(CellRangeAddress poiXlsRegion)
		{
			this.poiXlsRegion = poiXlsRegion;
		}
		public int getBottom()
		{
			return poiXlsRegion.getLastRow();
		}
		public int getLeft()
		{
			return poiXlsRegion.getFirstColumn();
		}
		public int getRight()
		{
			return poiXlsRegion.getLastColumn();
		}
		public int getTop()
		{
			return poiXlsRegion.getFirstRow();
		}

}
	

	/**
	 *
	 */

	public int getMinColumnIndex(int rownum)
	{
		Row row = sheet.getRow(rownum);
		
		return row == null ? 0 : row.getFirstCellNum();
	}

	/**
	 *
	 */

	public int getMinRowIndex()
	{
		return sheet.getFirstRowNum();
	}

	/**
	 *
	 */

	public int getNumberOfMergedRegions()
	{
		if (numMergedRegions == -1)
		{
			try
			{
				numMergedRegions = sheet.getNumMergedRegions();
			}
			catch (NullPointerException e)
			{
				numMergedRegions = 0;
			}
		}
		return numMergedRegions;
	}


	/**
	 * Returns a string value of an Excel cell.
	 * If the cell belongs to a merged region and it is not the first in the region
	 * Excel returns null as its value. This methods instead of null
	 * returns a value from the first region's cell (JF)
	 */
	public String getStringCellValue(int column, int row)
	{
		if (column<0) {
//			Log.debug("CELL Index < 0");
			return null;
		}
		Cell cell = getCell(column, row);
		if (cell == null)
			return null;

		Object res = getCellValue(cell);
		
		// JF: if the cell belongs to a merged region and it is not the first in the region
		// Excel returns null as its value. We will make its value to be the same 
		// as in the first region's cell
		if (res == null) {
			// check if the cell belongs to a merged region and use the region's value
			//Region region = getRegionContaining(column, row);
			CellRangeAddress region = getRegionContaining(column, row);
			if (region != null) {
				int c = region.getFirstColumn();
				int r = region.getFirstRow();
				if (c != column || r != row)
					res = getStringCellValue(c, r);			
			}
		}

		return res == null ? null : String.valueOf(res);
	}


	public String getCellUri(int column, int row)
	{
		return xlsCellPresentation(column, row);
	}

	static public  String xlsCellPresentation(int x, int y)
	{
		StringBuffer buf = new StringBuffer();
		int div = 'Z' - 'A' + 1;
		
		int xx = x;
		while(xx >= div)
		{
			int dd = xx/div;
			buf.append((char)('A' + dd - 1));
			xx -= dd * div;
		}

		buf.append((char)('A' + xx));

		
		buf.append(y+1);
		return buf.toString(); 
	}

	/**
	 *
	 */

	public ICellStyle getCellStyle(int column, int row)
	{
		if (column<0) {
//			Log.debug("CELL Index < 0");
			return null;
		}
		Cell cell = getCell(column, row);
		
		CellStyle style = cell == null ? null : cell.getCellStyle();
		return style == null ? null : new XlsCellStyle(style);
	}

}
