/*
 * Created on Sep 19, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package com.openrules.table.xls;

import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.util.Region;
import org.openl.IOpenSourceCodeModule;

import com.openrules.table.IGrid;
import com.openrules.table.IGridRegion;
import com.openrules.table.ui.ICellStyle;

/**
 * @author snshor
 * Jan. 11, 2008: JF expanded merged regions inside getStringCellValue()
 *
 */
public class XlsSheetGridModelWithDeprecatedRegion implements IGrid
{
	HSSFSheet sheet;
	IOpenSourceCodeModule xlsModule;
	int numMergedRegions = -1;


	String name;

	public XlsSheetGridModelWithDeprecatedRegion(
		HSSFSheet sheet,
		String name,
		IOpenSourceCodeModule xlsModule)
	{
		this.sheet = sheet;
		this.name = name;
		this.xlsModule = xlsModule;
	}



	public HSSFCell getCell(int x, int y)
	{
		HSSFRow row = sheet.getRow(y);
		if (row == null)
			return null;

		//return row.getCell((short) x); - param short deprecated in 3.2 // JF
		return row.getCell(x);
	}

	public boolean isEmpty(int x, int y)
	{
		HSSFRow row = sheet.getRow(y);
		if (row == null)
			return true;

		//HSSFCell cell = row.getCell((short) x); - param short deprecated in 3.2 // JF
		HSSFCell cell = row.getCell(x);
		if (cell == null)
			return true;

		return cell.getCellType() == HSSFCell.CELL_TYPE_BLANK;

	}


	public boolean isPartOfTheMergedRegion(int x, int y)
	{
		return getRegionContaining(x, y) != null;
	}
	
	public static boolean isTopLeftOfTheMergedRegion(int x, int y, Region reg)
	{
		return reg.getColumnFrom() == x && reg.getColumnFrom() == y; 
	} 
	
	
	
	public Region getRegionContaining(int x, int y)
	{
		int nregions = getNumberOfMergedRegions();
		for (int i = 0; i < nregions; i++)
		{
			Region reg = sheet.getMergedRegionAt(i);
			//if (reg.contains(y, (short) x))
			// JF: this is in place implementation of the removed "contains"
			int row = y;
			short col = (short) x;
			if ((reg.getRowFrom() <= row) && (reg.getRowTo() >= row)
	                && (reg.getColumnFrom() <= col) && (reg.getColumnTo() >= col))
				return reg;
				return reg;
		}
		return null;
	}




	static Object getCellValue(HSSFCell cell)
	{
		switch (cell.getCellType())
		{
			case HSSFCell.CELL_TYPE_BLANK :
				return null;
			case HSSFCell.CELL_TYPE_BOOLEAN :
				return new Boolean(cell.getBooleanCellValue());
			case HSSFCell.CELL_TYPE_FORMULA :
			case HSSFCell.CELL_TYPE_NUMERIC :
				double value = cell.getNumericCellValue();
				return value == (int) value
					? (Object) new Integer((int) value)
					: (Object) new Double(value);
			case HSSFCell.CELL_TYPE_STRING :
				return cell.getStringCellValue();
			default :
				return "unknown type: " + cell.getCellType();
		}
	}

	/**
	 * @return
	 */
	public String getName()
	{
		return name;
	}
	
	public String getUri()
	{
		String xlsUri = xlsModule == null ? "" : xlsModule.getUri(0);
		return xlsUri;// + "#" + name;
		
	}

	/**
	 *
	 */

	public int getCellHeight(int column, int row)
	{
		
		Region reg = getRegionContaining(column, row);
		if (reg == null)
		  return 1;
		
		if (reg.getColumnFrom() == column && reg.getRowFrom() == row)
		  return reg.getRowTo() - reg.getRowFrom() + 1;  
		return 1;
	}

	/**
	 *
	 */

	public int getCellWidth(int column, int row)
	{
		Region reg = getRegionContaining(column, row);
		if (reg == null)
		  return 1;
		
		if (reg.getColumnFrom() == column && reg.getRowFrom() == row)
		  return reg.getColumnTo() - reg.getColumnFrom() + 1;  
		return 1;
	}

	/**
	 *
	 */

	public int getMaxColumnIndex(int rownum)
	{
		HSSFRow row = sheet.getRow(rownum);
		
		return row == null ? 0 : row.getLastCellNum();
	}

	/**
	 *
	 */

	public int getMaxRowIndex()
	{
		return sheet.getLastRowNum();
	}


	XlsGridRegion[] regions = null;

	public synchronized IGridRegion getMergedRegion(int i)
	{
		if (regions == null)
		{
			int n = getNumberOfMergedRegions();
			regions = new XlsGridRegion[n];
			for (int j = 0; j < n; j++)
			{
				regions[i] = new XlsGridRegion(sheet.getMergedRegionAt(i));
			} 
		}
		
		return regions[i];
	}
	
	
	
	static class XlsGridRegion implements IGridRegion
	{
		Region poiXlsRegion;
		
		XlsGridRegion(Region poiXlsRegion)
		{
			this.poiXlsRegion = poiXlsRegion;
		}
		public int getBottom()
		{
			return poiXlsRegion.getRowTo();
		}
		public int getLeft()
		{
			return poiXlsRegion.getColumnFrom();
		}
		public int getRight()
		{
			return poiXlsRegion.getColumnTo();
		}
		public int getTop()
		{
			return poiXlsRegion.getRowFrom();
		}

}
	

	/**
	 *
	 */

	public int getMinColumnIndex(int rownum)
	{
		HSSFRow row = sheet.getRow(rownum);
		
		return row == null ? 0 : row.getFirstCellNum();
	}

	/**
	 *
	 */

	public int getMinRowIndex()
	{
		return sheet.getFirstRowNum();
	}

	/**
	 *
	 */

	public int getNumberOfMergedRegions()
	{
		if (numMergedRegions == -1)
		{
			try
			{
				numMergedRegions = sheet.getNumMergedRegions();
			}
			catch (NullPointerException e)
			{
				numMergedRegions = 0;
			}
		}
		return numMergedRegions;
	}


	/**
	 * Returns a string value of an Excel cell.
	 * If the cell belongs to a merged region and it is not the first in the region
	 * Excel returns null as its value. This methods instead of null
	 * returns a value from the first region's cell (JF)
	 */
	public String getStringCellValue(int column, int row)
	{
		HSSFCell cell = getCell(column, row);
		if (cell == null)
			return null;

		Object res = getCellValue(cell);
		
		// JF: if the cell belongs to a merged region and it is not the first in the region
		// Excel returns null as its value. We will make its value to be the same 
		// as in the first region's cell
		if (res == null) {
			// check if the cell belongs to a merged region and use the region's value
			Region region = getRegionContaining(column, row);
			if (region != null) {
				int c = region.getColumnFrom();
				int r = region.getRowFrom();
				if (c != column || r != row)
					res = getStringCellValue(c, r);			
			}
		}

		return res == null ? null : String.valueOf(res);
	}


	public String getCellUri(int column, int row)
	{
		return xlsCellPresentation(column, row);
	}

	static public  String xlsCellPresentation(int x, int y)
	{
		StringBuffer buf = new StringBuffer();
		int div = 'Z' - 'A' + 1;
		
		int xx = x;
		while(xx >= div)
		{
			int dd = xx/div;
			buf.append((char)('A' + dd - 1));
			xx -= dd * div;
		}

		buf.append((char)('A' + xx));

		
		buf.append(y+1);
		return buf.toString(); 
	}

	/**
	 *
	 */

	public ICellStyle getCellStyle(int column, int row)
	{
		HSSFCell cell = getCell(column, row);
		
		HSSFCellStyle style = cell == null ? null : cell.getCellStyle();
		return style == null ? null : new XlsCellStyle(style);
	}

}
