package com.openrules.tools;

/*
 * Copyright (c) 2005-2007 OpenRules, Inc. 	
 */

import java.io.PrintStream;
import java.sql.*;
import java.util.HashMap;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.openl.util.Log;

public class DbUtil
{

	static HashMap map = new HashMap();

	static public DbUtil getDBInstance(String dbName)
	{
		return getDBInstance(dbName, null, null, null);
	}

	static synchronized public DbUtil getDBInstance(
		String dbName,
		String user,
		String pwd,
		String driver)
	{
		DbEnv env = new DbEnv();
		env.driver = driver;
		env.dsn = dbName;
		env.user_name = user;
		env.password = pwd;

		DbUtil dbu = (DbUtil) map.get(env);
		if (dbu == null)
		{
			dbu = new DbUtil(env);
			map.put(env, dbu);
		}
		return dbu;
	}

	Connection connection;
	DbEnv env;

	public DbUtil(DbEnv env)
	{
		this.env = env;
	}

	public synchronized int count(String tableName) throws Exception
	{
		String sql = "SELECT count(*) from " + tableName;

		ResultSet rs = executeQuery(sql);

		rs.next();

		int count = rs.getInt(1);

		rs.close();
		return count;
	}

	public synchronized ResultSet selectAll(String tableName) throws Exception, Exception
	{
		String sql = "SELECT * from " + tableName;
		ResultSet rs = executeQuery(sql);
		return rs;
	}

	public synchronized String[] getArray(String tableName, String columnName)
		throws Exception
	{
		int n = count(tableName);
		String[] ary = new String[n];

	
		ResultSet rs =
			executeQuery("SELECT " + columnName + " from " + tableName);
		
		try
		{
			int columnIndex = rs.findColumn(columnName);

			for (int i = 0; i < n; i++)
			{
				rs.next();
				ary[i] = rs.getString(columnIndex);
			}
		}
		finally
		{
			rs.getStatement().close();
//			rs.close();
		}
		return ary;
	}
	public String[] getArrayFromDB(String dbName, String tableName, String columnName)
			  throws Exception
	{
		return getDBInstance(dbName).getArray(tableName,columnName);			
	}

	public synchronized ResultSet executeQuery(String sql) throws Exception
	{
		Statement stmt = getConnection().createStatement();

		ResultSet rs = stmt.executeQuery(sql);
		
		

		return rs;
	}

	protected synchronized void initConnection() throws Exception
	{
		String driverName = env.driver;

		if (driverName == null)
			driverName = "sun.jdbc.odbc.JdbcOdbcDriver";

		try
		{
			Class.forName(driverName); //loads the driver
		}
		catch (ClassNotFoundException ex)
		{
			Log.error(ex.getMessage());
			//ex.printStackTrace(System.err);
			throw ex;
		}

		String dsnName = env.dsn;
		String usrName = env.user_name;
		String usrPwd = env.password;

		out().println(
			"Connecting to '" + dsnName + "' using '" + driverName + "' ...");

		connection =
			DriverManager.getConnection(
				"jdbc:odbc:" + dsnName,
				usrName,
				usrPwd);

		connection.setAutoCommit(false);

		out().println(
			"Connecting to '"
				+ dsnName
				+ "' using '"
				+ driverName
				+ "' ... Done");
	}

	public synchronized void freeConnection() throws Exception
	{
		out().println("Free Connection ...");
		try
		{
			if (connection != null)
			{
				out().println("Close Connection");
				connection.close();
				connection = null;
			}
		}
		catch (Exception ex)
		{
			//ex.printStackTrace(System.err);
			Log.error(ex.getMessage());
			throw ex;
		}
		Log.info("Free Connection ...Done");
	}

	public synchronized Connection getConnection() throws Exception
	{
		if (connection == null)
		{
			initConnection();
		}
		return connection;
	}

	PrintStream out()
	{
		return System.out;
	}

	static public class DbEnv
	{
		public String driver, dsn, user_name, password;

		/* 
		 */
		public boolean equals(Object obj)
		{

			if (obj == null || !(obj instanceof DbEnv))
				return false;
			DbEnv dbe = (DbEnv) obj;

			return new EqualsBuilder()
				.append(driver, dbe.driver)
				.append(dsn, dbe.dsn)
				.append(user_name, dbe.user_name)
				.append(password, dbe.password)
				.isEquals();
		}

		/* 
		 */
		public int hashCode()
		{
			return new HashCodeBuilder()
				.append(driver)
				.append(dsn)
				.append(user_name)
				.append(password)
				.toHashCode();
		}

	}

}