package com.openrules.feel;

import org.openl.util.Log;

public class BasicFeelEvaluator implements FeelEvaluator {

	Context context;

	public BasicFeelEvaluator(Context context) {
		this.context = context;
	}
	
	public boolean evaluateAsBoolean(String feelExpression) {
		return evaluateAsBoolean(feelExpression, false);
	}

	public boolean evaluateAsBoolean(String feelExpression, boolean suppressMessages) {
		String result = evaluate(feelExpression,suppressMessages);
		return toBoolean(result);
	}
	
	public double evaluateAsDouble(String feelExpression) {
		return evaluateAsDouble(feelExpression,false);
	}

	public double evaluateAsDouble(String feelExpression, boolean suppressMessages) {
		String result = evaluate(feelExpression,suppressMessages);
		return toReal(result);
	}
	
	public int evaluateAsInt(String feelExpression) {
		return evaluateAsInt(feelExpression,false);
	}
	
	public int evaluateAsInt(String feelExpression, boolean suppressMessages) {
		String result = evaluate(feelExpression,suppressMessages);
		return toInt(result);
	}
	
	public long evaluateAsLong(String feelExpression) {
		return evaluateAsLong(feelExpression,false);
	}
	
	public long evaluateAsLong(String feelExpression, boolean suppressMessages) {
		String result = evaluate(feelExpression,suppressMessages);
		return toLong(result);
	}
	
	public String evaluate(String feelExpression) {
		return evaluate(feelExpression,false);
	}

	public String evaluate(String feelExpression, boolean suppressMessages) {
		// FEEL requires ** instead of ^ to express x in power y
		feelExpression = feelExpression.replaceAll("\\*\\*","^");
		// FEEL requires <> instead of != to express inequality (expr also uses <>)
		feelExpression = feelExpression.replaceAll("!=","<>");
		// replace double spaces with one space
		feelExpression = feelExpression.replaceAll("  "," ");
		Expr expr;
		try {
			Parser parser = new Parser(context);
			expr = parser.parseString(feelExpression);
			// System.out.println(feelExpression + ": " + com.openrules.expr.value());
		} catch (SyntaxException e) {
			String msg = e.explain();
			if (!suppressMessages)
				log(msg);
			Log.debug(msg);
			throw new RuntimeException(msg);
		}

		// calculate
		return "" + expr.value();
	}
	
	public void log(String text) {
		Log.error(text);
	}
	
	public int toInt(String result) {
		return (int) Math.round(toReal(result));
	}
	
	public long toLong(String result) {
		return Math.round(toReal(result));
	}

	public double toReal(String result) {
		return Double.parseDouble(result);
	}

	public boolean toBoolean(String result) {
		if ("true".equalsIgnoreCase(result) || result.startsWith("1") || result.equalsIgnoreCase("Yes"))
			return true;
		return false;
	}

	public Context getContext() {
		return context;
	}

}
