/*
Added by OpenRules, Inc. to avoid the use of static variables
in the package "expr"
*/

package com.openrules.feel;

import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;

/**
 * A context is a placeholder for variables
 */
abstract public class Context {
	
	private Hashtable<String, Variable> variables;
	
	static public char APOSTROPHE = '\'';
	private char longNameIndicator;

	public Context() {
		variables = new Hashtable<String, Variable>();
		// Built-in constants
		Variable pi = makeVariable("pi");
		pi.setValue(Math.PI);
		
		longNameIndicator = APOSTROPHE;
	}
	
	abstract public double getDoubleValue(String variableName);
	
	public Variable[] getVariables() {
		int size = variables.size();
		Variable[] vars = new Variable[size];
		Iterator it = variables.entrySet().iterator();
		int n = 0;
		while (it.hasNext()) {
			Map.Entry pairs = (Map.Entry) it.next();
			vars[n++] = (Variable)pairs.getValue();
		}
		return vars;
	}
	
	public Variable addVariable(String name, double value) {
		Variable variable = new Variable(this,name);
		variable.setValue(value);
		variables.put(name,variable);
		return variable;
	}
	
	public Variable getVariable(String name) {
		return variables.get(name);
	}

	/**
	 * Return a unique variable named `name'. There can be only one variable
	 * with the same name returned by this method; that is, make(s1) == make(s2)
	 * if and only if s1.equals(s2).
	 * 
	 * @param name
	 *            the variable's name
	 * @return the variable; create it initialized to 0 if it doesn't yet exist
	 */
	public Variable makeVariable(String name) { // JF removed "synchronized"
		Variable result = (Variable) variables.get(name);
		if (result == null)
			variables.put(name, result = new Variable(this,name));
		return result;
	}
	
	public char getLongNameIndicator() {
		return longNameIndicator;
	}

	public void setLongNameIndicator(char longNameIndicator) {
		this.longNameIndicator = longNameIndicator;
	}
}
