package com.openrules.feel;

public interface FeelEvaluator {
	
	/**
	 * Evaluates a string that is assumed to me a valid FEEL expression
	 * @param feelExpression
	 * @return a String that contains the result of the evaluation 
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public String evaluate(String feelExpression);
	
	/**
	 * Evaluates a string that is assumed to me a valid FEEL expression
	 * @param feelExpression
	 * @param suppressMessages if true error messages will be suppressed
	 * @return a String that contains the result of the evaluation 
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public String evaluate(String feelExpression,boolean suppressMessages);

	
	/**
	 * Evaluates a string "feelExpression" that is assumed to me a valid FEEL expression
	 * and converts the result to a boolean
	 * @param feelExpression
	 * @return true if the feelExpression is evaluated as true
	 * @return false if the feelExpression is evaluated as false
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public boolean evaluateAsBoolean(String feelExpression);
	
	/**
	 * Evaluates a string "feelExpression" that is assumed to me a valid FEEL expression
	 * and converts the result to a boolean
	 * @param feelExpression
	 * @param suppressMessages if true error messages will be suppressed
	 * @return true if the feelExpression is evaluated as true
	 * @return false if the feelExpression is evaluated as false
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public boolean evaluateAsBoolean(String feelExpression, boolean suppressMessages);
	
	/**
	 * Evaluates a string "feelExpression" that is assumed to be a valid FEEL expression
	 * and converts the result to a double
	 * @param feelExpression
	 * @return a calculated double value or Double.POSITIVE_INFINITY
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public double evaluateAsDouble(String feelExpression);
	
	/**
	 * Evaluates a string "feelExpression" that is assumed to be a valid FEEL expression
	 * and converts the result to a double
	 * @param feelExpression
	 * @param suppressMessages if true error messages will be suppressed
	 * @return a calculated double value or Double.POSITIVE_INFINITY
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public double evaluateAsDouble(String feelExpression, boolean suppressMessages); 
	
	/**
	 * Evaluates a string "feelExpression" that is assumed to be a valid FEEL expression
	 * and converts the result to a int
	 * @param feelExpression
	 * @return a calculated int value or Integer.MAX_VALUE
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public int evaluateAsInt(String feelExpression); 
	
	/**
	 * Evaluates a string "feelExpression" that is assumed to be a valid FEEL expression
	 * and converts the result to a int
	 * @param feelExpression
	 * @param suppressMessages if true error messages will be suppressed
	 * @return a calculated int value or Integer.MAX_VALUE
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public int evaluateAsInt(String feelExpression, boolean suppressMessages);
	
	/**
	 * Evaluates a string "feelExpression" that is assumed to be a valid FEEL expression
	 * and converts the result to a long
	 * @param feelExpression
	 * @return a calculated long value or Long.MAX_VALUE
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public long evaluateAsLong(String feelExpression); 
	
	/**
	 * Evaluates a string "feelExpression" that is assumed to be a valid FEEL expression
	 * and converts the result to a long
	 * @param feelExpression
	 * @param suppressMessages if true error messages will be suppressed
	 * @return a calculated long value or Long.MAX_VALUE
	 * @throws a RuntimeException with an explanation if "feelExpression" is not valid FEEL expression 
	 */
	public long evaluateAsLong(String feelExpression, boolean suppressMessages);
	
	/**
	 * This method is used to log warnings and errors found during FEEL expression evaluations
	 * @param text
	 */
	public void log(String text);
	
	/**
	 * 
	 * @return the evaluation context
	 */
	public Context getContext();
	
	
}
