package com.openrules.types;
/*
 * Copyright (C) 2007 OpenRules, Inc.
 */
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

public class CompareToDate {

	Oper oper;
	Date date;

	/**
	 * Parameter expression is a text that consists of an operator and a number,
	 * e.g. "<= date" means "before or on date", "> date" means "after date",
	 * "< date" means "before date" "date" means "on date"
	 * 
	 * Acceptable operators are: <,<=,>,>=,!= You cannot use an explicit
	 * operator "=" (prohibited by Excel). The absence of operator means
	 * equality.
	 * 
	 * @param expression
	 */
	public CompareToDate(String expression) {
		String[] array = expression.trim().split("\\s");
		if (array.length > 2)
			throw new RuntimeException("Invalid CompareToDate expression: "
					+ expression);
		// to do: throw an actual syntax error
		String operator;
		
		String exp = expression.trim();
		int start = 0;
		char c1 = exp.charAt(0);
		if (c1 == '<' || c1 == '>' || c1 == '!') {
			char c2 = exp.charAt(1);
			if (c2 == '=') {
				operator = exp.substring(0, 2);
				start = 2;
			} else {
				operator = exp.substring(0, 1);
				start = 1;
			}
		} else
			operator = "="; // no operator means '='

		oper = new Oper(operator);
		String dateAsString = exp.substring(start).trim();
		DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
		//SimpleDateFormat df = new SimpleDateFormat();
		try {
			date = df.parse(dateAsString);
		} catch (Exception e) {
			date = DateUtil.getDateFromString(dateAsString);
//			throw new RuntimeException("Invalid CompareToDate: "
//					+ expression);
		}
	}

	public boolean compare(Date arg) {
		return oper.compare(arg,date);
	}

	/**
	 * TEST
	 */
	public static void main(String[] args) {
		String condition = "<= 01/01/2012";
		String myDate = "01/01/2012 15:08";
		DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
		//SimpleDateFormat df = new SimpleDateFormat();
		Date d = new Date();
		String result;
		try {
			d = df.parse(myDate);
			CompareToDate cmp = new CompareToDate(condition);
			
			result = (cmp.compare(d)) ? "TRUE" : "FALSE";

			System.out.println("Statement <" + d + "> " + condition + 
					" is "	+ result);
			
		} catch (Exception e) {
			System.out.println("Parsing error for "+myDate);
			e.printStackTrace();
		}
		
		//DateFormat myDateFormat = new SimpleDateFormat("EEE MMM dd HH:mm:ss z yyyy");
		SimpleDateFormat myDateFormat = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy");
		//DateFormat myDateFormat = DateFormat.getDateInstance(DateFormat.FULL);
		//DateFormat myDateFormat = DateFormat.getDateInstance();
		//String sd = "Sun Jan 2 16:39:56 EST 2008";
		String sd = d.toString();
		System.out.println("sd=<"+sd+">");
		Date date;
		try {
			try {
				date = myDateFormat.parse(myDate);
			} catch (Exception e) {
				date = df.parse(myDate);
				//date = df.parse(sd);
			}
//			IString2DataConvertor convertor = String2DataConvertorFactory.getConvertor(Date.class);
//			String SIMPLE_DATE_FORMAT = "EEE MMM dd HH:mm:ss zzz yyyy";
//			date = (Date) convertor.parse(myDate, SIMPLE_DATE_FORMAT);
			System.out.println(date);
			System.out.println(df.format(date));
		} catch (Exception e) {
			System.out.println("parsing error: " + sd);
		}
		
		
	}
	

}
