package com.openrules.types;
/*
 * Copyright (C) 2007 OpenRules, Inc.
 */
public class CompareToInt {

	String operator;

	int number;

	/**
	 * Parameter expression is a text that consists of an operator and a number,
	 * e.g. "<= 3" means "less or equal 3" ">0" means "more than 0" "< 50"
	 * means "less than 50" "27" means "equals to 27"
	 * 
	 * Acceptable operators are: <,<=,>,>=,!= You cannot use an explicit
	 * operator "=" (prohibited by Excel). The absence of operator means
	 * equality.
	 * 
	 * @param expression
	 */
	public CompareToInt(String expression) {
		String[] array = expression.trim().split("\\s");
		if (array.length > 2)
			throw new RuntimeException("Invalid CompareToInt expression: "
					+ expression);
		// to do: throw an actual syntax error
		try {
			String exp = expression.trim();
			int start = 0;
			char c1 = exp.charAt(0);
			if (c1 == '<' || c1 == '>' || c1 == '!') {
				char c2 = exp.charAt(1);
				if (c2 == '=') {
					operator = exp.substring(0, 2);
					start = 2;
				} else {
					operator = exp.substring(0, 1);
					start = 1;
				}
			} else
				operator = "="; // no operator means '='

			String numberAsString = exp.substring(start).trim();
			number = Integer.parseInt(numberAsString);
		} catch (Exception e) {
			throw new RuntimeException("Invalid CompareToInt number: "
					+ expression);
		}
	}

	public boolean compare(int arg) {
		if (operator.equals("<"))
			return arg < number;
		if (operator.equals("<="))
			return arg <= number;
		if (operator.equals("="))
			return arg == number;
		if (operator.equals(">"))
			return arg > number;
		if (operator.equals(">="))
			return arg >= number;
		if (operator.equals("!="))
			return number != arg;

		throw new RuntimeException("Invalid CompareToInt operator: " + operator);
	}

	/**
	 * TEST
	 */
	public static void main(String[] args) {
		String condition = "<=40";
		CompareToInt age = new CompareToInt(condition);
		int value = 30;
		String result = (age.compare(value)) ? "TRUE" : "FALSE";

		System.out.println("Statement " + value + " " + condition + " is "
				+ result);
	}
}
