package com.openrules.types;



/*
 * Copyright (C) 2007 OpenRules, Inc.
 */
public class CompareToString {

	String operator;

	String string;
	DomainStringC domain;

	/**
	 * Parameter expression is a text that consists of an operator and a number,
	 * e.g. 
	 * "<= ABC" means "substring of ABC": "", "A", "B", "AB", or "ABC" 
	 * ">ABC" means "more than ABC": ABCD, DABC, XXXXABCFFF 
	 * "< ABC" means "less than ABC": "", "A", "B", "AB" 
	 * "ABC" means "equals to ABC"
	 * "A,B,C" means "one of A or B or C"
	 * 
	 * Acceptable operators are: <,<=,>,>=,!= You cannot use an explicit
	 * operator "=" (prohibited by Excel). The absence of operator means
	 * equality.
	 * 
	 * @param expression
	 */
	public CompareToString(String expression) {
//		String[] array = expression.trim().split("\\s");
//		if (array.length > 2)
//			throw new RuntimeException("Invalid CompareToString expression: "
//					+ expression);
		// to do: throw an actual syntax error
		domain = null;
		try {
			String exp = expression.trim();
			int start = 0;
			char c1 = exp.charAt(0);
			if (c1 == '<' || c1 == '>' || c1 == '!') {
				char c2 = exp.charAt(1);
				if (c2 == '=') {
					operator = exp.substring(0, 2);
					start = 2;
				} else {
					operator = exp.substring(0, 1);
					start = 1;
				}
			} else {
				operator = "="; // no operator means '='
				if (exp.indexOf(",") > 0) {
					operator = "IsOneOf";
					domain = new DomainStringC(exp);
				}
			}
			
			string = exp.substring(start).trim();
			
		} catch (Exception e) {
			throw new RuntimeException("Invalid CompareToString expression: "
					+ expression);
		}
	}

	public boolean compare(String arg) {
		if (operator.equals("<"))
			return string.indexOf(arg) >= 0 && arg.length() < string.length(); 
		if (operator.equals("<="))
			return string.indexOf(arg) >= 0;
		if (operator.equals("="))
			return string.equals(arg);
		if (operator.equals(">"))
			return arg.indexOf(string) >= 0 && arg.length()> string.length();
		if (operator.equals(">="))
			return arg.indexOf(string) >= 0;
		if (operator.equals("!="))
			return !string.equals(arg);
		if (operator.equals("IsOneOf")) {
			return domain.contains(arg);
		}

		throw new RuntimeException("Invalid CompareToString operator: " + operator);
	}

	/**
	 * TEST
	 */
	public static void main(String[] args) {
		String condition = "<= ABC";
		CompareToString cond = new CompareToString(condition);
		String value = "ABC";
		String result = (cond.compare(value)) ? "TRUE" : "FALSE";

		System.out.println("Statement " + value + " " + condition + " is "
				+ result);
		
		condition = ">= ABC";
		cond = new CompareToString(condition);
		value = "XABCX";
		result = (cond.compare(value)) ? "TRUE" : "FALSE";

		System.out.println("Statement " + value + " " + condition + " is "
				+ result);
		
		condition = " AX,B, C";
		cond = new CompareToString(condition);
		value = "X";
		result = (cond.compare(value)) ? "TRUE" : "FALSE";

		System.out.println("Statement " + value + " is one of " + condition + " is "
				+ result);
	}
}
