package com.openrules.types;
/*
 * Copyright (C) 2011 OpenRules, Inc.
 */
import java.math.BigDecimal;
import java.util.Date;

import org.apache.commons.beanutils.PropertyUtils;
import org.openl.types.impl.DynamicObject;
import org.openl.util.Log;

public class DataMap {
	
	String name;
	int intValue;
	long longValue;
	String stringValue;
	double doubleValue;
	BigDecimal bigDecimalValue;
	boolean booleanValue;
	Date dateValue;
	Object objectValue;
	
	static public final String TYPE_BASE_INT = "int";
	static public final String TYPE_INT = "java.lang.Integer";
	static public final String TYPE_BASE_LONG = "long";
	static public final String TYPE_LONG = "java.lang.Long";
	static public final String TYPE_BIG_DECIMAL = "java.math.BigDecimal";
	static public final String TYPE_BASE_DOUBLE = "double";
	static public final String TYPE_DOUBLE = "java.lang.Double";
	static public final String TYPE_STRING = "java.lang.String";
	static public final String TYPE_BASE_BOOL = "boolean";
	static public final String TYPE_BOOL = "java.lang.Boolean";
	static public final String TYPE_DATE = "java.util.Date";
	static public final String TYPE_ARRAY = "[]";
	static public final String TYPE_UNDEFINED = "?";
	
	static public double precision = 0.00000001;

	public DataMap() {
		init("?");
	}
	
	public DataMap(String name) {
		init(name);
	}
	
	public final void init(String name) {
		this.name=name;
		intValue = Integer.MIN_VALUE;
		longValue = Long.MIN_VALUE;
		stringValue = null;
		doubleValue = Double.NEGATIVE_INFINITY;
		dateValue = null;
		booleanValue = false;
		objectValue = null;
	}
	
	public final void get(Object object, String name) {
		try {
			Object property;
			String type;
			if (object instanceof DynamicObject) {
				property = ((DynamicObject)object).getFieldValue(name);
				if (property == null) {
					//throw new Exception("Cannot find " + name);
					Log.info("Property " + name + " is null");
					type = TYPE_UNDEFINED;
				}
				else {
					type = property.getClass().getName();
				}
			}
			else {
				//property = PropertyUtils.getSimpleProperty(object, name);
				property = PropertyUtils.getProperty(object, name);
				if (property == null) {
					//throw new Exception("Cannot find " + name);
					Log.info("Property " + name + " is null");
					type = TYPE_UNDEFINED;
				}
				else
					type = PropertyUtils.getPropertyType(object, name).getName();
			}
			//System.out.println(name + " has type " + type);
			
			if (type.equals(TYPE_BASE_DOUBLE) || type.equals(TYPE_DOUBLE)) {
				setDoubleValue(((Double) property).doubleValue());
			} else if (type.equals(TYPE_BIG_DECIMAL)) {
				setBigDecimalValue((BigDecimal) property);
			} else if (type.equals(TYPE_STRING)) {
				setStringValue((String) property);
			} else if (type.equals(TYPE_BASE_INT) || type.equals(TYPE_INT)) {
				setIntValue(((Integer) property).intValue());
			} else if (type.equals(TYPE_BASE_LONG) || type.equals(TYPE_LONG)) {
				setLongValue(((Long) property).longValue());
			} else if (type.equals(TYPE_BASE_BOOL) || type.equals(TYPE_BOOL)) {
				setBooleanValue(((Boolean) property).booleanValue());
			} else if (type.equals(TYPE_DATE)) {
				setDateValue((Date) property);
			} else {
				setObjectValue(property);
				//System.out.println("ERROR in DataMap.get(" + object.toString() + "," + name + ")");
			}
		} catch (Exception e) {
			String err = "ERROR in DataMap.get(" + object.toString() + "," + name + ")";
			Log.error(err);
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
	
	public final void get(Object object, String name, int index) {
		try {
			Object[] array;
			Object property;
			String type;
			if (object instanceof DynamicObject) {
				array = (Object[]) ((DynamicObject)object).getFieldValue(name);
				property = array[index];
			}
			else {
				property = PropertyUtils.getIndexedProperty(object, name, index);
				//type = PropertyUtils.getPropertyType(object, name).getName();
			}
			type = property.getClass().getName();
			//System.out.println(name + " has type " + type);
			
			if (type.equals(TYPE_BASE_DOUBLE) || type.equals(TYPE_DOUBLE)) {
				setDoubleValue(((Double) property).doubleValue());
			} else if (type.equals(TYPE_BIG_DECIMAL)) {
				setBigDecimalValue((BigDecimal) property);
			} else if (type.equals(TYPE_STRING)) {
				setStringValue((String) property);
			} else if (type.equals(TYPE_BASE_INT) || type.equals(TYPE_INT)) {
				setIntValue(((Integer) property).intValue());
			} else if (type.equals(TYPE_BASE_LONG) || type.equals(TYPE_LONG)) {
				setLongValue(((Integer) property).intValue());
			} else if (type.equals(TYPE_BASE_BOOL) || type.equals(TYPE_BOOL)) {
				setBooleanValue(((Boolean) property).booleanValue());
			} else if (type.equals(TYPE_DATE)) {
				setDateValue((Date) property);
			} else {
				setObjectValue(property);
				//System.out.println("ERROR in DataMap.get(" + object.toString() + "," + name + ")");
			}
		} catch (Exception e) {
			String err = "ERROR in DataMap.get(" + object.toString() + "," + name + "," + index +")";
			Log.error(err);
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
	
	public final void set(Object object, String name) {
		try {
			Object property;
			String type;
			boolean dynamic = false;
			if (object instanceof DynamicObject) {
				dynamic = true;
				property = ((DynamicObject)object).getFieldValue(name);
				type = property.getClass().getName();
			}
			else {
				property = PropertyUtils.getProperty(object, name);
				type = PropertyUtils.getPropertyType(object, name).getName();
			}
			//System.out.println(name + " has type " + type);
			Object newValue; 
			if (type.equals(TYPE_BASE_DOUBLE) || type.equals(TYPE_DOUBLE)) {
				newValue = new Double(getDoubleValue());
			} else if (type.equals(TYPE_STRING)) {
				newValue = getStringValue();
			} else if (type.equals(TYPE_BASE_INT) || type.equals(TYPE_INT)) {
				newValue = new Integer(getIntValue());
			} else if (type.equals(TYPE_BASE_BOOL) || type.equals(TYPE_BOOL)) {
				newValue = new Boolean(isBooleanValue());
			} else if (type.equals(TYPE_DATE)) {
				newValue = getDateValue();
			} else {
				newValue = getObjectValue();
				//System.out.println("ERROR in DataMap.get(" + object.toString() + "," + name + ")");
			}
			if (dynamic)
				((DynamicObject)object).setFieldValue(name, newValue);
			else
				PropertyUtils.setSimpleProperty(object,name,newValue);
		} catch (Exception e) {
			String err = "ERROR in DataMap.set(" + object.toString() + "," + name + ")";
			Log.error(err);
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
	
	public final void set(Object object, String name, int index) {
		try {
			Object[] array = new Object[] {};
			Object property;
			String type;
			boolean dynamic = false;
			if (object instanceof DynamicObject) {
				dynamic = true;
				array = (Object[]) ((DynamicObject)object).getFieldValue(name);
				property = array[index];
			}
			else {
				property = PropertyUtils.getIndexedProperty(object, name, index);
			}
			type = property.getClass().getName();
			//System.out.println(name + " has type " + type);
			Object newValue; 
			if (type.equals(TYPE_BASE_DOUBLE) || type.equals(TYPE_DOUBLE)) {
				newValue = new Double(getDoubleValue());
			} else if (type.equals(TYPE_BIG_DECIMAL)) {
				newValue = new BigDecimal((String) property);
			} else if (type.equals(TYPE_STRING)) {
				newValue = getStringValue();
			} else if (type.equals(TYPE_BASE_INT) || type.equals(TYPE_INT)) {
				newValue = new Integer(getIntValue());
			} else if (type.equals(TYPE_BASE_LONG) || type.equals(TYPE_LONG)) {
				newValue = new Long(getLongValue());
			} else if (type.equals(TYPE_BASE_BOOL) || type.equals(TYPE_BOOL)) {
				newValue = new Boolean(isBooleanValue());
			} else if (type.equals(TYPE_DATE)) {
				newValue = getDateValue();
			} else {
				newValue = getObjectValue();
				//System.out.println("ERROR in DataMap.get(" + object.toString() + "," + name + ")");
			}
			if (dynamic) {
				//((DynamicObject)object).setFieldValue(name, newValue);
				Object elementIndex = array[index]; 
				((DynamicObject)elementIndex).setFieldValue(name, newValue);
			}
			else {
				//PropertyUtils.setSimpleProperty(object,name,newValue);
				PropertyUtils.setIndexedProperty(object, name, index, newValue);
			}
		} catch (Exception e) {
			String err = "ERROR in DataMap.set(" + object.toString() + "," + name + "," + index + ")";
			Log.error(err);
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}
	
	
	public final void show(Object object, String property) {
		setName(property);
		get(object, property);
		Log.info(toString());
	}
	
	public final void show(Object object, String property, int index) {
		setName(property + "["+index+"]");
		get(object, property, index);
		Log.info(toString());
	}

//	public static void main(String[] args) {
//
//		Customer customer = new Customer();
//		customer.setName("Robinson");
//		customer.setGender("Female");
//		customer.setMaritalStatus("Married");
//		customer.setAge(20);
//		customer.setSalary(20000);
//		customer.setGender("Male");
//		
//		DataMap getMap = new DataMap();
//		getMap.show(customer, "age");
//		getMap.show(customer, "maritalStatus");
//		getMap.show(customer, "salary");
//		getMap.show(customer, "dob");
//		getMap.show(customer, "retired");
//		getMap.show(customer, "gender");
//		getMap.show(customer, "name");
//		
//		DataMap map2 = new DataMap();
//		map2.show(customer, "age");
//		map2.setIntValue(60);
//		map2.set(customer,"age");
//		map2.show(customer, "age");
//	}
	
	public final String getName() {
		return name;
	}

	public final void setName(String name) {
		this.name = name;
	}

	public final int getIntValue() {
		if (!isIntValueDefined()) {
			String msg = "Integer variable " + name + " is not defined";
			//Log.info(msg);
			throw new RuntimeException(msg);
		}
		return intValue;
	}

	public final void setIntValue(int intValue) {
		this.intValue = intValue;
	}
	
	public final long getLongValue() {
		if (!isLongValueDefined()){
			String msg = "Long variable " + name + " is not defined";
			//Log.info(msg);
			throw new RuntimeException(msg);
		}
		return longValue;
	}

	public final void setLongValue(long longValue) {
		this.longValue = longValue;
	}

	public final String getStringValue() {
		return stringValue;
	}

	public final void setStringValue(String stringValue) {
		this.stringValue = stringValue;
	}

	public final double getDoubleValue() {
		if (!isDoubleValueDefined()) {
			String msg = "Real Variable " + name + " is not defined";
			//Log.info(msg);
			throw new RuntimeException(msg);
		}
		return doubleValue;
	}
	
	public final BigDecimal getBigDecimalValue() {
//		if (!isBigDecimalValueDefined())
//			Log.info("BigDecimal Variable " + name + " is not defined");
		return bigDecimalValue;
	}

	public final boolean isBooleanValue() {
		return booleanValue;
	}

	public final void setBooleanValue(boolean booleanValue) {
		this.booleanValue = booleanValue;
	}

	public final Date getDateValue() {
		return dateValue;
	}

	public final void setDateValue(Date dateValue) {
		this.dateValue = dateValue;
	}

	public final void setDoubleValue(double doubleValue) {
		this.doubleValue = doubleValue;
	}
	
	public final void setBigDecimalValue(BigDecimal value) {
		this.bigDecimalValue = value;
	}
	
	public final boolean isIntValueDefined() {
		return intValue != Integer.MIN_VALUE;
	}
	
	public final boolean isLongValueDefined() {
		return longValue != Long.MIN_VALUE;
	}
	
	public final boolean isDoubleValueDefined() {
		return doubleValue != Double.NEGATIVE_INFINITY;
	}
	
	public final Object getObjectValue() {
		return objectValue;
	}

	public final void setObjectValue(Object objectValue) {
		this.objectValue = objectValue;
	}

	public final String toString() {
		if (stringValue != null) 
			return name + " has value " + stringValue;
		if (isIntValueDefined()) 
			return name + " has value " + intValue;
		if (isDoubleValueDefined())
			return name + " has value " + doubleValue;
		if (dateValue != null) 
			return name + " has value " + dateValue;
		if (objectValue != null) 
			return name + " has value " + objectValue;
		return name + " has value " + booleanValue;
	}
	
	public static boolean compareTwoObjects(Object obj1, Object obj2, String propertyName) {
		try {
			Object property1, property2;
			String type;
			if (obj1 instanceof DynamicObject) {
				property1 = ((DynamicObject)obj1).getFieldValue(propertyName);
				type = property1.getClass().getName();
				property2 = ((DynamicObject)obj2).getFieldValue(propertyName);
			}
			else {
				property1 = PropertyUtils.getProperty(obj1, propertyName);
				type = PropertyUtils.getPropertyType(obj1, propertyName).getName();
				property2 = PropertyUtils.getProperty(obj2, propertyName);
			}
			//System.out.println(propertyName + " has type " + type);
			Oper op = new Oper("=");	
			if (type.equals(TYPE_BASE_DOUBLE) || type.equals(TYPE_DOUBLE)) {
				double d1 = ((Double)property1).doubleValue();
				double d2 = ((Double)property2).doubleValue();
				return op.compare(d1, d2);
			} else if (type.equals(TYPE_BIG_DECIMAL)) {
				BigDecimal bd1 = (BigDecimal) property1;
				BigDecimal bd2 = (BigDecimal) property2;
				return op.compare(bd1, bd2);
			} else if (type.equals(TYPE_STRING)) {
				String s1 = (String) property1;
				String s2 = (String) property2;
				return op.compare(s1, s2);
			} else if (type.equals(TYPE_BASE_INT) || type.equals(TYPE_INT)) {
				int i1 = ((Integer)property1).intValue();
				int i2 = ((Integer)property2).intValue();
				return i1 == i2;
			} else if (type.equals(TYPE_BASE_LONG) || type.equals(TYPE_LONG)) {
				long l1 = ((Long)property1).longValue();
				long l2 = ((Long)property2).longValue();
				return l1 == l2;
			} else if (type.equals(TYPE_BASE_BOOL) || type.equals(TYPE_BOOL)) {
				boolean b1 = ((Boolean)property1).booleanValue();
				boolean b2 = ((Boolean)property1).booleanValue();
				return op.compare(b1, b2);
			} else if (type.equals(TYPE_DATE)) {
				Date d1 = (Date) property1;
				Date d2 = (Date) property1;
				return op.compare(d1, d2);
			} else {
				String err = "ERROR in DataMap.compareTwoObjects: unknown type of the object " 
			        + obj1.toString();
				throw new RuntimeException(err);
			}
		} catch (Exception e) {
				String err = "ERROR in DataMap.compareTwoObjects by property " + propertyName
						+ "\n" + obj1.toString() 
						+ "\n" +obj2.toString();
				Log.error(err);
				e.printStackTrace();
				throw new RuntimeException(e);
		}
	}
	

}
