package com.openrules.types;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;

import org.apache.commons.lang.time.DateUtils;

public class  DateUtil {
	static Locale defaultLocale = Locale.getDefault();
//	static Locale defaultLocale = new Locale("Cs", "CZ");
	static final String COMPLEX_DATE_FORMAT = "EEE MMM dd HH:mm:ss zzz yyyy";
	static DateFormat DateFormatComplex = new SimpleDateFormat(COMPLEX_DATE_FORMAT,defaultLocale);
	static DateFormat DateFormatMMDDYYYY = new SimpleDateFormat("MM/dd/yyyy",defaultLocale);
	static DateFormat DateFormatShort = DateFormat.getDateInstance(DateFormat.SHORT,defaultLocale);
	static DateFormat DateFormatSimple = new SimpleDateFormat("MM/dd/yy HH:mm",defaultLocale);
	
	static DateFormat DateFormatYYYYMMDD = new SimpleDateFormat("yyyy-MM-dd",defaultLocale);

	//static DateFormat fullDateFormat = DateFormat.getDateInstance(DateFormat.FULL);
	
	public static Object parse(String value) {
		try {
			//DateFormat simpleDateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
			return DateFormatComplex.parse(value);
		} catch (Exception e) {
			//return dateFormat.parse(value);
			return getDateFromString(value);
		}
	}
	
	public static Date cleanTime(Date date) {
		Calendar cal = Calendar.getInstance();  
		cal.setTime(date); 		  
		// Set time fields to zero  
		cal.set(Calendar.HOUR_OF_DAY, 0);  
		cal.set(Calendar.MINUTE, 0);  
		cal.set(Calendar.SECOND, 0);  
		cal.set(Calendar.MILLISECOND, 0);  
		// Put it back in the Date object  
		return cal.getTime();
	}
	
	public static Date getDateFromString(String value) {
		Date date;
		try { // try first if the value defined as a text like 10/19/1979
			//DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
			date = DateFormatSimple.parse(value);
		} catch (Exception e) {
			try {
				// probably value was defined as a Date in Excel 
				// see also String2DateConvertor in String2DataConvertorFactory
				int dateValue = Integer.parseInt(value);
				Calendar cc = Calendar.getInstance();
				cc.set(1900, 0, 1);
				//cc.add(Calendar.DATE, value - 1);
				cc.add(Calendar.DATE, dateValue - 2); 
				// JF: changed to -2 to fix a notorious Excel's error considering 1900 as a leap year
				date = cc.getTime();
			} catch (Exception e2) {
				try {
					//DateFormat simpleDateFormat = new SimpleDateFormat(SIMPLE_DATE_FORMAT);
					date = DateFormatComplex.parse(value);
				} 
				catch (Exception e3) {
					try {
						date = DateFormatMMDDYYYY.parse(value);
					} catch (Exception e4) {
						try {
							date = DateFormatShort.parse(value);
						} catch (Exception e5) {
							try {
								date = DateFormatYYYYMMDD.parse(value);
							} catch (Exception e6) {
								throw new RuntimeException(e6);
							}
						}
					}
				}
			}
		}
		return date;
	}
	
	public static String getStringFromDate(Date date) {
		//DateFormat dateFormat = DateFormat.getDateInstance(DateFormat.SHORT);
		String value = DateFormatMMDDYYYY.format(date);
		return value;
	}
	
	public Date addDays(Date date, int days) {
		return DateUtils.addDays(date, days);
	}
	
	public static void main(String[] args) {
		
		Locale defaultLocale = Locale.getDefault();
		System.out.println("Default locale name: " + defaultLocale 
				           + " country="+defaultLocale.getDisplayCountry());
		
		String value = "01/01/2100";
		Date d1 = DateUtil.getDateFromString(value);
		String v1 = DateUtil.getStringFromDate(d1);
		System.out.println(value + " = " + d1 + " v1=" + v1);
		
		
		
		
		value = "Thu Dec 31 15:56:48 EST 2015";
		Date d2 = (Date)DateUtil.parse(value);
		System.out.println(value + " = " + d2);
		value = "2015-1-15";
		Date d3 = (Date)DateUtil.parse(value);
		System.out.println(value + " = " + d3);
		value = "1/15/2015 13:45";
		Date d4 = (Date)DateUtil.parse(value);
		System.out.println(value + " = " + d4);
		
		String czDate = "15.01.1972"; //"30/06/09";
//		Default locale fails
//		Date d5 = (Date)DateUtil.parse(czDate);
//		System.out.println(czDate + " = " + d5);
		
		Locale locale = new Locale("Cs", "CZ");
		DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT,locale);
		System.out.println("Locale name : " + locale + " country="+locale.getDisplayCountry());
		
		//Locale locale = new Locale("french", "FR"); 
//		Locale locale = new Locale("it", "IT");
//		DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT,locale);
//		System.out.println("Locale name : " + locale + " country="+locale.getDisplayCountry());
		
		try {
			Date d = df.parse(czDate);
			System.out.println(d);
		} catch (Exception e) {
			System.out.println(e);
		}
		
		
		
	}
}