package com.openrules.types;


/*
 * Copyright (C) 2007 OpenRules, Inc.
 */

/**
 * This type is used to represent possible domains of Strings used inside rulesInMemory
 * tables. The domain may be represented by strings like "mom son daugther sister".
 * The expected divider is a whitespace.
 * 
 */

public class DomainString extends DomainType {

	public DomainString(String domain) {
		this(domain,"\\s");
	}
	
	public DomainString(String domain, String delim) {
		super(domain,delim);
	}

	/**
	 * Returns true if the domain contains the text passed as a parameter.
	 * The comparison is case sensitive
	 * 
	 * @param String
	 * @return true or false
	 */
	public boolean contains(String text) {
		for (int i = 0; i < array.length; i++) {
			String token = array[i];
			if (token.equals(""))
				continue;
			if (token.equals(text))
				return true;
		}
		return false;
	}
	
	/**
	 * Returns true if the domain contains the text passed as a parameter.
	 * The comparison is NOT case sensitive
	 * 
	 * @param String
	 * @return true or false
	 */
	public boolean containsIgnoreCase(String text) {
		for (int i = 0; i < array.length; i++) {
			String token = array[i];
			if (token.equals(""))
				continue;
			if (token.equalsIgnoreCase(text))
				return true;
		}
		return false;
	}
	
	public boolean validate() {
		return true;
	}
	
	public String[] getElements() {
		return array;
	}
	
	public static String toString(String[] array) {
		char delim = ',';
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < array.length; i++) {
			if (i>0)
				buf.append(delim);
			buf.append(array[i]);
		}
		return buf.toString();
	}
	
	public boolean intersectsWith(DomainString domain) {
		String[] strings = getElements();
		for (int i = 0; i < strings.length; i++) {
			if (domain.contains(strings[i]))
				return true;
		}
		return false;
	}

	/**
	 * TEST
	 */
	public static void main(String[] args) {
		String[] domains = { "5  son daugther 	sisters", " ", "son", " A B C" };
		String[] texts = new String[] { "son", "sisters", "c" };

		for (int i = 0; i < domains.length; i++) {
			DomainString domain = new DomainString(domains[i]);
			for (int j = 0; j < texts.length; j++) {
				String result = " contains ";
				if (!domain.containsIgnoreCase(texts[j]))
					result = " does not contain ";
				System.out.println("Domain \"" + domain + "\"" + result
						+ texts[j]);
			}
		}
		
		DomainString domain = new DomainString(domains[0]);
		System.out.println("Elements in domain: "+domain);
		String[] elements = domain.getElements();
		for (int i = 0; i < elements.length; i++) {
			System.out.println(elements[i]);
		}
		
		System.out.println(DomainString.toString(texts));
		
		DomainString domain2 = new DomainString(domains[2]); 
		System.out.println("Domain <" + domain + "> intersects with domain <" + domain2 + ">: " +
				domain.intersectsWith(domain2));
	}
}
