package com.openrules.types;

import java.text.DateFormat;
import java.util.Arrays;
import java.util.Date;

import org.openl.util.Log;

/*
 * Copyright (c) 2005-2012 OpenRules, Inc. 	
 * Author: JF
 * Aug. 26, 2014: added Exclude All
 */

public class Oper {
	
	static public final String EQ = "=";
	static public final String LT = "<";
	static public final String LE = "<=";
	static public final String GT = ">";
	static public final String GE = ">=";
	static public final String NE = "!=";
	static public final String WITHIN = "WITHIN";
	static public final String OUTSIDE = "OUTSIDE";
	static public final String ISONEOF = "ISONEOF";
	static public final String ISNOTONEOF = "ISNOTONEOF";
	static public final String INCLUDE = "INCLUDE";
	static public final String EXCLUDE = "EXCLUDE";
	static public final String DOES_NOT_INCLUDE = "DOESNOTINCLUDE"; 
	static public final String EXCLUDE_ALL = "EXCLUDEALL";
	static public final String INTERSECT = "INTERSECT";
	static public final String ARE = "ARE";
	static public final String ADD = "ADD";
	static public final String ASSIGN_PLUS = "ASSIGNPLUS";
	static public final String ASSIGN_MINUS = "ASSIGNMINUS";
	static public final String ASSIGN_MULTIPLY = "ASSIGNMULTIPLY";
	static public final String ASSIGN_DIVIDE = "ASSIGNDIVIDE";
	static public final String ISTRUE = "ISTRUE";
	static public final String ISFALSE = "ISFALSE";
	static public final String ISEMPTY = "ISEMPTY";
	static public final String STARTSWITH = "STARTSWITH";
	static public final String CONTAINS = "CONTAINS";
	static public final String DOES_NOT_CONTAIN = "DOESNOTCONTAIN";
	static public final String MATCH = "MATCH";
	static public final String NO_MATCH = "NO MATCH";
	static public final String MAX = "MAX";
	static public final String MIN = "MIN";
	static public final String AVERAGE = "AVERAGE";
	static public final String COUNT = "COUNT";
	static public final String SUM = "SUM";
	static public final String PRODUCT = "PRDUCT";
	
	String op;
	boolean compareTwoArrays;
	boolean ignoreTime;
	String id;
	String delimiter; // for Add and Add Separated by
	ComparableOperator operator;
	
	public Oper(String op) {
//		System.out.println("Oper: "+op);
		this.op = op;
		id = op;
		compareTwoArrays = false;
		ignoreTime = true;
		// remove the ending "time"
		int timeIndex = op.indexOf("time");
		if (timeIndex > 0) {
			op = op.substring(0, timeIndex);
			ignoreTime = false;
		}
		String s = removeSpaces(op); //op.replaceAll(" ", "");
		
//		System.out.println("Compressed Oper: "+s);
		if (OperName.isLT(s)) {
			operator = OperLT;
			id = LT; 
		}
		else if (OperName.isLE(s)) {
			operator = OperLE;
			id = LE;
		}
		else if (OperName.isEQ(s)) {
			operator = OperEQ;
			id = EQ;
		}
		else if (OperName.isNE(s)) {
			operator = OperNE;
			id = NE;
		}
		else if (OperName.isGT(s)) {
			operator = OperGT;
			id = GT;
		}
		else if (OperName.isGE(s)) {
			operator = OperGE;
			id = GE;
		}
		else if (OperName.isStartWith(s)) {
			operator = OperSTARTS_WITH;
			id = STARTSWITH;
		}
		else if (OperName.isEmpty(s)) {
			operator = OperIsEmpty;
			id = ISEMPTY;
		}
		else if (OperName.isContains(s)) {
			operator = OperCONTAINS;
			id = CONTAINS;
		}
		else if (OperName.isDoesNotContain(s)) {
			operator = OperDOES_NOT_CONTAIN;
			id = DOES_NOT_CONTAIN;
		}
		else if (OperName.isMatch(s)) {
			operator = OperMATCH;
			id = MATCH;
		}
		else if (OperName.isNoMatch(s)) {
			operator = OperNO_MATCH;
			id = NO_MATCH;
		}
		else if (OperName.isWithin(s)) {
			operator = OperWITHIN;
			id = WITHIN;
		}
		else if (OperName.isOutside(s)) {
			operator = OperOUTSIDE;
			id = OUTSIDE;
		}
		else if (OperName.isOneOf(s)) {
			operator = OperIsOneOf;
			id = ISONEOF;
			defineDelimiterFor(op);
		}
		else if (OperName.isNotOneOf(s)) {
			operator = OperIsNotOneOf;
			id = ISNOTONEOF;
			defineDelimiterFor(op);
		}
		else if (OperName.isInclude(s)) {
			operator = OperINCLUDE;
			compareTwoArrays = true;
			id = INCLUDE;
			defineDelimiterFor(op);
		}
		else if (OperName.isDoesNotInclude(s)) {
			operator = OperDoesNotInclude;
			compareTwoArrays = true;
			id = DOES_NOT_INCLUDE;
			defineDelimiterFor(op);
		}
		else if (OperName.isExcludeAll(s)) {
			operator = OperExcludeAll;
			compareTwoArrays = true;
			id = EXCLUDE_ALL;
			defineDelimiterFor(op);
		}
		else if (OperName.isExclude(s)) {
			operator = OperEXCLUDE;
			compareTwoArrays = true;
			id = EXCLUDE;
			defineDelimiterFor(op);
		}
		else if (OperName.isIntersect(s)) {
			operator = OperINTERSECT;
			compareTwoArrays = true;
			id = INTERSECT;
			defineDelimiterFor(op);
		}
		else if (OperName.isTrue(s)) {
			id = ISTRUE;
		} 
		else if (OperName.isFalse(s)) {
			id = ISFALSE;
		}
		else if (OperName.isAre(s)) {
			operator = OperARE;
			id = ARE;
			defineDelimiterFor(op);
//			if (s.length() == 3)
//				delimiter = ",";
//			else 
//				delimiter = s.substring(s.length()-1);
		}
		else if (OperName.isAdd(s)) {
			operator = OperADD;
			id = ADD;
			defineDelimiterFor(op);
//			if (s.length() == 3)
//				delimiter = ",";
//			else 
//				delimiter = s.substring(s.length()-1);
		}
		else if (OperName.isAssignPlus(s)) {
			operator = OperAssignPlus;
			id = ASSIGN_PLUS;
		}
		else if (OperName.isAssignMinus(s)) {
			operator = OperAssignMinus;
			id = ASSIGN_MINUS;
		}
		else if (OperName.isAssignMultiply(s)) {
			operator = OperAssignMultiply;
			id = ASSIGN_MULTIPLY;
		}
		else if (OperName.isAssignDivide(s)) {
			operator = OperAssignDivide;
			id = ASSIGN_DIVIDE;
		}
		else if (OperName.isMax(s)) {
			operator = OperMax;
			id = MAX;
		}
		else if (OperName.isMin(s)) {
			operator = OperMin;
			id = MIN;
		}
		else if (OperName.isAverage(s)) {
			operator = OperAverage;
			id = AVERAGE;
		}
		else if (OperName.isCount(s)) {
			operator = OperCount;
			id = COUNT;
		}
		else if (OperName.isSum(s)) {
			operator = OperSum;
			id = SUM;
		}
		else if (OperName.isProduct(s)) {
			operator = OperProduct;
			id = PRODUCT;
		}
		else
			throw (new RuntimeException("Oper " + op + " is not defined"));
	}
	
	void defineDelimiterFor(String s) {
		s = s.trim();
		char space = s.charAt(s.length()-2);
		if (space == ' ')
			delimiter = s.substring(s.length()-1);
		else
			delimiter = ",";
	}

	public boolean isCompoundAssignment() {
		if (id.equals(ASSIGN_PLUS) || id.equals(ASSIGN_MINUS) ||
			id.equals(ASSIGN_MULTIPLY) || id.equals(ASSIGN_DIVIDE))
			return true;
		return false;
	}
	
	public boolean isAssignPlus() {
		return id.equals(ASSIGN_PLUS);
	}
	
	public boolean isAssignMinus() {
		return id.equals(ASSIGN_MINUS);
	}
	
	public boolean isAssignMultiply() {
		return id.equals(ASSIGN_MULTIPLY);
	}
	
	public boolean isAssignDivide() {
		return id.equals(ASSIGN_DIVIDE);
	}
	
	public boolean isArrayOperator() {
		return OperName.isArrayOperator(id);
	}
	
	public boolean isEmpty() {
		return id.equals(ISEMPTY);
	}
	
	public boolean isMax() {
		return id.equals(MAX);
	}
	public boolean isMin() {
		return id.equals(MIN);
	}
	
	public String getId() {
		return id;
	}
	
	public String getName() {
		return op;
	}
	
	public boolean is(String id) {
		return this.id.equals(id);
	}

	public boolean compare(Comparable c1, Comparable c2) {
		return operator.compare(c1, c2);
	}

	public boolean compare(int i1, int i2) {
		return operator.compare(new Integer(i1), new Integer(i2));
	}
	
	public boolean compare(int one, int[] many) {
		for (int i = 0; i < many.length; i++) {
			if (one == many[i])
				return true;
		}
		return false;
	}
	
	boolean isIn(int a, int[] array) {
		for (int i = 0; i < array.length; i++) {
			if (a == array[i])
				return true;
		}
		return false;
	}
	boolean isIn(double a, double[] array) {
		for (int i = 0; i < array.length; i++) {
			if (Math.abs(a - array[i]) < precision)
				return true;
		}
		return false;
	}
	boolean isIn(long a, long[] array) {
		for (int i = 0; i < array.length; i++) {
			if (a == array[i])
				return true;
		}
		return false;
	}
	
	public boolean compare(int[] arr1, int[] arr2) {
		if (id.equals(INCLUDE)) { // arr1 includes arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					continue;
				else
					return false;
			}
			return true;
		}
		if (id.equals(EXCLUDE)) { // arr1 does not includes arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (!isIn(arr2[i],arr1))
					return true;
			}
			return false;
		}
		if (id.equals(EXCLUDE_ALL)) { // arr1 doesn't include any element of arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					return false;
			}
			return true;
		}
		if (id.equals(INTERSECT)) { // arr1 has common elements with arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					return true;
			}
			return true;
		}
		Log.info("Invalid operator " + op + " for arrays");
		return false;
	}
	
	public boolean compare(double[] arr1, double[] arr2) {
		if (id.equals(INCLUDE)) { // arr1 includes arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					continue;
				else
					return false;
			}
			return true;
		}
		if (id.equals(EXCLUDE)) { // arr1 does not includes arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (!isIn(arr2[i],arr1))
					return true;
			}
			return false;
		}
		if (id.equals(EXCLUDE_ALL)) { // arr1 doesn't include any element of arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					return false;
			}
			return true;
		}
		if (id.equals(INTERSECT)) { // arr1 has common elements with arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					return true;
			}
			return true;
		}
		Log.info("Invalid operator " + op + " for arrays");
		return false;
	}

	public boolean compare(long[] arr1, long[] arr2) {
		if (id.equals(INCLUDE)) { // arr1 includes arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					continue;
				else
					return false;
			}
			return true;
		}
		if (id.equals(EXCLUDE)) { // arr1 does not includes arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (!isIn(arr2[i],arr1))
					return true;
			}
			return false;
		}
		if (id.equals(EXCLUDE_ALL)) { // arr1 doesn't include any element of arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					return false;
			}
			return true;
		}
		if (id.equals(INTERSECT)) { // arr1 has common elements with arr2?
			for (int i = 0; i < arr2.length; i++) {
				if (isIn(arr2[i],arr1))
					return true;
			}
			return true;
		}
		Log.info("Invalid operator " + op + " for arrays");
		return false;
	}
	
	public boolean compare(long one, long[] many) {
		for (int i = 0; i < many.length; i++) {
			if (one == many[i])
				return true;
		}
		return false;
	}

	public boolean compare(long l1, long l2) {
		return operator.compare(new Long(l1), new Long(l2));
	}
	
	public boolean compare(String s1, boolean b2) {
		boolean b1 = (s1 == null) || (s1.trim().length() == 0);
		return b1 == b2;
	}
	
	public boolean compare(boolean b1, boolean b2) {
		return operator.compare(new Boolean(b1), new Boolean(b2)); // since 1.5
//		if (id.equals(EQ_ID)) {
//			if (b1 && b2)
//				return true;
//			if (!b1 && !b2)
//				return true;
//			return false;
//		}
//		else  { // id.equals(NEQ_ID) 
//			if (b1 && !b2)
//				return true;
//			if (!b1 && b2)
//				return true;
//			return false;
//		}
	}
	
	public boolean compare(boolean boolValue) {
		if (id.equals(ISTRUE))
			return boolValue == true;
		else
			return boolValue == false;
	}

	static public double precision = 0.00000001;
	public boolean compare(double d1, double d2) {
		if (EQ.equals(op))
			return (Math.abs(d1 - d2) < precision);
		if (NE.equals(op))
			return (Math.abs(d1 - d2) >= precision);
		return operator.compare(new Double(d1), new Double(d2));
	}
	
	public boolean compare(double one, double[] many) {
		for (int i = 0; i < many.length; i++) {
			if (Math.abs(one - many[i]) < precision)
				return true;
		}
		return false;
	}

	public boolean compare(Date d1, Date d2) {
		// We need this to avoid milliseconds that could fail an equality
		// comparison of two dates
		// Otherwise the entire method could be omitted
//		Date dd1 = d1;
//			//new Date(d1.getYear(), d1.getMonth(), d1.getMinutes(), d1.getHours(), d1.getMinutes());
//		Date dd2 = d2;
//			//new Date(d2.getYear(), d2.getMonth(), d2.getMinutes(), d2.getHours(), d2.getMinutes());
		  
//		d1.setHours(0);d1.setMinutes(0);d1.setSeconds(0);
//		d2.setHours(0);d2.setMinutes(0);d2.setSeconds(0);
		Date dd1 = d1;
		Date dd2 = d2;
		if (ignoreTime) {
			dd1 = DateUtil.cleanTime(d1);
			dd2 = DateUtil.cleanTime(d2);
		}
		boolean result = operator.compare(dd1, dd2);
		//String ft = "yyyy-MM-dd'T'HH:mm:ss.SSSZ";
		//DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
//		SimpleDateFormat df = new SimpleDateFormat();
//		System.out.println("Compare " +
//				" d1=" + df.format(d1) + " " + op + " " + " d2=" + df.format(d2) +
//				" --- " +result);
		return result;
	}
	
	public boolean compare(Comparable one, Comparable[] many) {
		return operator.compare(one, many);
	}
	
	public boolean compare(Comparable[] array1, Comparable[] array2) {
		return operator.compare(array1, array2);
	}
	
	public final boolean isCompareTwoArrays() {
		return compareTwoArrays;
	}

	public final void setCompareTwoArrays(boolean compareTwoArrays) {
		this.compareTwoArrays = compareTwoArrays;
	}
	
	public String toString() {
		return op;
	}
	
	public boolean validateInt(String domain, String value) {
		boolean result = true;
		FromToInt domainInterval = null;
		try {
			domainInterval = new FromToInt(domain);
		} 
		catch (Exception e) {
			Log.error("Invalid domain: " + domain + ". It should be a valid integer interval");
		}
		if (this.is(Oper.WITHIN)) {
			FromToInt valueInterval = new FromToInt(value);
			if (valueInterval.getNumberOfBounds() < 2) {
				Log.error("Invalid expression: " + value 
					+ " - both lower and upper bounds should be defined for an integer interval");
				result = false;
			}
			else // == 2
			if (domainInterval != null && !domainInterval.intersectsWith(valueInterval)) {
				Log.error("Interval <" + value 
						+ "> does not intersect with the domain <" + domain + ">");
				result = false;
			}
		}
		else 
		{
			int val = Integer.parseInt(value);
			if (domainInterval != null && !domainInterval.contains(val)) {
				Log.error("Value <" + value 
						+ "> is not inside the domain <" + domain + ">");
				result = false;
			}
		}
		return result;
	}
	
	public boolean validateLong(String domain, String value) {
		boolean result = true;
		FromToLong domainInterval = null;
		try {
			domainInterval = new FromToLong(domain);
		} 
		catch (Exception e) {
			Log.error("Invalid domain: " + domain + ". It should be a valid long interval");
		}
		if (this.is(Oper.WITHIN)) {
			FromToLong valueInterval = new FromToLong(value);
			if (valueInterval.getNumberOfBounds() < 2) {
				Log.error("Invalid expression: " + value 
					+ " - both lower and upper bounds should be defined for an long interval");
				result = false;
			}
			else // == 2
			if (domainInterval != null && !domainInterval.intersectsWith(valueInterval)) {
				Log.error("Interval <" + value 
						+ "> does not intersect with the domain <" + domain + ">");
				result = false;
			}
		}
		else 
		{
			long val = Long.parseLong(value);
			if (domainInterval != null && !domainInterval.contains(val)) {
				Log.error("Value <" + value 
						+ "> is not inside the domain <" + domain + ">");
				result = false;
			}
		}
		return result;
	}
	
	public boolean validateDouble(String domain, String value) {
		boolean result = true;
		FromToDouble domainInterval = null;
		try {
			domainInterval = new FromToDouble(domain);
		} 
		catch (Exception e) {
			Log.error("Invalid domain: " + domain + ". It should be a valid double interval");
		}
		if (this.is(Oper.WITHIN)) {
			FromToDouble valueInterval = new FromToDouble(value);
			if (valueInterval.getNumberOfBounds() < 2) {
				Log.error("Invalid expression: " + value 
					+ " - both lower and upper bounds should be defined for an double interval");
				result = false;
			}
			else // == 2
			if (domainInterval != null && !domainInterval.intersectsWith(valueInterval)) {
				Log.error("Interval <" + value 
						+ "> does not intersect with the domain <" + domain + ">");
				result = false;
			}
		}
		else {
			double val = Double.parseDouble(value);
			if (domainInterval != null && !domainInterval.contains(val)) {
				Log.error("Value <" + value 
						+ "> is not inside the domain <" + domain + ">");
				result = false;
			}
		}
		return result;
	}
	
	public boolean validateBool(String domain, String value) {
		return validateString(domain,value);
	}
	
	public boolean validateString(String domain, String value) {
		// validate expression <factNameVar> <oper> <value>
		if (domain != null) { 
			DomainString d1 = new DomainStringC(domain);
			DomainString d2 = new DomainStringC(value);
			if (!d1.intersectsWith(d2)) {
				Log.error("Domain <" + value 
						+ "> does not intersect with the domain <" + domain + ">");
				return false;
			}
		}
		return true;
	}

	//static interface ComparableOperator {
	static abstract class ComparableOperator {
		public boolean compare(Comparable c1, Comparable c2) {
			throw new RuntimeException("Oper " + this + 
			": Method compare(Comparable c1, Comparable c2) not defined");
		}
		public boolean compare(Comparable c1, Comparable[] c2) { 
			throw new RuntimeException("Oper " + this + 
				": Method compare(Comparable c1, Comparable[] c2) not defined");
		}
		
		public boolean compare(Comparable[] c1, Comparable[] c2) { 
			throw new RuntimeException("Oper " + this + 
				": Method compare(Comparable[] c1, Comparable[] c2) not defined");
		}
		
	}
	
	static ComparableOperator OperLT = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			if (c1==null && c2==null)
				return true;
			if (c1==null && c2!=null)
				return false;
			return c1.compareTo(c2) < 0;
		}
	};

	static ComparableOperator OperLE = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			if (c1==null && c2==null)
				return true;
			if (c1==null && c2!=null)
				return false;
			return c1.compareTo(c2) <= 0;
		}
	};

	static ComparableOperator OperGE = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			if (c1==null && c2==null)
				return true;
			if (c1==null && c2!=null)
				return false;
			return c1.compareTo(c2) >= 0;
		}
	};

	static ComparableOperator OperGT = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			if (c1==null && c2==null)
				return true;
			if (c1==null && c2!=null)
				return false;
			return c1.compareTo(c2) > 0;
		}
	};

	static ComparableOperator OperEQ = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			if (c1==null && c2==null)
				return true;
			if (c1==null && c2!=null)
				return false;
			return c1.compareTo(c2) == 0;
		}
	};

	static ComparableOperator OperNE = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			if (c1==null && c2==null)
				return true;
			if (c1==null && c2!=null)
				return false;
			return c1.compareTo(c2) != 0;
		}
	};
	
	static ComparableOperator OperWITHIN = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			String interval = (String)c2;
			if (c1 instanceof Integer) {
				int value = ((Integer)c1).intValue();
				FromToInt fromTo = new FromToInt(interval);
				return fromTo.contains(value);
			} else if (c1 instanceof Double) {
				double value = ((Double)c1).doubleValue(); 
				FromToDouble fromTo = new FromToDouble(interval);
				return fromTo.contains(value);
			} else {
				throw new RuntimeException("Oper WITHIN can compare int/double with from-to intervals only");
			}
		}
	};
	
	static ComparableOperator OperOUTSIDE = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			String interval = (String)c2;
			if (c1 instanceof Integer) {
				int value = ((Integer)c1).intValue();
				FromToInt fromTo = new FromToInt(interval);
				return !fromTo.contains(value);
			} else if (c1 instanceof Double) {
				double value = ((Double)c1).doubleValue(); 
				FromToDouble fromTo = new FromToDouble(interval);
				return !fromTo.contains(value);
			} else {
				throw new RuntimeException("Oper OUTSIDE can compare int/double with from-to intervals only");
			}
		}
	};
	
	static ComparableOperator OperSTARTS_WITH = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			String string1 = ((String)c1).toLowerCase();
			String string2 = ((String)c2).toLowerCase();
			return string1.startsWith(string2);
		}
	};
	
	static ComparableOperator OperIsEmpty = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			if (c2 == null)
				c2 ="true";
			String s2 = (String) c2;
			if ("true".equalsIgnoreCase(s2)) {
				if (c1 == null)
					return true;
				String s1 = (String)c1;
				if (s1.trim().length() == 0)
					return true;
				return false;
			}
			else { // s2 = false
				if (c1 == null)
					return false;
				String s1 = (String)c1;
				if (s1.trim().length() == 0)
					return false;
				return true;
			}
		}
	};
	
	static ComparableOperator OperCONTAINS = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			String string1 = ((String)c1).toLowerCase();
			String string2 = ((String)c2).toLowerCase();
			return string1.contains(string2);
		}
	};
	
	static ComparableOperator OperDOES_NOT_CONTAIN = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			String string1 = ((String)c1).toLowerCase();
			String string2 = ((String)c2).toLowerCase();
			return !string1.contains(string2);
		}
	};
	
	static ComparableOperator OperMATCH = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			String string = (String)c1;
			String regex = (String)c2;
			if ("*".equals(regex)) // invalid regular expression but frequently used as "any"
				return true; //regex =".*";
			return string.matches(regex);
		}
	};
	
	static ComparableOperator OperNO_MATCH = new ComparableOperator() {
		public boolean compare(Comparable c1, Comparable c2) {
			String string = (String)c1;
			String regex = (String)c2;
			if ("*".equals(regex)) // invalid regular expression but frequently used as "any"
				return false; // regex =".*";
			return !string.matches(regex);
		}
	};
	
	static ComparableOperator OperIsOneOf = new ComparableOperator() {
		public boolean compare(Comparable one, Comparable[] many) {
			if (one==null && many==null)
				return true;
			if (one==null && many!=null)
				return false;
			for (int i = 0; i < many.length; i++) {
				if (one.compareTo(many[i]) == 0)
					return true;
			}
			return false;
		}
		public boolean compare(Comparable c1, Comparable c2) {
			String delim = ",";
			String domain = (String) c2;
			String[] array;
			if (domain != null)
				array = domain.split(delim);
			else
				array = new String[] {domain};
			
			if (c1 instanceof String)  {
				String s1 = (String) c1;
				for (int i = 0; i < array.length; i++) {
					String s2 = array[i].trim();
					if (s1.equalsIgnoreCase(s2))
						return true;
				}
				return false;
			}
			if (c1 instanceof Integer)  {
				int i1 = ((Integer) c1).intValue();
				for (int i = 0; i < array.length; i++) {
					int i2 = Integer.parseInt(array[i].trim());
					if (i1 == i2)
						return true;
				}
				return false;
			}
			if (c1 instanceof Double)  {
				double d1 = ((Double) c1).doubleValue();
				for (int i = 0; i < array.length; i++) {
					double d2 = Double.parseDouble(array[i].trim());
					if (Math.abs(d1 - d2) <= precision)
						return true;
				}
				return false;
			}
			if (c1 instanceof Long)  {
				long l1 = ((Long) c1).longValue();
				for (int i = 0; i < array.length; i++) {
					long l2 = Long.parseLong(array[i].trim());
					if (l1 == l2)
						return true;
				}
				return false;
			}
			return false;
		}
	};
	
	static ComparableOperator OperIsNotOneOf = new ComparableOperator() {
		public boolean compare(Comparable one, Comparable[] many) {
			if (one==null && many==null)
				return true;
			if (one==null && many!=null)
				return false;
			for (int i = 0; i < many.length; i++) {
				if (one.compareTo(many[i]) == 0)
					return false;
			}
			return true;
		}
		public boolean compare(Comparable c1, Comparable c2) {
			if (c1==null && c2==null)
				return true;
			if (c1==null && c2!=null)
				return false;
			
			String delim = ",";
			String domain = (String) c2;
			String[] array;
			if (domain != null)
				array = domain.split(delim);
			else
				array = new String[] {domain};
			
			if (c1 instanceof String)  {
				String s1 = (String) c1;
				for (int i = 0; i < array.length; i++) {
					String s2 = array[i].trim();
					if (s1.equalsIgnoreCase(s2))
						return false;
				}
				return true;
			}
			if (c1 instanceof Integer)  {
				int i1 = ((Integer) c1).intValue();
				for (int i = 0; i < array.length; i++) {
					int i2 = Integer.parseInt(array[i].trim());
					if (i1 == i2)
						return false;
				}
				return true;
			}
			if (c1 instanceof Double)  {
				double d1 = ((Double) c1).doubleValue();
				for (int i = 0; i < array.length; i++) {
					double d2 = Double.parseDouble(array[i].trim());
					if (Math.abs(d1 - d2) <= precision)
						return false;
				}
				return true;
			}
			if (c1 instanceof Long)  {
				long l1 = ((Long) c1).longValue();
				for (int i = 0; i < array.length; i++) {
					long l2 = Long.parseLong(array[i].trim());
					if (l1 == l2)
						return false;
				}
				return true;
			}
			return true;
		}
	};
	
	static ComparableOperator OperINCLUDE = new ComparableOperator() {
		// array one includes array two?
		public boolean compare(Comparable[] one, Comparable[] two) {
			if (one==null && two==null)
				return true;
			if (one==null && two!=null)
				return false;
			if (one!=null && two==null)
				return true;
			for (int i = 0; i < two.length; i++) {
				boolean found = false;
				for (int j = 0; j < one.length; j++) {
					if (two[i].compareTo(one[j]) == 0) {
						found = true;
						break;
					}
				}
				if (!found)
					return false; // one does not contain two[i]
			}
			return true;
		}
		
	};
	
	static ComparableOperator OperDoesNotInclude = new ComparableOperator() {
		// array one includes array two?
		public boolean compare(Comparable[] one, Comparable[] two) {
			return !OperINCLUDE.compare(one,two);
		}
	};
	
	static ComparableOperator OperEXCLUDE = new ComparableOperator() {
		// array one does not include all elements of array two?
		public boolean compare(Comparable[] one, Comparable[] two) {
			if (one==null)
				return true;
			if (two==null)
				return false;
			for (int i = 0; i < two.length; i++) {
				boolean found = false;
				for (int j = 0; j < one.length; j++) {
					if (two[i].compareTo(one[j]) == 0) {
						found = true;
						break;
					}
				}
				if (!found)
					return true; // one does not contain two[i]
			}
			return false;
		}
		
	};
	
	static ComparableOperator OperExcludeAll = new ComparableOperator() {
		// array one excludes all elements of array two?
		public boolean compare(Comparable[] one, Comparable[] two) {
			if (one==null)
				return true;
			if (two==null)
				return false;
			for (int i = 0; i < two.length; i++) {
				for (int j = 0; j < one.length; j++) {
					if (two[i].compareTo(one[j]) == 0) {
						return false;
					}
				}
			}
			return true;
		}
		
	};
	
	static ComparableOperator OperINTERSECT = new ComparableOperator() {
		// array one intersects with array two?
		public boolean compare(Comparable[] one, Comparable[] two) {
			if (one==null && two==null)
				return true;
			if (one==null && two!=null)
				return true;
			if (one!=null && two==null)
				return false;
			for (int i = 0; i < two.length; i++) {
				for (int j = 0; j < one.length; j++) {
					if (two[i].compareTo(one[j]) == 0) {
						return true;
					}
				}
			}
			return false;
		}
		
	};
	
	static ComparableOperator OperARE = new ComparableOperator() {
		// implemented in Glossary.assign(...)
	};
	
	static ComparableOperator OperADD = new ComparableOperator() {
		// implemented in Glossary.assign(...)
	};
	
	static ComparableOperator OperAssignPlus = new ComparableOperator() {
		// implemented in Glossary.assign(...)
	};
	
	static ComparableOperator OperAssignMinus = new ComparableOperator() {
		// implemented in Glossary.assign(...)
	};
	static ComparableOperator OperAssignMultiply = new ComparableOperator() {
		// implemented in Glossary.assign(...)
	};
	static ComparableOperator OperAssignDivide = new ComparableOperator() {
		// implemented in Glossary.assign(...)
	};
	static ComparableOperator OperMax = new ComparableOperator() {
		// implemented in Glossary.max(...)
	};
	static ComparableOperator OperMin = new ComparableOperator() {
		// implemented in Glossary.min(...)
	};
	static ComparableOperator OperAverage = new ComparableOperator() {
	};
	static ComparableOperator OperCount = new ComparableOperator() {
	};
	static ComparableOperator OperSum = new ComparableOperator() {
	};
	static ComparableOperator OperProduct = new ComparableOperator() {
	};

	public static void main(String[] args) {
		
		Date date1 = new Date(107, 5, 15, 5, 30);
		Date date2 = new Date(107, 5, 15, 6, 30);
		Oper eq = new Oper("==");
		boolean result = eq.compare(date1, date2);
		String sign = (result) ? " == " : " != ";
		System.out.println(date1.toString() + sign + date2.toString());

		Oper le = new Oper("<=");
		result = le.compare(date1, date2);
		sign = (result) ? " <= " : " > ";
		System.out.println(date1.toString() + sign + date2.toString());

		Oper ne = new Oper("!=");
		result = ne.compare(date1, date2);
		sign = (result) ? " != " : " == ";
		System.out.println(date1.toString() + sign + date2.toString());

		DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
		try {
			String dd = "2/15/2007 13:30";
			Date d = df.parse(dd);
			System.out.println("Parse " + dd + " = " + d);
			Date d1 = new Date(2011-1900,4-1,10);
			Date d2 = df.parse("04/10/2011");
			Oper comparedate = new Oper("<=");
			result = comparedate.compare(d1, d2);
			System.out.println("" + d1 + " " + comparedate + " " + d2 + ((result) ? ": true" : ": false"));
		} catch (Exception e) {
			System.out.println("DateFormat exception: " + e);
			e.getStackTrace();
		}
		
		Oper eq1 = new Oper("Is");
		String s1 = "Mama";
		String s2 = "Mom";
		result = eq1.compare(s1, s2);
		sign = (result) ? " = " : " != ";
		System.out.println(s1 + sign + s2);
		
		double r1 = 3.14111111110000000000115;
		double r2 = 3.14111111110000000000114;
		result = eq1.compare(r1, r2);
		sign = (result) ? " = " : " != ";
		System.out.println(r1 + sign + r2);
		
		//String op = "Is Less Than Or Equal To";
		String op = ">";
		Oper eq2 = new Oper(op);
		int i1 = 25;
		int i2 = 32;
		result = eq2.compare(i1, i2);
		System.out.println("" + i1 + " " + op + " " + i2 + ((result) ? ": true" : ": false"));
		
		op = "!=";
		eq2 = new Oper(op);
		boolean b1 = true;
		boolean b2 = true;
		result = eq2.compare(b1, b2);
		System.out.println("" + b1 + " " + op + " " + b2 + ((result) ? ": true" : ": false"));
		
		op = "=";
		eq2 = new Oper(op);
		b1 = true;
		b2 = true;
		result = eq2.compare(b1, b2);
		System.out.println("" + b1 + " " + op + " " + b2 + ((result) ? ": true" : ": false"));
		
		op = "IsTrue";
		eq2 = new Oper(op);
		b1 = true;
		result = eq2.compare(b1);
		System.out.println("" + b1 + " " + op  + ((result) ? ": true" : ": false"));
		
//		op = "IsTrue";
//		eq2 = new Oper(op);
//		b1 = true;
//		b2 = true;
//		result = eq2.compare(b1, b2);
//		System.out.println("" + b1 + " " + op + " " + b2 + ((result) ? ": true" : ": false"));
		
		op = "Is Empty";
		eq2 = new Oper(op);
		String testString = null; // "  ";
		b2 = true;
		result = eq2.compare(testString, b2);
		System.out.println("" + testString + " " + op + " " + b2 + ((result) ? ": true" : ": false"));
		
		op = "Starts with";
		eq2 = new Oper(op);
		s1 = "House of candles";
		s2 = "ho";
		result = eq2.compare(s1, s2);
		System.out.println("" + s1 + " " + op + " " + s2 + ((result) ? ": true" : ": false"));
		
		op = "Match";
		eq2 = new Oper(op);
		s1 = "123-45-6789";
		//String regex = "999-99-9999";
		String regex = "\\d{3}-\\d{2}-\\d{4}";
		result = eq2.compare(s1, regex);
		System.out.println("" + s1 + " " + op + " " + regex + ((result) ? ": true" : ": false"));
		
		op = "Is Not Like";
		eq2 = new Oper(op);
		s1 = "123-45-6789";
		//String regex = "999-99-9999";
		regex = "\\d{3}-\\d{2}-\\d{4}";
		result = eq2.compare(s1, regex);
		System.out.println("" + s1 + " " + op + " " + regex + ((result) ? ": true" : ": false"));
		
		op = "Match";
		eq2 = new Oper(op);
		s1 = "ALEXANDER";
		regex = "[A-Z]*";
		result = eq2.compare(s1, regex);
		System.out.println("" + s1 + " " + op + " " + regex + ((result) ? ": true" : ": false"));
		
		op = "No Match";
		eq2 = new Oper(op);
		s1 = "ALEXANDER(12#)";
		regex = "*";
		result = eq2.compare(s1, regex);
		System.out.println("" + s1 + " " + op + " " + regex + ((result) ? ": true" : ": false"));
		
		op = "Match";
		eq2 = new Oper(op);
		s1 = "ALEXANDER(12#)";
		regex = ".*";
		result = eq2.compare(s1, regex);
		System.out.println("" + s1 + " " + op + " " + regex + ((result) ? ": true" : ": false"));
		
		op = "Match";
		eq2 = new Oper(op);
		s1 = "ALEXANDER";
		regex = "ALEXANDER";
		result = eq2.compare(s1, regex);
		System.out.println("" + s1 + " " + op + " " + regex + ((result) ? ": true" : ": false"));
		
		op = "Is One of";
		Oper isOneOfMany = new Oper(op);
		String one = "AA";
		String[] many = { "mmm", "NNN", "AA", "A", "123" };
		StringBuffer buf = new StringBuffer();
		buf.append("{");
		for (int i = 0; i < many.length; i++) {
			buf.append(" "+many[i]);
		}
		buf.append(" }");
		result = isOneOfMany.compare(one, many);
		System.out.println("" + one + " " + op + " " + buf + ((result) ? ": true" : ": false"));
		String domain = "NNN mmm aa 123";
		result = isOneOfMany.compare(one, domain);
		System.out.println("" + one + " " + op + " " + domain + ((result) ? ": true" : ": false"));
		
		op = "Is Not One Of";
		Oper isNotOneOf = new Oper(op);
		result = isNotOneOf.compare(one, many);
		System.out.println("" + one + " " + op + " " + buf + ((result) ? ": true" : ": false"));
		result = isNotOneOf.compare(one, domain);
		System.out.println("" + one + " " + op + " " + domain + ((result) ? ": true" : ": false"));
		
		op = "Is One of";
		isOneOfMany = new Oper(op);
		int x1 = 100;
		int[] xx = { 99,100,101,102,103 };
		result = isOneOfMany.compare(x1, xx);
		System.out.println("" + x1 + " " + op + " { 99,100,101,102,103 }" + ((result) ? ": true" : ": false"));
		
		
		op = "INTERSECT";
		Oper intersect = new Oper(op);
		String[] arr1 = { "xxx", "NNN", "AA", "B", "1263" };
		String[] arr2 = {  "126", "NNN", "AA" };
		String[] arr3 = {  "126", "NN", "AAA" };
		StringBuffer buf1 = new StringBuffer();
		buf1.append("{");
		for (int i = 0; i < arr1.length; i++) {
			buf1.append(" "+arr1[i]);
		}
		buf1.append(" }");
		StringBuffer buf2 = new StringBuffer();
		buf2.append("{");
		for (int i = 0; i < arr2.length; i++) {
			buf2.append(" "+arr2[i]);
		}
		buf2.append(" }");
		StringBuffer buf3 = new StringBuffer();
		buf3.append("{");
		for (int i = 0; i < arr3.length; i++) {
			buf3.append(" "+arr3[i]);
		}
		buf3.append(" }");
		result = intersect.compare(arr1, arr2);
		System.out.println("" + buf1 + " " + op + " " + buf2 + ((result) ? ": true" : ": false"));
		
		int[] intArr1 = { 1,2,3,4,5,6};
		int[] intArr2 = { 4,5,7 };
		result = intersect.compare(intArr1, intArr2);
		System.out.println(Arrays.toString(intArr1) + " " + op + " " + Arrays.toString(intArr2) + ((result) ? ": true" : ": false"));
		
		op = "Contains";
		String string1 = "House";
		String string2 = "use";
		Oper contains = new Oper(op);
		result = contains.compare(string1, string2);
		System.out.println("" + string1 + " " + op + " " + string2 + ((result) ? ": true" : ": false"));
		
		op = "Does Not Contain";
		string2 = "user";
		Oper containsNo = new Oper(op);
		result = containsNo.compare(string1, string2);
		System.out.println("" + string1 + " " + op + " " + string2 + ((result) ? ": true" : ": false"));
		
		op = "Include";
		Oper include = new Oper(op);
		result = include.compare(arr1, arr2);
		System.out.println("" + buf1 + " " + op + " " + buf2 + ((result) ? ": true" : ": false"));
		
		result = include.compare(intArr1, intArr2);
		System.out.println(Arrays.toString(intArr1) + " " + op + " " + Arrays.toString(intArr2) + ((result) ? ": true" : ": false"));
		
		int int1 = 5;
		result = include.compare(int1, intArr2);
		System.out.println("" + int1 + " " + op + " " + Arrays.toString(intArr2) + ((result) ? ": true" : ": false"));
		
		op = "Does not Include";
		Oper noinclude = new Oper(op);
		result = noinclude.compare(arr1, arr2);
		System.out.println("" + buf1 + " " + op + " " + buf2 + ((result) ? ": true" : ": false"));
		result = noinclude.compare(arr1, arr1);
		System.out.println("" + buf1 + " " + op + " " + buf1 + ((result) ? ": true" : ": false"));
		
		op = "Exclude";
		Oper exclude = new Oper(op);
		result = exclude.compare(arr1, arr2);
		System.out.println("" + buf1 + " " + op + " " + buf2 + ((result) ? ": true" : ": false"));
		result = exclude.compare(arr1, arr1);
		System.out.println("" + buf1 + " " + op + " " + buf1 + ((result) ? ": true" : ": false"));
		
		result = exclude.compare(intArr1, intArr2);
		System.out.println(Arrays.toString(intArr1) + " " + op + " " + Arrays.toString(intArr2) + ((result) ? ": true" : ": false"));
		
		op = "Exclude All";
		Oper excludeAll = new Oper(op);
		result = excludeAll.compare(arr1, arr2);
		System.out.println("" + buf1 + " " + op + " " + buf2 + ((result) ? ": true" : ": false"));
		
		op = "Exclude All";
		excludeAll = new Oper(op);
		result = excludeAll.compare(arr1, arr3);
		System.out.println("" + buf1 + " " + op + " " + buf3 + ((result) ? ": true" : ": false"));
		
		result = excludeAll.compare(intArr1, intArr2);
		System.out.println(Arrays.toString(intArr1) + " " + op + " " + Arrays.toString(intArr2) + ((result) ? ": true" : ": false"));
		
		op = "Within";
		Oper within = new Oper(op);
		int value = 20;
		String interval = "[20;30)";
		result = within.compare(value, interval);
		System.out.println("Value " + value + " " + op + " " + interval + ((result) ? ": true" : ": false"));
		
		op = "Outside";
		Oper outside = new Oper(op);
		result = outside.compare(value, interval);
		System.out.println("Value " + value + " " + op + " " + interval + ((result) ? ": true" : ": false"));

	}
	
	public String getDelimiter() {
		return delimiter;
	}

	public void setDelimiter(String delimiter) {
		this.delimiter = delimiter;
	}

	public void setId(String id) {
		this.id = id;
	}

	public static String removeSpaces(String s) {
		return s.replaceAll(" ", "");
	}

}
