package com.openrules.datasource;

/*
 * Copyright (C) 2019 OpenRules, Inc.
 * 
 */

/**
 *	URLDataSource is a DataSource for different URLs
 *  
 *  @author IK
 *  
 */

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;

import org.apache.commons.codec.binary.Base64;

public class ClassPathDataSource extends AbstractDataSource {

	private String path;
	private ClassLoader loader;

	public ClassPathDataSource(ClassLoader loader, String path) {
		this.loader = loader;
		this.path = path;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.datasource.IDataSource#getDescription()
	 */
	public String getDescription() {
		return toString();
	}

	public InputStream getInputStream() throws IOException {
		if (loader == null || path == null)
			return null;

		return loader.getResourceAsStream(path);
	}

	@Override
	public URL getUrl() {
		if (path == null) {
			return null;
		}
		try {
			return new URL("classpath:/" + path);
		} catch (MalformedURLException e) {
			return null;
		}
	}

	public String toString() {
		return path;
	}

	public URI getURI() {

		if (path == null) {
			return null;
		}

		try {
			return new URI("classpath:/" + path);
		} catch (URISyntaxException e) {
			return null;
		}
	}

}
