package com.openrules.datasource;

import java.net.MalformedURLException;

import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.Properties;

import org.openl.util.Log;

public class ClasspathDataSourceFactoryHandler implements IDataSourceProtocolHandler {

	public IDataSource fromURI(URI uri, IDataSourceFactory dataSourceFactory) {
		if (uri == null)
			throw new IllegalArgumentException("uri must not be null");
		Log.debug("Creating Classpath DataSource from " + uri.toASCIIString());


	    String path = uri.getPath();
	    //make relative to root
	    if ( path.startsWith("/") ) {
	    	path = path.substring(1);
	    }

		URL url = null;
		ClassLoader loader = null;

		// try to get it from the user resource context ( if given )
		if (dataSourceFactory != null) {

			if (dataSourceFactory.getUserContext() != null) {
				loader = dataSourceFactory.getUserContext().getUserClassLoader();
				if (loader != null)
					url = loader.getResource(path);
			}

			// try to get it from the main resource context ( if given )
			if (url == null && dataSourceFactory.getResourceContext() != null) {
				loader = dataSourceFactory.getResourceContext().getClassLoader();
				url = loader.getResource(path);
			}
		}
		// finally try to get it from the current ClassLoader
		if (url == null) {
			loader = getClass().getClassLoader();
			url = loader.getResource(path);
		}

		if ( url == null ) {
			return new ClassPathDataSource(loader, null);
		}
		
		return new ClassPathDataSource(loader, path);
	}

	public void initialize(Properties options) {
		// nothing
	}

}
