package com.openrules.datasource;

/*
 * Copyright (C) 2006-2009 OpenRules, Inc.
 * 
 * Created on August, 2006 by IK
 * 
 */

/**
 *	DataSourceFactory is responsible for the creation of DataSources
 *  that provide a unified access to different data sources such as xls- and 
 *  xml-files using prefixes such as "classpath:", "file:", "http:", "ftp:" 
 *  
 *  @author IK
 *  
 */

import java.io.File;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.Properties;

import org.openl.util.Log;

public class FileDataSourceFactoryHandler implements IDataSourceProtocolHandler {

	public IDataSource fromURI(URI uri, IDataSourceFactory dataSourceFactory) {
		//Log.debug("Create DataSource from " + uri);
		if (uri == null)
			throw new IllegalArgumentException();

		String path = uri.getSchemeSpecificPart();

		URLDataSource source = new URLDataSource();

		URL url = null;

		try {
			File f = new File(path);
			if (f.isAbsolute()) 
				url = f.toURI().toURL();

			if ( url == null && dataSourceFactory != null) {
				// try user home ( if given )
				if ( dataSourceFactory.getUserContext() != null) {
					String userHome = dataSourceFactory.getUserContext().getUserHome();
					userHome = (userHome == null ? "" : userHome.trim());
					if (userHome.length() > 0) {
						f = new File(userHome, path);
						if (f.exists())
							url = f.toURI().toURL();
					}
				}

				// try main context ( if given )
				if (url == null && dataSourceFactory.getResourceContext() != null) {
					f = dataSourceFactory.getResourceContext().findFileSystemResource(path);
					if (f != null)
						url = f.toURI().toURL();
				}
			}

			if (url == null)
				url = uri.toURL();

			source.setUrl(url);
			return source;
			
		} catch (MalformedURLException e) {
			throw new RuntimeException( e );
		}
	}

	public void initialize(Properties options) {
		//nothing
	}
	
}
