package com.openrules.datasource;

/*
 * Copyright (C) 2006-2009 OpenRules, Inc.
 * 
 * Created on August, 2006 by IK
 * 
 */

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.apache.commons.lang.StringUtils;

/**
 * 
 * default preprocessor for DataSourceFactory URI strings
 * it replaces all substrings '${property_name} with the result of System.getProperty("property_name")
 *  
 * for example 'ftp://localhost/${SEASON}/data.xml' may be transformed to 'ftp://localhost/SUMMER/data.xm
 * if the property is not defined then no replacement happened  
 *
 */
public class SystemPropertyResolver implements IDataSourcePreprocessor {

	private Pattern pattern = Pattern.compile("\\$\\{([^\\}]*)\\}");

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.openrules.datasource.IDataSourcePreprocessor#process(java.lang.String)
	 */
	public String process(String input) {
		if (input == null)
			return null;

		StringBuffer result = new StringBuffer();
		Matcher matcher = pattern.matcher(input);
		while (matcher.find()) {
			String value = resolveProperty( matcher.group(1) );
			matcher.appendReplacement(result, value == null ? "$0" : value );
		}
		matcher.appendTail(result);
		return result.toString();
	}

	protected String resolveProperty(String name) {
		if ( ! System.getProperties().containsKey(name) )
			return null;
		return StringUtils.stripToEmpty( System.getProperty(name) );
	}

}
