package com.openrules.datasource;

/*
 * Copyright (C) 2006-2009 OpenRules, Inc.
 * 
 * Created on August, 2006 by IK
 * 
 */

/**
 *	URLDataSource is a DataSource for different URLs
 *  
 *  @author IK
 *  
 */

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.net.URLConnection;

import org.apache.commons.codec.binary.Base64;

public class URLDataSource extends AbstractDataSource {

    private URL url;
    
    public URLDataSource() {
    }
    
    public URLDataSource( String path ) throws MalformedURLException {
        setUrl( new URL(path) );
    }

    public URLDataSource( URL url ) throws MalformedURLException {
        setUrl( url );
    }
    
    /* (non-Javadoc)
     * @see com.openrules.datasource.IDataSource#getDescription()
     */
    public String getDescription() {
        return toString();
    }
    
    public URL getUrl() {
        return url;
    }
    public void setUrl(URL url) {
        this.url = url;
    }

    public InputStream getInputStream() throws IOException {
    	// JD added URL authentication
    	if (url == null)
    		return null;
    	
    	URLConnection urlConnection = url.openConnection();
    	String protocol = url.getProtocol();
    	// http://username:password@file
    	if (protocol != null && protocol.startsWith("http")) {
    		String authority = url.getAuthority();
    		if (authority != null) {
	    		int atIndex = authority.indexOf('@'); 
	    		if (atIndex > 0) {
	    			String authInfo = authority.substring(0, atIndex);
	    			if (authInfo.indexOf(':') > 0) {
	    				byte[] authEncBytes = Base64.encodeBase64(authInfo.getBytes());
	    				String authInfoEnc = new String(authEncBytes);
	   					urlConnection.setRequestProperty("Authorization", "Basic " + authInfoEnc);
	    			}
	    		}
    		}
    	}
    	InputStream is = urlConnection.getInputStream();   	 
    	
        return is;
    }

    public String toString() {
    	String className = getClass().getName();
        return className.substring( className.lastIndexOf(".")+1) + ":" + url;
    }

	public URI getURI() {
		try {
			if ( url != null )
				return new URI( Utils.urify( url.toExternalForm() ) );
		} catch (URISyntaxException e) {
			//ignored
		}
		return null;
	}


}


