package com.openrules.datasource;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 * Created on August, 2006 by IK
 * 
 */

/**
 *	com.openrules.datasource.Utils provides common static methods that deal
 *  with input and output streams
 *  
 *  @author IK
 *  
 */

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Reader;
import java.io.StringWriter;
import java.io.Writer;
import java.util.Properties;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Utils {

	/**
	 * 
	 * @param input InputStream 
	 * @return a string representing content of the stream
	 * @throws IOException
	 */
	public static String copyToString( InputStream input ) throws IOException {
		return copyToString( new InputStreamReader(input) );
	}

	/**
	 * 
	 * @param reader Reader 
	 * @return a string representing content of the Reader
	 * @throws IOException
	 */
	public static String copyToString( Reader reader ) throws IOException {
		StringWriter sw = new StringWriter();
        char[] buffer = new char[4048];
        int cnt = 0;
        while (-1 != (cnt = reader.read(buffer))) {
            sw.write(buffer, 0, cnt);
        }
		return sw.toString();
	}
	/**
	 * 
	 * @param input InpuStream
	 * @return a byet array representing content of the stream
	 * @throws IOException
	 */
    public static byte[] copyToByteArray(InputStream input) throws IOException {
        ByteArrayOutputStream output = new ByteArrayOutputStream();
        byte[] buffer = new byte[4048];
        int cnt = 0;
        while (-1 != (cnt = input.read(buffer))) {
            output.write(buffer, 0, cnt);
        }
        return output.toByteArray();
    }
    
    /**
     * 
     * @param stream 
     * 
     * silently close the stream
     */
    public static void justClose( InputStream stream ) {
    	try {
    		if ( stream != null )
    			stream.close();
		} catch (IOException e) {
			//ignoring
		}
    }
    
    /**
     * 
     * @param stream 
     * 
     * silently close the stream
     */
    public static void justClose( OutputStream stream ) {
    	try {
    		if ( stream != null )
    			stream.close();
		} catch (IOException e) {
			//ignoring
		}
    }
    
    /**
     * 
     * @param reader 
     * 
     * silently close the reader
     */
    public static void justClose( Reader reader ) {
    	try {
    		if ( reader != null )
    			reader.close();
		} catch (IOException e) {
			//ignoring
		}
    }
    
    /**
     * 
     * @param writer
     * 
     * silently close the writer
     */
    public static void justClose( Writer writer  ) {
    	try {
    		if ( writer != null )
    			writer.close();
		} catch (IOException e) {
			//ignoring
		}
    }
    
    
	//replace all spaces with URI_SPACE
	//replace all backslashes with forward slashes
	private static Pattern p = Pattern.compile("[ \\\\]");

	public static String urify( String input ) {
		StringBuffer result = new StringBuffer();
		Matcher matcher = p.matcher(input);
		while (matcher.find()) {
			char target = matcher.group().charAt(0);
			matcher.appendReplacement(result, target == ' ' ? "%20" : "/" );
		}
		matcher.appendTail(result);
		return result.toString();
	}
	
	public static Properties loadProperties(IDataSource src) throws IOException {
		InputStream is = src.getInputStream();
		if ( is == null )
			return null;
		Properties props = new Properties();
		try { 
			props.load(is);
		} finally {
			is.close();
		}
		return props;
	}
    
	public static String removePath(String input) {
		int len = input.length();
		int i;
		for (i = len-1; i >=0; i--){
		    char c = input.charAt(i); 
		    if (c == '\\' || c == ':' || c == '/')
		    	break;
		}
		return input.substring(i+1);
	}
	
	public static void main(String[] args) {
		String fileName = "file:./rules/Decision.xlsx"; 
		String name = Utils.removePath(fileName);
		System.out.println(name);
	}
}
