/*
 * Created on Oct 6, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package org.openl;

import org.openl.binding.IBindingContext;
import org.openl.binding.IBindingContextDelegator;
import org.openl.binding.IBoundCode;
import org.openl.binding.IBoundMethodHeader;
import org.openl.binding.IBoundMethodNode;
import org.openl.binding.IBoundNode;
import org.openl.binding.impl.ANodeBinder;
import org.openl.binding.impl.Binder;
import org.openl.binding.impl.BindingContext;
import org.openl.binding.impl.BindingContextDelegator;
import org.openl.binding.impl.CastNode;
import org.openl.binding.impl.TypeBoundNode;
import org.openl.binding.impl.module.MethodBindingContext;
import org.openl.syntax.IParsedCode;
import org.openl.syntax.ISyntaxError;
import org.openl.syntax.SyntaxErrorException;
import org.openl.types.IOpenCast;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenMethodHeader;
import org.openl.types.impl.CompositeMethod;
import org.openl.types.java.JavaOpenClass;

/**
 * @author snshor
 *  
 */
public class OpenlTool
{

	/**
	 * 
	 */
	public OpenlTool()
	{
		super();
		// TODO Auto-generated constructor stub
	}

	static public IOpenClass getType(String typeName, OpenL openl)
	{
		IOpenBinder binder = openl.getBinder();
		return binder.makeBindingContext().findType("org.openl.this", typeName);
	}

	static public IOpenMethodHeader getMethodHeader(
		IOpenSourceCodeModule src,
		OpenL openl,
		IBindingContextDelegator cxt)
	{
		if (cxt== null)
		  cxt = new BindingContextDelegator(new BindingContext((Binder)openl.getBinder(), JavaOpenClass.VOID));
		
		try
		{
			cxt.pushErrors();
			IOpenParser parser = openl.getParser();

			IParsedCode pc = parser.parseAsMethodHeader(src);
			ISyntaxError[] error = pc.getError();
			if (error.length > 0)
			{
				throw new SyntaxErrorException("Parsing Error:", error);
			}

			IOpenBinder binder = openl.getBinder();

			IBoundCode bc = binder.bind(pc, cxt);
			error = bc.getError();
			if (error.length > 0)
			{
				throw new SyntaxErrorException("Binding Error:", error);
			}

			return ((IBoundMethodHeader) bc.getTopNode()).getMethodHeader();
		}
		finally
		{
			cxt.popErrors();
		}

	}

	static public CompositeMethod makeMethod(
		IOpenSourceCodeModule src,
		OpenL openl,
		IOpenMethodHeader header,
		IBindingContext cxt)
	{
		try
		{
			cxt.pushErrors();
			IOpenParser parser = openl.getParser();

			IParsedCode pc = parser.parseAsMethodBody(src);
			ISyntaxError[] error = pc.getError();
			if (error.length > 0)
			{
				throw new SyntaxErrorException("Parsing Error:", error);
			}

			IOpenBinder binder = openl.getBinder();

			MethodBindingContext mbc = new MethodBindingContext(header, cxt);

			IBoundCode bc = binder.bind(pc, mbc);
			error = bc.getError();
			if (error.length > 0)
			{
				throw new SyntaxErrorException("Binding Error:", error);
			}

			
//			IBoundMethodNode ibmn = bindMethodType((IBoundMethodNode) bc.getTopNode(), cxt, header.getType());
			

			return new CompositeMethod(
				header,
			(IBoundMethodNode) bc.getTopNode()	
				);
		}
		finally
		{
			cxt.popErrors();
		}
	}

	static public IOpenClass makeType(
		IOpenSourceCodeModule src,
		OpenL openl,
		IBindingContextDelegator cxt)
	{
		try
		{

			if (cxt == null)
				cxt =
					new BindingContextDelegator(
						openl.getBinder().makeBindingContext());
			cxt.pushErrors();
			IOpenParser parser = openl.getParser();

			IParsedCode pc = parser.parseAsType(src);
			ISyntaxError[] error = pc.getError();
			if (error.length > 0)
			{
				throw new SyntaxErrorException("Parsing Error:", error);
			}

			IOpenBinder binder = openl.getBinder();

			IBoundCode bc = binder.bind(pc, cxt);
			error = bc.getError();
			if (error.length > 0)
			{
				throw new SyntaxErrorException("Binding Error:", error);
			}

			return ((TypeBoundNode) bc.getTopNode()).getType();

		}
		finally
		{
			cxt.popErrors();
		}

	}

	/* (non-Javadoc)
	 * @see org.openl.binding.INodeBinder#bindType(org.openl.syntax.ISyntaxNode, org.openl.binding.IBindingContext, org.openl.types.IOpenClass)
	 */
	static IBoundMethodNode bindMethodType(
		IBoundMethodNode bnode,
		IBindingContext bindingContext,
		IOpenClass type)
		throws Exception
	{

//		if (bnode.getType().equals(type))
//			return bnode;

		IOpenCast cast = ANodeBinder.getCast(bnode.getType(), type, bindingContext);

		if (cast == null)
		  return bnode;

		return new MethodCastNode(bnode, cast, type);

	}
	
	static class MethodCastNode extends CastNode implements IBoundMethodNode
	{
		
			/**
		 * @param bnode
		 * @param cast
		 * @param castedType
		 */
		public MethodCastNode(
			IBoundNode bnode,
			IOpenCast cast,
			IOpenClass castedType)
		{
			super(bnode, cast, castedType);
		}

		/**
		 *
		 */

		public int getLocalFrameSize()
		{
			return ((IBoundMethodNode) bnode).getLocalFrameSize();
		}

		/**
		 *
		 */

		public int getParametersSize()
		{
			return ((IBoundMethodNode) bnode).getParametersSize();
		}

}

}
