/*
 * Created on Jul 8, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package org.openl.conf;

import java.io.File;
import java.net.URL;

import org.openl.OpenConfigurationException;
import org.openl.types.IOpenFactory;
import org.openl.types.IOpenSchema;
import org.openl.types.ITypeLibrary;
import org.openl.util.RuntimeExceptionWrapper;

/**
 * @author snshor
 * 
 * Modified by IK on 08/07/2006: add location
 * 
 */
public class OpenSchemaConfiguration extends AConfigurationElement implements
		ITypeFactoryConfigurationElement {

	URL url;

	String location;

	String factoryName;

	IOpenSchema schema;

	/**
	 * 
	 */
	public OpenSchemaConfiguration() {
		super();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.conf.ITypeFactoryConfigurationElement#getLibrary(org.openl.conf.IConfigurableResourceContext)
	 */
	public ITypeLibrary getLibrary(IConfigurableResourceContext cxt) {
		if (schema == null) {
			try {
				IOpenFactory factory = cxt.getConfiguration().getOpenFactory(
						factoryName);
				schema = factory.getSchema( location, false);
			} catch (Exception ex) {
				throw RuntimeExceptionWrapper.wrap(ex);
			}
		}
		return schema;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.conf.IConfigurationElement#validate(org.openl.conf.IConfigurableResourceContext)
	 */
	public void validate(IConfigurableResourceContext cxt)
			throws OpenConfigurationException {
		if (factoryName == null)
			throw new OpenConfigurationException(
					"Attribute factory must be set for schema", getUri(), null);

		if ( location == null )
			throw new OpenConfigurationException(
					"Attribute <location> must be set for schema",
					getUri(), null);

		if (cxt.getConfiguration().getOpenFactory(factoryName) == null)
			throw new OpenConfigurationException("Factory " + factoryName
					+ " does not exist", getUri(), null);

	}

	/**
	 * @deprecated use <code>setLocation(String location)</code> //IK
	 */
	public void setFile(File f) throws Exception {
		location  = f.toURL().toExternalForm();
	}

	/**
	 * @deprecated use <code>setLocation()</code> //IK
	 */
	public void setURL(String x) throws Exception {
		this.location = new URL(x).toExternalForm();
	}

	//IK
	public void setLocation(String location) {
		this.location = location;
	}

	/**
	 * @param string
	 */
	public void setFactory(String string) {
		factoryName = string;
	}

}
