/*
 * Created on May 13, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package org.openl.syntax.impl;

import java.util.Iterator;
import java.util.Map;

import org.openl.IOpenSourceCodeModule;
import org.openl.syntax.ISyntaxNode;
import org.openl.util.AIndexedIterator;
import org.openl.util.TreeIterator;
import org.openl.util.text.ILocation;
import org.openl.util.text.TextInterval;

/**
 * @author snshor
 */
public abstract class ASyntaxNode implements ISyntaxNode
{

	String type;
	//  String namespace;

	IOpenSourceCodeModule module;

	Map properties;

	ILocation location;

	public ASyntaxNode(
		String type,
		ILocation location,
		Map properties,
		IOpenSourceCodeModule module)
	{
		this.type = type;
		this.location = location;
		this.properties = properties;
		this.module = module;
		//    this.namespace = namespace;
	}

	public ILocation getSourceLocation()
	{
		if (location == null)
		{
			int n = getNumberOfChildren();
			switch (n)
			{
				case 0 :
					return null;
				case 1 :
					return getChild(0).getSourceLocation();
				default :
					return new TextInterval(
						getChild(0).getSourceLocation().getStart(),
						getChild(n - 1).getSourceLocation().getEnd());

			}

		}

		return location;
	}

	/* (non-Javadoc)
	 * @see org.openl.parser.SyntaxNode#getType()
	 */
	public String getType()
	{
		return type;
	}

	/**
	 * @param string
	 */
	public void setType(String string)
	{
		type = string;
	}

	static class SyntaxTreeAdaptor implements TreeIterator.TreeAdaptor
	{
		public Iterator children(Object obj)
		{
			ISyntaxNode syntaxNode = (ISyntaxNode) obj;
			return new SyntaxNodeChildrenIterator(syntaxNode);

		}

		static class SyntaxNodeChildrenIterator extends AIndexedIterator
		{
			ISyntaxNode node;

			public SyntaxNodeChildrenIterator(ISyntaxNode node)
			{
				super(0, node.getNumberOfChildren(), 1);
				this.node = node;
			}

			/* (non-Javadoc)
			 * @see org.openl.util.AIndexedIterator#indexedElement(int)
			 */
			protected Object indexedElement(int i)
			{
				return node.getChild(i);
			}

		}

	}

	static public final TreeIterator.TreeAdaptor TREE_ADAPTOR =
		new SyntaxTreeAdaptor();

	/**
	 * @return
	 */
	public Map getProperties()
	{
		return properties;
	}

	protected void printMySelf(int level, StringBuffer buf)
	{
		for (int j = 0; j < level; j++)
		{
			buf.append("  ");
		}
		buf.append(getType());
		
	}

	public void print(int level, StringBuffer buf)
	{
		int nkids = getNumberOfChildren();
		
		printMySelf(level, buf);
		buf.append('\n');
		for (int i = 0; i < nkids; i++)
		{
				getChild(i).print(level + 1, buf);
		}
	}

	/**
	 * @return
	 */
	public IOpenSourceCodeModule getModule()
	{
		return module;
	}

	/**
	 *
	 */

	public String toString()
	{
		StringBuffer buf = new StringBuffer();
		print(0, buf);
		return buf.toString();
	}

}
