/*
 * Created on Oct 23, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */
 
package org.openl.syntax.impl;

import java.io.IOException;
import java.io.Reader;
import java.util.Vector;

import org.openl.IOpenSourceCodeModule;
import org.openl.util.RuntimeExceptionWrapper;
import org.openl.util.text.AbsolutePosition;
import org.openl.util.text.TextInterval;

/**
 * @author snshor
 *
 */
public class TokenizerParser
{

	String delim;

	static final int EOF = -1;	
	

	static public IdentifierNode[] tokenize(IOpenSourceCodeModule src, String delim)
	{
		return new TokenizerParser(delim).parse(src);		
	}

	public TokenizerParser(String delim)
	{
		this.delim = delim;
	}

	
	boolean isDelim(int c)
	{
		return delim.indexOf(c) >= 0;
	}
	
	
	IdentifierNode[] parse(IOpenSourceCodeModule src)
	{
		try
		{
			Vector nodes = new Vector();
			Reader reader = src.getCharacterStream();
			
			int c;
			
			StringBuffer buf = null;		
			int startToken = 0;	
			
			int pos = 0;
			for (;; pos++)
			{
				c = reader.read();
				if (isDelim(c) || c == EOF)
				{
					if (buf != null)
					{
						IdentifierNode node = new IdentifierNode("token", new TextInterval(new AbsolutePosition(startToken), new AbsolutePosition(pos))  ,  buf.toString(),src );
						nodes.add(node);
						buf = null;
					}
					if (c == EOF)
					  break;
				}
				else
				{
					if (buf == null)
					{
						buf = new StringBuffer();
						startToken = pos;
					}
					buf.append((char)c);
				}					
			}
			
			return (IdentifierNode[]) nodes.toArray(new IdentifierNode[nodes.size()]);
			
		}
		catch (IOException e)
		{
			throw RuntimeExceptionWrapper.wrap(e);
		}
		
	}


}
