package com.openrules.datasource;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 * Created on August, 2006 by IK
 * 
 */

/**
 *	IDataSourceFactory is is an interface for DataSource
 *  that provide a unified access to different data sources such as xls- and 
 *  xml-files using prefixes such as "classpath:", "file:", "http:", "ftp:" 
 *  
 *  @author IK
 *  
 */

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.net.URI;
import java.net.URL;

public interface IDataSource {

    /**
     * 
     * @return null if source does not exist
     * @deprecated - use getURI
     */
	public URL getUrl();

    /**
     * 
     * @return URI of the datasource
     */
	public URI getURI();
	
    /**
     * 
     * @return null if source does not exist
     */
    public InputStream getInputStream() throws IOException;

    /**
     * 
     * @return null if source does not exist
     */
	public Reader getReader() throws IOException;
    
    /**
     * 
     * @return content as a byte array
     */
    public byte[] getByteArray() throws IOException;
    
    /**
     * 
     * @return content as a String
     */
    public String getString() throws IOException;
    
    /**
     * @return String represention descriptions of the datasource
     * it might be file path, url etc.
     */
    public String getDescription();


    
}


