package com.openrules.datasource;

/*
 * Copyright (C) 2006 OpenRules, Inc.
 * 
 * Created on August, 2006 by IK
 * 
 */
 
/**
 *	IDataSourceFactory is an interface for DataSourceFactory class responsible
 *  for the creation of different DataSource(s) 
 *  that provide a unified access to different data sources such as xls- and 
 *  xml-files using prefixes such as "classpath:", "file:", "http:", "ftp:" 
 *  
 *  @author IK
 *  
 */

import java.net.MalformedURLException;

import org.openl.conf.IConfigurableResourceContext;
import org.openl.conf.IUserContext;

public interface IDataSourceFactory {

	/**
	 * creates IDataSource from URI 
	 * 
	 * fromString("file:/data/aaa.xls")
	 * 
	 * @param path 
	 * @return IDataSource
	 * @throws MalformedURLException
	 */
	public IDataSource fromString(String path) throws MalformedURLException;

	/**
	 * creates IDataSource from the path relative to parent URI
	 * 
	 * fromString( "classpath:/org.example.base", "../aaa/bbb.xml")
	 *   
	 * @param parent  parent location URI
	 * @param path relative path
	 * @return IDataSource
	 * @throws MalformedURLException
	 */
	public IDataSource fromString(String parent, String path) throws MalformedURLException;

	/**
	 * @return configurable resource context 
	 */
	public IConfigurableResourceContext getResourceContext();

	/**
	 * @return user context 
	 */
	public IUserContext getUserContext();
	
	/**
	 * registers a protocol handler for the scheme
	 * 
	 * register("classpath", new ClassPathHandler() )
	 * 
	 * @param scheme
	 * @param dsh  protocol handler @see IDataSourceProtocolHandler 
	 * @return existing protocol handler for the scheme (or null)
	 */
	public IDataSourceProtocolHandler register( String scheme, IDataSourceProtocolHandler dsh );
	
	
	/**
	 * DataSourceFactory uses this default protocol handler 
	 * for all unregistered schemes
	 *   
	 * @return default protocol handler
	 */
	public IDataSourceProtocolHandler getDefaultHandler();
	
	
	/**
	 * @see getDefaultHandler()
	 * @param dsh
	 */
	public void setDefaultHandler( IDataSourceProtocolHandler dsh );
	
	
	/**
	 * @see <code>IDataSourcePreprocessor</code>
	 * @return
	 */
	public IDataSourcePreprocessor getPreprocessor();
	
	/**
	 * injection of data source URI preprocessor
	 * 
	 * @param preprocessor ( set null to disable preprocessing ) 
	 */
	public void setPreprocessor( IDataSourcePreprocessor preprocessor );
		

}
