/*
 * Created on May 30, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package org.openl.conf;

import java.io.File;
import java.util.Properties;
import org.openl.util.Log;
import org.openl.OpenConfigurationException;


/**
 * 
 * This class is responsible for configuration of a particular OpenL instance designated by it's instance_name.
 *
 * @author snshor
 * 
 * Mofified on 2005/04/19 by SM: lowercased "OpenL.properties"
 * 
 * Modified on 2005/07/15 by JF: renamed lang.config to openrules.config
 * 
 * Modified on 2005/08/26 by JF: renamed openl.properties to openrules.properties
 * 
 * Modified on 2006/08/04 by JF: removed dependencies of AntOpenLBuilder. 
 * Setup a new DEFAULT_BUILDER_CLASS_NAME as "com.openrules.java.OpenRulesLBuilder";
 *
 */

public class OpenLConfigurator extends Configurator
{

	static public final String OPENL_ALT_CONFIG_ROOT = "openrules.config";

  static final public String OPENL_DEFAULT_PROPERTY_FILE_NAME =
    "openrules.properties"; 
  static final public String OPENL_PROPERTY_FILE_PROPERTY = "properties.file";

  static final public String DEFAULT_BUILDER_CLASS_PROPERTY = 
  	  "org.openl.builderclass";
  	    
  	  
  static final public String BUILDER_CLASS = ".builderclass";
  
  static final public String DEFAULT_BUILDER_CLASS_PATH_PROPERTY =
    "org.openl.builder.classpath";
	static final public String BUILDER_CLASS_PATH =
	      ".builder.classpath";
  static final public String DEFAULT_BUILDER_CLASS_NAME =
     //"org.openl.conf.ant.AntOpenLBuilder";
	 "com.openrules.java.OpenRulesLBuilder";
  
   static PropertyFileLoader OPENL_GLOBAL_PROPERTY_PROVIDER =
    new PropertyFileLoader(
      OPENL_DEFAULT_PROPERTY_FILE_NAME,
      "org.openl." + OPENL_PROPERTY_FILE_PROPERTY,
      new ConfigurableResourceContext(null),
      null);

//  ClassLoaderFactory clFactory = new ClassLoaderFactory();

  Properties mergeProperties(Properties properties, Properties defaults)
  {
    Properties result = null;

    if (defaults != null && defaults != PropertyFileLoader.NO_PROPERTIES)
    {
      result = new Properties(defaults);
    }

    if (properties != null && properties != PropertyFileLoader.NO_PROPERTIES)
    {
      if (result == null)
        result = new Properties();

      result.putAll(properties);
    }

    return result != null ? result : PropertyFileLoader.NO_PROPERTIES;
  }
  
  
  public String makeAlternativeHome(String userHome)
  {
  	try
  	{
  		String name = userHome + "/../" + OPENL_ALT_CONFIG_ROOT;
  		String altHome = new File(name).getCanonicalPath();
  		/*if (altHome!=null)
  			Log.info("ALT_HOME=<"+altHome+">");
  		else
  			Log.info("ALT_HOME <" + name +"> not found");
  		*/
  		return altHome;
  	}
  	catch(Throwable t)
  	{
  		return null;
  	}
  }

  public synchronized IOpenLBuilder getBuilder(
    String openlName,
    IUserContext ucxt)
    throws OpenConfigurationException
  {
 //    ClassLoaderFactory.setCurrentFactory(clFactory);

	 String userHome = ucxt.getUserHome();
	 String altHome = makeAlternativeHome(userHome);

		String[] homes = altHome == null ? new String[]{userHome} : new String[]{userHome, altHome};

    ConfigurableResourceContext cxt =
      new ConfigurableResourceContext(
        ucxt.getUserClassLoader(),
        homes);

    PropertyFileLoader propertyLoader =
      new PropertyFileLoader(
        openlName + "." + OPENL_DEFAULT_PROPERTY_FILE_NAME,
        openlName + "." + OPENL_PROPERTY_FILE_PROPERTY,
        cxt,
        new PropertyFileLoader(
          OPENL_DEFAULT_PROPERTY_FILE_NAME,
          "org.openl." + OPENL_PROPERTY_FILE_PROPERTY,
          cxt,
          null));

    Properties pp = propertyLoader.getProperties();
    pp = mergeProperties(pp, ucxt.getUserProperties());
    if (pp != PropertyFileLoader.NO_PROPERTIES)
      cxt.setProperties(pp);

    try
    {
      IOpenLBuilder builder = makeBuilder(openlName,cxt, ucxt);
      builder.setConfigurableResourceContext(cxt, ucxt);

      return builder;
    }
    catch (Exception ex)
    {
      throw new OpenConfigurationException(
        "Error creating builder: ",
        null,
        ex);
    }
  }

  IOpenLBuilder makeBuilder(
    String openl,
    IConfigurableResourceContext cxt,
    IUserContext ucxt)
    throws Exception
  {
    String builderClassName = getBuilderClassName(openl, cxt);
    String builderClassPath = getBuilderClassPath(openl, cxt);

    ClassLoader cl = ClassLoaderFactory.getOpenlCoreLoader();
    if (builderClassPath != null)
    {
      cl =
	        ClassLoaderFactory.createUserClassloader(
          "builder.classloader:",
          builderClassPath,
          ClassLoaderFactory.getOpenlCoreLoader(),
          ucxt);
    }

    return (IOpenLBuilder)ClassFactory.newInstanceForName(builderClassName, cl);
  }

  String getBuilderClassName(String openl, IConfigurableResourceContext cxt)
  {

    String builderClassName = cxt.findProperty(openl + BUILDER_CLASS);
		if (builderClassName == null)
		  builderClassName = cxt.findProperty(DEFAULT_BUILDER_CLASS_PROPERTY);
		
    return builderClassName == null
      ? DEFAULT_BUILDER_CLASS_NAME
      : builderClassName;
  }

  String getBuilderClassPath(String openl, IConfigurableResourceContext cxt)
  {

    String builderClassPath = cxt.findProperty(openl + BUILDER_CLASS_PATH);
    if (builderClassPath == null)
      builderClassPath = cxt.findProperty(DEFAULT_BUILDER_CLASS_PATH_PROPERTY);

    return builderClassPath;
  }

}
