/*
 * Created on Jun 24, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */
 
package org.openl.types.xml;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.openl.types.IAggregateInfo;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenField;
import org.openl.types.IOpenIndex;
import org.openl.types.IOpenMethod;
import org.openl.types.IOpenSchema;
import org.openl.types.impl.AAggregateInfo;
import org.openl.types.impl.AOpenField;
import org.openl.types.impl.AOpenMethod;
import org.openl.types.impl.ArrayLengthOpenField;
import org.openl.types.impl.ArrayOpenClass;
import org.openl.types.impl.MethodSignature;
import org.openl.types.java.JavaOpenClass;
import org.openl.vm.IRuntimeEnv;
import org.w3c.dom.Node;

/**
 * @author snshor
 *
 */
public class DOMElementArrayClass extends ArrayOpenClass
{

	HashMap methodMap;


  /**
   * @param schema
   * @param componentClass
   * @param lengthOpenField
   * @param index
   */
  public DOMElementArrayClass(
    IOpenSchema schema,
    IOpenClass componentClass)
  {
    super(schema, componentClass, new NodeArrayLengthField());
    addMethods();
  }
  
  
  void addMethods()
  {
  	methodMap = new HashMap();
  	 IOpenMethod m = new AddMethod(new IOpenClass[]{ componentClass},this);
  	
  	methodMap.put(new MethodKey(m), m);
  }
  
  static class AddMethod extends AOpenMethod
  {
  	
    /**
     * @param name
     * @param typeClass
     * @param parameterTypes
     * @param declaringClass
     * @param isStatic
     * @param info
     */
    public AddMethod(
      IOpenClass[] parameterTypes,
      IOpenClass declaringClass
      )
    {
      super("add", JavaOpenClass.VOID, new MethodSignature( parameterTypes), declaringClass);
    }



      /* (non-Javadoc)
     * @see org.openl.types.IMethodCaller#invoke(java.lang.Object, java.lang.Object[])
     */
    public Object invoke(Object target, Object[] params, IRuntimeEnv env)
    {
    	DOMArrayAccessor array = (DOMArrayAccessor)target;
    	array.parent.appendChild((Node)params[0]);
      return null;
    }

}
  

  /* (non-Javadoc)
   * @see org.openl.types.IOpenClass#getInstanceClass()
   */
  public Class getInstanceClass()
  {
    // TODO Auto-generated method stub
    return null;
  }

  /* (non-Javadoc)
   * @see org.openl.types.IOpenClass#isAssignableFrom(java.lang.Class)
   */
  public boolean isAssignableFrom(Class c)
  {
    // TODO Auto-generated method stub
    return false;
  }

  /* (non-Javadoc)
   * @see org.openl.types.IOpenClass#isAssignableFrom(org.openl.types.IOpenClass)
   */
  public boolean isAssignableFrom(IOpenClass ioc)
  {
    return ioc == this;
  }

  /* (non-Javadoc)
   * @see org.openl.types.IOpenClass#isInstance(java.lang.Object)
   */
  public boolean isInstance(Object instance)
  {
    // TODO Auto-generated method stub
    return false;
  }

  /* (non-Javadoc)
   * @see org.openl.types.IOpenClass#superClasses()
   */
  public Iterator superClasses()
  {
    // TODO Auto-generated method stub
    return null;
  }
  
  static class DOMArrayAccessor 
  {
  	String name;
  	Node parent;
		DOMArrayAccessor(String name, Node parent)
		{
			this.name = name;
			this.parent = parent;
		}
  }


	static class NodeArrayLengthField extends ArrayLengthOpenField
	{
		
	    /* (non-Javadoc)
     * @see org.openl.types.ArrayLengthOpenField#getLength(java.lang.Object)
     */
    public int getLength(Object target)
    {
    	DOMArrayAccessor array = (DOMArrayAccessor)target;
      return DomUtil.countNodes(array.name, array.parent);
    }

}

	static public IOpenField getOpenField(String name, IOpenClass nodeType)
	{
		return new NodeArrayField(name, nodeType);
	}

	static class NodeArrayField extends AOpenField
	{
		
    /**
     * @param name
     * @param type
     */
    public NodeArrayField(String name, IOpenClass type)
    {
    	//TODO schema not null
      super(name, new DOMElementArrayClass(null,  type));
    }

	    /* (non-Javadoc)
     * @see org.openl.types.IOpenField#get(java.lang.Object)
     */
    public Object get(Object target, IRuntimeEnv env)
    {
      return new DOMArrayAccessor(name, (Node)target);
    }

    /* (non-Javadoc)
     * @see org.openl.types.IOpenField#set(java.lang.Object, java.lang.Object)
     */
    public void set(Object target, Object value, IRuntimeEnv env)
    {
			throw new UnsupportedOperationException();
    }

} 
  
  class NodeArrayIndex implements IOpenIndex
  {
      /* (non-Javadoc)
     * @see org.openl.types.IOpenIndex#getElementType()
     */
    public IOpenClass getElementType()
    {
      return componentClass;
    }

    /* (non-Javadoc)
     * @see org.openl.types.IOpenIndex#getIndexType()
     */
    public IOpenClass getIndexType()
    {
      return JavaOpenClass.INT;
    }

    /* (non-Javadoc)
     * @see org.openl.types.IOpenIndex#getValue(java.lang.Object, java.lang.Object)
     */
    public Object getValue(Object container, Object index)
    {
    	DOMArrayAccessor array = (DOMArrayAccessor)container;
			int i = ((Integer)index).intValue();
    	
      return DomUtil.findNode(array.name, i, array.parent);
    }

    /* (non-Javadoc)
     * @see org.openl.types.IOpenIndex#isWritable()
     */
    public boolean isWritable()
    {
      return false;
    }

    /* (non-Javadoc)
     * @see org.openl.types.IOpenIndex#setValue(java.lang.Object, java.lang.Object, java.lang.Object)
     */
    public void setValue(Object container, Object index, Object value)
    {
    	throw new UnsupportedOperationException();
    }

}
  
  
  
  

  /* (non-Javadoc)
   * @see org.openl.types.ArrayOpenClass#index()
   */
  protected IOpenIndex index()
  {
    return new NodeArrayIndex();
  }

  /* (non-Javadoc)
   * @see org.openl.types.impl.AOpenClass#methodMap()
   */
  protected Map methodMap()
  {
    return methodMap;
  }

  /* (non-Javadoc)
   * @see org.openl.types.IOpenClass#getAggregateInfo()
   */
  public IAggregateInfo getAggregateInfo()
  {
    return info;
  }
  
  static final IAggregateInfo info = new DOMArrayAggregateInfo();
  
  static class DOMArrayAggregateInfo extends AAggregateInfo
  {
  	
      /* (non-Javadoc)
     * @see org.openl.types.IAggregateInfo#getComponentType(org.openl.types.IOpenClass)
     */
    public IOpenClass getComponentType(IOpenClass aggregateType)
    {
      return ((DOMElementArrayClass)aggregateType).componentClass;
    }

    /* (non-Javadoc)
     * @see org.openl.types.IAggregateInfo#getIndex(org.openl.types.IOpenClass)
     */
    public IOpenIndex getIndex(IOpenClass aggregateType, IOpenClass indexType)
    {
    	if (indexType != JavaOpenClass.INT)
    	  return null;
    	  
		return ((DOMElementArrayClass)aggregateType).index();
    }

    /* (non-Javadoc)
     * @see org.openl.types.IAggregateInfo#getIterator(java.lang.Object)
     */
    public Iterator getIterator(Object aggregate)
    {
      // TODO Auto-generated method stub
      return null;
    }

    /* (non-Javadoc)
     * @see org.openl.types.IAggregateInfo#isAggregate()
     */
    public boolean isAggregate(IOpenClass type)
    {
      return true;
    }

}
  

  /* (non-Javadoc)
   * @see org.openl.types.IOpenClass#newInstance(org.openl.vm.IRuntimeEnv)
   */
  public Object newInstance(IRuntimeEnv env)
  {
    // TODO Auto-generated method stub
    throw new UnsupportedOperationException();
  }

}
