/*
 * Created on Jun 18, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */

package org.openl.types.xml;

import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.net.MalformedURLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.openl.binding.OpenLRuntimeException;
import org.openl.types.IAggregateInfo;
import org.openl.types.IMemberMetaInfo;
import org.openl.types.IMethodSignature;
import org.openl.types.IOpenClass;
import org.openl.types.IOpenFactory;
import org.openl.types.IOpenMethod;
import org.openl.types.impl.AOpenClass;
import org.openl.types.impl.AOpenMethod;
import org.openl.types.impl.MethodSignature;
import org.openl.types.java.JavaNoAggregateInfo;
import org.openl.types.java.JavaOpenClass;
import org.openl.util.AOpenIterator;
import org.openl.util.ASelector;
import org.openl.util.ISelector;
import org.openl.util.OpenIterator;
import org.openl.util.RuntimeExceptionWrapper;
import org.openl.vm.IRuntimeEnv;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;

import com.openrules.datasource.DataSourceFactory;
import com.openrules.datasource.IDataSource;
import com.openrules.datasource.IDataSourceFactory;

/**
 * This class works with any XML Document read using DOM Parser. Because we
 * don't know any schema information in advance we are just going to use
 * run-time values of the node to provide an "illusion" of an OpenClass; it is
 * very dynamic
 * 
 * @author snshor
 * 
 */

public class DOMOpenClass extends AOpenClass {
	String longName;

	HashMap fieldMap = null;

	HashMap methodMap = null;

	DOMOpenSchema.NodeInfo info;

	IDataSourceFactory dataSourceFactory;

	public DOMOpenClass(String longName, DOMOpenSchema schema,
			DOMOpenSchema.NodeInfo info) {
		super(schema);
		this.longName = longName;
		this.info = info;
	}

	public IDataSourceFactory getDataSourceFactory() {
		return dataSourceFactory;
	}

	public void setDataSourceFactory(IDataSourceFactory dataSourceFactory) {
		this.dataSourceFactory = dataSourceFactory;
	}

	final DOMOpenSchema domSchema() {
		return (DOMOpenSchema) schema;
	}

	public String getTagName() {
		int idx = longName.lastIndexOf('.');
		return idx < 0 ? longName : longName.substring(idx + 1);
	}

	DOMOpenConstructor domConstructor;

	synchronized DOMOpenConstructor getConstructor() {
		if (domConstructor == null)
			domConstructor = new DOMOpenConstructor(this);
		return domConstructor;
	}

	protected synchronized Map methodMap() {
		if (methodMap != null)
			return methodMap;

		methodMap = new HashMap();

		DOMOpenConstructor oc = getConstructor();

		methodMap.put(new MethodKey(oc), oc);

		SaveMethod sm = new SaveMethod();

		methodMap.put(new MethodKey(sm), sm);

		LoadMethod lm = new LoadMethod();

		methodMap.put(new MethodKey(lm), lm);

		return methodMap;

	}

	class SaveMethod extends AOpenMethod {

		public SaveMethod() {
			super("save", JavaOpenClass.VOID, new MethodSignature(
					new IOpenClass[] { JavaOpenClass.STRING },
					new String[] { "fileName" }), DOMOpenClass.this);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IMethodCaller#invoke(java.lang.Object,
		 *      java.lang.Object[])
		 */
		public Object invoke(Object target, Object[] params, IRuntimeEnv env) {
			try {
				DomUtil.saveNodeAsDocument((String) params[0], (Node) target);
			} catch (Exception ex) {
				throw RuntimeExceptionWrapper.wrap(ex);
			}
			return null;
		}

	}

	class LoadMethod extends AOpenMethod {

		public LoadMethod() {
			super("load", DOMOpenClass.this, new MethodSignature(
					new IOpenClass[] { JavaOpenClass.STRING }),
					DOMOpenClass.this, true, null);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IMethodCaller#invoke(java.lang.Object,
		 *      java.lang.Object[])
		 */
		public Object invoke(Object target, Object[] params, IRuntimeEnv env) {
			try {
				return DomUtil.getDocument(
						resolveInputSource((String) params[0])).getFirstChild();
			} catch (Exception ex) {
				throw RuntimeExceptionWrapper.wrap(ex);
			}
		}

	}

	protected InputSource resolveInputSource(String uri) throws Exception {
		Reader r = schema.getFactory().getDataSourceFactory().fromString(uri)
				.getReader();
		if (r == null)
			throw new Exception("Document is not found: " + uri);
		return new InputSource(r);
	}

	protected synchronized Map fieldMap() {
		if (fieldMap != null)
			return fieldMap;

		fieldMap = new HashMap();

		for (Iterator iter = info.elementNames.entrySet().iterator(); iter
				.hasNext();) {
			Map.Entry element = (Map.Entry) iter.next();
			int cnt = ((Integer) element.getValue()).intValue();
			String name = (String) element.getKey();
			IOpenClass componentType = schema.getType(getName() + '.' + name);

			if (cnt == 1) {
				fieldMap.put(name, new DOMElementNodeField(name, this,
						componentType));
			} else {
				fieldMap.put(name, DOMElementArrayClass.getOpenField(name,
						componentType));
			}

		}

		if (info.hasText)
			fieldMap.put("_cdata_", new DOMTextNodeField("_cdata_", this));

		for (Iterator iter = info.attrNames.iterator(); iter.hasNext();) {
			String attr = (String) iter.next();
			fieldMap.put(attr, new DOMAttributeNodeField(attr, this));

		}

		// release info
		info = null;
		return fieldMap;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#getFactory()
	 */
	public IOpenFactory getFactory() {
		return null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#superClasses()
	 */
	public Iterator superClasses() {
		return OpenIterator.EMPTY;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#isAbstract()
	 */
	public boolean isAbstract() {
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#isInstance(java.lang.Object)
	 */
	public boolean isInstance(Object instance) {
		return (instance instanceof Node);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#isAssignableFrom(org.openl.types.IOpenClass)
	 */
	public boolean isAssignableFrom(IOpenClass ioc) {
		return ioc == this;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#isAssignableFrom(java.lang.Class)
	 */
	public boolean isAssignableFrom(Class c) {
		return Node.class.isAssignableFrom(c);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#getInstanceClass()
	 */
	public Class getInstanceClass() {
		return Node.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.base.INamedThing#getName()
	 */
	public String getName() {
		return longName;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.binding.IMethodFactory#getMatchingMethod(java.lang.String,
	 *      org.openl.types.IOpenClass[])
	 */
	static class DOMTextNodeField extends DOMField {

		/**
		 * @param name
		 * @param declaringClass
		 */
		public DOMTextNodeField(String name, IOpenClass declaringClass) {
			super(name, declaringClass);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IOpenField#get(java.lang.Object)
		 */
		public Object get(Object target, IRuntimeEnv env) {
			Node parent = (Node) target;
			return getText(parent);
		}

		public IOpenClass getType() {
			return JavaOpenClass.STRING;
		}
	}

	static Node findFirstNode(Node parent, ISelector sel) {
		return (Node) new NodeIterator(parent).select(sel).next();
	}

	static String getText(Node parent) {
		Node text = findFirstNode(parent, new NodeTypeSelector(Node.TEXT_NODE)
				.or(new NodeTypeSelector(Node.CDATA_SECTION_NODE)));
		if (text == null)
			return null;
		return text.getNodeValue();
	}

	static class NodeIterator extends AOpenIterator {

		Node child;

		NodeIterator(Node parent) {
			child = parent.getFirstChild();
		}

		public boolean hasNext() {
			return child != null;
		}

		public Object next() {
			return nextNode();
		}

		public Node nextNode() {
			Node ret = child;
			child = child.getNextSibling();
			return ret;
		}

	}

	static class NodeTypeSelector extends ASelector.IntValueSelector {
		/**
		 * @param value
		 */
		public NodeTypeSelector(int value) {
			super(value);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.util.ASelector.IntValueSelector#getIntValue(java.lang.Object)
		 */
		protected int getIntValue(Object test) {
			return ((Node) test).getNodeType();
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#isArray()
	 */
	public boolean isArray() {
		// TODO Auto-generated method stub
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#getArrayType(int)
	 */
	public IOpenClass getArrayType(int dim) {
		// TODO Auto-generated method stub
		return null;
	}

	public Object newInstance(IRuntimeEnv env) {
		return getConstructor().invoke(null, null, env);
	}

	static class DOMOpenConstructor implements IOpenMethod {

		DOMOpenClass openClass;

		DOMOpenConstructor(DOMOpenClass openClass) {
			this.openClass = openClass;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IMethodCaller#invoke(java.lang.Object,
		 *      java.lang.Object[])
		 */
		public Object invoke(Object target, Object[] params, IRuntimeEnv env) {
			return openClass.domSchema().getDocument().createElement(
					openClass.getTagName());
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IOpenMember#getDeclaringClass()
		 */
		public IOpenClass getDeclaringClass() {
			return openClass;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IOpenMember#getInfo()
		 */
		public IMemberMetaInfo getInfo() {
			return null;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IOpenMember#getType()
		 */
		public IOpenClass getType() {
			return openClass;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IOpenMember#isStatic()
		 */
		public boolean isStatic() {
			return true;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IMethodCaller#getMethod()
		 */
		public IOpenMethod getMethod() {
			return this;
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.base.INamedThing#getName()
		 */
		public String getName() {
			return openClass.getName();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.openl.types.IOpenMethodHeader#getSignature()
		 */
		public IMethodSignature getSignature() {
			return IMethodSignature.VOID;
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.openl.types.IOpenClass#getAggregateInfo()
	 */
	public IAggregateInfo getAggregateInfo() {
		return JavaNoAggregateInfo.NO_AGGREGATE;
	}

}
