/*
 * Created on Jun 18, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */
 
package org.openl.types.xml;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import org.openl.types.IOpenFactory;
import org.openl.types.impl.AOpenSchema;
import org.openl.util.Log;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;

/**
 * @author snshor
 *
 */
public class DOMOpenSchema extends AOpenSchema
{
	Document doc;
	
	
//	HashMap infoMap = null;

	
	public DOMOpenSchema(IOpenFactory	factory, Document doc)
	{
		super(factory);
		this.doc = doc;
	}
	
	public Document getDocument()
	{
		return doc;
	}
	
	
//	public void setFile(File f) throws Exception
//	{
//		InputSource xmlSource = new InputSource(new FileReader(f));
//		doc = DomUtil.getDocument(xmlSource);
////		infoMap = analyzeDoc(doc);
//	}
	
	
	
	
	synchronized HashMap analyzeDoc(Node topNode)
	{
		HashMap map = new HashMap();
		if (topNode != null)
		  topNode = topNode.getFirstChild();
		analyzeNode(topNode, topNode.getNodeName(), map);
		return map;
		
	}
	
	void analyzeNode(Node node, String name, Map map)
	{
		if (node == null)
		  return;
		
//		System.out.println("--- " + name);
		
		NodeInfo oldInfo = (NodeInfo) map.get(name);

		NodeInfo info = new NodeInfo(); 
		
		Node child = node.getFirstChild();
		

		for (; child != null ; child=child.getNextSibling())
		{
			switch (child.getNodeType())
			{
				case Node.CDATA_SECTION_NODE :
				case Node.TEXT_NODE :
					info.hasText = true;
//					System.out.println("--- " + "#text");
					
					break;
				case Node.ELEMENT_NODE:
				  info.addElement(child.getNodeName());
//					System.out.println("--- " + child.getNodeName());
				  
				  analyzeNode(child, name + '.' + child.getNodeName(), map);
					break;
				default :
					Log.debug("Unprocessed node type");
			}
		}
    
   
	 	NamedNodeMap attrs = node.getAttributes();
    if (attrs != null)
    {
			for( int i = 0; i < attrs.getLength(); ++i)
			{
				Node attr = attrs.item(i);
				info.attrNames.add(attr.getNodeName()); 
			}
		}
	
		
		
		if (oldInfo != null)
		  oldInfo.merge(info);
		else
		  map.put(name, info);
		
	}
	
	
	static class NodeInfo
	{
		
		HashSet attrNames = new HashSet();
		HashMap elementNames = new HashMap();
		boolean hasText = false;
		
		void addElement(String elementName)
		{
			Integer value = (Integer) elementNames.get(elementName);
			elementNames.put(elementName, new Integer(value == null ? 1 : value.intValue() + 1));
		}
		
		public String toString()
		{
			StringBuffer buf = new StringBuffer();
			boolean first = true;
			buf.append('[');
			for (Iterator iter = attrNames.iterator(); iter.hasNext();)
      {
      	if (!first)
      	  buf.append(",");
        String element = (String)iter.next();
        buf.append(element);
      }
      buf.append(']');
			
			
			for (Iterator iter = elementNames.entrySet().iterator(); iter.hasNext();)
      {
        Map.Entry element = (Map.Entry)iter.next();
      	buf.append('<').append(element.getKey()).append(',').append(element.getValue()).append('>');  
      }
			
			
			return buf.toString();
		}
		
		void merge(NodeInfo anInfo)
		{
			this.hasText = hasText || anInfo.hasText;
			
			attrNames.addAll(anInfo.attrNames);
			
			for (Iterator iter = anInfo.elementNames.entrySet().iterator(); iter.hasNext();)
      {
        Map.Entry element = (Map.Entry)iter.next();
        Integer value = (Integer) elementNames.get(element.getKey());
        if (value == null)
        
          elementNames.put(element.getKey(), element.getValue());
        else
        {
					int n1 = value.intValue();
					int n2 = ((Integer)element.getValue()).intValue();
					if (n2 > n1)
					  elementNames.put(element.getKey(), element.getValue());  
        }
      }
			
		}
	}
	

	



  /* (non-Javadoc)
   * @see org.openl.types.impl.AOpenSchema#buildAllClasses()
   */
  protected Map buildAllClasses()
  {
  	Map map = new HashMap();
  	
  	Map infoMap = analyzeDoc(doc);
		
		for (Iterator iter = infoMap.entrySet().iterator(); iter.hasNext();)
    {
      Map.Entry entry = (Map.Entry)iter.next();
      NodeInfo info = (NodeInfo)entry.getValue();
      
      DOMOpenClass openClass = new DOMOpenClass((String)entry.getKey(), this, info);
      map.put(entry.getKey(), openClass);
    }  	
  	
    return map;
  }

}
