/*
 * Created on Jun 3, 2003
 *
 * Developed by Intelligent ChoicePoint Inc. 2003
 */
 
package org.openl;

import java.io.File;
import java.io.FileReader;
import java.net.URL;
import java.net.URLClassLoader;

import org.openl.binding.OpenLRuntimeException;
import org.openl.conf.IUserContext;
import org.openl.conf.UserContext;
import org.openl.syntax.impl.StringSourceCodeModule;
import org.openl.util.Log;
import org.openl.util.RuntimeExceptionWrapper;

//import org.openl.util.IBlock;

import junit.framework.Assert;
import junit.framework.TestCase;

/**
 * @author snshor
 *
 */
public class OpenLTest extends TestCase
{

  /**
   * Constructor for OpenLTest.
   * @param arg0
   */
  public OpenLTest(String arg0)
  {
    super(arg0);
  }

	static IUserContext userContext = null;

	static IUserContext getUserContext() 
	{
    try
    {
			if (userContext == null)
			{
	      String projectDir = "c:/_SVN/openrules.dev/openl.test.simple.conf.test";
	      URL url = new File(projectDir+ "/classes").getCanonicalFile().toURL();
	      
	      ClassLoader userLoader = new URLClassLoader( new URL[]{url} );
	      userContext = new UserContext(userLoader , projectDir);
      
			}
      return userContext;
    }
    catch (Exception e)
    {
      throw RuntimeExceptionWrapper.wrap("", e);
    }
	}

	public static Object evaluate(String code, String openl) throws OpenLRuntimeException
	{
		
		OpenL op = OpenL.getInstance(openl, getUserContext());
		return op.evaluate(new StringSourceCodeModule(code, null));
	}

	public static Object evaluateFile(String filename, String openl) throws Exception
	{
	
		filename = "c:/_SVN/openrules.dev/openl.test.simple.conf.test/" + filename;
			
		Log.error("======== Evaluating " + filename);	
		FileReader reader = new FileReader(filename);
		
		int c;
		StringBuffer buf = new StringBuffer();
		while((c = reader.read()) != -1)
			buf.append((char)c);		
		
		return evaluate(buf.toString(), openl);
		
	}




	public static void _testEvaluateFile(String filename, Object result, String openl) throws Exception
	{
		
		FileReader reader = new FileReader("c:/Eclipse.dev/workspace/openl.test.simple.conf.test/" + filename);
		
		int c;
		StringBuffer buf = new StringBuffer();
		while((c = reader.read()) != -1)
		  buf.append((char)c);		
		
		_testEvaluate(buf.toString(), result, openl);
		
	}


	public static void _testEvaluate(String code, Object result, String openl) throws Exception
	{
		OpenL op = OpenL.getInstance(openl, getUserContext());
		
		System.out.print(code + " = ");

		Object res = op.evaluate(new StringSourceCodeModule(code, null));
		
		System.out.println(res);
		
		Assert.assertEquals(result, res);
	}

	public static void _testError(String code, String openl)
	{
		OpenL op = OpenL.getInstance(openl, getUserContext());
		boolean failed = false;
		try
		{
			op.evaluate(new StringSourceCodeModule(code, null));
		}
		catch(Throwable t)
		{
			failed = true;
		}
		Assert.assertTrue(failed);
		
	}
	
	public void testScience() throws Exception
	{
		_testEvaluate("sin(5) + 4", new Double(Math.sin(5) + 4), "org.openl.j.science");
		
	}
	
	public void testUnits() throws Exception
	{
//		_testPrint("15[m] + 7[cm]", "15.07m");
//		_testPrint("36[km/h]", "10m/s");

		OpenLTest._testEvaluate(
			"IMultiplicativeExpression dist = 15[m] + 7[cm];" 
			+ "String s2 = dist.printAs(m, \"m\", 2);" 
			+ "s2" ,
			"15.07m",
			"org.openl.j.science");


		OpenLTest._testEvaluate(
			"(15[m] + 7[cm]).printAs(m, \"m\", 2)",
			"15.07m",
			"org.openl.j.science");
			
			
		OpenLTest._testEvaluate(
			"(15[m] + 7[cm]).printAs(cm, \"cm\", 2)",
			"1,507cm",
			"org.openl.j.science");
		OpenLTest._testEvaluate(
			"90[mph].printAs(km/h, \"km/h\", 2)",
			"144.81km/h",
			"org.openl.j.science");

		OpenLTest._testEvaluate(
			"(90[mph] + 11[km/h]).printAs(km/h, \"km/h\", 2)",
			"155.81km/h",
			"org.openl.j.science");

		OpenLTest._testEvaluate(
			"10[m/s].printAs(km/h, \"km/h\", 2)",
			"36km/h",
			"org.openl.j.science");

		OpenLTest._testEvaluate(
			"double x = 90; (x[mi]/60[min] + 11[km/h] - 20[m/s]).printAs(km/h, \"km/h\", 2)",
			"83.81km/h",
			"org.openl.j.science");
		
		
	}
	
	
	public void testEvaluate() throws Exception
	{

		_testEvaluate("5.5 + 4", new Double(9.5), "org.openl.j");
		_testEvaluate("5+3", new Integer(8), "org.openl.j");
		_testEvaluate("5.5 + 4.5", new Double(10), "org.openl.j");
		_testEvaluate("5.5 + 4", new Double(9.5), "org.openl.j");

		_testEvaluate("\"ab\\0123c\"", "ab\0123c", "org.openl.j");	
		_testEvaluate("\"ab\\012c\"", "ab\012c", "org.openl.j");	
		_testEvaluate("\"ab\\77c\"", "ab\77c", "org.openl.j");	
		_testEvaluate("\"ab\\u7777c\"", "ab\u7777c", "org.openl.j");	

		_testEvaluate("\"abc\"", "abc", "org.openl.j");	
		_testEvaluate("\"ab\\nc\"", "ab\nc", "org.openl.j");	

		_testEvaluate("\"abc\" + \"def\"", "abcdef", "org.openl.j");	

		_testEvaluate("double z = 5; z", new Double(5), "org.openl.j");
		_testEvaluate("int x=5; x + 4", new Integer(9), "org.openl.j");	
		
		_testEvaluate("int x = 5, y = x + 1; x = y + 2; x", new Integer(8), "org.openl.j");
		
		_testEvaluate("int x = 5, y = x + 1; y", new Integer(6), "org.openl.j");

		_testEvaluate("int sum = 0; for(int i = 0; i < 10; ++i) sum = sum + i; sum", new Integer(45), "org.openl.j");

		_testEvaluate("int sum = 0, sum2 = 10; for(int i = 0; i < 10; ++i) {sum = sum + i; sum2 = sum2 + i;} sum2", new Integer(55), "org.openl.j");

  	
		_testEvaluate("Object x = \"abc\"; String y = (String)x + \"d\"; y", "abcd", "org.openl.j" );


		_testEvaluate("long y = -11; int x = (int)y; x", new Integer(-11), "org.openl.j" );
		  	
		_testEvaluate("long y = -11; while(y < 0)  y+=2; y", new Long(1), "org.openl.j" );

		_testEvaluate("double y = -1; if (y > 0) ++y;else y--;  y--", new Double(-2), "org.openl.j" );

		_testEvaluate("double y = -1; if (y < 0) ++y;  ++y", new Double(1), "org.openl.j" );

		_testEvaluate("double[] y = new double[2];  y[0] = 3; y[1] = 4;y[0] + y[1]", new Double(7), "org.openl.j" );
  	
		_testEvaluate("double[][] y = {{1, 2}, {3, 4}};  y[0][0] + y[1][1]", new Double(5), "org.openl.j" );

		_testEvaluate("String[] x = {\"abc\", \"def\"}; Object y = x; String[] z = (String[])y; z[0] + z[1] ", "abcdef", "org.openl.j" );


	 _testEvaluate("int x = 4;int[] y = {1, 2, 3, x};  y[2] + y[3]", new Integer(7), "org.openl.j" );

		
	}

  public void oldTestEvaluate() throws Exception
  {

		_testBenchmark("5+3", new Integer(8), "org.openl.j", 500,1000);
		_testEvaluate("Customer c = new Customer(); c.setName(\"abc\");c.getName()", "abc", "org.openl.j.test" );
		_testEvaluate("Vector v = new Vector(); v.add(\"abc\");v.size()", new Integer(1), "org.openl.j.test" );

  	
		_testEvaluate("staticStringAry1[0] = \"abcd\"; staticStringAry1[0].length()", new Integer(4), "org.openl.j.science");
  	
  	
  	
		_testEvaluate("staticStringAry1.length + 1",new Integer(3), "org.openl.j.science");
		_testEvaluate("staticStringAry1[1]","def", "org.openl.j.science");
  	_testEvaluate("staticIndexedInt[2]", new Integer(2), "org.openl.j.science");
		_testEvaluate("new Double(5)", new Double(5), "org.openl.j.science");




		_testEvaluate("sin(5) + 4", new Double(Math.sin(5) + 4), "org.openl.j.science");
		_testEvaluate("cos(0) + 4", new Double(5), "org.openl.j.science");
		_testEvaluate("sin(PI/6)", new Double(Math.sin(Math.PI/6)), "org.openl.j.science");
//		_testEvaluate("getRuntime()", Runtime.getRuntime(), "org.openl.j.science");

		
		_testEvaluate("staticStringABCDEF.length()", new Integer(6), "org.openl.j.science");
		_testEvaluate("instance1.memberStringABC", "ABC", "org.openl.j.science");
		_testEvaluate("instance1.memberStringABC.length()", new Integer(3), "org.openl.j.science");
		_testEvaluate("staticStringABCDEF.indexOf(instance1.memberStringABC)", new Integer(0), "org.openl.j.science");
		
		
  }


	public void _testBenchmark(final String code, final Object res, final String openl, int nTimes, final long maxMicros)
	{
//		IBlock ib = new IBlock()
//		{
//			public void evaluate(Object obj)
//			{
//				try
//				{
//					_testEvaluate(code, res,  openl);
//				}
//				catch(Exception ex)
//				{
//					throw RuntimeExceptionWrapper.wrap(ex);
//				}
//				
//			}
//		};
//		benchmark(ib, nTimes, "Code(" + code + ")", maxMicros);
	}
	
//	void benchmark(IBlock block,  int nTimes, String descr, long maxMicros)
//	{
//		long start = System.currentTimeMillis();
//		
//		for(int i = 0; i < nTimes; ++i)
//			block.evaluate(null);
//			
//		long end = System.currentTimeMillis();
//		
//		long mks = (end - start) * 1000 /nTimes;
//		System.out.println("Benchmark " + descr + ": " + mks + "mks. Expected "  + maxMicros);
//		
//	}

}
