/*
 * Created on Apr 30, 2004
 *
 * Developed by OpenRules.com 2003-2004
 */
package org.openl.domain;

import java.util.BitSet;
import java.util.Iterator;

import org.openl.util.AOpenIterator;

/**
 * @author snshor
 */
public class EnumDomain // implements ISetDomain
{

	BitSet bits;
	Enum enm;

	public EnumDomain(Enum enm, Object[] objs)
	{
		bits = new BitSet(enm.size());
		this.enm = enm;

		for (int i = 0; i < objs.length; i++)
		{
			int idx = enm.getIndex(objs[i]);
			bits.set(idx);
		}
	}

	public EnumDomain(Enum enm, BitSet bits)
	{
		this.bits = bits;
		this.enm = enm;
	}

	/**
	 *
	 */

	public EnumDomain and(EnumDomain sd)
	{
		checkOperand(sd);

		//		if (!bits.intersects(sd.bits))
		//		{
		//			return enm.empty();
		//		}

		if (bits.equals(sd.bits))
			return this;

		BitSet copy = (BitSet) bits.clone();
		copy.and(sd.bits);
		return new EnumDomain(enm, copy);

	}

	void checkOperand(EnumDomain sd)
	{

		if (sd.getEnum() != enm)
			throw new RuntimeException("Can not use subsets of different domains");

	}

	/**
	 *
	 */

	public boolean contains(Object obj)
	{
		int idx = enm.getIndex(obj);
		return bits.get(idx);
	}

	/**
	 *
	 */

	public Iterator iterator()
	{
		return new EnumDomainIterator();
	}

	/**
	 *
	 */

	public EnumDomain not()
	{
		int size = enm.size();

		BitSet bs = (BitSet) bits.clone();

		bs.flip(0, size);

		return new EnumDomain(enm, bs);
	}

	/**
	 *
	 */

	public EnumDomain or(EnumDomain sd)
	{
		checkOperand(sd);

		if (bits.equals(sd.bits))
			return this;

		BitSet copy = (BitSet) bits.clone();
		copy.or(sd.bits);
		return new EnumDomain(enm, copy);
	}

	/**
	 *
	 */

	public EnumDomain sub(EnumDomain sd)
	{
		checkOperand(sd);

		if (bits.equals(sd.bits))
			return this;

		BitSet copy = (BitSet) bits.clone();
		copy.andNot(sd.bits);
		return new EnumDomain(enm, copy);
	}

	/**
	 * @return
	 */
	public Enum getEnum()
	{
		return enm;
	}

	class EnumDomainIterator extends AOpenIterator
	{

		BitSetIterator bsi = new BitSetIterator(bits);

		/**
			 *
			 */

		public boolean hasNext()
		{
			return bsi.hasNext();
		}

		/**
		 *
		 */

		public Object next()
		{
			int idx = bsi.nextInt();
			return enm.objs[idx];
		}

	}
	
	
	public int size()
	{
		return bits.cardinality();
	}
	

	/**
	 *
	 */

	public boolean equals(Object obj)
	{
		if (obj == null || !(obj instanceof EnumDomain))
		  return false;
		  
		EnumDomain ed = (EnumDomain)obj;
		
		return enm.equals(ed.enm) && bits.equals(ed.bits);  
		
	}

	/**
	 *
	 */

	public int hashCode()
	{
		return enm.hashCode() * 37 + bits.hashCode();
	}

}
