package org.openl.util;

import java.io.File;
import java.io.PrintStream;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

/*
 * Dec 25, 2016: Previous implementation used static method for the class Report.
 * So, it did not support multi-threading and could be used for design time analysis only.
 * All global method replaced and changes made in the methods finalizeDecision and 
 * customInitializeDecisionRun in the file "DecisionTableExecuteTemplates.xls"
 */

public class Report implements IReport { 
	String decisionName;
	String path;
	String runPath;
	PrintStream out;
	boolean on = false;
	int numberOfTables = 0;
	int tableNumber = 0;
	HashMap<String, ReportDT> reports;
	
	ReportInMemory reportInMemory;
	
//	private static Thread currentThread = Thread.currentThread();
	
	static final String ruleColor = "'background-color:Navy;color:White'";
//	static final String condColor = "'background-color:Yellow'";
	static final String condColor = "'background-color:LightCyan'";
//	static final String actionColor = "'background-color:YellowGreen'";
	static final String actionColor = "'background-color:LightPink'";
	
	public Report() {
		
	}
	
	public ReportInMemory getReportInMemory() {
		return reportInMemory;
	}
	public void setReportInMemory(ReportInMemory reportInMemory) {
		this.reportInMemory = reportInMemory;
	}
	
	public void resetMemoryReport() {
		reportInMemory = new ReportInMemory(this);
	}
	
	public void setPath(String runPath) {
		this.path += "/" + runPath;
	}
	
	public String getPath() {
		return path;
	}
	
	// open report
	public void open(String path, String decisionName) {
		open(path,decisionName,null);
	}
	
//	public boolean isCurrentThread() {
//		Thread thread = Thread.currentThread();
//		if (!thread.equals(currentThread)) {
//			currentThread = thread;
//			return false;
//		}
//		return true;
//	}
	
	public void open(String path, String decisionName, String decisionRunName) {
//		if (!isCurrentThread())
//			return;
		on = true;
		this.decisionName = decisionName;
		this.path = path;
		tableNumber = 0;
		reports = new HashMap<String,ReportDT>();
		Date d = Calendar.getInstance().getTime();
		//String time = d.getMonth()+"-"+d.getDate()+"."+d.getHours()+"-"+d.getMinutes()+"-"+d.getSeconds();
		
		this.runPath = path;
		if (decisionRunName != null)
			this.runPath = path + "/" + decisionRunName;
		else
			this.runPath = path + "/" + decisionName;
		
		File dir = new File(path);
		if(!dir.exists()){
			Log.info("Create report directory: " + this.path);
			dir.mkdirs();
		}
		
//		File dir = new File(runPath);
//		if(!dir.exists()){
//			Log.info("Create report directory: " + this.runPath);
//			dir.mkdirs();
//		}
//
//		String filename = this.runPath + "/Report." + decisionName + ".html";
//		//System.out.println("Generate " + filename + "...");
//		try {
//			out = new PrintStream(filename, "UTF-8");
//			
//		} catch (Exception e) {
//			System.out.println("Cannot create output file " + filename);
//		}
//		out.println("<meta http-equiv=\"Content-Type\" content=\"text/html;charset=utf-8\">");
//		out.println("<h3>OpenRules Execution Report of " + d + "</h3>");
//		out.println("<h3> Decision \"" + decisionName + "\"</h3>");
//		out.println("<table border='2' bordercolor='black' cellpadding='10'>");
//		// title row
//		out.println("<tr align='center'>");
//		out.println("<th style=" + ruleColor + "> # </th>");
//		out.println("<th style=" + ruleColor + "> Decision Table </th>");
//		out.println("</tr>");
		numberOfTables = 0;
		
		resetMemoryReport();
	}
	
	public void close() {
//		out.println("</table>");
		on = false;
		
		reportInMemory.html();
	}
	
	
	public boolean isOn() {
		return on;
	}
	
	public void startDecisionTable(String tableName, String type,
										  String[] conditionLabels, String[] conditionDescriptions,
										  String[] actionLabels, String[] actionDescriptions) {
//		if (!isCurrentThread())
//			return;
		tableNumber++;
		ReportDT report = new ReportDT(this,tableNumber);
		reports.put(tableName,report);
		String filename = report.startDecisionTable(tableName, type, conditionLabels, conditionDescriptions, 
				                                    actionLabels, actionDescriptions);
//		out.println("<tr align='center'>");
//		out.println("<th>" + tableNumber + "</th>");
//		String back = "../";
//		if (!runPath.equals(path))
//			back = "../../";
//		out.println("<th> <a href=\"" + back + filename + "\" target=\"_blank\">" + tableName + "</a> </th>");
//		out.println("</tr>");	
	}
	
	public void addRule(String tableName, int ruleNumber) {
//		if (!isCurrentThread())
//			return;
		ReportDT report = reports.get(tableName);
		report.addRule(ruleNumber);
	}
	
	public void addRuleCondition(String tableName, String value, int conditionNumber) {
//		if (!isCurrentThread())
//			return;
		ReportDT report = reports.get(tableName);
		report.addRuleCondition(value,conditionNumber);
	}
	
	public void addRuleAction(String tableName, String value, int actionNumber) {
		ReportDT report = reports.get(tableName);
		report.addRuleAction(value,actionNumber);
	}
	
	public void endDecisionTable(String tableName) {
		ReportDT report = reports.get(tableName);
		report.endDecisionTable();
		reports.remove(report);
	}

	public HashMap<String, ReportDT> getReports() {
		return reports;
	}

}
