package org.openl.util;

import java.io.PrintStream;
import java.util.ArrayList;

public class ReportInMemory {
	
	Report report;
	ArrayList<RuleInMemory> rulesInMemory;
	
	RuleInMemory current;
	boolean firstAction;

	public ReportInMemory(Report report) {
		this.report = report;
		this.rulesInMemory = new ArrayList<RuleInMemory>();
	}
	
	public void newRuleInMemory(String table, String number) {
		current = new RuleInMemory();
		current.table = table;
		current.number = number;
		firstAction = true;
		rulesInMemory.add(current);
	}
	
	public ArrayList<RuleInMemory> getRulesInMemory() {
		return rulesInMemory;
	}

	public void addCondition(String condition) {
		if (condition == null)
			return;
		if (current.rule == null) {
			current.rule = "<b>IF</b> " + condition;
		}
		else {
			current.rule += " <b>AND</b> " + condition;
		}
	}
	
	public void addAction(String action) {
		if (action == null)
			return;
		if (firstAction) {
			firstAction = false;
			if (current.rule == null)
				current.rule = action;
			else
				current.rule += " <b>THEN</b> " + action;
		}
		else {
			current.rule += " <b>AND</b> " + action;
		}
	}
	
	
	public void addCurrentRule() {
		rulesInMemory.add(current);
	}
	
	public RuleInMemory getCurrentRule() {
		return current;
	}
	
	public void log() {
		Log.debug("\nReport in Memory");
		for (int i = 0; i < rulesInMemory.size(); i++) {
			RuleInMemory r = rulesInMemory.get(i);
			Log.debug("\n" + r.rule + " || " + r.table + " || " + r.number);
		}
	}
	
	public void html() {
		String filename;
		PrintStream out;
		filename = report.runPath + ".html"; //"/AllExecutedRules.html";
		//System.out.println("Generate " + filename + "...");
		try {
			out = new PrintStream(filename,"UTF-8");
			
		} catch (Exception e) {
			Log.info("Cannot create output file " + filename);
			return;
		}
		out.println("<meta http-equiv=\"Content-Type\" content=\"text/html;charset=utf-8\">");
		out.println("<h3>Decision \"" + report.decisionName + "\" (" + filename + ")</h3>");
		out.println("<h3>Executed Decision Tables and Rules (" + new java.util.Date() + ") </h3>");
		out.println("<table border='2' bordercolor='black' cellpadding='5'>");
		// title row
		out.println("<tr align='center'>");
		out.println("<td style=" + Report.ruleColor + "> Decision Table : Rule#</td>");
		out.println("<td style=" + Report.ruleColor + "> Executed Rule</td>");
		out.println("<td style=" + Report.ruleColor + "> Variables and Values</td>");
		out.println("</tr>");
		// table rows
		for (int i = 0; i < rulesInMemory.size(); i++) {
			RuleInMemory r = rulesInMemory.get(i);
			Log.debug("\n" + r.rule + " || " + r.table + " || " + r.number);
			out.println("<tr>");
			out.println("<td>" + r.table + ":" + r.number + "</td>");
			out.println("<td>" + r.rule + "</td>");
			out.println("<td>");
			for (int j = 0; j < r.getVariables().size(); j++) {
				String variable = r.getVariables().get(j);
				Object value = r.getValues().get(j);
				String valueString;
				if (value == null)
					valueString = "";
				else
					valueString = value.toString();
				if (j > 0)
					out.println("<br>");
				out.println(variable + "=" + valueString);
			}
			out.println("</td>");
			out.println("</tr>");
		}
		out.println("</table>");
		out.close();
	}

}
